/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig;

import java.awt.Component;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.text.DateFormat;
import java.util.Date;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import org.exolab.castor.xml.MarshalException;
import org.exolab.castor.xml.Marshaller;
import org.exolab.castor.xml.ValidationException;

import com.iver.andami.Launcher;
import com.iver.andami.PluginServices;
import com.iver.andami.messages.Messages;
import com.iver.andami.messages.NotificationManager;
import com.iver.andami.plugins.Extension;
import com.iver.cit.gvsig.fmap.layers.LayerFactory;
import com.iver.cit.gvsig.gui.layout.Layout;
import com.iver.cit.gvsig.gui.project.OpenException;
import com.iver.cit.gvsig.gui.project.ProjectWindow;
import com.iver.cit.gvsig.project.Project;
import com.iver.cit.gvsig.project.ProjectFactory;
import com.iver.cit.gvsig.project.ProjectMap;
import com.iver.cit.gvsig.project.ProjectView;
import com.iver.utiles.GenericFileFilter;
import com.iver.utiles.XMLEntity;
import com.iver.utiles.xmlEntity.generate.XmlTag;


/**
 * Extension que proporciona controles para crear proyectos nuevos, abrirlos y
 * guardarlos. Adems los tipos de tabla que soporta el proyecto son aadidos
 * en esta clase.
 *
 * @author Fernando Gonzlez Corts
 */
public class ProjectExtension extends Extension {
	private ProjectWindow projectFrame;
	private Project p;
	/**
	 * @see com.iver.mdiApp.plugins.IExtension#initialize()
	 */
	public void initialize() {
	    try {
            Class.forName("javax.media.jai.EnumeratedParameter");
        } catch (ClassNotFoundException e) {
            NotificationManager.addError("La mquina virtual con la que se ejecuta gvSIG no tiene JAI instalado", e);
        }

		LayerFactory.setWritersPath(PluginServices.getPluginServices(this)
				  .getPluginDirectory()
				  .getAbsolutePath() +
				  File.separator + "drivers");
        
        LayerFactory.setDriversPath(PluginServices.getPluginServices(this)
				  .getPluginDirectory()
				  .getAbsolutePath() +
				  File.separator + "drivers");

        // Recuperamos el ltimo argumento, que se supone
        // que ser el fichero .gvp que queremos abrir.
        // (por enmedio pueden venir o no otros argumentos,
        // por ejemplo el idioma)
        // TODO: Aqu Jaume podra meter lo del backup del proyecto
        // que ha hecho para ValenciaUrbanstica

		/*
		pe = (ProjectExtension) PluginServices.getExtension(com.iver.cit.gvsig.ProjectExtension.class);
		if (projectFile.length()!=0){
			p=pe.readProject(projectFile);
		}
		else
			p = restorePreviousProject();
		pe.setProject(p); */
        String[] theArgs = PluginServices.getArguments();
        String lastArg = theArgs[theArgs.length-1];
        if ((lastArg.endsWith(".gvp")) ||
        		(lastArg.endsWith(".GVP")))
        {
        	PluginServices.getLogger().debug("Intentando cargar el proyecto " + lastArg);
        	File projectFile = new File(lastArg);
        	p = readProject(projectFile);
        	PluginServices.getMainFrame().setTitle(p.getName());
        }
        else
        {
			p = ProjectFactory.createProject();
			p.setName(PluginServices.getText(this, "untitled"));
			p.setModified(false);
			PluginServices.getMainFrame().setTitle(PluginServices.getText(this, "sin_titulo"));
        }
    	projectFrame = new ProjectWindow(this);
		projectFrame.setProject(p);
		showProjectWindow();
	}

	/**
	 * Muestra la ventana con el gestor de proyectos.
	 */
	public void showProjectWindow() {
		PluginServices.getMDIManager().addView(projectFrame);
		if (Project.getSeedProjectWindow()!=null) {
			PluginServices.getMDIManager().changeViewInfo(projectFrame, Project.getSeedProjectWindow());
		}
	}

	/**
	 * Guarda el proyecto actual en disco.
	 */
	private void guardar() {
		if (p.getPath() == null) {
			guardarDialogo();
		} else {
			writeProject(new File(p.getPath()), p);
			projectFrame.refreshControls();
		}
	}
	private void guardarDialogo(){
		JFileChooser jfc = new JFileChooser();
		jfc.addChoosableFileFilter(new GenericFileFilter("gvp",
				PluginServices.getText(this, "tipo_fichero_proyecto")));

		if (jfc.showSaveDialog((Component) PluginServices.getMainFrame()) == JFileChooser.APPROVE_OPTION) {
			File file=jfc.getSelectedFile();
			if (!(file.getPath().endsWith(".gvp") || file.getPath().endsWith(".GVP"))){
				file=new File(file.getPath()+".gvp");
			}
			writeProject(file, p);
			projectFrame.refreshControls();
		}
	}
	/**
	 * Guarda si el proyecto ha sido modificado.
	 *
	 * @return True si se ha guardado correctamente.
	 */
	private boolean modificado() {
		if (p.isModified()) {
		///if (true) {
			//TODO de momento se queda como modificado siempre menos cuando est totalmente vacio que se tomar como no modificado,
			//para poder controlar perfectamente cuando un proyecto ha sido modificado
			//hay que recoger los eventos de cambio de leyenda, cambio de extent, aadir una capa, etc que se encuentran en FMap.

			int res = JOptionPane.showConfirmDialog((Component) PluginServices.getMainFrame(),
					PluginServices.getText(this, "guardar_cambios"),
					"gvSIG",
					JOptionPane.YES_NO_CANCEL_OPTION,
					JOptionPane.INFORMATION_MESSAGE);

			if (res == JOptionPane.YES_OPTION) {
				guardar();
			} else if (res == JOptionPane.CANCEL_OPTION) {
				return false;
			}
		}

		return true;
	}

	/**
	 * @see com.iver.mdiApp.plugins.IExtension#updateUI(java.lang.String)
	 */
	public void execute(String actionCommand) {
		if (actionCommand.equals("NUEVO")) {
			//Si est modificado se pregunta si se quiere guardar el anterior
			if (!modificado()) {
				return;
			}
			ProjectView.numViews=0;
            ProjectMap.numMaps = 0;
			PluginServices.getMDIManager().closeAllViews();
			p = ProjectFactory.createProject();
			p.setName(PluginServices.getText(this, "untitled"));
			p.setModified(false);
			projectFrame.setProject(p);
			showProjectWindow();
			PluginServices.getMainFrame().setTitle(PluginServices.getText(this, "sin_titulo"));
		} else if (actionCommand.equals("ABRIR")) {
			//Si est modificado se pregunta si se quiere guardar el anterior
			if (!modificado()) {
				ProjectView.numViews=0;
                ProjectMap.numMaps = 0;
				return;
			}

			JFileChooser jfc = new JFileChooser();
			jfc.addChoosableFileFilter(new GenericFileFilter("gvp",
					PluginServices.getText(this, "tipo_fichero_proyecto")));

			if (jfc.showOpenDialog((Component) PluginServices.getMainFrame()) == JFileChooser.APPROVE_OPTION) {
				ProjectView.numViews=0;
                ProjectMap.numMaps = 0;
				PluginServices.getMDIManager().closeAllViews();
				Project o = readProject(jfc.getSelectedFile());
				if (o != null) {
					p = o;
				}

				projectFrame.setProject(p);
				PluginServices.getMainFrame().setTitle(p.getName());
				projectFrame.refreshControls();
				showProjectWindow();
			}
		} else if (actionCommand.equals("GUARDAR")) {
			guardar();
		} else if (actionCommand.equals("GUARDAR_COMO")) {
			guardarDialogo();
		} else if (actionCommand.equals("SALIR")){
			int option = JOptionPane.showConfirmDialog(null,
					Messages.getString("MDIFrame.quiere_salir"),
					Messages.getString("MDIFrame.salir"),
					JOptionPane.YES_NO_OPTION);

			if (option == JOptionPane.YES_OPTION) {
				Launcher.closeApplication();
			}
		} else if (actionCommand.compareTo("OPENTEMPLATE")==0){
			openLayout();
		}
	}
	public void openLayout() {
		//Project project = ((ProjectExtension) PluginServices.getExtension(ProjectExtension.class)).getProject();
		Layout layout=null;
		JFileChooser jfc = new JFileChooser();
		jfc.addChoosableFileFilter(new GenericFileFilter("gvt",
				PluginServices.getText(this, "plantilla")));

		if (jfc.showOpenDialog((Component) PluginServices.getMainFrame()) == JFileChooser.APPROVE_OPTION) {
			File file=jfc.getSelectedFile();
			if (!(file.getPath().endsWith(".gvt") || file.getPath().endsWith(".GVT"))){
				file=new File(file.getPath()+".gvt");
			}
			try {
				File xmlFile = new File(file.getAbsolutePath());
				FileReader reader;
				reader = new FileReader(xmlFile);

				XmlTag tag = (XmlTag) XmlTag.unmarshal(reader);
				try {
					layout = Layout.createLayout(new XMLEntity(tag),p);
				} catch (OpenException e) {
					e.showError();
				}
				//fPanelLegendManager.setRenderer(LegendFactory.createFromXML(new XMLEntity(tag)));
			} catch (FileNotFoundException e) {
				NotificationManager.addError(PluginServices.getText(this, "Al_leer_la_leyenda"), e);
			} catch (MarshalException e) {
				NotificationManager.addError(PluginServices.getText(this, "Al_leer_la_leyenda"), e);
			} catch (ValidationException e) {
				NotificationManager.addError(PluginServices.getText(this, "Al_leer_la_leyenda"), e);
			}
			ProjectMap pmap = ProjectFactory.createMap(file.getName());
			pmap.setModel(layout);
			pmap.getModel().setProjectMap(pmap);

			p.addMap(pmap);
			PluginServices.getMDIManager().addView(layout);


		}
	}
	/**
	 * Escribe el proyecto en XML.
	 *
	 * @param file Fichero.
	 * @param p Proyecto.
	 */
	public void writeProject(File file, Project p) {
		// write it out as XML
		try {
			FileWriter writer = new FileWriter(file.getAbsolutePath());
			Marshaller m = new Marshaller(writer);
			m.setEncoding("ISO-8859-1");
			p.setName(file.getName());
			p.setPath(file.toString());
			p.setModificationDate(DateFormat.getDateInstance().format(new Date()));
			p.setModified(false);
			m.marshal(p.getXMLEntity().getXmlTag());
			PluginServices.getMainFrame().setTitle(file.getName());
		} catch (Exception e) {
			NotificationManager.addError("Error guardando el proyecto", e);
		}

	}

	/**
	 * Lee del XML el proyecto.<br><br>
	 *
	 * Reads the XML of the project.<br>
	 * It returns n project object holding all needed info that is not linked to the Project Dialog. <br>In case you want the project to be
	 * linked to the window you must to set this object to the extension:<br>
	 *
	 * <b>Example:</b><br>
	 *
	 * ...<br>
	 * ...<br>
	 * Project p = ProjectExtension.readProject(projectFile);<br>
	 * ProjectExtension.setProject(p);
	 * ...<br>
	 * ...<br>
	 * @param file Fichero.
	 *
	 * @return Project
	 *
	 */
	public Project readProject(File file) {
		Project proj = null;

		try {
			File xmlFile = new File(file.getAbsolutePath());
			FileReader reader;
			reader = new FileReader(xmlFile);
			XmlTag tag = (XmlTag) XmlTag.unmarshal(reader);
			XMLEntity xml=new XMLEntity(tag);
			String VERSION=xml.getStringProperty("VERSION");

			try {
				// if ((VERSION!=null) && (VERSION.equals("0.5") || VERSION.equals("0.4") || (VERSION.indexOf("GISPLANET") != -1))){
				if (VERSION != null) {
					proj = Project.createFromXML(xml);
				}else{
					proj = Project.createFromXML03(new XMLEntity(tag));
				}
				return proj;
			} catch (OpenException e){
				e.showError();
				//NotificationManager.addInfo("Al leer el proyecto", e);
			}
		} catch (FileNotFoundException e) {
			JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),PluginServices.getText(this, "fichero_incorrecto"));
			//NotificationManager.addError("Al leer el proyecto", e);
		} catch (MarshalException e) {
			JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),PluginServices.getText(this, "formato_incorrecto"));
			//NotificationManager.addError("Al leer el proyecto", e);
		} catch (ValidationException e) {
			JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),PluginServices.getText(this, "formato_incorrecto"));
			//NotificationManager.addError("Al leer el proyecto", e);
		}

		return null;
	}

	/**
	 * Devuelve el proyecto.
	 *
	 * @return Proyecto.
	 */
	public Project getProject() {
		return p;
	}

	/**
	 * @see com.iver.andami.plugins.IExtension#isEnabled()
	 */
	public boolean isEnabled() {
		return true;
	}

	/**
	 * @see com.iver.andami.plugins.IExtension#isVisible()
	 */
	public boolean isVisible() {
		return true;
	}

	/**
	 * Sets the project
	 * @param p
	 */
	public void setProject(Project p){
		projectFrame.setProject(p);
		this.p=p;
	}
}
