/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.IOException;
import java.text.ParseException;
import java.util.BitSet;
import java.util.Enumeration;
import java.util.EventListener;
import java.util.EventObject;

import javax.swing.BorderFactory;
import javax.swing.DefaultCellEditor;
import javax.swing.DefaultListSelectionModel;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableColumnModelEvent;
import javax.swing.event.TableColumnModelListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

import org.apache.log4j.Logger;

import com.hardcode.driverManager.DriverLoadException;
import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueFactory;
import com.iver.andami.PluginServices;
import com.iver.andami.messages.NotificationManager;
import com.iver.andami.ui.mdiFrame.MainFrame;
import com.iver.andami.ui.mdiManager.SingletonView;
import com.iver.andami.ui.mdiManager.ViewInfo;
import com.iver.andami.ui.mdiManager.ViewListener;
import com.iver.andami.ui.mdiManager.ViewTransform;
import com.iver.cit.gvsig.fmap.FMap;
import com.iver.cit.gvsig.fmap.core.DefaultFeature;
import com.iver.cit.gvsig.fmap.core.DefaultRow;
import com.iver.cit.gvsig.fmap.core.IFeature;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.core.IRow;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.edition.EditionEvent;
import com.iver.cit.gvsig.fmap.edition.EditionException;
import com.iver.cit.gvsig.fmap.edition.IEditableSource;
import com.iver.cit.gvsig.fmap.edition.IRowEdited;
import com.iver.cit.gvsig.fmap.edition.VectorialEditableAdapter;
import com.iver.cit.gvsig.fmap.edition.commands.CommandListener;
import com.iver.cit.gvsig.fmap.edition.writers.dbf.DbfWriter;
import com.iver.cit.gvsig.fmap.edition.writers.shp.ShpWriter;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrVect;
import com.iver.cit.gvsig.fmap.layers.LayerFactory;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;
import com.iver.cit.gvsig.fmap.layers.SelectionEvent;
import com.iver.cit.gvsig.fmap.layers.SelectionListener;
import com.iver.cit.gvsig.gui.tables.Column;
import com.iver.cit.gvsig.gui.tables.EditionTable;
import com.iver.cit.gvsig.gui.tables.PopupMenu;
import com.iver.cit.gvsig.project.ProjectTable;
import com.iver.utiles.SimpleFileFilter;
import com.iver.utiles.swing.jtable.FieldSelectionEvent;
import com.iver.utiles.swing.jtable.FieldSelectionListener;
import com.iver.utiles.swing.jtable.SelectionHeaderSupport;

/**
 * DOCUMENT ME!
 *
 * @author Fernando Gonzlez Corts
 */
public class Table extends JPanel implements SingletonView, EditionTable,
    ViewListener,CommandListener, ViewTransform {
    private static Logger logger = Logger.getLogger(Table.class.getName());
    private javax.swing.JScrollPane jScrollPane = null;
    protected javax.swing.JTable table = null;
    protected ProjectTable model = null;
    protected JLabel jLabelStatus = null;
    protected FMap fmap;
    private boolean updating = false;
    private TableSelectionListener selectionListener = new TableSelectionListener();
    private long numReg = 0;
    protected SelectionHeaderSupport headerSelectionSupport = new SelectionHeaderSupport();
    private long[] orderIndexes = null;
    private long[] orderIndexesInverted = null;
    private IRow[] rowsCopied = null;
    private ViewInfo m_viewInfo = null;
	private boolean isPalette=false;

    /**
     * This is the default constructor
     */
    public Table() {
        super();
        initialize();
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public ProjectTable getModel() {
        return model;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public BitSet getSelectedFieldIndices() {
    	BitSet bs=headerSelectionSupport.getSelectedColumns();
    	BitSet newbs=new BitSet();
    	for (int i =1;i<bs.length();i++){
    		if (bs.get(i))
    			newbs.set(i-1);
    	}
        return newbs;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public int[] getSelectedRowIndices() {
        return getTable().getSelectedRows();
    }

    /**
     * DOCUMENT ME!
     */
    protected void refreshControls() {
        try {
            MainFrame mF = PluginServices.getMainFrame();

            if (mF != null) {
                PluginServices.getMDIManager().getViewInfo(Table.this).setTitle(PluginServices.getText(
                        this, "Tabla") + ": " + model.getName());
            }

            if (model.getAssociatedTable() != null) {
                this.fmap = ((FLayer) model.getAssociatedTable()).getFMap();
            } else {
                this.fmap = null;
            }

            SelectableDataSource dataSource = (SelectableDataSource) model.getModelo()
                                                                          .getRecordset();
            logger.debug("dataSource.start()");
            dataSource.start();


            ColumnModel cm=new ColumnModel();
            cm.setDataModel(model);
            getTable().setColumnModel(cm);

            AbstractTableModel dataModel = new DataSourceDataModel(model);

            getTable().setModel(dataModel);
            TableColumn column = null;
			for (int i = 0; i < model.getColumnCount(); i++) {
			    column = table.getColumnModel().getColumn(i);
			    int w=model.getColumn(i).getWidth();
			    column.setPreferredWidth(w); //sport column is bigger

			}
			headerSelectionSupport.setTableHeader(getTable().getTableHeader());
            headerSelectionSupport.addFieldSelectionListener(new FieldSelectionListener() {
                    public void fieldSelected(FieldSelectionEvent e) {
                        if (PluginServices.getMainFrame() != null) {
                            PluginServices.getMainFrame().enableControls();
                        }
                    }
                });

            model.getModelo().getRecordset().addSelectionListener(selectionListener);

            updateSelection();
        } catch (DriverException e) {
            NotificationManager.addError("No se pudo leer la informacin", e);
        } catch (DriverLoadException e) {
            NotificationManager.addError("No se pudo leer la informacin", e);
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param table DOCUMENT ME!
     */
    public void setModel(ProjectTable table) {
        model = table;

        //Gestin del nombre de la ventana
        model.addPropertyChangeListener(new PropertyChangeListener() {
                public void propertyChange(PropertyChangeEvent evt) {
                    if (evt.getPropertyName().equals("name")) {
                        PluginServices.getMDIManager().getViewInfo(Table.this)
                                      .setTitle(PluginServices.getText(this,
                                "Tabla") + ": " + (String) evt.getNewValue());
                    } else if (evt.getPropertyName().equals("model")) {
                        refreshControls();
                    }
                }
            });

        refreshControls();
        TableColumn tc=getTable().getColumnModel().getColumn(0);
        tc.setCellRenderer(new MyTableCellRenderer());


    }
    class MyTableCellRenderer extends JLabel implements TableCellRenderer {
        final Border grayBorder = BorderFactory.createLineBorder(Color.darkGray);

        MyTableCellRenderer() {

        }

        public Component getTableCellRendererComponent(javax.swing.JTable table,
            Object value, boolean isSelected, boolean hasFocus, int row,
            int col) {
        	JLabel label=new JLabel(String.valueOf(row+1));
         	label.setForeground(Color.blue);
        	label.setOpaque(true);
         	if (isSelected)
        		label.setBackground(Color.yellow);
        	else
        		label.setBackground(UIManager.getColor("Button.background"));

        	label.setBorder(grayBorder);
        	return label;
        }
      }

    /**
     *
     */
    public void updateSelection() {
        updating = true;

        try {
            DefaultListSelectionModel sm = (DefaultListSelectionModel) getTable()
                                                                           .getSelectionModel();
            sm.clearSelection();

            BitSet bs = (model.getModelo().getRecordset()).getSelection();
            sm.setValueIsAdjusting(true);

            if (orderIndexes != null) {
                for (int i = 0; i < orderIndexes.length; i++) {
                    if (bs.get(i)) {
                        sm.addSelectionInterval((int) orderIndexesInverted[i],
                            (int) orderIndexesInverted[i]);
                    }
                }
            } else {
                for (int i = bs.nextSetBit(0); i >= 0;
                        i = bs.nextSetBit(i + 1)) {
                    sm.addSelectionInterval(i, i);
                }
            }

            sm.setValueIsAdjusting(false);
            try {
                numReg = model.getModelo().getRowCount();
            } catch (IOException e) {
                e.printStackTrace();
            }

            jLabelStatus.setText(" " +
                ((SelectableDataSource) model.getModelo().getRecordset()).getSelection()
                 .cardinality() + " / " + numReg + " " +
                PluginServices.getText(this, "registros_seleccionados_total") +
                ".");
        } catch (DriverIOException e) {
            e.printStackTrace();
        } catch (DriverLoadException e) {
            e.printStackTrace();
        }

        updating = false;
    }

    /**
     * This method initializes this
     */
    protected void initialize() {
        jLabelStatus = new JLabel();
        setLayout(new java.awt.BorderLayout());
        jLabelStatus.setText("");
        jLabelStatus.setName("");
        jLabelStatus.setPreferredSize(new java.awt.Dimension(100, 18));
        add(getJScrollPane(), java.awt.BorderLayout.CENTER);
        this.add(jLabelStatus, java.awt.BorderLayout.SOUTH);
    }

    /**
     * This method initializes table
     *
     * @return javax.swing.JTable
     */
    public javax.swing.JTable getTable() {
        if (table == null) {
            table = new javax.swing.JTable();
            table.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_OFF);
            table.setSelectionModel(new DefaultListSelectionModel());
            table.getTableHeader().addMouseListener(new MouseHandler());
            table.addKeyListener(new TableKeyListener());
            table.addMouseListener(new MouseRow());
            table.setSelectionForeground(Color.blue);
    		table.setSelectionBackground(Color.yellow);
    		table.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
                    public void valueChanged(ListSelectionEvent e) {

                    	if (updating) {
                         	return;
                        }

                        SelectableDataSource dataModel = null;

                        try {
                            dataModel = Table.this.model.getModelo()
                                                        .getRecordset();
                        } catch (DriverLoadException e1) {
                            e1.printStackTrace();
                        }

                        DefaultListSelectionModel model = (DefaultListSelectionModel) table.getSelectionModel();
                        BitSet selection = dataModel.getSelection();

                        for (int i = e.getFirstIndex(); i <= e.getLastIndex();
                                i++) {
                            if (orderIndexes != null) {
                                selection.set((int) orderIndexes[i],
                                    model.isSelectedIndex(i));
                            } else {
                                selection.set(i, model.isSelectedIndex(i));
                            }
                        }

                        if (e.getValueIsAdjusting() == false) {
                            if (fmap != null) {
                                fmap.endAtomicEvent();
                            }

                            dataModel.fireSelectionEvents();
                        } else {
                            if (fmap != null) {
                                fmap.beginAtomicEvent();
                            }
                        }

                        jLabelStatus.setText(" " + selection.cardinality() +
                            " / " + numReg + " " +
                            PluginServices.getText(this,
                                "registros_seleccionados_total") + ".");

                    }
                });

       		JTextField tf=new JTextField();
    		table.setDefaultEditor(Number.class, new DefaultEditor(tf));
            table.setDefaultEditor(String.class, new DefaultEditor(tf));
            table.setDefaultEditor(Object.class, new DefaultEditor(tf));
        }

        return table;
    }
    protected class DefaultEditor extends DefaultCellEditor {
		public Component getTableCellEditorComponent(javax.swing.JTable table,
				Object value, boolean isSelected, int row, int column) {
			JTextField tf=(JTextField)super.getTableCellEditorComponent(table, value, isSelected,
					row, column);
			if (isSelected){
				tf.setBackground(new Color(230,220,220));
				tf.selectAll();

			}
			return tf;
		}

		public DefaultEditor(JTextField tf) {
			super(tf);
			 //MouseListener[] mouseListeners=getTable().getEditorComponent().getMouseListeners();
	           // if (!mouseListeners[mouseListeners.length-1].getClass().equals(MouseRow.class)){
	           tf.addMouseListener(new MouseRow());
	           // }
			getComponent().addKeyListener(new KeyAdapter() {
				int keyPressed = 0;

				public void keyPressed(KeyEvent ke) {
					if (ke.getKeyCode() != KeyEvent.VK_TAB)
						keyPressed++;
					JTextField tf = (JTextField) getComponent();


					if (ke.getKeyCode() == KeyEvent.VK_RIGHT
							|| ke.getKeyCode() == KeyEvent.VK_ENTER) {
						int caretPosition = tf.getCaretPosition();
						if (caretPosition >= tf.getText().length()) {
							int x = table.getSelectedColumn();
							int y = table.getSelectedRow();
							if (x + 1 >= table.getColumnCount()) {
								x = 0;
								y++;
							} else {
								x++;
							}
							//table.changeSelection(y, x, false, true);
							//initEditField(x,y);
							getComponent().setEnabled(false);
						}

					} else if (ke.getKeyCode() == KeyEvent.VK_LEFT) {
						int caretPosition = tf.getCaretPosition();
						if (caretPosition <= 0) {
							int x = table.getSelectedColumn();
							int y = table.getSelectedRow();
							if (x == 0) {
								x = table.getColumnCount() - 1;
								if (y - 1 < 0)
									y = table.getRowCount() - 1;
								else
									y--;
							} else {
								x--;
							}
							//table.changeSelection(y, x, false, true);
							//initEditField(x,y);
							getComponent().setEnabled(false);
						}

					}
				}

				public void keyReleased(KeyEvent ke) {
					JComponent editor = (JComponent) DefaultEditor.this
							.getComponent();
					if (editor instanceof JTextField) {
						if (keyPressed == 0) {
							((JTextField) editor).setText(String.valueOf(ke
									.getKeyChar()));
						}
					}
					keyPressed--;
					if (keyPressed < 0)
						keyPressed = 0;
					getComponent().setEnabled(true);
				}
			});

		}

		public Object getCellEditorValue() {
			String s = ((JTextField) (DefaultEditor.this.getComponent()))
					.getText();
			getComponent().setEnabled(true);
			return s;
		}

		public boolean isCellEditable(EventObject event) {
			// IF NUMBER OF CLICKS IS LESS THAN THE CLICKCOUNTTOSTART RETURN
			// FALSE
			// FOR CELL EDITING.
			if (event instanceof MouseEvent) {
				return ((MouseEvent) event).getClickCount() >= getClickCountToStart();
			}

			return true;
		}
	}




    /**
	 * This method initializes jScrollPane
	 *
	 * @return javax.swing.JScrollPane
	 */
    protected javax.swing.JScrollPane getJScrollPane() {
        if (jScrollPane == null) {
            jScrollPane = new javax.swing.JScrollPane();
            jScrollPane.setViewportView(getTable());
        }

        return jScrollPane;
    }

    /**
	 * @see com.iver.mdiApp.ui.MDIManager.SingletonView#getViewModel()
	 */
    public Object getViewModel() {
        return model;
    }

    /**
	 * This method is used to get <strong>an initial</strong> ViewInfo object
	 * for this Table. It is not intended to retrieve the ViewInfo object in a
	 * later time. <strong>Use PluginServices.getMDIManager().getViewInfo(view)
	 * to retrieve the ViewInfo object at any time after the creation of the
	 * object.
	 *
	 * @see com.iver.mdiApp.ui.MDIManager.View#getViewInfo()
	 */
    public ViewInfo getViewInfo() {
    	if (m_viewInfo==null) {
    		m_viewInfo = new ViewInfo(ViewInfo.ICONIFIABLE |
    	            ViewInfo.MAXIMIZABLE | ViewInfo.RESIZABLE);
    		m_viewInfo.setTitle(PluginServices.getText(this, "Tabla")+ " : " +model.getName());
    		m_viewInfo.setWidth(300);
    		m_viewInfo.setHeight(200);
    	}
        return m_viewInfo;
    }

    /**
     * DOCUMENT ME!
     *
     * @param indexes
     *
     * @throws IOException
     */
    public void setOrder(long[] indexes) throws IOException {
        orderIndexes = indexes;
        orderIndexesInverted = new long[orderIndexes.length];

        for (int i = 0; i < indexes.length; i++) {
            orderIndexesInverted[(int) indexes[i]] = i;
        }

        updating = true;
        ((DataSourceDataModel) getTable().getModel()).fireTableDataChanged();
        updating = false;
        updateSelection();
    }

    /*  public void startEditing() throws DriverException{
       // dw = model.getModelo().getDataWare(DataSourceFactory.DATA_WARE_COHERENT_ROW_ORDER);
       // dw.beginTrans();
       }
     */

    /**
     * DOCUMENT ME!
     */

    /*  public void cancelEditing() throws DriverException {
       dw.rollBackTrans();
       dw = null;
       }
     */

    /**
     * DOCUMENT ME!
     */

    /*  public void stopEditing() throws DriverException {
       // dw.commitTrans();
       // dw = null;
       // model.getModelo().stop();
       // model.getModelo().start();
       }
     */

    /**
     * DOCUMENT ME!
     */

    /*  public boolean isEditing() {
       return dw != null;
       }
     */

    /**
     * Quita los campos seleccionados
     */
    public void clearSelectedFields() {
        headerSelectionSupport.clearSelectedColumns();
    }

    /**
     * DOCUMENT ME!
     *
     * @param index DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public Value[] getValueRow(int index) {
        DataSourceDataModel dsdm = (DataSourceDataModel) getTable().getModel();
        Value[] values = new Value[dsdm.getColumnCount()];

        for (int i = 0; i < dsdm.getColumnCount(); i++) {
            values[i] = (Value) dsdm.getValueAt(index, i);
        }

        return values;
    }

    /* private void refresh() throws DriverException{
       //dw.commitTrans();
       //model.getModelo().stop();
       //dw.beginTrans();
           //DataSourceDataModel dsdm=(DataSourceDataModel)getTable().getModel();
           //dsdm.fireTableDataChanged();
       }*/
    /*  public void addEmptyRow() throws DriverException{
       ValueCollection valuePK=new ValueCollection();
           valuePK.setValues(new Value[]{ValueFactory.createValue(dw.getRowCount())});
           dw.insertEmptyRow(valuePK);
           refresh();
       }
     */
    /*        public void copySelectedRows() throws DriverException{
       int[] sel=getSelectedRowIndices();
       for(int i=0;i<sel.length;i++){
               rowsCopied.add(getValueRow(sel[i]));
       }
       }
     */
    /*        public void addSelectionToEnd() throws DriverException {
       for (int i=0;i<rowsCopied.size();i++){
               dw.insertFilledRow((Value[])rowsCopied.get(i));
       }
       refresh();
       }
     */
    /*public void delSelectionRow() throws DriverException{
       int[] sel=getSelectedRowIndices();
       for(int i=sel.length-1;i>=0;i--){
               dw.deleteRow(sel[i]);
       }
       refresh();
       }
     */
    /*public boolean isCopy(){
       return !rowsCopied.isEmpty();
       }
     */
    /*
       public void addSelectionToRow() throws DriverException {
                   int[] sel=getSelectedRowIndices();
                           dw.insertFilledRow((Value[])rowsCopied.get(i),sel[0]);
                   refresh();
           }
     */
    public void startEditing() throws EditionException {
       getModel().getModelo().startEdition(EditionEvent.ALPHANUMERIC);
       //int row=table.getSelectedRow();
       //int column=table.getSelectedColumn();
       //if (row!=-1 && column!=-1)
       //   initEditField(column,row);

//       View[] views = (View[]) PluginServices.getMDIManager().getAllViews();
//
//		for (int i=0 ; i<views.length ; i++){
//			if (views[i] instanceof com.iver.cit.gvsig.gui.View){
//				com.iver.cit.gvsig.gui.View view=(com.iver.cit.gvsig.gui.View)views[i];
//				FLayers layers =view.getMapControl().getMapContext().getLayers();
//				for (int j=0;j<layers.getLayersCount();j++){
//					if (layers.getLayer(j).equals(this.getModel().getAssociatedTable())){
//						view.showConsole();
//						((FLyrVect)layers.getLayer(j)).setActive(true);
//						((FLyrVect)layers.getLayer(j)).setEditing(true);
//					}
//				}
//			}
//		}
    }
    private void initEditField(int[] x,int[] y){
    	if (getTable().getRowCount()>0){
    	if (isEditing()) {

			if (x.length == 1 && y.length == 1)
				table.editCellAt(y[0], x[0]);
			JTextField tf = (JTextField) table.getEditorComponent();
			if (tf != null) {
				tf.selectAll();
				tf.requestFocus();
			}
			if (y.length > 0 && x.length > 0) {
				getTable().setRowSelectionInterval(y[0], y[y.length - 1]);
				getTable().setColumnSelectionInterval(x[0], x[x.length - 1]);
			}

		}
    	}
    }
    /**
	 * DOCUMENT ME!
	 */
    public void stopEditing() {
        try {
            FLyrVect lyr = (FLyrVect) getModel().getAssociatedTable();

            if ((lyr != null) &&
                    lyr.getSource() instanceof VectorialEditableAdapter) {
                JFileChooser jfc = new JFileChooser();
                SimpleFileFilter filterShp = new SimpleFileFilter("shp",
                        "Ficheros .shp");
                jfc.setFileFilter(filterShp);

                if (jfc.showSaveDialog(
                            (Component) PluginServices.getMainFrame()) == JFileChooser.APPROVE_OPTION) {
                    File newFile = jfc.getSelectedFile();

                    //VectorialEditableAdapter vea = (VectorialEditableAdapter) layer.getSource();
                    // File newFile = vea.getDriver().
                    VectorialEditableAdapter vea = (VectorialEditableAdapter) lyr.getSource();
                    ShpWriter writer = (ShpWriter)LayerFactory.getWM().getWriter("Shape Writer");
                    writer.setFile(newFile);
                    writer.initialize(lyr);
                    vea.stopEdition(writer,EditionEvent.ALPHANUMERIC);
                    lyr.setSource(vea.getOriginalAdapter());
                    lyr.setEditing(false);


//                    View[] views = (View[]) PluginServices.getMDIManager().getAllViews();
//
//            		for (int i=0 ; i<views.length ; i++){
//            			if (views[i] instanceof com.iver.cit.gvsig.gui.View){
//            				com.iver.cit.gvsig.gui.View view=(com.iver.cit.gvsig.gui.View)views[i];
//            				FLayers layers =view.getMapControl().getMapContext().getLayers();
//            				for (int j=0;j<layers.getLayersCount();j++){
//            					if (layers.getLayer(i).equals(this.getModel().getAssociatedTable())){
//            						view.showConsole();
//            						((FLyrVect)layers.getLayer(i)).setEditing(false);
//            					}
//            				}
//            			}
//            		}



                    ///vista.hideConsole();
                }
            } else {

            	 JFileChooser jfc = new JFileChooser();
                 SimpleFileFilter filterShp = new SimpleFileFilter("dbf",
                         "Ficheros .dbf");
                 jfc.setFileFilter(filterShp);

                 if (jfc.showSaveDialog(
                             (Component) PluginServices.getMainFrame()) == JFileChooser.APPROVE_OPTION) {
                     File newFile = jfc.getSelectedFile();

                     IEditableSource ies=getModel().getModelo();
                     DbfWriter writer = (DbfWriter)LayerFactory.getWM().getWriter("DBF Writer");
                     writer.initialize(newFile,getModel().getModelo());
                     ies.stopEdition(writer,EditionEvent.ALPHANUMERIC);

                     ///vista.hideConsole();
                 }

/*
                GdbmsWriter gdbmswriter = new GdbmsWriter(getModel().getModelo()
                                                              .getRecordset()
                                                              .getDataWare(DataSourceFactory.DATA_WARE_COHERENT_ROW_ORDER));
                gdbmswriter.preProcess();

                for (int i = 0; i < getModel().getModelo().getRowCount();
                        i++) {
                    gdbmswriter.process(getModel().getModelo().getRow(i));
                }

                gdbmswriter.postProcess();
  */
            }

            //layer.setSource(getModel().getModelo().getOriginalAdapter());
            //layer.setEditing(false);
            //vista.hideConsole();
            //}
        } catch (Exception e) {
            NotificationManager.addError("No se pudo guardar la edicin", e);
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param index DOCUMENT ME!
     */
    public void hideColumns(int[] index) {
        // TODO Auto-generated method stub
    }

    /**
     * DOCUMENT ME!
     *
     * @param index DOCUMENT ME!
     */
    public void setUneditableColumns(int[] index) {
        // TODO Auto-generated method stub
    }

    /**
     * DOCUMENT ME!
     *
     * @param numColumns DOCUMENT ME!
     * @param values DOCUMENT ME!
     */
    public void setDefaultValues(int[] numColumns, Value[] values) {
        // TODO Auto-generated method stub
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public Value getDefaultValue() {
        // TODO Auto-generated method stub
        return null;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public int[] getSelectedColumns() {
        return null;
    }

    /**
     * DOCUMENT ME!
     *
     * @throws IOException DOCUMENT ME!
     */
    public void cancelEditing() throws IOException {
        getModel().getModelo().cancelEdition(EditionEvent.ALPHANUMERIC);
        //refresh();
        //refreshControls();
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public boolean isEditing() {
        return getModel().getModelo().isEditing();
    }

    /**
     * DOCUMENT ME!
     */
    public void refresh() {
    	int row=table.getSelectedRow();
    	int column=table.getSelectedColumn();
    	if (row!=-1 && column!=-1 && table.getEditorComponent()!=null){
    		Value[] values=getValueRow(row);
    		JTextField jtf=(JTextField)table.getEditorComponent();
    		jtf.setText(values[column].toString());
    	}
        updating = true;
        ((DataSourceDataModel) getTable().getModel()).fireTableDataChanged();

        updating = false;
        updateSelection();
        PluginServices.getMainFrame().enableControls();

    }

    /**
     * Add the rows that are passed like parameter and if parameter is null a
     * row is added empties.
     *
     * @param rows Rows to add or null.
     *
     * @throws DriverIOException
     * @throws IOException
     */
    public void addRow(IRow[] rows) throws DriverIOException, IOException {
        try {
            if (rows == null) {
            	if (getModel().getAssociatedTable()!=null){
            		JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),"No se puede aadir una fila a una tabla asociada a una capa.");
            		return;
            	}
                IRow row;
                int numAttr=getModel().getModelo()
                	.getRecordset()
                	.getFieldCount();
            	Value[] values=new Value[numAttr];
            	for (int i=0;i<numAttr;i++){
            		values[i]=ValueFactory.createNullValue();
            	}
                row = new DefaultRow(values);
                getModel().getModelo().addRow(row,"Fila en blanco",EditionEvent.ALPHANUMERIC);
            } else {
            	getModel().getModelo().startComplexRow();
                for (int i = 0; i < rows.length; i++) {
                    getModel().getModelo().addRow(((IRowEdited) rows[i]).getLinkedRow(),"Pegar filas",EditionEvent.ALPHANUMERIC);
                }
                getModel().getModelo().endComplexRow();
            }
        } catch (DriverException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (DriverLoadException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        refresh();

    }

    /**
     * Copy in the arraylist the rows that be selected just then.
     *
     * @throws DriverIOException
     * @throws IOException
     */
    public void copyRow() throws DriverIOException, IOException {
        int[] index = getSelectedRowIndices();
        rowsCopied = new IRow[index.length];

        for (int i = 0; i < index.length; i++) {
            rowsCopied[i] = getModel().getModelo().getRow(index[i]);
        }
    }

    /**
     * Cut the rows that be selected just then.
     *
     * @throws DriverIOException
     * @throws IOException
     */
    public void cutRow() throws DriverIOException, IOException {
        int[] index = getSelectedRowIndices();
        rowsCopied = new IRow[index.length];

        for (int i = 0; i < index.length; i++) {
            rowsCopied[i] = getModel().getModelo().getRow(index[i]);
        }

        removeRow();
    }

    /**
     * Remove in the rows that be selected just then.
     *
     * @throws DriverIOException
     * @throws IOException
     */
    public void removeRow() throws DriverIOException, IOException {
    	int[] index = getSelectedRowIndices();
        getModel().getModelo().startComplexRow();
        for (int i = index.length-1; i >=0; i--) {
            getModel().getModelo().removeRow(index[i],"Eliminar fila", EditionEvent.ALPHANUMERIC);
        }
        int[] sel={0};
        getTable().setEditingRow(0);
        initEditField(sel,sel);

        getModel().getModelo().endComplexRow();
        getTable().clearSelection();

        refresh();
        //repaintAssociatedView();
    }

//    private void repaintAssociatedView(){
//    	 View[] views = (View[]) PluginServices.getMDIManager().getAllViews();
//
// 		for (int i=0 ; i<views.length ; i++){
// 			if (views[i] instanceof com.iver.cit.gvsig.gui.View){
// 				com.iver.cit.gvsig.gui.View view=(com.iver.cit.gvsig.gui.View)views[i];
// 				FLayers layers =view.getMapControl().getMapContext().getLayers();
// 				for (int j=0;j<layers.getLayersCount();j++){
// 					if (layers.getLayer(j).equals(this.getModel().getAssociatedTable())){
// 						view.getMapControl().getMapContext().invalidate();
// 					}
// 				}
// 			}
// 		}
//    }
    /**
     * DOCUMENT ME!
     */
    public void addColumn() {
        // TODO Auto-generated method stub
        refresh();
        //repaintAssociatedView();
    }

    /**
     * DOCUMENT ME!
     */
    public void removeColumn() {
        // TODO Auto-generated method stub
        refresh();
        //repaintAssociatedView();
    }

    /**
     * Return if we have rows copied or not.
     *
     * @return True if we have rows copied.
     */
    public boolean isCopied() {
        return (rowsCopied != null);
    }

    /**
     * Paste the arraylist rows.
     *
     * @throws DriverIOException
     * @throws IOException
     */
    public void pasteRow() throws DriverIOException, IOException {
        addRow(rowsCopied);
        //repaintAssociatedView();
    }

    /**
     * DOCUMENT ME!
     */
    public void viewActivated() {
        //if (isEditing() && getModel().getModelo() instanceof VectorialEditableAdapter)
        updateSelection();
    }

    /**
     * DOCUMENT ME!
     */
    public void viewClosed() {
        // TODO Auto-generated method stub
    }

    /**
     * DOCUMENT ME!
     *
     * @author Fernando Gonzlez Corts
     */
    public class DataSourceDataModel extends AbstractTableModel {
        //private SelectableDataSource dataSource;
        private IEditableSource des;
        private String[] aliases;
        private int[] mapping;

        /**
         * Crea un nuevo DataSourceDataModel.
         *
         * @param pt DOCUMENT ME!
         */
        public DataSourceDataModel(ProjectTable pt) {
            this.des = pt.getModelo();

            //try {
            //dataSource = pt.getModelo().getRecordset();
            //} catch (DriverLoadException e) {
            // TODO Auto-generated catch block
            //	e.printStackTrace();
            //}
            aliases = pt.getAliases();
            mapping = pt.getMapping();
        }

        /**
         * Returns the name of the field.
         *
         * @param col index of field
         *
         * @return Name of field
         */
        public String getColumnName(int col) {
        	return aliases[mapping[col]];
        }

        /**
         * Returns the number of fields.
         *
         * @return number of fields
         */
        public int getColumnCount() {
            return mapping.length;
        }

        /**
         * Returns number of rows.
         *
         * @return number of rows.
         */
        public int getRowCount() {
            try {
                return (int) des.getRowCount();
            } catch (DriverIOException e) {
                return 0;
            } catch (IOException e) {
                return 0;
            }
        }

        /**
         * DOCUMENT ME!
         *
         * @param row DOCUMENT ME!
         * @param col DOCUMENT ME!
         *
         * @return DOCUMENT ME!
         */
        public Object getValueAt(int row, int col) {

           if (col==0){
        	   return null;
           }
           col--;

        	try {
				if (orderIndexes != null) {
					Object obj =null;

					IRowEdited rowEdited=des.getRow((int)orderIndexes[row]);
					obj= rowEdited.getAttribute(mapping[col]);
					return obj;
				} else {
					Object obj =null;

						IRowEdited rowEdited=des.getRow(row);
						obj= rowEdited.getAttribute(mapping[col]);
						//obj = sds.getFieldValue(row, col);
					return obj;
				}
				} catch (DriverIOException e1) {
					return ValueFactory.createValue("").toString();
				} catch (IOException e1) {
					return ValueFactory.createValue("").toString();
				}
	    }

        /**
         * DOCUMENT ME!
         *
         * @param rowIndex DOCUMENT ME!
         * @param columnIndex DOCUMENT ME!
         *
         * @return DOCUMENT ME!
         */
        public boolean isCellEditable(int rowIndex, int columnIndex) {
        	if (columnIndex==0)
                return false;

        	return isEditing();
        }

        /**
         * DOCUMENT ME!
         *
         * @param aValue DOCUMENT ME!
         * @param rowIndex DOCUMENT ME!
         * @param columnIndex DOCUMENT ME!
         *
         * @throws RuntimeException DOCUMENT ME!
         */
        public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        	 if (columnIndex==0)
                 throw new UnsupportedOperationException("Row Number is a read-only column");
        	 columnIndex--;
        	Value v;
            if (getValueAt(rowIndex,columnIndex+1)==null || getValueAt(rowIndex,columnIndex+1).toString().equals(aValue))
            	return;
            try {
                v = ValueFactory.createValueByType(aValue.toString(),
                        des.getRecordset().getFieldType(columnIndex));

                IRowEdited row = des.getRow(rowIndex); //.getAttribute(columnIndex);
                Value[] values = row.getAttributes();
                values[columnIndex] = v;

                IRow newRow = null;

                if (row.getLinkedRow() instanceof IFeature) {
                    IGeometry geometry = ((DefaultFeature) row.getLinkedRow()).getGeometry();
                    newRow = new DefaultFeature(geometry, values,row.getID());
                } else {
                    newRow = new DefaultRow(values,row.getID());
                }

                des.modifyRow(rowIndex, newRow,"Editar valor", EditionEvent.ALPHANUMERIC);
            } catch (DriverException e1) {
                throw new RuntimeException(e1);
            } catch (ParseException e) {
                throw new RuntimeException(e);
            } catch (DriverLoadException e) {
                throw new RuntimeException(e);
            } catch (DriverIOException e) {
                throw new RuntimeException(e);
            } catch (IOException e) {
                throw new RuntimeException(e);
            } catch (NumberFormatException e) {
			/*	NotificationManager.addError(PluginServices.-getText(this,"numero_incorrecto")+
						"\n"+PluginServices.-getText(this,"fila")+" = "+rowIndex+
						"\n"+PluginServices.-getText(this,"columna")+ " = "+columnIndex,e);
			*/
			}
        }
    }
    public class ColumnModel extends DefaultTableColumnModel{
    	private ProjectTable pt;
		public ColumnModel() {
		}

		public void setDataModel(ProjectTable model) {
			this.pt=model;
		}

		public void addColumn(TableColumn tc) {
			super.addColumn(tc);
		}

		public TableColumn getColumn(int i) {
			TableColumn tc=super.getColumn(i);
			return tc;
		}

		public int getColumnCount() {
			return super.getColumnCount();
		}

		public void removeColumn(TableColumn arg0) {
			super.removeColumn(arg0);
		}

		public int getColumnIndex(Object arg0) {
			return super.getColumnIndex(arg0);
		}

		public int getColumnIndexAtX(int arg0) {
			return super.getColumnIndexAtX(arg0);
		}

		public int getColumnMargin() {
			return super.getColumnMargin();
		}

		public Enumeration getColumns() {
			return super.getColumns();
		}

		public int getTotalColumnWidth() {
			int w=0;
			for (int i=0;i<getColumnCount();i++){
				w+=pt.getColumn(i).getWidth();
			}
			return super.getTotalColumnWidth();
		}

		public void moveColumn(int arg0, int arg1) {
			//super.moveColumn(arg0, arg1);
			if (arg0==arg1){
				super.moveColumn(arg0,arg1);
			}
		}

		public void addColumnModelListener(TableColumnModelListener arg0) {
			super.addColumnModelListener(arg0);
		}

		protected ListSelectionModel createSelectionModel() {
			return super.createSelectionModel();
		}

		protected void fireColumnAdded(TableColumnModelEvent arg0) {
			super.fireColumnAdded(arg0);
		}

		protected void fireColumnMarginChanged() {
			super.fireColumnMarginChanged();
		}

		protected void fireColumnMoved(TableColumnModelEvent arg0) {
			super.fireColumnMoved(arg0);
		}

		protected void fireColumnRemoved(TableColumnModelEvent arg0) {
			super.fireColumnRemoved(arg0);
		}

		protected void fireColumnSelectionChanged(ListSelectionEvent arg0) {
			super.fireColumnSelectionChanged(arg0);
		}

		public TableColumnModelListener[] getColumnModelListeners() {
			return super.getColumnModelListeners();
		}

		public boolean getColumnSelectionAllowed() {
			return super.getColumnSelectionAllowed();
		}

		public EventListener[] getListeners(Class arg0) {
			return super.getListeners(arg0);
		}

		public int getSelectedColumnCount() {
			return super.getSelectedColumnCount();
		}

		public int[] getSelectedColumns() {
			return super.getSelectedColumns();
		}

		public ListSelectionModel getSelectionModel() {
			return super.getSelectionModel();
		}

		public void propertyChange(PropertyChangeEvent arg0) {
			try{
			super.propertyChange(arg0);
			int w=((Integer)arg0.getNewValue()).intValue();
			if (arg0.getSource() instanceof TableColumn && (w!=75)){
				TableColumn tc=(TableColumn)arg0.getSource();
				Column column=pt.getColumn(tc.getModelIndex());
				column.setWidth(w);
				System.out.println("Index guardar = "+tc.getModelIndex()+" , "+"Anchura = "+w);
			}
			}catch (Exception e) {
				System.out.println("PETA");
			}

		}
		protected void recalcWidthCache() {
			super.recalcWidthCache();
		}

		public void removeColumnModelListener(TableColumnModelListener arg0) {
			super.removeColumnModelListener(arg0);
		}

		public void setColumnMargin(int arg0) {
			super.setColumnMargin(arg0);
		}

		public void setColumnSelectionAllowed(boolean arg0) {
			super.setColumnSelectionAllowed(arg0);
		}

		public void setSelectionModel(ListSelectionModel arg0) {
			super.setSelectionModel(arg0);
		}

		public void valueChanged(ListSelectionEvent arg0) {
			super.valueChanged(arg0);
		}

		public int hashCode() {
			return super.hashCode();
		}

		protected void finalize() throws Throwable {
			super.finalize();
		}

		protected Object clone() throws CloneNotSupportedException {
			return super.clone();
		}

		public boolean equals(Object arg0) {
			return super.equals(arg0);
		}

		public String toString() {
			return super.toString();
		}
	}
    /**
     * DOCUMENT ME!
     *
     * @author Vicente Caballero Navarro
     */
    public class TableSelectionListener implements SelectionListener {
        /**
         * @see com.iver.cit.gvsig.fmap.layers.LegendListener#selectionChanged(com.iver.cit.gvsig.fmap.layers.LayerEvent)
         */
        public void selectionChanged(SelectionEvent e) {
            updateSelection();
            Table.this.repaint();
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @author Vicente Caballero Navarro
     */
    private class MouseHandler extends MouseAdapter {
        /**
         * DOCUMENT ME!
         *
         * @param e DOCUMENT ME!
         */
        public void mouseClicked(MouseEvent e) {

            /* JTableHeader h = (JTableHeader) e.getSource();
               TableColumnModel columnModel = h.getColumnModel();
               int viewColumn = columnModel.getColumnIndexAtX(e.getX());
               int column = columnModel.getColumn(viewColumn).getModelIndex();
               if (column != -1) {
               }*/
        }
    }
    /**
     * DOCUMENT ME!
     *
     * @author Vicente Caballero Navarro
     */
    private class TableKeyListener implements KeyListener {

		public void keyPressed(KeyEvent arg0) {
			//JTextField tf=(JTextField)table.getEditorComponent();
			//table.setCellSelectionEnabled(true);
			//FocusManager fm = FocusManager.getCurrentManager();
			//fm.focusPreviousComponent(table);
		}

		public void keyReleased(KeyEvent ke) {
			int[] row=table.getSelectedRows();
		    int[] column=table.getSelectedColumns();
		    initEditField(column,row);
		}

		public void keyTyped(KeyEvent arg0) {
			// TODO Auto-generated method stub

		}

    }

    /**
     * DOCUMENT ME!
     *
     * @author Vicente Caballero Navarro
     */
    private class MouseRow extends MouseAdapter {
       	public void mouseReleased(MouseEvent arg0) {
			super.mouseReleased(arg0);
			int[] row=table.getSelectedRows();
		    int[] column=table.getSelectedColumns();
		    initEditField(column,row);
		}

		/**
         * DOCUMENT ME!
         *
         * @param e DOCUMENT ME!
         */
        public void mouseClicked(MouseEvent e) {
            super.mouseClicked(e);


        	PluginServices.getMainFrame().enableControls();

            if (e.getButton() == MouseEvent.BUTTON3) {
                new PopupMenu(e.getPoint());
            }
        }
    }

	public void commandRepaint() {
		refresh();
	}

	public void commandRefresh() {
		commandRepaint();
	}

	public void toPalette() {
		isPalette=true;
		m_viewInfo = new ViewInfo(ViewInfo.ICONIFIABLE |
				ViewInfo.MODELESSDIALOG | ViewInfo.RESIZABLE | ViewInfo.PALETTE);
	PluginServices.getMDIManager().changeViewInfo(this,getViewInfo());
	}

	public void restore() {
		isPalette=false;
		m_viewInfo = new ViewInfo(ViewInfo.ICONIFIABLE |
	            ViewInfo.MAXIMIZABLE | ViewInfo.RESIZABLE);
		PluginServices.getMDIManager().changeViewInfo(this,getViewInfo());
	}

	public boolean isPalette() {
		return isPalette;
	}


	private void addNumbers(){
		TableColumn tc=new TableColumn();
		tc.setWidth(40);
		tc.setResizable(false);
		getTable().addColumn(tc);
	}
	public static void main(String[] args) {
		JTextField tf=new JTextField("hola");
		tf.selectAll();
		JFrame frame=new JFrame();
		frame.getContentPane().add(tf);
		frame.show(true);
	}
}
