/*
 * Created on 27-jul-2004
 *
 */
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.gui.layout;

import java.awt.Cursor;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.gui.layout.fframes.IFFrame;
import com.iver.cit.gvsig.gui.layout.fframes.IFFrameEditableVertex;
import com.iver.cit.gvsig.gui.layout.fframes.IFFrameGroupSelectable;


/**
 * Eventos que se realizan sobre el Layout.
 *
 * @author Vicente Caballero Navarro
 */
public class EventsHandler implements ActionListener, ComponentListener,
    MouseMotionListener, MouseListener, KeyListener {
    private Layout layout = null;
    private Point2D.Double m_pointSelected = null;
    private int index = 0;
    private ArrayList lastSelect = new ArrayList();
    private FLayoutFunctions events = null;
    private FLayoutZooms zooms = null;
    private static IFFrame[] selectFFrames;
    private int difX;
    private int difY;
    /**
     * Crea un nuevo EventsHandler.
     *
     * @param l Referencia al Layout.
     */
    public EventsHandler(Layout l) {
        layout = l;
        events = new FLayoutFunctions(layout);
        zooms = new FLayoutZooms(layout);
    }

    /**
     * @see java.awt.event.ComponentListener#componentHidden(java.awt.event.ComponentEvent)
     */
    public void componentHidden(ComponentEvent arg0) {
    }

    /**
     * @see java.awt.event.ComponentListener#componentMoved(java.awt.event.ComponentEvent)
     */
    public void componentMoved(ComponentEvent arg0) {
    }

    /**
     * @see java.awt.event.ComponentListener#componentResized(java.awt.event.ComponentEvent)
     */
    public void componentResized(ComponentEvent arg0) {
        layout.fullRect();
    }

    /**
     * @see java.awt.event.ComponentListener#componentShown(java.awt.event.ComponentEvent)
     */
    public void componentShown(ComponentEvent arg0) {
    }

    /**
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent arg0) {
        layout.repaint();
    }

    /**
     * @see java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent)
     */
    public void mouseDragged(MouseEvent e) {
        if (e.getButton() != MouseEvent.BUTTON3) {
            layout.setLastPoint(e.getPoint());
            layout.repaint();
        }
        if (layout.getTool() == Layout.EDIT){
        	IFFrame[] fframes=layout.getFFrames();
        	for (int i=0;i<fframes.length;i++){
        		IFFrame frame=fframes[i];
        		if (frame instanceof IFFrameEditableVertex){
        			((IFFrameEditableVertex)frame).pointDragged(FLayoutUtilities.toSheetPoint(e.getPoint(),layout.getAT()));
        			layout.setStatus(Layout.GRAPHICS);

        		}
        	}

        }
    }

    /**
     * @see java.awt.event.MouseMotionListener#mouseMoved(java.awt.event.MouseEvent)
     */
    public void mouseMoved(MouseEvent E) {
       if ((layout.getTool() == Layout.POLYLINE) ||
                (layout.getTool() == Layout.POLYGON) ||
                (layout.getTool() == Layout.LINE) ||
                (layout.getTool() == Layout.CIRCLE) ||
                (layout.getTool() == Layout.RECTANGLESIMPLE)) {
            layout.getGeometryAdapter().pointPosition(FLayoutUtilities.toSheetPoint(
                    E.getPoint(), layout.getAT()));
            layout.setStatus(Layout.GRAPHICS);
            layout.repaint();
        }

        if (layout.getTool() == Layout.SELECT) {
            Cursor cursor = null;
            Point2D.Double p = new Point2D.Double(E.getX(), E.getY());
            IFFrame[] fframes=layout.getFFrameSelected();
            for (int i = 0; i < fframes.length; i++) {
                if (fframes[i].getContains(p)!=IFFrame.NOSELECT){
                	cursor=fframes[i].getMapCursor(p);
                }
                if (cursor != null) {
                    layout.setMapCursor(cursor);
                } else {
                    layout.setMapCursor(Layout.icrux);
                }
            }
            ///layout.setStatus(Layout.SELECT);
        }
    }

    /**
     * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
     */
    public void mouseClicked(MouseEvent E) {
    	if ((E.getClickCount() == 2) &&
                ((layout.getTool() == Layout.POLYLINE) ||
                (layout.getTool() == Layout.POLYGON))) {
    		delLastPoint();
            endGraphic();
        } else {
            if (E.getButton() == MouseEvent.BUTTON1) {
                if (layout.getTool() == Layout.SELECT) {
                    m_pointSelected = new Point2D.Double(E.getX(), E.getY());
                    IFFrame[] fframes=layout.getFFrames();
                    if (fframes.length > 0) {
                        ArrayList listSelect = new ArrayList();
                        for (int j = 0; j < fframes.length; j++) {
                            if (fframes[j].getContains(
                                        m_pointSelected) != IFFrame.NOSELECT) {
                                listSelect.add(fframes[j]);
                            }
                        }

                        if (listSelect.size() > 0) {
                            for (int k = 0; k < listSelect.size(); k++) {
                                if (((IFFrame) listSelect.get(k)).getSelected() != IFFrame.NOSELECT) {
                                    index = listSelect.size() - k;

                                    break;
                                }
                            }

                            if (!FLayoutUtilities.isEqualList(listSelect,
                                        lastSelect) ||
                                    (index > (listSelect.size() - 1))) {
                                index = 0;
                            }
                            for (int j = 0; j < fframes.length;
                                    j++) {
                                IFFrame fframe = fframes[j];

                                if (!E.isShiftDown()) {
                                    fframe.setSelected(false);
                                } else {
                                    if (fframe.getSelected() != IFFrame.NOSELECT) {
                                        if (fframe.getContains(m_pointSelected) != IFFrame.NOSELECT) {
                                            fframe.setSelected(false);
                                        }
                                    }
                                }
                            }

                            ((IFFrame) listSelect.get((listSelect.size() - 1 -
                                index))).setSelected(true);
                            index++;
                            lastSelect = listSelect;
                        }

                        layout.setStatus(Layout.SELECT);
                        layout.repaint();

                        if (E.getClickCount() > 1) {
                            FLayoutGraphics flg = new FLayoutGraphics(layout);
                            flg.openFFrameDialog();
                            layout.setStatus(Layout.DESACTUALIZADO);
                            layout.repaint();

                            //layout.setStatus(Layout.SELECT);
                        }
                    }

                    PluginServices.getMainFrame().enableControls();
                }
            } else if (E.getButton() == MouseEvent.BUTTON2) {
            }
        }
    }

    /**
     * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
     */
    public void mouseEntered(MouseEvent arg0) {
        /* TODO        PluginServices.getMainFrame().getStatusBar().setMessage("0",
           layout.getAtributes().getNameUnit());

        if (layout.getTool() == Layout.PAN) {
            layout.setMapCursor(Layout.iLayoutpan);
        } else if (layout.getTool() == Layout.ZOOM_MAS) {
            layout.setMapCursor(Layout.iLayoutzoomin);
        } else if (layout.getTool() == Layout.ZOOM_MENOS) {
            layout.setMapCursor(Layout.iLayoutzoomout);
        } else if (layout.getTool() == Layout.VIEW_PAN) {
            layout.setMapCursor(Layout.ipan);
        } else if (layout.getTool() == Layout.VIEW_ZOOMIN) {
            layout.setMapCursor(Layout.izoomin);
        } else if (layout.getTool() == Layout.VIEW_ZOOMOUT) {
            layout.setMapCursor(Layout.izoomout);
        }
        */
    }

    /**
     * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
     */
    public void mouseExited(MouseEvent arg0) {
        ///TODO	PluginServices.getMainFrame().getStatusBar().setMessage("0", "");
    }

    /**
     * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
     */
    public void mousePressed(MouseEvent E) {
        if (E.getButton() == MouseEvent.BUTTON1) {
            Point pScreen = new Point(E.getX(), E.getY());
            layout.setPointAnt(pScreen);
            layout.setFirstPoint(layout.getPointAnt());

            if ((layout.getTool() == Layout.ZOOM_MAS) ||
                    (layout.getTool() == Layout.VIEW_ZOOMIN)) {
                layout.setStatus(Layout.ZOOM_MAS);
            } else if ((layout.getTool() == Layout.RECTANGLEVIEW) ||
                    (layout.getTool() == Layout.RECTANGLEPICTURE) ||
                    (layout.getTool() == Layout.RECTANGLESCALEBAR) ||
                    (layout.getTool() == Layout.RECTANGLELEGEND) ||
                    (layout.getTool() == Layout.RECTANGLENORTH) ||
                    (layout.getTool() == Layout.RECTANGLETEXT)||
                    (layout.getTool() == Layout.RECTANGLEBOX)) {
                layout.setStatus(Layout.RECTANGLE);
            } else if ((layout.getTool() == Layout.POINT) ) {
            	layout.getGeometryAdapter().addPoint(FLayoutUtilities.toSheetPoint(
                        E.getPoint(), layout.getAT()));
            	endGraphic();
                //layout.setStatus(Layout.GRAPHICS);
            } else if ((layout.getTool() == Layout.POLYLINE) ||
                    (layout.getTool() == Layout.POLYGON)) {
                layout.getGeometryAdapter().addPoint(FLayoutUtilities.toSheetPoint(
                        E.getPoint(), layout.getAT()));
                layout.setStatus(Layout.GRAPHICS);
            } else if ((layout.getTool() == Layout.LINE) ||
                    (layout.getTool() == Layout.CIRCLE) ||
                    (layout.getTool() == Layout.RECTANGLESIMPLE)) {
                if (layout.getGeometryAdapter().addPoint(FLayoutUtilities.toSheetPoint(
                                E.getPoint(), layout.getAT())) == 2) {
                    endGraphic();
                    layout.setStatus(Layout.DESACTUALIZADO);
                } else {
                    layout.setStatus(Layout.GRAPHICS);
                }
            } else if (layout.getTool() == Layout.PAN) {
                layout.getRectOrigin().setLocation(layout.getRect().x,
                    layout.getRect().y);
                layout.setStatus(Layout.PAN);
            } else if (layout.getTool() == Layout.VIEW_PAN) {
                /*        layout.getRectOrigin().setLocation(layout.getRect().x,
                   layout.getRect().y);
                 */
                layout.setStatus(Layout.VIEW_PAN);
            } else if (layout.getTool() == Layout.ZOOM_MENOS) {
                layout.setCancelDrawing(true);
            } else if ((layout.getTool() == Layout.ZOOM_MAS) ||
                    (layout.getTool() == Layout.PAN)) {
                layout.setCancelDrawing(true);
            } else if (layout.getTool() == Layout.SELECT) {
                m_pointSelected = new Point2D.Double(E.getX(), E.getY());
                IFFrame[] fframes=layout.getFFrames();
                for (int i = 0; i < fframes.length; i++) {
                    IFFrame fframe = fframes[i];

                    if (m_pointSelected != null) {
                        if (!E.isShiftDown()) {
                            if ((fframe.getSelected() != IFFrame.NOSELECT)) {
                                fframe.setSelected(m_pointSelected,E);
                            }
                        }else if (fframe instanceof IFFrameGroupSelectable){
                        	    fframe.setSelected(m_pointSelected,E);
                        }
                    }

                    if (fframe.getSelected() != IFFrame.NOSELECT) {
                        layout.setIsReSel(false);
                    }
                }

                if ((layout.getLastPoint() != null) &&
                        (layout.getFirstPoint() != null)) {
                    layout.getLastPoint().setLocation(layout.getFirstPoint());
                }

                if (E.getClickCount() < 2) {
                    layout.setStatus(Layout.SELECT);
                    layout.repaint();
                }
            } else if (layout.getTool() == Layout.SET_TAG) {
                m_pointSelected = new Point2D.Double(E.getX(), E.getY());
                IFFrame[] fframes=layout.getFFrames();
                for (int i = 0; i < fframes.length; i++) {
                    IFFrame fframe = fframes[i];

                    if (m_pointSelected != null) {
                        if (fframe.contains(m_pointSelected)) {
                            fframe.openTag();
                        }
                    }
                }
            } else if (layout.getTool() == Layout.EDIT){
            	IFFrame[] fframes=layout.getFFrames();
            	for (int i=0;i<fframes.length;i++){
            		IFFrame frame=fframes[i];
            		if (frame instanceof IFFrameEditableVertex){
            			((IFFrameEditableVertex)frame).pointPressed(FLayoutUtilities.toSheetPoint(E.getPoint(),layout.getAT()));
            		}
            	}
            }
        } else if (E.getButton() == MouseEvent.BUTTON3) {
            new Popupmenu(layout, E.getPoint());
        }
    }

    /**
     * @see java.awt.event.MouseListener#mouseReleassed(java.awt.event.MouseEvent)
     */
    public void mouseReleased(MouseEvent E) {
        if (E.getButton() != MouseEvent.BUTTON3) {
            layout.setLastPoint(E.getPoint());
        }

        if (E.getButton() == MouseEvent.BUTTON1) {
            Point p1 = layout.getFirstPoint();
            Point p2 = new Point(E.getX(), E.getY());

            if (layout.getTool() == Layout.ZOOM_MAS) {
                zooms.setZoomIn(p1, p2);
                layout.refresh();
            } else if ((layout.getTool() == Layout.ZOOM_MENOS) ||
                    (E.getButton() == MouseEvent.BUTTON3)) {
                zooms.setZoomOut(p2);
                layout.refresh();
            } else if (layout.getTool() == Layout.SELECT) {
                events.setSelect();
                layout.refresh();
            } else if ((layout.getTool() == Layout.RECTANGLEVIEW) ||
                    (layout.getTool() == Layout.RECTANGLEPICTURE) ||
                    (layout.getTool() == Layout.RECTANGLESCALEBAR) ||
                    (layout.getTool() == Layout.RECTANGLELEGEND) ||
                    (layout.getTool() == Layout.RECTANGLETEXT) ||
                    (layout.getTool() == Layout.RECTANGLENORTH)||
                    (layout.getTool() == Layout.RECTANGLEBOX)) {
                events.addFFrame();
                PluginServices.getMainFrame().enableControls();
                layout.refresh();
            } else if (layout.getTool() == Layout.VIEW_ZOOMIN) {
                zooms.setViewZoomIn(p1, p2);
                layout.refresh();
            } else if (layout.getTool() == Layout.VIEW_ZOOMOUT) {
                zooms.setViewZoomOut(p2);
                layout.refresh();
            } else if (layout.getTool() == Layout.VIEW_PAN) {
                events.setViewPan(p1, p2);
                layout.refresh();
            } else if ((layout.getTool() == Layout.POLYLINE) ||
                    (layout.getTool() == Layout.POLYGON)) {
                layout.setStatus(Layout.GRAPHICS);
                layout.repaint();
            } else if (layout.getTool() == Layout.EDIT){
            	IFFrame[] fframes=layout.getFFrames();
            	for (int i=0;i<fframes.length;i++){
            		IFFrame frame=fframes[i];
            		if (frame instanceof IFFrameEditableVertex){
            			if (frame.getSelected()!=IFFrame.NOSELECT && ((IFFrameEditableVertex)frame).isEditing()){
            				/*IFFrame fframeAux=frame.cloneFFrame(layout);
            				//((IFFrameEditable)fframeAux).startEditing();
            				((IFFrameEditable)fframeAux).pointReleased(FLayoutUtilities.toSheetPoint(E.getPoint(),layout.getAT()));
            				layout.getEFS().modifyFFrame(frame,fframeAux);
            				layout.updateFFrames();*/
            				IFFrame fframeAux=frame.cloneFFrame(layout);
            				((IFFrameEditableVertex)fframeAux).startEditing();
            				((IFFrameEditableVertex)fframeAux).pointReleased(FLayoutUtilities.toSheetPoint(E.getPoint(),layout.getAT()),((IFFrameEditableVertex)frame).getGeometry());
            				layout.getEFS().modifyFFrame(frame,fframeAux);
            				fframeAux.getBoundingBox(layout.getAT());
            				layout.updateFFrames();
            			}
            		}
            	}
            	layout.refresh();
            } else if (layout.getTool() == Layout.PAN) {
                layout.refresh();
            }
            ///layout.refresh();
            layout.setCancelDrawing(false);
        } else if (E.getButton() == MouseEvent.BUTTON3) {
        }


    }
    private void delLastPoint(){
    	layout.getGeometryAdapter().delLastPoint();
    }
    /**
     * DOCUMENT ME!
     */
    private void endGraphic() {
        if ((layout.getTool() == Layout.LINE) ||
                (layout.getTool() == Layout.POLYLINE) ||
                (layout.getTool() == Layout.POLYGON) ||
                (layout.getTool() == Layout.CIRCLE) ||
                (layout.getTool() == Layout.RECTANGLESIMPLE) ||
                (layout.getTool() == Layout.POINT)) {
            layout.getGeometryAdapter().end();
            PluginServices.getMainFrame().enableControls();
            events.addFFrame();
            layout.initGeometryAdapter();
            layout.refresh();
        }
    }

	public void keyPressed(KeyEvent arg0) {
		IFFrame[] fframes=layout.getFFrameSelected();
		int dif=10;
		if (arg0.getModifiers()==KeyEvent.CTRL_MASK){
		switch(arg0.getKeyCode()){
		case (KeyEvent.VK_C):
			selectFFrames=new IFFrame[fframes.length];
			for (int i=0;i<fframes.length;i++){
				selectFFrames[i]=fframes[i].cloneFFrame(layout);
			}
			break;
		case KeyEvent.VK_X:
			selectFFrames=new IFFrame[fframes.length];
			for (int i=0;i<fframes.length;i++){
				selectFFrames[i]=fframes[i].cloneFFrame(layout);
			}
			layout.delFFrameSelected();
			layout.refresh();
			break;

		}
		}else{
		switch (arg0.getKeyCode()) {
		case KeyEvent.VK_LEFT:
			difX=difX-dif;
			break;
		case KeyEvent.VK_RIGHT:
			difX=difX+dif;
			break;
		case KeyEvent.VK_UP:
			difY=difY-dif;
			break;
		case KeyEvent.VK_DOWN:
			difY=difY+dif;
			break;
		}
	/*	if (isFirst){
			layout.setFirstPoint(new Point((int)fframes[0].getBoundingBox(layout.getAT()).getX(),(int)fframes[0].getBoundingBox(layout.getAT()).getY()));
			isFirst=false;
		}
		layout.setLastPoint(new Point((int)layout.getFirstPoint().getX()+difX,(int)layout.getFirstPoint().getY()+difY));
		layout.setStatus(Layout.SELECT);
		layout.repaint();
		*/
		}
	}

	public void keyReleased(KeyEvent arg0) {
		IFFrame[] fframes=layout.getFFrameSelected();
		if (arg0.getModifiers()==KeyEvent.CTRL_MASK){
			switch(arg0.getKeyCode()){
			case KeyEvent.VK_V:
				IFFrame copyFFrame=null;
				layout.getEFS().startComplexCommand();
				for (int i=0;i<selectFFrames.length;i++){
					copyFFrame=selectFFrames[i].cloneFFrame(layout);
					if (i==0)
						layout.addFFrame(copyFFrame,true,true);
					else
						layout.addFFrame(copyFFrame,false,true);
				}
				layout.getEFS().endComplexCommand();
				layout.refresh();
				break;
			case KeyEvent.VK_Z:
				layout.getEFS().undo();
				layout.updateFFrames();
				layout.refresh();
				PluginServices.getMainFrame().enableControls();
				break;
			case KeyEvent.VK_Y:
				layout.getEFS().redo();
				layout.updateFFrames();
				layout.refresh();
				PluginServices.getMainFrame().enableControls();
				break;
			}
		}else{
			switch (arg0.getKeyCode()) {
			case KeyEvent.VK_LEFT:
			case KeyEvent.VK_RIGHT:
			case KeyEvent.VK_UP:
			case KeyEvent.VK_DOWN:

				layout.getEFS().startComplexCommand();
				for (int i=0;i<fframes.length;i++){
					IFFrame fframeAux=fframes[i].cloneFFrame(layout);
					Rectangle2D r=getRectMove(fframes[i].getBoundingBox(layout.getAT()),difX,difY);
					fframeAux.setBoundBox(FLayoutUtilities.toSheetRect(r,layout.getAT()));
					layout.getEFS().modifyFFrame(fframes[i],fframeAux);
				}
				layout.getEFS().endComplexCommand();
				layout.updateFFrames();
				layout.refresh();
				break;
			}
		}
			difX=0;
			difY=0;

	}

	public void keyTyped(KeyEvent arg0) {
		// TODO Auto-generated method stub

	}
	private Rectangle2D getRectMove(Rectangle2D r,int difX,int difY) {
		Rectangle2D rectMove=new Rectangle2D.Double();
		rectMove.setRect(r.getX()+difX,r.getY()+difY,r.getWidth(),r.getHeight());
		return rectMove;
	}
}
