/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.gui.project;

import java.awt.Component;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.text.DateFormat;

import javax.swing.ButtonGroup;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import com.hardcode.driverManager.DriverLoadException;
import com.hardcode.gdbms.engine.data.DataSource;
import com.hardcode.gdbms.engine.data.DataSourceFactory;
import com.hardcode.gdbms.engine.data.driver.DBDriver;
import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.data.driver.FileDriver;
import com.iver.andami.PluginServices;
import com.iver.andami.messages.NotificationManager;
import com.iver.andami.ui.mdiManager.SingletonView;
import com.iver.andami.ui.mdiManager.View;
import com.iver.andami.ui.mdiManager.ViewInfo;
import com.iver.cit.gvsig.ProjectExtension;
import com.iver.cit.gvsig.fmap.drivers.GeorreferencedRasterDriver;
import com.iver.cit.gvsig.fmap.edition.EditableAdapter;
import com.iver.cit.gvsig.fmap.layers.LayerFactory;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;
import com.iver.cit.gvsig.gui.DataBaseOpenDialog;
import com.iver.cit.gvsig.gui.FOpenDialog;
import com.iver.cit.gvsig.gui.FileOpenDialog;
import com.iver.cit.gvsig.gui.Table;
import com.iver.cit.gvsig.gui.layout.Layout;
import com.iver.cit.gvsig.project.Project;
import com.iver.cit.gvsig.project.ProjectElement;
import com.iver.cit.gvsig.project.ProjectFactory;
import com.iver.cit.gvsig.project.ProjectMap;
import com.iver.cit.gvsig.project.ProjectTable;
import com.iver.cit.gvsig.project.ProjectView;
import com.iver.utiles.DefaultListModel;


/**
 * Clase principal del proyecto donde se puede operar para crear vistas,
 * tablas, mapas, ...
 *
 * @author Fernando Gonzlez Corts
 */
public class ProjectWindow extends JPanel implements PropertyChangeListener,
	View, SingletonView {
	private javax.swing.JPanel jPanel = null;
	private javax.swing.JRadioButton btnVistas = null;
	private javax.swing.JRadioButton btnTablas = null;
	private javax.swing.JRadioButton btnMapas = null;
	private ButtonGroup grupo = new ButtonGroup();
	private javax.swing.JPanel jPanel1 = null;
	private javax.swing.JList lstDocs = null;
	private javax.swing.JPanel jPanel2 = null;
	private javax.swing.JButton btnNuevo = null;
	private javax.swing.JButton btnPropiedades = null;
	private javax.swing.JButton btnAbrir = null;
	private javax.swing.JButton btnBorrar = null;
	private javax.swing.JButton btnRenombrar = null;
	private javax.swing.JPanel jPanel3 = null;
	private javax.swing.JLabel jLabel = null;
	private javax.swing.JLabel lblNombreSesion = null;
	private javax.swing.JLabel jLabel1 = null;
	private javax.swing.JLabel lblGuardado = null;
	private javax.swing.JLabel jLabel2 = null;
	private javax.swing.JLabel lblFecha = null;
	private javax.swing.JButton btnImportar = null;
	private javax.swing.JButton btnExportar = null;
	private javax.swing.JButton btnEditar = null;
	// The properties of the Project Manager window (size, position, etc):
	private ViewInfo m_viewInfo=null;

	/** Proyecto representado en la vista */
	private Project p;

	/**
	 * Extension con la informacin, entre otras, sobre las extensiones para
	 * las tablas
	 */
	private ProjectExtension projectExtension;
	private javax.swing.JScrollPane jScrollPane = null;
	private javax.swing.JPanel jPanel4 = null;

	/**
	 * This is the default constructor
	 *
	 * @param project Extension
	 */
	public ProjectWindow(ProjectExtension project) {
		super();

		initialize();

		projectExtension = project;

		refreshControls();
	}

	/**
	 * This is the default constructor
	 *
	 * @param project Extension
	 */
	public ProjectWindow() {
		super();

		initialize();
		refreshControls();
	}

	public void setProjectExtension(ProjectExtension p) {
		projectExtension = p;
	}


	/**
	 * Asigna el proyecto a la ventana
	 *
	 * @param p Proyecto con el que se operar a travs de este dilogo
	 */
	public void setProject(Project p) {
		this.p = p;
		p.addPropertyChangeListener(this);
		refreshControls();
	}

	/**
	 * Activa los botones de la botonera del medio o los desactiva en funcin
	 * de que est seleccionado o no un elemento de proyecto en la lista del
	 * medio
	 */
	private void activarBotones() {
		if (lstDocs.getSelectedIndex() != -1) {
			btnAbrir.setEnabled(true);
			btnBorrar.setEnabled(true);
			btnRenombrar.setEnabled(true);
			btnPropiedades.setEnabled(true);
		} else {
			btnAbrir.setEnabled(false);
			btnBorrar.setEnabled(false);
			btnRenombrar.setEnabled(false);
			btnPropiedades.setEnabled(false);
		}
	}

	/**
	 * Refresca la lista de elementos de proyecto con vistas, mapas o tablas,
	 * segn la opcin activada
	 */
	private void refreshList() {
		if (p != null) {
			DefaultListModel model;

			if (btnMapas.isSelected()) {
				model = new DefaultListModel(p.getMaps());

				//				getBtnNuevo().setEnabled(true);
			} else if (btnTablas.isSelected()) {
				model = new DefaultListModel(p.getTables());

				//                getBtnNuevo().setEnabled(false);
			} else {
				model = new DefaultListModel(p.getViews());

				//				getBtnNuevo().setEnabled(true);
			}

			lstDocs.setModel(model);

			activarBotones();
		}
	}

	/**
	 * Refresca las etiquetas con la informacin del proyecto
	 */
	private void refreshProperties() {
		if (p != null) {
			String aux;

			lblNombreSesion.setText(p.getName());

			String path = p.getPath();

			if (path != null) {
				File f = new File(path);
				lblGuardado.setText(f.toString());
			} else {
				lblGuardado.setText("");
			}

			lblFecha.setText(p.getCreationDate());
		}
	}

	/**
	 * Refresca todo el dilogo
	 */
	public void refreshControls() {
		refreshList();
		refreshProperties();
	}

	/**
	 * This method initializes this
	 */
	private void initialize() {
		setLayout(new java.awt.FlowLayout());
		add(getJPanel(), null);
		add(getJPanel1(), null);
		add(getJPanel3(), null);

		this.setSize(400, 450);
		this.setPreferredSize(new java.awt.Dimension(400, 430));
		grupo.add(btnVistas);
		grupo.add(btnTablas);
		grupo.add(btnMapas);

		btnVistas.setSelected(true);
	}
	private String ask(String title, String content){
		JOptionPane pane = new JOptionPane();
		pane.setMessage(content);
		pane.setMessageType(JOptionPane.QUESTION_MESSAGE);
		pane.setWantsInput(true);
		pane.setInitialSelectionValue("");

		JDialog dlg = pane.createDialog((Component) PluginServices.getMainFrame(),
				title);
		dlg.setModal(true);
		dlg.show();

		return pane.getInputValue().toString().trim();
	}

	/**
	 * Crea un nuevo project element
	 *
	 * @throws Exception DOCUMENT ME!
	 */
	private void nuevo() throws Exception {
		DateFormat df = DateFormat.getDateInstance();

		if (btnMapas.isSelected()) {
			ProjectMap map = ProjectFactory.createMap(PluginServices.getText(
						this, "untitled"));
			map.setModel(new Layout());
			map.getModel().setProjectMap(map);
			p.addMap(map);
		} else if (btnTablas.isSelected()) {
			try {
				FOpenDialog fopen = new FOpenDialog();
				FileOpenDialog fod = new FileOpenDialog(new Class[] {
							FileDriver.class}, false);
				DataBaseOpenDialog dbod = new DataBaseOpenDialog();
				dbod.setClasses(new Class[] { DBDriver.class });
				fopen.addTab("Fichero", fod);
				fopen.addTab("Base de datos", dbod);
				PluginServices.getMDIManager().addView(fopen);

				if (fopen.isAccepted()) {
					JPanel panel = fopen.getSelectedTab();
					ProjectExtension ext = (ProjectExtension) PluginServices.getExtension(ProjectExtension.class);

					if (panel instanceof FileOpenDialog) {
						File[] files = fod.getFiles();
						String[] driverNames = fod.getDriverNames();

						for (int i = 0; i < files.length; i++) {
							String name = files[i].getName();

							LayerFactory.getDataSourceFactory()
										.addFileDataSource(driverNames[i],
								name, files[i].getAbsolutePath());

							DataSource dataSource = LayerFactory.getDataSourceFactory()
																.createRandomDataSource(name, DataSourceFactory.AUTOMATIC_OPENING);
							SelectableDataSource sds = new SelectableDataSource(dataSource);
							EditableAdapter auxea=new EditableAdapter();
		                	auxea.setOriginalDataSource(sds);
							ProjectTable projectTable = ProjectFactory.createTable(name,
									auxea);
							ext.getProject().addTable(projectTable);

							Table t = new Table();
							t.setModel(projectTable);
							projectTable.setAndamiView(t);
							PluginServices.getMDIManager().addView(t);
						}
					} else if (panel instanceof DataBaseOpenDialog) {
						String driverName = dbod.getDriverName();
						int port = -1;

						try {
							port = Integer.parseInt(dbod.getPort());
						} catch (NumberFormatException e) {
						}

						String name = dbod.getHost() + "/" +
							dbod.getDataBase();

						if (port != -1) {
							name = dbod.getHost() + ":" + port + "/" +
								dbod.getDataBase();
						}

						String user = dbod.getUser().trim();
						String password = dbod.getPassword();

						if (user.equals("")) {
							user = null;
							password = null;
						}

						LayerFactory.getDataSourceFactory().addDBDataSourceByTable(name,
							dbod.getHost(), port, user, password,
							dbod.getDataBase(), dbod.getTable(), driverName);

						DataSource dataSource = LayerFactory.getDataSourceFactory()
															.createRandomDataSource(name, DataSourceFactory.AUTOMATIC_OPENING);
						SelectableDataSource sds = new SelectableDataSource(dataSource);
						EditableAdapter auxea=new EditableAdapter();
	                	auxea.setOriginalDataSource(sds);
						ProjectTable projectTable = ProjectFactory.createTable(name,
								auxea);
						ext.getProject().addTable(projectTable);

						Table t = new Table();
						t.setModel(projectTable);
						projectTable.setAndamiView(t);
						PluginServices.getMDIManager().addView(t);
					}
				}
			} catch (DriverLoadException e) {
				NotificationManager.addError("Error al cargar los drivers", e);
			}
		} else {
			ProjectView vista = ProjectFactory.createView(PluginServices.getText(
						this, "untitled"));
			p.addView(vista);
		}
	}

	/**
	 * Abre la ventana de un nuevo project element
	 */
	private void abrir() {
		if (btnMapas.isSelected()) {
			int index = lstDocs.getSelectedIndex();

			if (index == -1) {
				return;
			}
            ProjectMap projectMap = (ProjectMap) p.getMaps().get(index);
			Layout l = projectMap.getModel();
            // System.out.println("l.getName " + l.getName() + " projectMap.getName " + projectMap.getName());
            projectMap.setName(l.getName());
            l.setProjectMap(projectMap);
			l.fullRect();
            l.getViewInfo().setTitle(PluginServices.getText(this,
            "Mapa") + " : " +l.getName());
			PluginServices.getMDIManager().addView(l);
		} else if (btnTablas.isSelected()) {
			com.iver.cit.gvsig.gui.Table vista = new com.iver.cit.gvsig.gui.Table();
			int index = lstDocs.getSelectedIndex();

			if (index == -1) {
				return;
			}
			ProjectTable pt=(ProjectTable) p.getTables().get(index);
			vista.setModel(pt);
			PluginServices.getMDIManager().addView(vista);
		} else {
			com.iver.cit.gvsig.gui.View vista = new com.iver.cit.gvsig.gui.View();
			int index = lstDocs.getSelectedIndex();

			if (index == -1) {
				return;
			}

			vista.setModel((ProjectView) p.getViews().get(index));
			PluginServices.getMDIManager().addView(vista);
		}
	}

	/**
	 * Cambia el nombre de un project element
	 */
	private void renombrar() {
		ProjectElement elem = null;
		int index = lstDocs.getSelectedIndex();

		if (index == -1) {
			return;
		}

		if (btnMapas.isSelected()) {
			elem = (ProjectElement) p.getMaps().get(index);
		} else if (btnTablas.isSelected()) {
			elem = (ProjectElement) p.getTables().get(index);
		} else {
			elem = (ProjectElement) p.getViews().get(index);
		}

		JOptionPane pane = new JOptionPane();
		pane.setMessage(PluginServices.getText(this, "introduce_nombre"));
		pane.setMessageType(JOptionPane.QUESTION_MESSAGE);
		pane.setWantsInput(true);
		pane.setInitialSelectionValue(elem.getName());
        pane.setInputValue(elem.getName());
		JDialog dlg = pane.createDialog((Component) PluginServices.getMainFrame(),
				PluginServices.getText(this, "renombrar"));
		dlg.setModal(true);
		dlg.show();

		String nuevoNombre = pane.getInputValue().toString().trim();

		if (nuevoNombre.length() == 0) {
			return;
		}

		if (nuevoNombre == null) {
			return;
		}

		for (int i=0; i<lstDocs.getModel().getSize(); i++){
			if (i==index) continue;
			if (((ProjectElement)lstDocs.getModel().getElementAt(i)).getName().equals(nuevoNombre)){
				JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(), PluginServices.getText(this, "elemento_ya_existe"));
				return;
			}

		}
		elem.setName(nuevoNombre);

		refreshList();
	}

	/**
	 * Borra un project element
	 */
	private void borrar() {
		int res = JOptionPane.showConfirmDialog((Component) PluginServices.getMainFrame(),
				PluginServices.getText(this, "confirmar_borrar"),
				PluginServices.getText(this, "borrar"),
				JOptionPane.YES_NO_OPTION, JOptionPane.INFORMATION_MESSAGE);

		if (res == JOptionPane.YES_OPTION) {
			int index = lstDocs.getSelectedIndex();

			if (btnMapas.isSelected()) {
				PluginServices.getMDIManager().closeSingletonView(p.getMaps()
																   .get(index));
				p.delMap(index);
			} else if (btnTablas.isSelected()) {
				PluginServices.getMDIManager().closeSingletonView(p.getTables()
																   .get(index));
				p.delTable(index);
			} else {
				PluginServices.getMDIManager().closeSingletonView(p.getViews()
																   .get(index));
				p.delView(index);
			}

			refreshList();
		}
	}

	/**
	 * Muestra el dilogo de propiedades de un project element
	 */
	private void propiedades() {
		int index = lstDocs.getSelectedIndex();

		if (index == -1) {
			return;
		}

		View dlg = null;

		if (btnMapas.isSelected()) {
			dlg = new MapProperties((ProjectMap) p.getMaps().get(index));
		} else if (btnTablas.isSelected()) {
			try {
                dlg = new TableProperties((ProjectTable) p.getTables().get(index));
            } catch (DriverException e) {
                NotificationManager.addError(e.getMessage(), e);
            }
		} else {
			dlg = new ViewProperties((ProjectView) p.getViews().get(index));
		}

		PluginServices.getMDIManager().addView(dlg);

		refreshControls();
		lstDocs.setSelectedIndex(index);
	}

	/**
	 * This method initializes jPanel
	 *
	 * @return javax.swing.JPanel
	 */
	private javax.swing.JPanel getJPanel() {
		if (jPanel == null) {
			jPanel = new javax.swing.JPanel();

			java.awt.FlowLayout layFlowLayout1 = new java.awt.FlowLayout();
			layFlowLayout1.setHgap(15);
			jPanel.setLayout(layFlowLayout1);
			jPanel.add(getBtnVistas(), null);
			jPanel.add(getBtnTablas(), null);
			jPanel.add(getBtnMapas(), null);
			jPanel.setName("tipoDocPanel");
			jPanel.setPreferredSize(new java.awt.Dimension(390, 125));
			jPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(
					null, PluginServices.getText(this, "tipos_de_documentos"),
					javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
					javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
		}

		return jPanel;
	}

	/**
	 * This method initializes btnVistas
	 *
	 * @return javax.swing.JRadioButton
	 */
	private javax.swing.JRadioButton getBtnVistas() {
		if (btnVistas == null) {
			btnVistas = new javax.swing.JRadioButton();
			btnVistas.setIcon(new javax.swing.ImageIcon(
					this.getClass().getClassLoader().getResource("images/Vista.png")));
			btnVistas.setSelectedIcon(new javax.swing.ImageIcon(
					this.getClass().getClassLoader().getResource("images/Vista_sel.png")));
			btnVistas.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
			btnVistas.setText(PluginServices.getText(this, "vistas"));
			btnVistas.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
			btnVistas.setName("vistasBtn");
			btnVistas.addChangeListener(new javax.swing.event.ChangeListener() {
					public void stateChanged(javax.swing.event.ChangeEvent e) {
						refreshList();
					}
				});
		}

		return btnVistas;
	}

	/**
	 * This method initializes btnTablas
	 *
	 * @return javax.swing.JRadioButton
	 */
	private javax.swing.JRadioButton getBtnTablas() {
		if (btnTablas == null) {
			btnTablas = new javax.swing.JRadioButton();
			btnTablas.setIcon(new javax.swing.ImageIcon(
					this.getClass().getClassLoader().getResource("images/tablas.png")));
			btnTablas.setSelectedIcon(new javax.swing.ImageIcon(
					this.getClass().getClassLoader().getResource("images/tablas_sel.png")));
			btnTablas.setText(PluginServices.getText(this, "tablas"));
			btnTablas.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
			btnTablas.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
			btnTablas.setName("tablasBtn");
			btnTablas.addChangeListener(new javax.swing.event.ChangeListener() {
					public void stateChanged(javax.swing.event.ChangeEvent e) {
						refreshList();
					}
				});
		}

		return btnTablas;
	}

	/**
	 * This method initializes btnMapas
	 *
	 * @return javax.swing.JRadioButton
	 */
	private javax.swing.JRadioButton getBtnMapas() {
		if (btnMapas == null) {
			btnMapas = new javax.swing.JRadioButton();
			btnMapas.setIcon(new javax.swing.ImageIcon(
					this.getClass().getClassLoader().getResource("images/mapas.png")));
			btnMapas.setSelectedIcon(new javax.swing.ImageIcon(
					this.getClass().getClassLoader().getResource("images/mapas_sel.png")));
			btnMapas.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
			btnMapas.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
			btnMapas.setText(PluginServices.getText(this, "mapas"));
			btnMapas.addChangeListener(new javax.swing.event.ChangeListener() {
					public void stateChanged(javax.swing.event.ChangeEvent e) {
						refreshList();
					}
				});
		}

		return btnMapas;
	}

	/**
	 * This method initializes jPanel1
	 *
	 * @return javax.swing.JPanel
	 */
	private javax.swing.JPanel getJPanel1() {
		if (jPanel1 == null) {
			jPanel1 = new javax.swing.JPanel();

			java.awt.BorderLayout layBorderLayout2 = new java.awt.BorderLayout();
			layBorderLayout2.setVgap(30);
			jPanel1.setLayout(layBorderLayout2);
			jPanel1.add(getJPanel2(), java.awt.BorderLayout.EAST);
			jPanel1.add(getJScrollPane(), java.awt.BorderLayout.CENTER);
			jPanel1.setPreferredSize(new java.awt.Dimension(390, 170));
			jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder(
					null,
					PluginServices.getText(this, "documentos_existentes"),
					javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
					javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
		}

		return jPanel1;
	}

	/**
	 * This method initializes lstDocs
	 *
	 * @return javax.swing.JList
	 */
	private javax.swing.JList getLstDocs() {
		if (lstDocs == null) {
			lstDocs = new javax.swing.JList();
			lstDocs.addMouseListener(new java.awt.event.MouseAdapter() {
					public void mouseClicked(java.awt.event.MouseEvent e) {
						if (e.getClickCount() == 2) {
							abrir();
						}
					}
				});
			lstDocs.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
					public void valueChanged(
						javax.swing.event.ListSelectionEvent e) {
						activarBotones();
					}
				});
		}

		return lstDocs;
	}

	/**
	 * This method initializes jPanel2
	 *
	 * @return javax.swing.JPanel
	 */
	private javax.swing.JPanel getJPanel2() {
		if (jPanel2 == null) {
			jPanel2 = new javax.swing.JPanel();

			java.awt.FlowLayout layFlowLayout11 = new java.awt.FlowLayout();
			layFlowLayout11.setVgap(5);
			jPanel2.setLayout(layFlowLayout11);
			jPanel2.add(getBtnNuevo(), null);
			jPanel2.add(getBtnAbrir(), null);
			jPanel2.add(getBtnRenombrar(), null);
			jPanel2.add(getBtnBorrar(), null);
			jPanel2.add(getBtnPropiedades(), null);
			jPanel2.setPreferredSize(new java.awt.Dimension(100, 50));
		}

		return jPanel2;
	}

	/**
	 * This method initializes btnNuevo
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnNuevo() {
		if (btnNuevo == null) {
			btnNuevo = new javax.swing.JButton();
			btnNuevo.setPreferredSize(new java.awt.Dimension(80, 23));
			btnNuevo.setName("btnNuevo");
			btnNuevo.setText(PluginServices.getText(this, "nuevo"));
			btnNuevo.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnNuevo.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent e) {
						try {
							nuevo();
						} catch (Exception e1) {
							NotificationManager.addError(e1.getLocalizedMessage(),
								e1);
						}
					}
				});
		}

		return btnNuevo;
	}

	/**
	 * This method initializes btnPropiedades
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnPropiedades() {
		if (btnPropiedades == null) {
			btnPropiedades = new javax.swing.JButton();
			btnPropiedades.setPreferredSize(new java.awt.Dimension(80, 23));
			btnPropiedades.setText(PluginServices.getText(this, "propiedades"));
			btnPropiedades.setName("btnPropiedades");
			btnPropiedades.setEnabled(false);
			btnPropiedades.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnPropiedades.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent e) {
						propiedades();
					}
				});
		}

		return btnPropiedades;
	}

	/**
	 * This method initializes btnAbrir
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnAbrir() {
		if (btnAbrir == null) {
			btnAbrir = new javax.swing.JButton();
			btnAbrir.setPreferredSize(new java.awt.Dimension(80, 23));
			btnAbrir.setName("btnAbrir");
			btnAbrir.setText(PluginServices.getText(this, "abrir"));
			btnAbrir.setEnabled(false);
			btnAbrir.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnAbrir.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent e) {
						abrir();
					}
				});
		}

		return btnAbrir;
	}

	/**
	 * This method initializes btnBorrar
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnBorrar() {
		if (btnBorrar == null) {
			btnBorrar = new javax.swing.JButton();
			btnBorrar.setPreferredSize(new java.awt.Dimension(80, 23));
			btnBorrar.setText(PluginServices.getText(this, "borrar"));
			btnBorrar.setName("btnBorrar");
			btnBorrar.setEnabled(false);
			btnBorrar.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnBorrar.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent e) {
						borrar();
					}
				});
		}

		return btnBorrar;
	}

	/**
	 * This method initializes btnRenombrar
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnRenombrar() {
		if (btnRenombrar == null) {
			btnRenombrar = new javax.swing.JButton();
			btnRenombrar.setPreferredSize(new java.awt.Dimension(80, 23));
			btnRenombrar.setName("btnRenombrar");
			btnRenombrar.setText(PluginServices.getText(this, "renombrar"));
			btnRenombrar.setEnabled(false);
			btnRenombrar.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnRenombrar.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent e) {
						renombrar();
					}
				});
		}

		return btnRenombrar;
	}

	/**
	 * This method initializes jPanel3
	 *
	 * @return javax.swing.JPanel
	 */
	private javax.swing.JPanel getJPanel3() {
		if (jPanel3 == null) {
			jPanel3 = new javax.swing.JPanel();

			java.awt.FlowLayout layFlowLayout3 = new java.awt.FlowLayout();
			layFlowLayout3.setHgap(5);
			layFlowLayout3.setAlignment(java.awt.FlowLayout.LEFT);
			jPanel3.setLayout(layFlowLayout3);
			jPanel3.add(getJLabel(), null);
			jPanel3.add(getLblNombreSesion(), null);
			jPanel3.add(getJLabel1(), null);
			jPanel3.add(getLblGuardado(), null);
			jPanel3.add(getJLabel2(), null);
			jPanel3.add(getLblFecha(), null);
			jPanel3.add(getJPanel4(), null);
			jPanel3.setPreferredSize(new java.awt.Dimension(390, 125));
			jPanel3.setBorder(javax.swing.BorderFactory.createTitledBorder(
					null, PluginServices.getText(this, "propiedades_sesion"),
					javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
					javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
		}

		return jPanel3;
	}

	/**
	 * This method initializes jLabel
	 *
	 * @return javax.swing.JLabel
	 */
	private javax.swing.JLabel getJLabel() {
		if (jLabel == null) {
			jLabel = new javax.swing.JLabel();
			jLabel.setText(PluginServices.getText(this, "nombre_sesion") + ":");
			jLabel.setPreferredSize(new java.awt.Dimension(150, 16));
		}

		return jLabel;
	}

	/**
	 * This method initializes lblNombreSesion
	 *
	 * @return javax.swing.JLabel
	 */
	private javax.swing.JLabel getLblNombreSesion() {
		if (lblNombreSesion == null) {
			lblNombreSesion = new javax.swing.JLabel();
			lblNombreSesion.setText("");
			lblNombreSesion.setName("lblNombreSesion");
			lblNombreSesion.setPreferredSize(new java.awt.Dimension(200, 16));
			lblNombreSesion.setAutoscrolls(true);
		}

		return lblNombreSesion;
	}

	/**
	 * This method initializes jLabel1
	 *
	 * @return javax.swing.JLabel
	 */
	private javax.swing.JLabel getJLabel1() {
		if (jLabel1 == null) {
			jLabel1 = new javax.swing.JLabel();
			jLabel1.setText(PluginServices.getText(this, "guardado") + ":");
			jLabel1.setPreferredSize(new java.awt.Dimension(150, 16));
		}

		return jLabel1;
	}

	/**
	 * This method initializes lblGuardado
	 *
	 * @return javax.swing.JLabel
	 */
	private javax.swing.JLabel getLblGuardado() {
		if (lblGuardado == null) {
			lblGuardado = new javax.swing.JLabel();
			lblGuardado.setText("");
			lblGuardado.setPreferredSize(new java.awt.Dimension(200, 16));
			lblGuardado.setAutoscrolls(true);
		}

		return lblGuardado;
	}

	/**
	 * This method initializes jLabel2
	 *
	 * @return javax.swing.JLabel
	 */
	private javax.swing.JLabel getJLabel2() {
		if (jLabel2 == null) {
			jLabel2 = new javax.swing.JLabel();
			jLabel2.setText(PluginServices.getText(this, "creation_date") +
				":");
			jLabel2.setPreferredSize(new java.awt.Dimension(150, 16));
		}

		return jLabel2;
	}

	/**
	 * This method initializes lblFecha
	 *
	 * @return javax.swing.JLabel
	 */
	private javax.swing.JLabel getLblFecha() {
		if (lblFecha == null) {
			lblFecha = new javax.swing.JLabel();
			lblFecha.setText("");
			lblFecha.setPreferredSize(new java.awt.Dimension(200, 16));
			lblFecha.setAutoscrolls(true);
		}

		return lblFecha;
	}

	/**
	 * This method initializes btnImportar
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnImportar() {
		if (btnImportar == null) {
			btnImportar = new javax.swing.JButton();
			btnImportar.setPreferredSize(new java.awt.Dimension(80, 23));
			btnImportar.setText(PluginServices.getText(this, "importar"));
			btnImportar.setMargin(new java.awt.Insets(2, 2, 2, 2));
		}

		return btnImportar;
	}

	/**
	 * This method initializes btnExportar
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnExportar() {
		if (btnExportar == null) {
			btnExportar = new javax.swing.JButton();
			btnExportar.setPreferredSize(new java.awt.Dimension(80, 23));
			btnExportar.setText(PluginServices.getText(this, "exportar"));
			btnExportar.setMargin(new java.awt.Insets(2, 2, 2, 2));
		}

		return btnExportar;
	}

	/**
	 * This method initializes btnEditar
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnEditar() {
		if (btnEditar == null) {
			btnEditar = new javax.swing.JButton();
			btnEditar.setPreferredSize(new java.awt.Dimension(80, 23));
			btnEditar.setText(PluginServices.getText(this, "propiedades"));

			btnEditar.setMargin(new java.awt.Insets(2, 2, 2, 2));

			final JPanel f = this;
			btnEditar.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent e) {
						ProjectProperties dlg = new ProjectProperties(p);
						PluginServices.getMDIManager().addView(dlg);
						refreshProperties();
					}
				});
		}

		return btnEditar;
	}

	/**
	 * This method initializes jScrollPane
	 *
	 * @return javax.swing.JScrollPane
	 */
	private javax.swing.JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new javax.swing.JScrollPane();
			jScrollPane.setViewportView(getLstDocs());
			jScrollPane.setPreferredSize(new java.awt.Dimension(200, 100));
			jScrollPane.setVerticalScrollBarPolicy(javax.swing.JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		}

		return jScrollPane;
	}

	/**
	 * @see com.iver.mdiApp.ui.MDIManager.SingletonView#getViewModel()
	 */
	public Object getViewModel() {
		return p;
	}

    /**
     * This method is used to get <strong>an initial</strong> ViewInfo object
     * for this Project Manager window. It is not intended to retrieve the
     * ViewInfo object in a later time.
     * <strong>Use PluginServices.getMDIManager().getViewInfo(view)
     * to retrieve the ViewInfo object at any time after the creation of the
     * object.
     *
     * @see com.iver.mdiApp.ui.MDIManager.View#getViewInfo()
     */
	public ViewInfo getViewInfo() {
		if (m_viewInfo==null) {
			m_viewInfo = new ViewInfo();
			m_viewInfo.setWidth(400);
			m_viewInfo.setHeight(430);
			m_viewInfo.setTitle(PluginServices.getText(this, "titulo"));
		}
		return m_viewInfo;
	}

	/**
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		refreshControls();
	}

	/**
	 * @see com.iver.mdiApp.ui.MDIManager.View#viewActivated()
	 */
	public void viewActivated() {
	}

	/**
	 * This method initializes jPanel4
	 *
	 * @return javax.swing.JPanel
	 */
	private javax.swing.JPanel getJPanel4() {
		if (jPanel4 == null) {
			jPanel4 = new javax.swing.JPanel();

			java.awt.FlowLayout layFlowLayout2 = new java.awt.FlowLayout();
			layFlowLayout2.setHgap(30);
			jPanel4.setLayout(layFlowLayout2);
			jPanel4.add(getBtnImportar(), null);
			jPanel4.add(getBtnExportar(), null);
			jPanel4.add(getBtnEditar(), null);
			jPanel4.setPreferredSize(new java.awt.Dimension(360, 30));
		}

		return jPanel4;
	}
}
