/*
 * Created on 02-mar-2004
 * 
 * gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.gui.toc;

import java.awt.Color;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;

import javax.swing.JColorChooser;
import javax.swing.JDialog;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.tree.DefaultMutableTreeNode;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.ProjectExtension;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.FMap;
import com.iver.cit.gvsig.fmap.MapControl;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.core.v02.FSymbol;
import com.iver.cit.gvsig.fmap.layers.CancelationException;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLayers;
import com.iver.cit.gvsig.fmap.layers.FLyrDefault;
import com.iver.cit.gvsig.fmap.layers.FLyrRaster;
import com.iver.cit.gvsig.fmap.layers.layerOperations.AlphanumericData;
import com.iver.cit.gvsig.fmap.layers.layerOperations.ClassifiableVectorial;
import com.iver.cit.gvsig.fmap.rendering.SingleSymbolLegend;
import com.iver.cit.gvsig.gui.View;
import com.iver.cit.gvsig.gui.Panels.AdjustTransparencyPanel;
import com.iver.cit.gvsig.gui.thememanager.FThemeManagerWindow;
import com.iver.cit.gvsig.project.Project;
import com.iver.cit.gvsig.project.ProjectTable;

/**
 * Realiza el cambio de color si se pulsa OK
 * @author Luis W. Sevilla (sevilla_lui@gva.es)
 */
class FSymbolChangeColorTocMenuEntry extends TocMenuEntry {
    private JMenuItem color;    
	public void initialize(FPopupMenu m) {
		super.initialize(m);
		boolean bShow = false;
		if (isTocItemBranch())
		{
			FLayer lyr = getNodeLayer();
    		if ((lyr instanceof ClassifiableVectorial))
    		{
    		    ClassifiableVectorial lyrVect = (ClassifiableVectorial) lyr;
    		    if (lyrVect.getLegend() instanceof SingleSymbolLegend)
    		        bShow = true;
    		}
		}
		else
		    bShow = true;
		if (bShow)
		{
        	color = new JMenuItem(PluginServices.getText(this, "Cambio_Color"));
        	color.setFont(FPopupMenu.theFont);
        	getMenu().add(color);        
        	getMenu().setEnabled(true);
	        //Cambio color
	        color.addActionListener(this);
		}
	}
	
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.gui.toc.TocMenuEntry#execute(com.iver.cit.gvsig.gui.toc.ITocItem)
	 */
	public void actionPerformed(ActionEvent e) {
		ITocItem tocItem = (ITocItem) getNodeUserObject();
        
        Color newColor = JColorChooser.showDialog(null,                
                PluginServices.getText(this, "Elegir_Color"),
                null);
        if (newColor != null)
        {
			if (isTocItemBranch())
			{
				//FLayer lyr = getNodeLayer();
                FLayer[] actives = getMapContext().getLayers().getActives();
                for (int i=0;i<actives.length;i++){
                    if ((actives[i] instanceof ClassifiableVectorial))
                    {
                        ClassifiableVectorial lyrVect = (ClassifiableVectorial) actives[i];
                        if (lyrVect.getLegend() instanceof SingleSymbolLegend)
                        {
                            SingleSymbolLegend leg = (SingleSymbolLegend) lyrVect.getLegend();
                            leg.getDefaultSymbol().setColor(newColor);
                        } 
                    }
                }
			}
			else
			{
    	        TocItemLeaf leaf = (TocItemLeaf) tocItem;
    	        FSymbol sym = leaf.getSymbol();
    	        sym.setColor(newColor);
    	        			    
			}
	        // TRUCO PARA REFRESCAR.
	        getMapContext().invalidate();                    

        }
        
	}
}
/**
 * Realiza un cambio de nombre en la capa seleccionada
 * 
 * @author Vicente Caballero Navarro
 */
class ChangeNameTocMenuEntry extends TocMenuEntry {
    private JMenuItem changename;    
	public void initialize(FPopupMenu m) {
		super.initialize(m);
		//boolean bShow = false;
		if (isTocItemBranch())
		{
			changename = new JMenuItem(PluginServices.getText(this, "cambio_nombre"));
        	changename.setFont(FPopupMenu.theFont);
        	getMenu().add(changename);        
        	getMenu().setEnabled(true);
            if (getMapContext().getLayers().getActives().length == 1) {
                changename.setEnabled(true);
            } else {
                changename.setEnabled(false);
            }
	        //Cambio nombre
	        changename.addActionListener(this);
    		
		}
	}
	
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.gui.toc.TocMenuEntry#execute(com.iver.cit.gvsig.gui.toc.ITocItem)
	 */
	public void actionPerformed(ActionEvent e) {
		//ITocItem tocItem = (ITocItem) getNodeUserObject();
        	if (isTocItemBranch())
			{
				FLayer lyr = getNodeLayer();
				ChangeName chn=new ChangeName(lyr.getName());
				PluginServices.getMDIManager().addView(chn);
				 getNodeLayer().setName(chn.getName());
			}
	        // TRUCO PARA REFRESCAR.
	        ///getMapContext().invalidate();                    

   }
}
/**
 * Muestra el men de propiedades del tema.
 * 
 * @author jmorell
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
class FLyrVectEditPropertiesTocMenuEntry extends TocMenuEntry {
    private JMenuItem prop;
	public void initialize(FPopupMenu m) {
		super.initialize(m);
		FLayer lyr = null;
		if (isTocItemBranch()) {
			getMenu().addSeparator();
			lyr = getNodeLayer();
            //if ((lyr instanceof ClassifiableVectorial) && (getMapContext().getLayers().getActives().length == 1)) {
            if ((lyr instanceof ClassifiableVectorial)) {
		    	prop = new JMenuItem(PluginServices.getText(this, "propiedades"));
				getMenu().add(prop);
				prop.setFont(FPopupMenu.theFont);
				if (getMapContext().getLayers().getActives().length == 1) {
                    prop.setEnabled(true);
                } else {
                    prop.setEnabled(false);
                }
                //Cambio estilo
				prop.addActionListener(this);
			}
		}
	}
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.gui.toc.TocMenuEntry#execute(com.iver.cit.gvsig.gui.toc.ITocItem)
	 */
	public void actionPerformed(ActionEvent e) {
		/*FThemeManagerWindow fThemeManagerWindow = new FThemeManagerWindow();
		try {
			fThemeManagerWindow.setMapContext(getMapContext());
		} catch (com.iver.cit.gvsig.fmap.DriverException e1) {
			e1.printStackTrace();
		}*/
		FLayer[] actives = getMapContext().getLayers().getActives();
		FThemeManagerWindow fThemeManagerWindow;
    	if (actives.length==1) {
    		System.out.println("openPliegoThemeProperties(): Una sola capa. Abrimos el panel para la capa " + actives[0]);
    		fThemeManagerWindow = new FThemeManagerWindow();
    		try {
    			fThemeManagerWindow.setMapContext(getMapContext());
    		} catch (com.iver.cit.gvsig.fmap.DriverException e1) {
    			e1.printStackTrace();
    		}
            //PluginServices.getMDIManager().addView(fThemeManagerWindow);
    		if (PluginServices.getMainFrame() == null) {
    			JDialog dlg = new JDialog();
    			fThemeManagerWindow.setPreferredSize(fThemeManagerWindow.getSize());
    			dlg.getContentPane().add(fThemeManagerWindow);
    			dlg.setModal(false);			
    			dlg.pack();
    			dlg.show();
    		} else {
    			PluginServices.getMDIManager().addView(fThemeManagerWindow);
    		}
    	} else {
        	for (int i = 0; i < actives.length; i++){
        		System.out.println("openPliegoThemeProperties(): Muchas capas. Abrimos el panel para la capa " + actives[i]);
        		fThemeManagerWindow = new FThemeManagerWindow(actives[i], getMapContext());
                //PluginServices.getMDIManager().addView(fThemeManagerWindow);
        		if (PluginServices.getMainFrame() == null) {
        			JDialog dlg = new JDialog();
        			fThemeManagerWindow.setPreferredSize(fThemeManagerWindow.getSize());
        			dlg.getContentPane().add(fThemeManagerWindow);
        			dlg.setModal(false);			
        			dlg.pack();
        			dlg.show();
        		} else {
        			PluginServices.getMDIManager().addView(fThemeManagerWindow);
        		}
        	}
    	}
		/*if (PluginServices.getMainFrame() == null) {
			JDialog dlg = new JDialog();
			fThemeManagerWindow.setPreferredSize(fThemeManagerWindow.getSize());
			dlg.getContentPane().add(fThemeManagerWindow);
			dlg.setModal(false);			
			dlg.pack();
			dlg.show();
		} else {
			PluginServices.getMDIManager().addView(fThemeManagerWindow);
		}*/
	}
}

class FLyrRasterAdjustTransparencyTocMenuEntry extends TocMenuEntry {
	private JMenuItem transparency;
	FLayer lyr = null;
	public void initialize(FPopupMenu m) {
		super.initialize(m);
		
		if (isTocItemBranch()) {
			lyr = getNodeLayer();
    		// Opcciones para capas raster
    		if ((lyr instanceof FLyrRaster)) {
    		    transparency = new JMenuItem(PluginServices.getText(this, "adjust_transparency"));
    			getMenu().add( transparency );
    			transparency.setFont(FPopupMenu.theFont);
    			getMenu().setEnabled(true);
    			//getMenu().addSeparator();
    	        //Cambio color
    	        transparency.addActionListener(this);
     		}
		}
	}
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.gui.toc.TocMenuEntry#execute(com.iver.cit.gvsig.gui.toc.ITocItem)
	 */
	public void actionPerformed(ActionEvent e) {
       	lyr = getNodeLayer();
        AdjustTransparencyPanel transPanel = new AdjustTransparencyPanel((FLyrDefault) lyr);
		transPanel.openJDialog();
        //getMapContext().invalidate();                    
	}
}

class ZoomAlTemaTocMenuEntry extends TocMenuEntry {
    private JMenuItem zoom;
	public void initialize(FPopupMenu m) {
		super.initialize(m);
		if (isTocItemBranch()) {
			zoom = new JMenuItem(PluginServices.getText(this, "Zoom_a_la_capa"));
			getMenu().add(zoom);
			zoom.setFont(FPopupMenu.theFont);
            
	        zoom.addActionListener(this);
		}
	}
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.gui.toc.TocMenuEntry#execute(com.iver.cit.gvsig.gui.toc.ITocItem)
	 */
	public void actionPerformed(ActionEvent e) {
    	/*FLayer lyr = getNodeLayer();
        try {
        	getMapContext().getViewPort().setExtent(lyr.getFullExtent());
		} catch (com.iver.cit.gvsig.fmap.DriverException e1) {
			e1.printStackTrace();
		}*/
		// 050209, jmorell: Para que haga un zoom a un grupo de capas seleccionadas.
		FLayer[] actives = getMapContext().getLayers().getActives();
		if (actives.length==1) {
	        try {
	        	getMapContext().getViewPort().setExtent(actives[0].getFullExtent());
			} catch (com.iver.cit.gvsig.fmap.DriverException e1) {
				e1.printStackTrace();
			}
		} else {
			try {
				Rectangle2D maxExtent = setMaxExtent(actives);
		    	getMapContext().getViewPort().setExtent(maxExtent);
			} catch (com.iver.cit.gvsig.fmap.DriverException e1) {
				e1.printStackTrace();
			}
		}
	}
	
	private Rectangle2D setMaxExtent(FLayer[] actives) throws DriverException {
		Rectangle2D extRef = actives[0].getFullExtent();
		double minXRef = extRef.getMinX();
		double maxYRef = extRef.getMaxY();
		double maxXRef = extRef.getMaxX();
		double minYRef = extRef.getMinY();
		for (int i=0;i<actives.length;i++) {
        	Rectangle2D extVar = actives[i].getFullExtent();
    		double minXVar = extVar.getMinX();
    		double maxYVar = extVar.getMaxY();
    		double maxXVar = extVar.getMaxX();
    		double minYVar = extVar.getMinY();
    		if (minXVar<=minXRef) minXRef=minXVar;
    		if (maxYVar>=maxYRef) maxYRef=maxYVar;
    		if (maxXVar>=maxXRef) maxXRef=maxXVar;
    		if (minYVar<=minYRef) minYRef=minYVar;
    		extRef.setRect(minXRef, minYRef, maxXRef-minXRef, maxYRef-minYRef);
		}
		return extRef;
	}
}

class EliminarCapaTocMenuEntry extends TocMenuEntry {
    private JMenuItem removeLayer;
	public void initialize(FPopupMenu m) {
		super.initialize(m);
		if (getNodeUserObject() instanceof TocItemBranch) {
			removeLayer = new JMenuItem(PluginServices.getText(this, "eliminar_capa"));
			removeLayer.setFont(FPopupMenu.theFont);
			removeLayer.addActionListener(this);
			getMenu().addSeparator();
			getMenu().add(removeLayer);
			getMenu().addSeparator();
		}
	}
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.gui.toc.TocMenuEntry#execute(com.iver.cit.gvsig.gui.toc.ITocItem)
	 */
	public void actionPerformed(ActionEvent e) {
    	// 050209, jmorell: Para poder borrar todas las capas seleccionadas desde el FPopUpMenu.
		//					Es necesario pulsar Maysculas a la vez que el botn derecho.
    	FMap fMap = getMapContext();
    	FLayer[] actives = fMap.getLayers().getActives();
    	for (int i = actives.length-1; i>=0; i--){
        	try {
				actives[i].getParentLayer().removeLayer(actives[i]);
				FLayers lyrs=getMapContext().getLayers();
				lyrs.addLayer(actives[i]);
				actives[i].getParentLayer().removeLayer(actives[i]);
				
                if (actives[i] instanceof AlphanumericData){
                    Project project = ((ProjectExtension)PluginServices.getExtension(ProjectExtension.class)).getProject();
                    ProjectTable pt = project.getTable((AlphanumericData) actives[i]);
                    
                    ArrayList tables = project.getTables();
                    for (int j = 0; j < tables.size(); j++) {
                        if (tables.get(j) == pt){
                            project.delTable(j);
                            break;
                        }
                    }
                    
                    PluginServices.getMDIManager().closeSingletonView(pt);
                }
                
				if (getMapContext().getLayers().getLayersCount()==0)
            	    PluginServices.getMainFrame().enableControls();
    		} catch (CancelationException e1) {
    			e1.printStackTrace();
    		}
    	}
		// 050209, jmorell: As solo borra una capa (sobre la que pulsas).
    	/*FLayer lyr = getNodeLayer();
        try {
        	getMapContext().getLayers().removeLayer(lyr);
        	if (getMapContext().getLayers().getLayersCount()==0)PluginServices.getMainFrame().enableControls();
		} catch (CancelationException e1) {
			e1.printStackTrace();
		}*/
    }
}
/**
 * Realiza una agrupacin de capas, a partir de las capas que se encuentren activas.
 * 
 * @author Vicente Caballero Navarro
 */
class LayersGroupTocMenuEntry extends TocMenuEntry {
    private JMenuItem agrupar;    
	public void initialize(FPopupMenu m) {
		super.initialize(m);
        if (isTocItemBranch()) {
            agrupar = new JMenuItem(PluginServices.getText(this, "agrupar_capas"));
            agrupar.setFont(FPopupMenu.theFont);
            getMenu().add(agrupar);        
            getMenu().setEnabled(true);
            if (getMapContext().getLayers().getActives().length > 1) {
                agrupar.setEnabled(true);
            } else {
                agrupar.setEnabled(false);
            }
            agrupar.addActionListener(this);
        }
	}
	
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.gui.toc.TocMenuEntry#execute(com.iver.cit.gvsig.gui.toc.ITocItem)
	 */
	public void actionPerformed(ActionEvent e) {
		//ITocItem tocItem = (ITocItem) getNodeUserObject();
        ChangeName changename=new ChangeName(null);
		PluginServices.getMDIManager().addView(changename);
		String nombre=changename.getName();
		
        if (nombre != null){
			if (isTocItemBranch()){
				FLayer[] actives = getMapContext().getLayers().getActives();
				
				if (actives.length>1) {
									
					FLayers layerGroup=new FLayers(getMapContext(),getNodeLayer().getParentLayer());
					View theView = (View) PluginServices.getMDIManager().getActiveView();
					theView.getMapControl().getMapContext().getLayers()
					   .addLayer(layerGroup);   
					for (int i=0;i<actives.length;i++){
						
						if(actives[i] instanceof FLyrRaster)
							((FLyrRaster)actives[i]).setRemoveRasterFlag(false);
						
						actives[i].getParentLayer().removeLayer(actives[i]);
						layerGroup.addLayer(actives[i]);
					}
					layerGroup.setName(nombre);
					     
	    		
	    		}
				
			}
			// TRUCO PARA REFRESCAR.
	        getMapContext().invalidate();                    
}
}
}
/**
 * Realiza una desagrupacin de capas, a partir de las capas que se encuentren activas.
 * 
 * @author Vicente Caballero Navarro
 */
class LayersUngroupTocMenuEntry extends TocMenuEntry {
    private JMenuItem desagrupar;    
	public void initialize(FPopupMenu m) {
		super.initialize(m);
        if (isTocItemBranch()) {
    		boolean isFLayers=true;
    		if (isTocItemBranch()){
    			FLayer lyr = getNodeLayer();
    			if (!(lyr instanceof FLayers)){
    				isFLayers=false;
    			}
    		}
    		if (isFLayers){
    			desagrupar = new JMenuItem(PluginServices.getText(this, "desagrupar_capas"));
            	desagrupar.setFont(FPopupMenu.theFont);
            	getMenu().add(desagrupar);        
            	getMenu().setEnabled(true);
    	        desagrupar.addActionListener(this);
    		}
        }
	}
	
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.gui.toc.TocMenuEntry#execute(com.iver.cit.gvsig.gui.toc.ITocItem)
	 */
	public void actionPerformed(ActionEvent e) {
		//ITocItem tocItem = (ITocItem) getNodeUserObject();
			if (isTocItemBranch()){
				FLayers agrupa = (FLayers)getNodeLayer();
				FLayers parent=agrupa.getParentLayer();
				if (parent!=null){
					for (int j=0;j<agrupa.getLayersCount();j++){
						parent.addLayer(agrupa.getLayer(j));
					}
					parent.removeLayer(agrupa);
				}
			}
	        // TRUCO PARA REFRESCAR.
	        getMapContext().invalidate();                    

        
        
	}
}
/**
 * Cambia la posicin actual del layer a la primera posicin.
 * 
 * @author Vicente Caballero Navarro
 */
class FirstLayerTocMenuEntry extends TocMenuEntry {
    private JMenuItem first;    
	public void initialize(FPopupMenu m) {
		super.initialize(m);
        if (isTocItemBranch()) {
	    	first = new JMenuItem(PluginServices.getText(this, "colocar_delante"));
        	first.setFont(FPopupMenu.theFont);
        	getMenu().add(first);        
        	getMenu().setEnabled(true);
	        first.addActionListener(this);
        }
	}
	
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.gui.toc.TocMenuEntry#execute(com.iver.cit.gvsig.gui.toc.ITocItem)
	 */
	public void actionPerformed(ActionEvent e) {
		ITocItem tocItem = (ITocItem) getNodeUserObject();
      
      	if (isTocItemBranch()){
            if (getMapContext().getLayers().getActives().length == 1) {
                FLayer layer=((TocItemBranch)tocItem).getLayer();
                FLayers layers=layer.getParentLayer();
                for (int i=0;i<layers.getLayersCount();i++){
                    if(layers.getLayer(i).equals(layer)){
                        layers.removeLayer(i);
                        layers.addLayer(layer);
                    }
                }
            } else if (getMapContext().getLayers().getActives().length > 1) {
                FLayer[] actives = getMapContext().getLayers().getActives();
                FLayers layers=actives[0].getParentLayer();
                //FLayers layerGroup=new FLayers(getMapContext(),getNodeLayer().getParentLayer());
                for (int i=0;i<actives.length;i++){
                    for (int j=0;j<layers.getLayersCount();j++){
                        if(layers.getLayer(j).equals(actives[i])){
                            layers.removeLayer(j);
                            layers.addLayer(actives[i]);
                        }
                    }
                }
            }
		}
		// TRUCO PARA REFRESCAR.
        getMapContext().invalidate();                    
	}
}
/** 
 * @author Nacho Brodin <brodin_ign@gva.es>
 *
 * Entrada de men para la activacin de la funcionalidad de zoom a un
 * pixel.
 */
class ZoomPixelTocMenuEntry extends TocMenuEntry {
	public static final int ZOOM_TO_IMAGE_CENTER = 0x1;
	public static final int ZOOM_TO_VIEW_CENTER = 0x2;
	private JMenuItem properties;
	FLayer lyr = null;
	public int zoomType = ZOOM_TO_VIEW_CENTER;
	
	public void initialize(FPopupMenu m) {
		super.initialize(m);
		
		if (isTocItemBranch()) {
			lyr = getNodeLayer();
    		// Opcciones para capas raster
    		if ((lyr instanceof FLyrRaster)) {
    			properties = new JMenuItem(PluginServices.getText(this, "Zoom_pixel"));
    			getMenu().add( properties );
    			properties.setFont(FPopupMenu.theFont);
    			getMenu().setEnabled(true);
    			
    			properties.addActionListener(this);
     		}
		}
	}
	
	public void actionPerformed(ActionEvent e) {

		FLayer[] actives = getMapContext().getLayers().getActives();
		if (actives.length==1) {
	        try {
	        	Rectangle2D r = actives[0].getFullExtent();
	      
	        	ArrayList attr = ((FLyrRaster)actives[0]).getSource().getAttributes();
	        	int width = 0, height = 0;
	       		for (int i=0; i<attr.size(); i++) {
	    			Object[] a = (Object []) attr.get(i);
	    			if (a[0].toString().equals("Width"))
	    				width = ((Integer)a[1]).intValue();
	    			if (a[0].toString().equals("Height"))
	    				height = ((Integer)a[1]).intValue();	    			    			
	    		}
	        	ViewPort v = getMapContext().getViewPort();
	        	
	        	if (zoomType == ZOOM_TO_IMAGE_CENTER) {
		        	//A centro de imagen
		        	/*double centroX = width/2;
					double centroY = height/2;
					int minX = (int)Math.round(centroX - (v.getImageWidth()/2));
					int minY = (int)Math.round(centroY - (v.getImageHeight()/2));
					int maxX = (int)Math.round(minX + v.getImageWidth());
					int maxY = (int)Math.round(minY + v.getImageHeight());
					double wcOriginWidth = r.getMaxX() - r.getMinX();
					double wcOriginHeight = r.getMaxY() - r.getMinY();
					
					double wcDstMinX = r.getMinX()+((minX*wcOriginWidth)/width);
					double wcDstMinY = r.getMinY()+((minY*wcOriginHeight)/height);
					double wcDstMaxX = r.getMinX()+((maxX*wcOriginWidth)/width);
					double wcDstMaxY = r.getMinY()+((maxY*wcOriginHeight)/height);
					
					double wcDstWidth = wcDstMaxX - wcDstMinX;
					double wcDstHeight = wcDstMaxY - wcDstMinY;*/
	        	} else if (zoomType == ZOOM_TO_VIEW_CENTER) {
		        	//A centro de vista
		        	double wcOriginCenterX = v.getExtent().getMinX()+((v.getExtent().getMaxX()-v.getExtent().getMinX())/2);
		        	double wcOriginCenterY = v.getExtent().getMinY()+((v.getExtent().getMaxY()-v.getExtent().getMinY())/2);
		        					
		        	//Hallamos la relacin entre el pixel y las WC a partir de la imagen de la capa
					double relacionPixelWcWidth =  (r.getMaxX() - r.getMinX())/width;
					double relacionPixelWcHeight = (r.getMaxY() - r.getMinY())/height;
					//double desplazamientoX = r.getMinX(); 
					//double desplazamientoY = r.getMinY();
					
					double wcOriginX = wcOriginCenterX - ((v.getImageWidth()*relacionPixelWcWidth)/2);
					double wcOriginY = wcOriginCenterY - ((v.getImageHeight()*relacionPixelWcHeight)/2);
					
					double wcDstMinX = wcOriginX; 
					double wcDstMinY = wcOriginY; 
					double wcDstMaxX = wcDstMinX + (v.getImageWidth()*relacionPixelWcWidth);
					double wcDstMaxY = wcDstMinY + (v.getImageHeight()*relacionPixelWcHeight);
					
					double wcDstWidth = wcDstMaxX - wcDstMinX;
					double wcDstHeight = wcDstMaxY - wcDstMinY;
									
		        	r = new Rectangle2D.Double(wcDstMinX, wcDstMinY, wcDstWidth, wcDstHeight);
		        	getMapContext().getViewPort().setExtent(r);
	        	}
			} catch (com.iver.cit.gvsig.fmap.DriverException e1) {
				e1.printStackTrace();
			}
		} 
		//View vista = (View) PluginServices.getMDIManager().getActiveView();
		//MapControl mapCtrl = vista.getMapControl();	
	}
}

/** 
 * @author Nacho Brodin <brodin_ign@gva.es>
 *
 * Entrada de men para la activacin de la funcionalidad de zoom a un
 * pixel centrado en el cursor.
 */
class ZoomPixelCursorTocMenuEntry extends TocMenuEntry {
	public static final int ZOOM_TO_IMAGE_CENTER = 0x1;
	public static final int ZOOM_TO_VIEW_CENTER = 0x2;
	private JMenuItem properties;
	FLayer lyr = null;
	public int zoomType = ZOOM_TO_VIEW_CENTER;
	
	public void initialize(FPopupMenu m) {
		super.initialize(m);
		
		if (isTocItemBranch()) {
			lyr = getNodeLayer();
    		// Opcciones para capas raster
    		if ((lyr instanceof FLyrRaster)) {
    			properties = new JMenuItem(PluginServices.getText(this, "Zoom_pixel"));
    			getMenu().add( properties );
    			properties.setFont(FPopupMenu.theFont);
    			getMenu().setEnabled(true);
    			properties.addActionListener(this);
     		}
		}
	}
	
	public void actionPerformed(ActionEvent e) {

		FLayer[] actives = getMapContext().getLayers().getActives();
		if (actives.length==1) {
	     
	        lyr = getNodeLayer();
	    	View vista = (View) PluginServices.getMDIManager().getActiveView();
	    	MapControl mapCtrl = vista.getMapControl();
	    	mapCtrl.setTool("zoom_pixel_cursor");
	            	 
		} 
		//View vista = (View) PluginServices.getMDIManager().getActiveView();
		//MapControl mapCtrl = vista.getMapControl();	
	}
}

/**
 * 
 * @author Nacho Brodin <brodin_ign@gva.es>
 *
 * Entrada de men para la activacin de la funcionalidad de salvar
 * a raster una parte de la vista.
 */
class SaveRasterTocMenuEntry extends TocMenuEntry{
	private JMenuItem properties;
	FLayer lyr = null;
	
	public void initialize(FPopupMenu m) {
		super.initialize(m);
		
		if (isTocItemBranch()) {
			lyr = getNodeLayer();
    		// Opcciones para capas raster
    		if ((lyr instanceof FLyrRaster)) {
    			properties = new JMenuItem(PluginServices.getText(this, "salvar_raster"));
    			getMenu().add( properties );
    			properties.setFont(FPopupMenu.theFont);
    			getMenu().setEnabled(true);
    			//getMenu().addSeparator();
    	        //Cambio color
    			properties.addActionListener(this);
     		}
		}
	}
	
	public void actionPerformed(ActionEvent e) {
		lyr = getNodeLayer();
		View vista = (View) PluginServices.getMDIManager().getActiveView();
		MapControl mapCtrl = vista.getMapControl();
		mapCtrl.setTool("saveRaster");
	}
}

/**
 * Menu de botn derecho para el TOC.
 * Se pueden aadir entradas facilmente desde una extensin,
 * creando una clase derivando de TocMenuEntry, y aadiendola en
 * esttico (o en tiempo de carga de la extensin) a FPopupMenu.
 * (Las entradas actuales estn hechas de esa manera).
 *
 * @author vcn To change the template for this generated type comment go to
 *         Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and
 *         Comments
 */

public class FPopupMenu extends JPopupMenu {
	private static ArrayList menuEntrys = new ArrayList();
    public DefaultMutableTreeNode nodo;
    protected FMap mapContext;
    //private JMenuItem capa;
    // Lo de fijar la fuente es porque en linux se vea mal si no se fija.
    // TODO: Esto no funcionar para idiomas como el chino. Hay que cambiarlo.
    public final static Font theFont = new Font("SansSerif", Font.PLAIN, 10);
    
    static {
    	FPopupMenu.addEntry(new FSymbolChangeColorTocMenuEntry());
    	FPopupMenu.addEntry(new ChangeNameTocMenuEntry());
    	menuEntrys.add(new FLyrVectEditPropertiesTocMenuEntry());
    	//FPopupMenu.addEntry(new FLyrRasterAdjustTransparencyTocMenuEntry());
    	FPopupMenu.addEntry(new ZoomAlTemaTocMenuEntry());
    	//FPopupMenu.addEntry(new ZoomPixelTocMenuEntry());
    	FPopupMenu.addEntry(new ZoomPixelCursorTocMenuEntry());
    	FPopupMenu.addEntry(new EliminarCapaTocMenuEntry());
    	menuEntrys.add(new LayersGroupTocMenuEntry());
		menuEntrys.add(new LayersUngroupTocMenuEntry());
		menuEntrys.add(new FirstLayerTocMenuEntry());
    	FPopupMenu.addEntry(new FLyrRasterAdjustPropertiesTocMenuEntry());
    }

    public static void addEntry(TocMenuEntry entry) {
    	menuEntrys.add(entry);
    }
    
    public static Object getEntry(String className) {
    	for(int i=0;i<menuEntrys.size();i++){
    		if(menuEntrys.get(i).getClass().getName().endsWith(className)){
    			return menuEntrys.get(i);
    		}
    	}
    	return null;
    }
    
    /**
     * Creates a new FPopupMenu object.
     *
     * @param nodo DOCUMENT ME!
     * @param vista DOCUMENT ME!
     */
    public FPopupMenu(FMap mc, DefaultMutableTreeNode node) {
        //super();
        this.mapContext = mc;
        this.nodo = node;
 
        //salir = new MenuItem("Salir");

        for (int i=0; i<menuEntrys.size(); i++) {
        	((TocMenuEntry) menuEntrys.get(i)).initialize(this);
        }
    }
    
    public FMap getMapContext() { return mapContext; }
	
}
