/**
 * 
 */
package com.iver.cit.gvsig.gui.toolListeners;

import java.awt.Cursor;
import java.awt.Event;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.InputEvent;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import javax.swing.ImageIcon;

import com.iver.cit.gvsig.fmap.MapControl;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.tools.BehaviorException;
import com.iver.cit.gvsig.fmap.tools.ZoomInListenerImpl;
import com.iver.cit.gvsig.fmap.tools.Events.MoveEvent;
import com.iver.cit.gvsig.fmap.tools.Events.RectangleEvent;
import com.iver.cit.gvsig.fmap.tools.Listeners.RectangleListener;
import com.iver.cit.gvsig.fmap.tools.Listeners.PanListener;
import com.iver.cit.gvsig.gui.MapOverview;

/**
 * @author jmvivo
 *
 */



public class MapOverviewChangeZoomListener implements RectangleListener, PanListener {
	private final Image izoomin = new ImageIcon(MapControl.class.getResource(
	"images/ZoomInCursor.gif")).getImage();
	private Cursor cur = Toolkit.getDefaultToolkit().createCustomCursor(izoomin,new Point(16, 16), "");
	protected MapControl mapControl;
	
	/**
	 * Crea un nuevo MapOverviewChangeZoomListener.
	 *
	 * @param mapControl Control de mapa asociado
	 */
	public MapOverviewChangeZoomListener(MapControl mapControl) {
		this.mapControl=mapControl;
	}
	
	/**
	 * Cambia el zoom. Es llamado cuando finaliza de
	 * pintar el recuadro
	 *
	 * 
	 *
	 * @see com.iver.cit.gvsig.fmap.tools.Listeners.RectangleListener#rectangle(com.iver.cit.gvsig.fmap.tools.Events.RectangleEvent)	
	 */
	public void rectangle(RectangleEvent event) throws BehaviorException {
		
		if (!checkModifiers(event.getEvent().getModifiers())){
			return;
		}
		MapOverview mov=(MapOverview) this.mapControl;
		ViewPort vpView=mov.getAssociatedMapContext().getViewPort();
		ViewPort vp = mov.getViewPort();
		
		if (vp.getExtent() != null && vpView.getExtent() != null) {
			
			// Recuperamos las coordenadas del evento en px
			Rectangle2D pxRectangle = event.getPixelCoordRect();
			// Recuperamos las coordenadas del evento en coordenadas de la vista de localizador
			Rectangle2D realRectangle = event.getWorldCoordRect();			
			
			if ((pxRectangle.getWidth() < 3) && (pxRectangle.getHeight() < 3))
			{
				// rectangulo < 3 px no hacemos nada
				return;
	
			} else {
				// Cambiamos la extension de la vista asociada al localizador
				vpView.setExtent(realRectangle);				
			}
			
		}


	}

	/** 
	 * (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.tools.Listeners.ToolListener#getCursor()
	 **/
	public Cursor getCursor() {
		return this.cur;
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.tools.Listeners.ToolListener#cancelDrawing()
	 */
	public boolean cancelDrawing() {		
		return true; //???
	}
	
	/** 
	 * Es llamado mientras se hace del 'Dragg' del 
	 * raton. Pinta el recuadro de la forma.
	 **/
	public void move(MoveEvent event) throws BehaviorException {
		
		
		if (!checkModifiers(event.getEvent().getModifiers())){
			return;
		}
		MapOverview mov=(MapOverview) this.mapControl;
		ViewPort vpView=mov.getAssociatedMapContext().getViewPort();
		ViewPort vp = mov.getViewPort();	
		
		if (vp.getExtent() != null && vpView.getExtent() != null) {

			// Creamos un recuadro con las coordenadas del raton
			// traducidas a la del mapa
			Rectangle2D realRectangle = new Rectangle2D.Double();			
			realRectangle.setFrameFromDiagonal(vp.toMapPoint(event.getFrom()),vp.toMapPoint(event.getTo()));			
			
			// Establecemos la forma
			mov.refreshOverView(realRectangle);
		}
		
		
		
	}
	
	/** 
	 * Comprueba que el estado de los botones
	 * sea el deseado para esta herramienta.
	 **/
	private boolean checkModifiers(int modifiers) {
		int keyPressedMask = InputEvent.CTRL_MASK | InputEvent.BUTTON1_MASK;
		return ((modifiers & keyPressedMask) != keyPressedMask); 
	}
	
}
