/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.gui.cad.tools;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.event.InputEvent;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.Point2D.Double;
import java.awt.image.ImagingOpException;
import java.io.IOException;
import java.util.ArrayList;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.CADExtension;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.core.DefaultFeature;
import com.iver.cit.gvsig.fmap.core.FShape;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.edition.DefaultRowEdited;
import com.iver.cit.gvsig.fmap.edition.EditionEvent;
import com.iver.cit.gvsig.fmap.edition.IEditableSource;
import com.iver.cit.gvsig.fmap.edition.IRowEdited;
import com.iver.cit.gvsig.fmap.edition.UtilFunctions;
import com.iver.cit.gvsig.fmap.edition.VectorialEditableAdapter;
import com.iver.cit.gvsig.gui.cad.CADTool;
import com.iver.cit.gvsig.gui.cad.DefaultCADTool;
import com.iver.cit.gvsig.gui.cad.exception.CommadException;
import com.iver.cit.gvsig.gui.cad.tools.smc.ScaleCADToolContext;
import com.iver.cit.gvsig.gui.cad.tools.smc.ScaleCADToolContext.ScaleCADToolState;
import com.iver.cit.gvsig.layers.VectorialLayerEdited;


/**
 * DOCUMENT ME!
 *
 * @author Vicente Caballero Navarro
 */
public class ScaleCADTool extends DefaultCADTool {
    private ScaleCADToolContext _fsm;
    private Point2D firstPoint;
    private Point2D lastPoint;
	private Point2D scalePoint;
	private Double orr;
	private Double frr;
	private Double ore;
	private Double fre;

    /**
     * Crea un nuevo PolylineCADTool.
     */
    public ScaleCADTool() {
    }

    /**
     * Mtodo de incio, para poner el cdigo de todo lo que se requiera de una
     * carga previa a la utilizacin de la herramienta.
     */
    public void init() {
        _fsm = new ScaleCADToolContext(this);
    }

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.gui.cad.CADTool#transition(com.iver.cit.gvsig.fmap.layers.FBitSet, double, double)
     */
    public void transition(double x, double y, InputEvent event) {
        _fsm.addPoint(x, y, event);
    }

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.gui.cad.CADTool#transition(com.iver.cit.gvsig.fmap.layers.FBitSet, double)
     */
    public void transition(double d) {
        _fsm.addValue(d);
    }

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.gui.cad.CADTool#transition(com.iver.cit.gvsig.fmap.layers.FBitSet, java.lang.String)
     */
    public void transition(String s) throws CommadException {
    	if (!super.changeCommand(s)){
    		_fsm.addOption(s);
    	}
    }

    /**
     * DOCUMENT ME!
     */
    public void selection() {
        ArrayList rowSelected=getSelectedRows();
        if (rowSelected.size() == 0 && !CADExtension.getCADTool().getClass().getName().equals("com.iver.cit.gvsig.gui.cad.tools.SelectionCADTool")) {
            CADExtension.setCADTool("_selection",false);
            ((SelectionCADTool) CADExtension.getCADTool()).setNextTool(
                "_scale");
        }
    }

    /**
     * Equivale al transition del prototipo pero sin pasarle como parmetro el
     * editableFeatureSource que ya estar creado.
     *
     * @param x parmetro x del punto que se pase en esta transicin.
     * @param y parmetro y del punto que se pase en esta transicin.
     */
    public void addPoint(double x, double y,InputEvent event) {
        ScaleCADToolState actualState = (ScaleCADToolState) _fsm.getPreviousState();
        String status = actualState.getName();

        if (status.equals("Scale.PointMain")) {
				firstPoint = new Point2D.Double(x, y);
			    scalePoint = firstPoint;
		} else if (status.equals("Scale.ScaleFactorOrReference")) {
			PluginServices.getMDIManager().setWaitCursor();
			lastPoint = new Point2D.Double(x, y);

			//double w;
			//double h;
			//w = lastPoint.getX() - firstPoint.getX();
			//h = lastPoint.getY() - firstPoint.getY();

			try {
				double size=getCadToolAdapter().getMapControl().getViewPort().toMapDistance(getCadToolAdapter().getMapControl().getWidth());
				scale(firstPoint.distance(lastPoint)/(size/40));
			} catch (DriverIOException e) {
				e.printStackTrace();
			} catch (IOException e1) {
				e1.printStackTrace();
			}

			PluginServices.getMDIManager().restoreCursor();
		} else if (status.equals("Scale.PointOriginOrScaleFactor")) {
			orr = new Point2D.Double(x, y);
		} else if (status.equals("Scale.EndPointReference")) {
			frr = new Point2D.Double(x, y);
		} else if (status.equals("Scale.OriginPointScale")) {
			ore = new Point2D.Double(x, y);
			firstPoint = ore;
		} else if (status.equals("Scale.EndPointScale")) {
			fre = new Point2D.Double(x, y);

			double distrr = orr.distance(frr);
			double distre = ore.distance(fre);
			double escalado = distre / distrr;

			try {
				scale(escalado);
			} catch (DriverIOException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

    }

    /**
     * Mtodo para dibujar la lo necesario para el estado en el que nos
     * encontremos.
     *
     * @param g Graphics sobre el que dibujar.
     * @param x parmetro x del punto que se pase para dibujar.
     * @param y parmetro x del punto que se pase para dibujar.
     */
    public void drawOperation(Graphics g, double x, double y) {
		ScaleCADToolState actualState = ((ScaleCADToolContext) _fsm).getState();
		String status = actualState.getName();
		ArrayList selectedRow = getSelectedRows();
		Point2D currentPoint = new Point2D.Double(x, y);

		if (status.equals("Scale.ScaleFactorOrReference")) {
			VectorialLayerEdited vle = getVLE();
			ViewPort vp = vle.getLayer().getFMap().getViewPort();
			Point2D point = vp.fromMapPoint(firstPoint.getX(), firstPoint
					.getY());
			double size = getCadToolAdapter().getMapControl().getViewPort()
					.toMapDistance(
							getCadToolAdapter().getMapControl().getWidth());
			double scale = firstPoint.distance(currentPoint) / (size / 40);
			drawLine((Graphics2D) g, firstPoint, currentPoint);
			if (selectedRow.size() < CADTool.TOPGEOMETRY) {
				for (int i = 0; i < selectedRow.size(); i++) {
					DefaultFeature fea = (DefaultFeature) ((DefaultRowEdited) selectedRow
							.get(i)).getLinkedRow();
					IGeometry geometry = fea.getGeometry().cloneGeometry();

					UtilFunctions.scaleGeom(geometry, firstPoint, scale, scale);
					geometry.draw((Graphics2D) g, getCadToolAdapter()
							.getMapControl().getViewPort(),
							CADTool.modifySymbol);

				}

			} else {
				AffineTransform at = new AffineTransform();
				at.setToTranslation(point.getX(), point.getY());
				at.scale(scale, scale);
				at.translate(-point.getX(), -point.getY());
				Image imgSel = vle.getSelectionImage();
				try {
					((Graphics2D) g).drawImage(imgSel, at, null);
				} catch (ImagingOpException e) {
				}
			}
			PluginServices.getMainFrame().getStatusBar().setMessage("5",
					"Factor = " + scale);
		} else if (status.equals("Scale.EndPointScale")) {
			VectorialLayerEdited vle = getVLE();
			ViewPort vp = vle.getLayer().getFMap().getViewPort();
			Point2D point = vp.fromMapPoint(scalePoint.getX(), scalePoint
					.getY());

			double distrr = orr.distance(frr);
			double distre = ore.distance(currentPoint);
			double escalado = distre / distrr;
			if (selectedRow.size() < CADTool.TOPGEOMETRY) {
				for (int i = 0; i < selectedRow.size(); i++) {
						DefaultFeature fea = (DefaultFeature) ((DefaultRowEdited) selectedRow
								.get(i)).getLinkedRow();
						IGeometry geometry = fea.getGeometry().cloneGeometry();
						UtilFunctions.scaleGeom(geometry, scalePoint, escalado,
								escalado);
						geometry.draw((Graphics2D) g, getCadToolAdapter()
								.getMapControl().getViewPort(),
								CADTool.modifySymbol);

					}
				} else {
					AffineTransform at = new AffineTransform();
					at.setToTranslation(point.getX(), point.getY());
					at.scale(escalado, escalado);
					at.translate(-point.getX(), -point.getY());
					Image imgSel = vle.getSelectionImage();
					try {
						((Graphics2D) g).drawImage(imgSel, at, null);
					} catch (ImagingOpException e) {
					}
				}
				drawLine((Graphics2D) g, firstPoint, new Point2D.Double(x, y));
			}
	}

    /**
	 * Add a diferent option.
	 *
	 * @param s
	 *            Diferent option.
	 */
    public void addOption(String s) {
    	ScaleCADToolState actualState = (ScaleCADToolState) _fsm.getPreviousState();
        String status = actualState.getName();
       if (status.equals("Scale.ScaleFactorOrReference")) {
		/*	try {
				scale(2);
			} catch (DriverIOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		*/
		}
	}

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.gui.cad.CADTool#addvalue(double)
     */
    public void addValue(double d) {
    	ScaleCADToolState actualState = (ScaleCADToolState) _fsm.getPreviousState();
        String status = actualState.getName();
        if (status.equals("Scale.ScaleFactorOrReference")) {
    			try {
    				scale(d);
    			} catch (DriverIOException e) {
    				e.printStackTrace();
    			} catch (IOException e1) {
    				e1.printStackTrace();
    			}

    	}
    }
    private void scale(double scaleFactor) throws DriverIOException, IOException {
    		VectorialEditableAdapter vea=getVLE().getVEA();
    		vea.startComplexRow();
    		ArrayList selectedRow=getSelectedRows();
    		ArrayList selectedRowAux=new ArrayList();
    		for (int i = 0; i < selectedRow.size(); i++) {
    			IRowEdited edRow = (IRowEdited) selectedRow.get(i);
    			DefaultFeature fea = (DefaultFeature) edRow.getLinkedRow().cloneRow();
				UtilFunctions.scaleGeom(fea.getGeometry(), scalePoint, scaleFactor, scaleFactor);
    				// df.getGeometry().scale(scalePoint, scaleFactor, scaleFactor);
    			int index=	vea.modifyRow(edRow.getIndex(), fea,getName(),EditionEvent.GRAPHIC);
    			selectedRowAux.add(new DefaultRowEdited(fea,IRowEdited.STATUS_MODIFIED,index));
    		}
    		vea.endComplexRow();
    		clearSelection();
    		selectedRow=selectedRowAux;
    	}

	public String getName() {
		return PluginServices.getText(this,"scale_");
	}

	public String toString() {
		return "_scale";
	}

}
