/*
 * Created on 02-feb-2006
 *
 * gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
*
* $Id: AttributeBufferVisitor.java,v 1.1 2006/05/24 21:14:41 azabala Exp $
* $Log: AttributeBufferVisitor.java,v $
* Revision 1.1  2006/05/24 21:14:41  azabala
* primera version en cvs despues de refactoring orientado a crear un framework extensible de geoprocessing
*
* Revision 1.8  2006/05/01 19:18:09  azabala
* revisin general del buffer (aadidos anillos concentricos, buffers interiores y exteriores, etc)
*
* Revision 1.7  2006/03/28 16:27:16  azabala
* *** empty log message ***
*
* Revision 1.6  2006/03/07 21:01:33  azabala
* *** empty log message ***
*
* Revision 1.5  2006/03/06 19:48:39  azabala
* *** empty log message ***
*
* Revision 1.2  2006/03/05 19:57:05  azabala
* *** empty log message ***
*
* Revision 1.1  2006/02/17 16:31:38  azabala
* *** empty log message ***
*
* Revision 1.3  2006/02/12 21:02:44  azabala
* *** empty log message ***
*
* Revision 1.2  2006/02/09 16:01:06  azabala
* First version in CVS
*
* Revision 1.1  2006/02/02 19:46:01  azabala
* First version in CVS
*
*
*/
package com.iver.cit.gvsig.geoprocess.buffer.fmap;

import com.hardcode.gdbms.engine.values.NumericValue;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;
import com.iver.cit.gvsig.fmap.layers.layerOperations.AlphanumericData;
import com.iver.cit.gvsig.fmap.layers.layerOperations.VectorialData;
import com.iver.cit.gvsig.fmap.operations.strategies.VisitException;
import com.iver.cit.gvsig.geoprocess.core.fmap.GeoprocessException;
import com.iver.cit.gvsig.geoprocess.core.fmap.GeoprocessingResultsProcessor;
import com.vividsolutions.jts.geom.Geometry;
/**
 * FeatureVisitor that computes buffer geometries from original geometrie
 * and an attribute value of IFeature
 * @author azabala
 *
 */
public class AttributeBufferVisitor extends BufferVisitor {
	/**
	 * Attribute name from which visitor will get buffer distances
	 */
	private String attributeName;
	/**
	 * It allows visitor to get attribute values link to geometries
	 */
	private SelectableDataSource recordset;
	/**
	 * Processes individual buffer results
	 * (we could persist them in a datastore, caching them, etc
	 */
	private GeoprocessingResultsProcessor resultProcessor;
	
	
	/**
	 * Constructor
	 * @param attributeName
	 * @param recordset
	 */
	public AttributeBufferVisitor(String attributeName) throws GeoprocessException{
		this.attributeName = attributeName;
	}

	public void stop(FLayer layer) {
		resultProcessor.finish();
	}

	public boolean start(FLayer layer) {
		if(layer instanceof AlphanumericData && layer instanceof VectorialData){
			try {
				this.recordset = ((AlphanumericData)layer).getRecordset();
			} catch (DriverException e) {
				return false;//must we throw an Exception??
			}
			return true;
		}
		return false;
	}

	public double getBufferDistance(IGeometry g, int index){
		NumericValue value = null;
		try {
			int fieldIndex = recordset.getFieldIndexByName(attributeName);
			//we could throw an AttributeBufferException, or
			//we could check preconditions in BufferGeoprocess
			//TODO to check attribute is a NUMBER
			 value = (NumericValue) recordset.
			 		getFieldValue(index, fieldIndex);
		} catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
			return -1d;
		}
		//TODO catch ClassCastException 
		return value.doubleValue();
	}

	public String getProcessDescription() {
		return "Computing buffers applying distances from attribute";
	}
}

