/*
 * Created on 28-mar-2006
 *
 * gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: GeoprocessPaneContainer.java,v 1.2 2006/05/25 08:20:43 jmvivo Exp $
 * $Log: GeoprocessPaneContainer.java,v $
 * Revision 1.2  2006/05/25 08:20:43  jmvivo
 * Modificado para que, si el proceso no ha podido ejecutarse, no cierre la ventana
 *
 * Revision 1.1  2006/05/24 21:13:09  azabala
 * primera version en cvs despues de refactoring orientado a crear un framework extensible de geoprocessing
 *
 * Revision 1.1  2006/04/11 17:55:51  azabala
 * primera version en cvs
 *
 * Revision 1.2  2006/04/07 19:00:58  azabala
 * *** empty log message ***
 *
 * Revision 1.1  2006/03/28 16:26:45  azabala
 * *** empty log message ***
 *
 *
 */
package com.iver.cit.gvsig.geoprocess.core.gui;

import javax.swing.JPanel;

import com.iver.andami.PluginServices;
import com.iver.andami.ui.mdiManager.View;
import com.iver.andami.ui.mdiManager.ViewInfo;
import com.iver.cit.gvsig.geoprocess.core.IGeoprocessController;

import javax.swing.JButton;
import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Window;
/**
 * This GUI class is a container to show gui components associated to
 * a geoprocess.
 * To correctly execute the geoprocess associated, this PaneContainer must
 * receive an AndamiCommand instance, which is controller class that 
 * knows how to invoque geoprocess
 * @author azabala
 *
 */
public class GeoprocessPaneContainer extends JPanel implements View {
	private static final long serialVersionUID = 1L;
	private JPanel buttonPanel = null;
	private JPanel mainPanel = null;
	private JButton okButton = null;
	private JButton cancelButton = null;
	
	private ViewInfo viewInfo = null;
	private IGeoprocessController controller;

	

	/**
	 * Constructor from GUI component associated to a geoprocess
	 * @param mainPanel
	 */
	public GeoprocessPaneContainer(JPanel mainPanel) {
		super();
		this.mainPanel = mainPanel;
		initialize();
	}
	/**
	 * Implementation of ViewInfo andami interface.
	 */
	public ViewInfo getViewInfo() {
		if (viewInfo == null) {
			viewInfo = new ViewInfo(ViewInfo.MODALDIALOG);
			viewInfo.setTitle(PluginServices.getText(this,
					"Herramientas_de_analisis"));
		}
		return viewInfo;
	}
	
	/**
	 * Sets the main panel (GUI component associated to a geoprocess)
	 * @param geoprocessPanel
	 */
	public void setMainPanel(JPanel geoprocessPanel){
		this.add(geoprocessPanel, BorderLayout.NORTH);
		this.validate();
	}
	/**
	 * Sets AndamiCmd instance, which knows how to run geoprocess
	 * from user inputs in main panel.
	 * @param command
	 */
	public void setCommand(IGeoprocessController controller){
		this.controller = controller;
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setLayout(new BorderLayout());
		this.setBounds(new java.awt.Rectangle(0,0,500,400));
		this.add(getButtonPanel(), java.awt.BorderLayout.SOUTH);
		this.add(getMainPanel(), java.awt.BorderLayout.CENTER);
		this.validate();
	}

	/**
	 * This method initializes buttonPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getButtonPanel() {
		if (buttonPanel == null) {
			buttonPanel = new JPanel();
			buttonPanel.add(getCancelButton(), null);
			buttonPanel.add(getOkButton(), null);
		}
		return buttonPanel;
	}

	/**
	 * This method initializes mainPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getMainPanel() {
		return mainPanel;
	}

	/**
	 * This method initializes okButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getOkButton() {
		if (okButton == null) {
			okButton = new JButton();
			okButton.setText(PluginServices.getText(this, "Terminar"));
			okButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					if (controller.launchGeoprocess()) {
						//So controller will launch geoprocess in background,
						//we can call cancel to close dialog
						cancel();
					}
				}
			});
		}
		return okButton;
	}

	/**
	 * This method initializes cancelButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setText(PluginServices.getText(this, "Cerrar"));
			cancelButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					cancel();
				}
			});
		}
		return cancelButton;
	}
	/**
	 * Closes parent dialog
	 *
	 */
	public void cancel(){
		Container container = getParent();
		Container parentOfContainer = null;
		while(! (container instanceof Window)){
			parentOfContainer = container.getParent();
			container = parentOfContainer;
		}
		((Window)container).dispose();	
	}

}  //  @jve:decl-index=0:visual-constraint="31,13"
