/*
 * Created on 21-feb-2006
 *
 * gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: IntersectVisitor.java,v 1.1 2006/05/24 21:10:40 azabala Exp $
 * $Log: IntersectVisitor.java,v $
 * Revision 1.1  2006/05/24 21:10:40  azabala
 * primera version en cvs despues de refactoring orientado a crear un framework extensible de geoprocessing
 *
 * Revision 1.6  2006/05/01 19:12:17  azabala
 * optimizaciones haciendo uso de strategy.process(rectangle) (el bitset se estaba recorriendo secuencial, no aleatorio)
 *
 * Revision 1.5  2006/03/26 20:03:06  azabala
 * *** empty log message ***
 *
 * Revision 1.4  2006/03/23 21:04:36  azabala
 * *** empty log message ***
 *
 * Revision 1.3  2006/03/21 19:29:18  azabala
 * *** empty log message ***
 *
 * Revision 1.2  2006/03/07 21:01:33  azabala
 * *** empty log message ***
 *
 * Revision 1.1  2006/03/06 19:48:39  azabala
 * *** empty log message ***
 *
 * Revision 1.2  2006/03/05 19:58:47  azabala
 * *** empty log message ***
 *
 * Revision 1.1  2006/02/26 20:54:38  azabala
 * *** empty log message ***
 *
 *
 */
package com.iver.cit.gvsig.geoprocess.intersection.fmap;

import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.values.Value;
import com.iver.cit.gvsig.fmap.core.IFeature;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.core.v02.FConverter;
import com.iver.cit.gvsig.fmap.drivers.FieldDescription;
import com.iver.cit.gvsig.fmap.drivers.ILayerDefinition;
import com.iver.cit.gvsig.fmap.edition.EditionException;
import com.iver.cit.gvsig.fmap.layers.FBitSet;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrVect;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;
import com.iver.cit.gvsig.fmap.layers.layerOperations.AlphanumericData;
import com.iver.cit.gvsig.fmap.layers.layerOperations.VectorialData;
import com.iver.cit.gvsig.fmap.operations.strategies.FeatureVisitor;
import com.iver.cit.gvsig.fmap.operations.strategies.Strategy;
import com.iver.cit.gvsig.fmap.operations.strategies.VisitException;
import com.iver.cit.gvsig.geoprocess.core.fmap.FeatureFactory;
import com.iver.cit.gvsig.geoprocess.core.fmap.FeatureProcessor;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.MultiPolygon;
import com.vividsolutions.jts.geom.Polygon;
import com.vividsolutions.jts.precision.EnhancedPrecisionOp;

/**
 * Computes intersections of features of a layer with features of an overlay
 * layer.
 * 
 * @author azabala
 * 
 * FIXME Crear una clase abstracta: OverlayVisitor, pues todos los visitors de
 * un overlay son muy parecidos.
 * 
 */
public class IntersectVisitor implements FeatureVisitor {

	/**
	 * Allows to get attributes of first layer features which is being
	 * intersected
	 */
	SelectableDataSource firstRs;

	/**
	 * Number of fields of first layer recordset
	 */
	int numFieldsA;

	/**
	 * looks for overlay features of the processed feauture by spatial criteria
	 * (queryByEnvelope)
	 */
	FLyrVect overlayLayer;

	/**
	 * Strategy to process overlay layer
	 */
	Strategy strategy;

	/**
	 * flag to decide if process onlye features selected in overlay layer.
	 */
	boolean onlyOverlaySelection;

	/**
	 * Gets attributes of second layer features which are being intersected.
	 */
	SelectableDataSource secondRs;

	/**
	 * Number of fields of second layer recordset
	 */
	int numFieldsB;

	/**
	 * Schema of the result layer
	 */
	ILayerDefinition layerDefinition;

	/**
	 * It processes features resulting of intersetions. It could saves them in
	 * persistent datastore, caching them, reprocess them, reprojects them, etc.
	 */
	FeatureProcessor featureProcessor;
	
	/**
	 * Constructor
	 * @param overlayLayer
	 * @param processor
	 * @param strategy
	 * @param onlyOverlaySelection
	 * @throws DriverException
	 * @throws com.iver.cit.gvsig.fmap.DriverException
	 */
	public IntersectVisitor(FLyrVect overlayLayer, FeatureProcessor processor,
			Strategy strategy, boolean onlyOverlaySelection)
			throws DriverException, com.iver.cit.gvsig.fmap.DriverException {
		this.overlayLayer = overlayLayer;
		this.featureProcessor = processor;
		this.strategy = strategy;
		this.onlyOverlaySelection = onlyOverlaySelection;
		secondRs = overlayLayer.getRecordset();
		numFieldsB = secondRs.getFieldCount();
	}

	public void visit(IGeometry g, final int index) throws VisitException {

		final Geometry firstJts = g.toJTSGeometry();
		final boolean onlyOverlay = onlyOverlaySelection;
		try {
			strategy.process(new FeatureVisitor(){
				public void visit(IGeometry g, int indexOverlay) throws VisitException {
					if(onlyOverlay){
						try {
							if(!overlayLayer.getRecordset().getSelection().get(index))
								return;
						} catch (com.iver.cit.gvsig.fmap.DriverException e) {
							throw new VisitException(
							"Error en interseccion: verificando si un posible overlay esta seleccionado", e);
						}//geometry g is not selected
					}				
					Geometry overlayJts = g.toJTSGeometry();					
					if (firstJts.intersects(overlayJts)) {
						Geometry newGeoJts = EnhancedPrecisionOp.intersection(firstJts, overlayJts);
						if (!(newGeoJts instanceof Polygon)
								&& !(newGeoJts instanceof MultiPolygon)) {
							// intersection of adjacent polygons is a linestring
							// but we are not interested in it
							return;
						}
						IFeature intersectionFeature;
						try {
							intersectionFeature = createFeature(newGeoJts,
									index, indexOverlay);
						} catch (DriverException e) {
							throw new VisitException(
									"Error al crear el feature resultante de la interseccion");
						}
						featureProcessor.processFeature(intersectionFeature);
					}// if intersects	
				}

				public String getProcessDescription() {
					return "Computing intersections of a polygon with its adjacents";
				}
				public void stop(FLayer layer) {
				}
				public boolean start(FLayer layer) {
					return true;
				}},g.getBounds2D());
			
		} catch (com.iver.cit.gvsig.fmap.DriverException e) {
			throw new VisitException(
					"Error buscando los overlays que intersectan con un feature");
		} 
	}

	public void stop(FLayer layer) {
		featureProcessor.finish();
	}

	public boolean start(FLayer layer) {
		if (layer instanceof AlphanumericData && layer instanceof VectorialData) {
			try {
				this.firstRs = ((AlphanumericData) layer).getRecordset();
				numFieldsA = firstRs.getFieldCount();
				this.featureProcessor.start();
			} catch (com.iver.cit.gvsig.fmap.DriverException e) {
				return false;
			} catch (DriverException e) {
				return false;
			} catch (EditionException e) {
				return false;
			}
			return true;
		}
		return false;
	}
	

	/**
	 * TODO Meter este metodo en FeatureFactory, porque lo estoy copypasteando
	 * en todos los Visitor
	 * 
	 * FIXME El esquema es resultado de la mezcla de los esquemas de dos capas
	 * Como saber qu campo de ILayerDefinition se toma de firstRs y cual de secondRs?
	 * Mirando DefinitionUtils, vemos que primero se recorre firstLayer,
	 * y luego secondLayer. REVISAR
	 */
	private IFeature createFeature(Geometry jtsGeometry, int firstLayerIndex,
			int overlayLayerIndex) throws DriverException {
		IFeature solution = null;
		IGeometry intersectGeometry = FConverter.jts_to_igeometry(jtsGeometry);
		
		
		Value[] featureAttr = new Value[numFieldsA + numFieldsB];
		for (int indexField = 0; indexField < numFieldsA; indexField++) {
			featureAttr[indexField] = firstRs.getFieldValue(firstLayerIndex,
					indexField);
		}
		for (int indexFieldB = 0; indexFieldB < numFieldsB; indexFieldB++) {
			featureAttr[numFieldsA + indexFieldB] = secondRs.getFieldValue(
					overlayLayerIndex, indexFieldB);
		}
		solution = FeatureFactory.createFeature(featureAttr, intersectGeometry);
		return solution;
		
	}

	public void setFeatureProcessor(FeatureProcessor featureProcessor) {
		this.featureProcessor = featureProcessor;
	}

	public String getProcessDescription() {
		return "Computing intersections between two layers";
	}

	public ILayerDefinition getLayerDefinition() {
		return layerDefinition;
	}

	public void setLayerDefinition(ILayerDefinition layerDefinition) {
		this.layerDefinition = layerDefinition;
	}

}
