/*
 * Created on 17-feb-2006
 *
 * gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
*
* $Id: UnionGeoprocess.java,v 1.1 2006/05/24 21:09:11 azabala Exp $
* $Log: UnionGeoprocess.java,v $
* Revision 1.1  2006/05/24 21:09:11  azabala
* primera version en cvs despues de refactoring orientado a crear un framework extensible de geoprocessing
*
* Revision 1.12  2006/05/01 19:04:47  azabala
* la cancelacion no solo para el ITask que ejecuta el geoproceso, adems llama al metodo cancel() del mismo (que se supone que debera hacer un drop() con los resultados del geoproceso inconcluso)
*
* Revision 1.11  2006/03/26 20:03:18  azabala
* *** empty log message ***
*
* Revision 1.10  2006/03/23 21:05:29  azabala
* *** empty log message ***
*
* Revision 1.9  2006/03/21 19:29:45  azabala
* *** empty log message ***
*
* Revision 1.8  2006/03/17 19:53:53  azabala
* *** empty log message ***
*
* Revision 1.7  2006/03/15 18:34:41  azabala
* *** empty log message ***
*
* Revision 1.6  2006/03/14 18:32:46  fjp
* Cambio con LayerDefinition para que sea compatible con la definicin de tablas tambin.
*
* Revision 1.5  2006/03/07 21:01:33  azabala
* *** empty log message ***
*
* Revision 1.4  2006/03/06 19:48:39  azabala
* *** empty log message ***
*
* Revision 1.3  2006/02/26 20:55:37  azabala
* *** empty log message ***
*
* Revision 1.2  2006/02/20 19:43:51  azabala
* *** empty log message ***
*
* Revision 1.1  2006/02/19 20:55:34  azabala
* *** empty log message ***
*
*
*/
package com.iver.cit.gvsig.geoprocess.union.fmap;

import java.util.Map;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.drivers.ILayerDefinition;
import com.iver.cit.gvsig.fmap.edition.EditionException;
import com.iver.cit.gvsig.fmap.layers.FBitSet;
import com.iver.cit.gvsig.fmap.layers.FLyrVect;
import com.iver.cit.gvsig.fmap.operations.strategies.Strategy;
import com.iver.cit.gvsig.fmap.operations.strategies.StrategyManager;
import com.iver.cit.gvsig.fmap.operations.strategies.VisitException;
import com.iver.cit.gvsig.geoprocess.core.fmap.AbstractGeoprocess;
import com.iver.cit.gvsig.geoprocess.core.fmap.DeferredFeaturePersisterProcessor;
import com.iver.cit.gvsig.geoprocess.core.fmap.DefinitionUtils;
import com.iver.cit.gvsig.geoprocess.core.fmap.FeaturePersisterProcessor2;
import com.iver.cit.gvsig.geoprocess.core.fmap.GeoprocessException;
import com.iver.cit.gvsig.geoprocess.core.fmap.IOverlayGeoprocess;
import com.iver.cit.gvsig.geoprocess.core.fmap.XTypes;
import com.iver.cit.gvsig.geoprocess.difference.fmap.DifferenceVisitor;
import com.iver.cit.gvsig.geoprocess.intersection.fmap.IntersectVisitor;
import com.iver.utiles.swing.threads.CancellableMonitorable;
import com.iver.utiles.swing.threads.DefaultCancellableMonitorable;
import com.iver.utiles.swing.threads.IMonitorableTask;
/**
 * Union geoprocess is known like "spatial or" because is an overlay
 * geoprocess (it computes intersections between features of
 * two layers) formed for geometries of one layer or another.
 * <br>
 *  Algorithm makes these three passes:
 *  a) computing intersections and saves them in a temp file.
 *  b) computing differences with first layer.
 *  c) computing differences with second layer.
 * @author azabala
 *
 *TODO Is very similar to ClipGeoprocess. Build an overlay abstract class.
 *
 *
 */
public class UnionGeoprocess extends AbstractGeoprocess 
							implements IOverlayGeoprocess {
	
	/**
	 * overlay layer
	 */
	private FLyrVect overlayLayer;
	
	/**
	 * Schema of the result layer
	 */
	private ILayerDefinition resultLayerDefinition;
	/**
	 * flag to only clip selection of input layer
	 */
	private boolean onlyFirstLayerSelection = false;
	/**
	 * flag to only clip with selection of clipping layer
	 */
	private boolean onlyClipLayerSelection = false;
	
	/**
	 * Constructor 
	 *
	 */
	public UnionGeoprocess(FLyrVect inputLayer){
		setFirstOperand(inputLayer);
	}
	
	public void setSecondOperand(FLyrVect secondLayer) {
		this.overlayLayer = secondLayer;
	}

	public void setFirstOperand(FLyrVect firstLayer) {
		this.firstLayer = firstLayer;

	}

	public void setParameters(Map params) throws GeoprocessException {
		Boolean firstLayerSelection = (Boolean) params.get("firstlayerselection");
		if(firstLayerSelection != null)
			this.onlyFirstLayerSelection = firstLayerSelection.booleanValue();
		
		Boolean secondLayerSelection = (Boolean) params.get("secondlayerselection");
		if(secondLayerSelection != null)
			this.onlyClipLayerSelection = secondLayerSelection.booleanValue();

	}

	public void checkPreconditions() throws GeoprocessException {
		if (firstLayer == null)
			throw new GeoprocessException("Union: capa de entrada a null");
		if (overlayLayer == null)
			throw new GeoprocessException("Union: capa de union a null");
		if(this.writer == null || 
		   this.schemaManager == null){
			throw new GeoprocessException("Operacion de union sin especificar capa de resultados");
		}
		try {
			if(firstLayer.getShapeType() != XTypes.POLYGON){
				throw new GeoprocessException("Primera capa de union no es de polgonos");
			}
			if(overlayLayer.getShapeType() != XTypes.POLYGON){
				throw new GeoprocessException("Segunda capa de union no es de polgonos");
			}
		} catch (com.iver.cit.gvsig.fmap.DriverException e) {
			throw new GeoprocessException("Error al tratar de chequear si las capas a unir son de polgonos");
		}

	}
	
	/**
	 * Runs execution of this geoprocess.
	 */
	public void process() throws GeoprocessException {
		/*
		 * Lo vamos a hacer en tres pasadas:
		 * a) intersecciones con el geoproceso Intersection,
		 * guardando en el fichero del usuario.
		 * b) diferencias de first con overlay, guardando en el
		 * mismo fichero
		 * c) diferencias de overlay con first, guardando en el 
		 * mismo fichero
		 */
		try {
			this.schemaManager.createOrAlterSchema(createLayerDefinition());
			writer.preProcess();
			
			Strategy strategy = StrategyManager.getStrategy(firstLayer);
			Strategy strategy2 = StrategyManager.getStrategy(overlayLayer);
			FeaturePersisterProcessor2 featureProcessor =
				new FeaturePersisterProcessor2(writer);
			
			IntersectVisitor visitor = new IntersectVisitor(overlayLayer, 
														featureProcessor,
														strategy2, 
														onlyClipLayerSelection);
			strategy.process(visitor);
			
			//diferencias entre input y overlay
			writer.preProcess();
			DifferenceVisitor visitor2 = new DifferenceVisitor(overlayLayer,
					featureProcessor, strategy2, onlyClipLayerSelection);
			visitor2.setLayerDefinition(resultLayerDefinition);
			strategy.process(visitor2);
			
			writer.preProcess();
			DifferenceVisitor visitor3 = new DifferenceVisitor(this.firstLayer,
					featureProcessor, strategy, onlyFirstLayerSelection);
			visitor3.setLayerDefinition(resultLayerDefinition);
			strategy2.process(visitor3);
			
		} catch (EditionException e) {
			throw new GeoprocessException("Error al crear el esquema/fichero de la nueva capa");
		} catch (DriverException e) {
			throw new GeoprocessException("Error de driver al calcular el geoproceso interseccion");
		} catch (VisitException e) {
			throw new GeoprocessException("Error al procesar el feature de una capa durante el geoproceso interseccion");
		} catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}

	public void cancel() {
		// TODO Auto-generated method stub

	}

	public ILayerDefinition createLayerDefinition() {
		if(resultLayerDefinition == null){
			try {
				resultLayerDefinition = DefinitionUtils.
						mergeLayerDefinitions(firstLayer, overlayLayer);
			} catch (Exception e) {
				//TODO Quizas createLayerDefinition deberia lanzar
				//una excepcion
				e.printStackTrace();
			}
		}
		return resultLayerDefinition;
	}

	public IMonitorableTask createTask() {
		try {
			return new UnionMonitorableTask();
		} catch (Exception e) {
			return null;
		}
	}
	
	/**
	 * IMonitorableTask that allows to run diff geoprocess in background,
	 * with cancelation requests.
	 * 
	 * @author azabala
	 * 
	 * FIXME INTERNACIONALIZAR TODOS LOS TEXTOS
	 * 
	 */
	class UnionMonitorableTask implements IMonitorableTask {
		private CancellableMonitorable cancelMonitor = null;
		String UNION_MESSAGE = PluginServices.getText(this, "Mensaje_union");
		String FIRST_PASS_NOTE = PluginServices.getText(this, "Mensaje_primera_pasada_union");
		String SECOND_PASS_NOTE = PluginServices.getText(this, "Mensaje_segunda_pasada_union");
		String THIRD_PASS_NOTE = PluginServices.getText(this, "Mensaje_tercera_pasada_union");
		String OF = PluginServices.getText(this, "De");
//		String FIRST_PASS_MESSAGE = "Computing intersections: ";
//		String SECOND_PASS_MESSAGE = "Computing A-B difference:";
//		String THIRD_PASS_MESSAGE = "Computing B-A difference:";
		
		String currentMessage = "";
		private boolean finished = false;

		UnionMonitorableTask() throws DriverIOException, DriverException {
			initialize();
		}
		void initialize() throws DriverIOException, DriverException {
			cancelMonitor = createCancelMonitor();
			currentMessage = "Initializing...";
		}

		private CancellableMonitorable createCancelMonitor()
				throws DriverIOException, DriverException {
			DefaultCancellableMonitorable monitor = new 
							DefaultCancellableMonitorable();
			monitor.setInitialStep(0);
			monitor.setDeterminatedProcess(true);
			int numSteps = 0;
			//num steps = 2 * firstlayer (intersect + diff A-B)
			//+ second layer (diff B-A)
			if (onlyFirstLayerSelection) {
				FBitSet selection = firstLayer.getRecordset().getSelection();
				numSteps += (2 * selection.cardinality());
			} else {
				numSteps += 2 * firstLayer.getSource().getShapeCount();
			}
			if(onlyClipLayerSelection) {
				FBitSet selection = overlayLayer.getRecordset().getSelection();
				numSteps += (2 * selection.cardinality());
			}else{
				numSteps += 2 * overlayLayer.getSource().getShapeCount();
			}
			monitor.setFinalStep(numSteps);
			return monitor;
		}

		public int getInitialStep() {
			return cancelMonitor.getInitialStep();
		}

		public int getFinishStep() {
			return cancelMonitor.getFinalStep();
		}

		public int getCurrentStep() {
			return cancelMonitor.getCurrentStep();
		}

		public String getStatusMessage() {
			return UNION_MESSAGE;
		}

		public String getNote() {
			return currentMessage + " "+
			getCurrentStep() + " "+
			OF + " " + getFinishStep();
		}

		public void cancel() {
			((DefaultCancellableMonitorable) cancelMonitor).setCanceled(true);
			UnionGeoprocess.this.cancel();
		}

		public void run() throws GeoprocessException {

			try {
				schemaManager.createOrAlterSchema(createLayerDefinition());
				writer.preProcess();
				
				Strategy strategy = StrategyManager.getStrategy(firstLayer);
				Strategy strategy2 = StrategyManager.getStrategy(overlayLayer);
				DeferredFeaturePersisterProcessor featureProcessor =
					new DeferredFeaturePersisterProcessor(writer);
				
				IntersectVisitor visitor = new IntersectVisitor(overlayLayer, 
															featureProcessor,
															strategy2, 
															onlyClipLayerSelection);
				//FIXME Meter las selecciones
				currentMessage = FIRST_PASS_NOTE;
				if(onlyFirstLayerSelection){
					FBitSet selection = firstLayer.getRecordset().getSelection();
					strategy.process(visitor, selection, cancelMonitor);
				}else{
					strategy.process(visitor, cancelMonitor);
				}
				
				currentMessage = SECOND_PASS_NOTE;
				DifferenceVisitor visitor2 = new DifferenceVisitor(overlayLayer,
						featureProcessor, strategy2, onlyClipLayerSelection);
				visitor2.setLayerDefinition(resultLayerDefinition);
				if(onlyFirstLayerSelection){
					FBitSet selection = firstLayer.getRecordset().getSelection();
					strategy.process(visitor2, selection, cancelMonitor);
				}else{
					strategy.process(visitor2, cancelMonitor);
				}
				currentMessage = THIRD_PASS_NOTE;
				DifferenceVisitor visitor3 = new DifferenceVisitor(firstLayer,
						featureProcessor, strategy, onlyFirstLayerSelection);
				visitor3.setLayerDefinition(resultLayerDefinition);
				if(onlyClipLayerSelection){
					FBitSet selection = overlayLayer.getRecordset().getSelection();
					strategy2.process(visitor3, selection, cancelMonitor);
				}else{
					strategy2.process(visitor3, cancelMonitor);
				}
				writer.postProcess();
				
			} catch (EditionException e) {
				throw new GeoprocessException("Error al crear el esquema/fichero de la nueva capa");
			} catch (DriverException e) {
				throw new GeoprocessException("Error de driver al calcular el geoproceso interseccion");
			} catch (VisitException e) {
				throw new GeoprocessException("Error al procesar el feature de una capa durante el geoproceso interseccion");
			} catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}finally{
				finished = true;
			}
		}

		public boolean isDefined() {
			return cancelMonitor.isDeterminatedProcess();
		}

		public boolean isCanceled() {
			return cancelMonitor.isCanceled();
		}

		public boolean isFinished() {
			return finished;
		}
	}

}

