/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package com.iver.cit.gvsig.fmap.layers;

import java.awt.geom.Point2D;

import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.gui.View;


/**
 * Este interfaz debe ser implementado por el dialogo de georreferenciacin
 * para que fmap pueda acceder a sus mtodos.
 * 
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public interface IGeoUi {
	/**
	 * Obtiene el punto de la lista que ha sido seleccionado
	 * @return
	 */
	public int getSelectedPoint();
	public int getWidth();
	public int getHeight();
	/**
	 * 
	 * @param vp
	 * @param p
	 * @param initVp
	 * @return
	 */
	public ViewPort getLeftInitViewport(ViewPort vp, Point2D center, ViewPort initVp);
	
	/**
	 * A partir de nuevas coordenadas actualiza la vista, minimagen, capa de puntos el
	 * dialogo y la tabla.
	 *
	 */
	public void updateData(int nPunto, Point2D pixel, Point2D map, View view);
	
	/**
	 * Funcin para salvar los puntos de control sobre el fichero .rmf asociado a la imagen.
	 * @param theView
	 */
	public void savePoints()throws Exception;
	
	/**
	 * Consulta si se desea incluir los errores cuando se exporta a fichero .csv 
	 * @param errorCVS true si se incluyen los errores dentro del fichero y false si no se incluyen
	 */
    public boolean isErrorCSV();
	
	/**
	 * Obtiene los errores en una matriz de Nx3 en la que cada fila es un punto y la
	 * columna 0 es el error en X, la columna 1 es el error en Y y la columna 2 es el error RMS. 
	 * @return matriz de errores
	 */
	public double[][] getErrors();
	
	/**
	 * Cierra el dialogo
	 */
	public void close();
}
