/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.tools.Behavior;

import java.awt.event.MouseEvent;


import com.iver.cit.gvsig.fmap.tools.BehaviorException;
import com.iver.cit.gvsig.fmap.tools.Behavior.Behavior;
import com.iver.cit.gvsig.fmap.tools.Events.PointEvent;
import com.iver.cit.gvsig.fmap.tools.Listeners.DraggedListener;
import com.iver.cit.gvsig.fmap.tools.Listeners.ToolListener;


/**
 * .
 *
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class DraggedBehavior extends Behavior {
	private DraggedListener listener;

	/**
	 * Crea un nuevo DraggedListener.
	 *
	 * @param l listener.
	 */
	public DraggedBehavior(DraggedListener l) {
		listener = l;
	}

	/**
	 * Reimplementacin del mtodo mousePressed de Behavior.
	 *
	 * @param e MouseEvent
	 */
	public void mousePressed(MouseEvent e)throws BehaviorException {
		PointEvent event = new PointEvent(e.getPoint(), e);
		listener.press(event);
	}

	/**
	 * Reimplementacin del mtodo mouseReleased de Behavior.
	 *
	 * @param e MouseEvent
	 *
	 * @throws BehaviorException 
	 */
	public void mouseReleased(MouseEvent e) throws BehaviorException {
		PointEvent event = new PointEvent(e.getPoint(), e);
		listener.release(event);
	}

	/**
	 * Reimplementacin del mtodo mouseDragged de Behavior.
	 *
	 * @param e MouseEvent
	 *
	 * @throws BehaviorException 
	 */
	public void mouseDragged(MouseEvent e) throws BehaviorException {
		PointEvent event = new PointEvent(e.getPoint(), e);
		listener.dragg(event);
	}
	
	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Behavior.Behavior#setListener(org.gvsig.fmap.tools.ToolListener)
	 */
	public void setListener(ToolListener listener) {
		this.listener = (DraggedListener) listener;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Behavior.Behavior#getListener()
	 */
	public ToolListener getListener() {
		return listener;
	}
}
