/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.tools.Behavior;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;

import javax.swing.ImageIcon;
import javax.swing.JOptionPane;

import org.cresques.px.Extent;
import org.gvsig.georeferencing.gui.dialogs.GeoreferencingDialog;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.MapControl;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrGeoRaster;
import com.iver.cit.gvsig.fmap.layers.GeoPoint;
import com.iver.cit.gvsig.fmap.tools.BehaviorException;
import com.iver.cit.gvsig.fmap.tools.Behavior.Behavior;
import com.iver.cit.gvsig.fmap.tools.Listeners.RectangleListener;
import com.iver.cit.gvsig.fmap.tools.Listeners.ToolListener;


/**
 * Behaviour que espera un listener de tipo RedimListener.
 * Nacho Brodin (brodin_ign@gva.es)
 *
 */
public class GeoRedimBehavior extends Behavior {
	//Params
	private int 							WIDTH_BORDER = 25; 
	private int 							LONG_CORNER = 35;
	private Color							rectangleColor = Color.RED;
	
	private RectangleListener listener;
	
	private final Image iconHoriz = new ImageIcon(getClass().getClassLoader().getResource(
		"images/FlechaHorizCursor.gif")).getImage(); 
	private final Image iconVert = new ImageIcon(getClass().getClassLoader().getResource(
		"images/FlechaVertCursor.gif")).getImage();
	private final Image iconInclDer = new ImageIcon(getClass().getClassLoader().getResource(
		"images/FlechaInclDerCursor.gif")).getImage();
	private final Image iconInclIzq = new ImageIcon(getClass().getClassLoader().getResource(
		"images/FlechaInclIzqCursor.gif")).getImage();
	private final Image defaultCursor = new ImageIcon(getClass().getClassLoader().getResource(
		"images/ZoomInCursor.gif")).getImage();
	
	/**
	 * Cada elemento del vector corresponde a un tipo de icono que
	 * estar a true si est activo y a false si no lo est.
	 * horizontal, vertical, inclinado derecha, inclinado izquierda
	 */
	private boolean[] 						iconActive = {false, false, false, false};
	private Point2D 						ul = null;
	private Point2D 						lr = null;
	private Point2D 						tmpUl = null;
	private Point2D 						tmpLr = null;
	
	/**
	 * Variable que si est a true permite que se pinte el marco de la imagen. Se activa al
	 * comenzar el redimensionado y se desactiva al terminar
	 */
	private boolean 						isResizable = false;
	
	/**
	 * Cada elemento corresponde a la acivacin de redimensionado en una
	 * direccin que estar activa a true.
	 * vertical derecha, vertical izquierda, horizontal arriba, horizontal abajo,
	 * inclinado
	 */
	private boolean[] 						redimActive = new boolean[5];
	private Point2D 						pointInit = null;
	private Rectangle2D 					rectInit = null;
	private FLyrGeoRaster 					lyrGeoRaster = null;
	private com.iver.cit.gvsig.gui.View 	theView = null;
	
		
	/**
	 * Crea un nuevo RectangleBehavior.
	 *
	 * @param zili listener.
	 */
	public GeoRedimBehavior(RectangleListener zili) {
		listener = zili;
		for(int i=0; i<redimActive.length;i++)
			redimActive[i] = false;
	}

	/**
	 * Funcin que carga la capa si todava no lo est.
	 */
	private void loadLayer() throws InstantiationException, ClassCastException{
		//Cargamos la capa
		
		theView = (com.iver.cit.gvsig.gui.View) PluginServices.getMDIManager().getActiveView();
		for(int i=0;i<theView.getMapControl().getMapContext().getLayers().getLayersCount();i++){
			FLayer lyr = theView.getMapControl().getMapContext().getLayers().getLayer(i);
			if(	lyr instanceof FLyrGeoRaster && 
				lyr.getName().startsWith("*"))
				this.lyrGeoRaster = (FLyrGeoRaster)lyr;
		}
		if(lyrGeoRaster == null)
			throw new InstantiationException("No se ha podido cargar la capa de georreferenciacin");
	}

	
	/**
	 * Cuando se produce un evento de pintado dibujamos el marco de la imagen para
	 * que el usuario pueda seleccionar y redimensionar.
	 */
	public void paintComponent(Graphics g) {
		BufferedImage img = getMapControl().getImage();
		g.drawImage(img, 0, 0, null);
		g.setColor(rectangleColor);
		ViewPort vp = getMapControl().getMapContext().getViewPort();
		try{
			loadLayer();
			Rectangle r = new Rectangle();
				
			if(tmpLr == null || tmpUl == null){
				ul = vp.fromMapPoint(lyrGeoRaster.getAssignExtent().getMin().getX(), lyrGeoRaster.getAssignExtent().getMin().getY());
				lr = vp.fromMapPoint(lyrGeoRaster.getAssignExtent().getMax().getX(), lyrGeoRaster.getAssignExtent().getMax().getY());
				r.setFrameFromDiagonal(ul, lr);
			}else
				r.setFrameFromDiagonal(tmpUl, tmpLr);
			if(isResizable)			
				g.drawRect(r.x, r.y, r.width, r.height);
		}catch(InstantiationException exc){
			JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
					PluginServices.getText(this, "error_capa_puntos"));
		}catch(ClassCastException exc){
			JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
					PluginServices.getText(this, "error_capa_puntos"));
		}
		
	}

	/**
	 * Reimplementacin del mtodo mousePressed de Behavior. Cuando se pulsa
	 * estando sobre el marco de la imagen activamos la posibilidad de arrastrar
	 * para redimensionar la imagen.
	 *
	 * @param e MouseEvent
	 */
	public void mousePressed(MouseEvent e) {
		if (e.getButton() == MouseEvent.BUTTON1) {
			isResizable = true;	
			try{
				loadLayer();
			}catch(InstantiationException exc){
				JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
						PluginServices.getText(this, "error_capa_puntos"));
				return;
			}catch(ClassCastException exc){
				JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
						PluginServices.getText(this, "error_capa_puntos"));
				return;
			}
			
			tmpUl = ul;
			tmpLr = lr;
			
			ViewPort vp = getMapControl().getMapContext().getViewPort();
			double wcX = vp.toMapPoint(e.getX(), e.getY()).getX();
			double wcY = vp.toMapPoint(e.getX(), e.getY()).getY();
			
			double minX = lyrGeoRaster.getAssignExtent().getMin().getX();
			double minY = lyrGeoRaster.getAssignExtent().getMin().getY();
			double maxX = lyrGeoRaster.getAssignExtent().getMax().getX();
			double maxY = lyrGeoRaster.getAssignExtent().getMax().getY();
			
			
			if(iconActive[0]){//Estirar en horizontal activado
				if(wcX > (maxX - WIDTH_BORDER) && wcX < maxX)
					redimActive[0] = true;
				if(wcX < (minX + WIDTH_BORDER) && wcX > minX)
					redimActive[1] = true;
			}
			if(iconActive[1]){//Estirar en vertical activado
				if(wcY > (maxY - WIDTH_BORDER) && wcY < maxY)
					redimActive[3] = true;
				if(wcY < (minY + WIDTH_BORDER) && wcY > minY)
					redimActive[2] = true;
			}
			if(iconActive[2] || iconActive[3]){ //Estirar en oblicuo activado
				redimActive[4] = true;
				pointInit = e.getPoint();
				rectInit =  new Rectangle2D.Double(tmpUl.getX(), tmpUl.getY(), tmpLr.getX() - tmpUl.getX(), tmpLr.getY() - tmpUl.getY());
			}
			
		}

		if (listener.cancelDrawing()) {
			getMapControl().cancelDrawing();
		}
		
		//getMapControl().repaint();
		//getMapControl().drawMap(true);
	}

	/**
	 * Reimplementacin del mtodo mouseReleased de Behavior. Desactivamos
	 * los flags de redimensionado y a partir de la seleccin del usuario 
	 * creamos un nuevo extent para la imagen. Con este extent creamos una nueva
	 * capa que sustituir a la anterior.
	 *
	 * @param e MouseEvent
	 *
	 * @throws BehaviorException Excepcin lanzada cuando el Behavior.
	 */
	public void mouseReleased(MouseEvent e) throws BehaviorException {
		
		if (e.getButton() == MouseEvent.BUTTON1 && isResizable) {
			//Desactivamos los flags de redimensionado
			for(int i=0;i<redimActive.length;i++)
				redimActive[i] = false;
			//Asignamos el nuevo extent a la imagen
			ViewPort vp = getMapControl().getMapContext().getViewPort();
			Extent ext = new Extent(vp.toMapPoint((int)tmpUl.getX(), (int)tmpUl.getY()).getX(),
									vp.toMapPoint((int)tmpUl.getX(), (int)tmpUl.getY()).getY(),
									vp.toMapPoint((int)tmpLr.getX(), (int)tmpLr.getY()).getX(),
									vp.toMapPoint((int)tmpLr.getX(), (int)tmpLr.getY()).getY());
			
						
			if(this.lyrGeoRaster != null){
				
				((FLyrGeoRaster)getMapControl().getMapContext().getLayers().getLayer(lyrGeoRaster.getName())).setAssignExtent(ext);
				
				//Si la capa tiene puntos de control hacemos un update de estos 
				//a su posicin para que se actualicen los controles
				for(int i=0; i<lyrGeoRaster.getFLyrPoints().getCountPoints(); i++){
							
					Point2D center = lyrGeoRaster.img2World(lyrGeoRaster.getFLyrPoints().getPoint(i).pixelPoint);
					lyrGeoRaster.getFLyrPoints().setMiniExtent(	i,
																center,
																//lyrGeoRaster.getGeoDialog().getZoomControlLeft().getCanvas().initViewPort(vp, center, lyrGeoRaster.getFLyrPoints().getPoint(i).leftViewPort),
																lyrGeoRaster.getGeoDialog().getLeftInitViewport(vp, center, lyrGeoRaster.getFLyrPoints().getPoint(i).leftViewPort),
																false);
				}
				
				//Repintamos la vista y las mini imgenes
				int selection = lyrGeoRaster.getGeoDialog().getSelectedPoint();
				GeoPoint gp = lyrGeoRaster.getFLyrPoints().getPoint(selection);
				Point2D px = null; 
				Point2D map = null;
				if(gp != null){
					px = gp.pixelPoint;
					map = gp.mapPoint;
				}
				
				lyrGeoRaster.updateData(selection + 1, px, map, theView);
				if(px == null && map == null)
					theView.getMapControl().getMapContext().invalidate();
				
				tmpUl = null;
				tmpLr = null;
				
			}
			isResizable = false;
		}
	}
	
	/**
	 * Al arrastrar cuando se ha pulsado sobre el marco de la imagen recalculamos
	 * el marco de la imagen 
	 *
	 * @param e MouseEvent
	 */
	public void mouseDragged(MouseEvent e) {
		double longLadoX = Math.abs(tmpLr.getX() - tmpUl.getX());
		double longLadoY = Math.abs(tmpLr.getY() - tmpUl.getY());
		double coordCentro = 0D;
		
		//Guardamos los puntos iniciales para poder reponerlos si excedemos el tamao mnimo
		Point2D antUl = tmpUl;
		Point2D antLr = tmpLr;
		
		coordCentro = tmpLr.getY() + (Math.abs(tmpLr.getY() - tmpUl.getY()) / 2);		
		if(redimActive[0]){//vertical derecha
			double newLadoX = (e.getX() - tmpUl.getX());
			double newLadoY = (newLadoX * longLadoY) / longLadoX;
			tmpUl = new Point2D.Double(tmpUl.getX(), coordCentro + (newLadoY/2));
			tmpLr = new Point2D.Double(e.getX(), coordCentro - (newLadoY/2));
		}
		if(redimActive[1]){//vertical izquierda
			double newLadoX = (e.getX() - tmpLr.getX());
			double newLadoY = (newLadoX * longLadoY) / longLadoX;
			tmpLr = new Point2D.Double(tmpLr.getX(), coordCentro + (newLadoY/2));
			tmpUl = new Point2D.Double(e.getX(), coordCentro - (newLadoY/2));
		}
		
		coordCentro = tmpLr.getX() - (Math.abs(tmpLr.getX() - tmpUl.getX()) / 2);
		if(redimActive[2]){//horizontal abajo
			double newLadoY = (e.getY() - tmpLr.getY());
			double newLadoX = (newLadoY * longLadoX) / longLadoY;
			tmpLr = new Point2D.Double(coordCentro + (newLadoX/2), tmpLr.getY());
			tmpUl = new Point2D.Double(coordCentro - (newLadoX/2), e.getY());
		}
		if(redimActive[3]){//horizontal arriba
			double newLadoY = (e.getY() - tmpUl.getY());
			double newLadoX = (newLadoY * longLadoX) / longLadoY;
			tmpUl = new Point2D.Double(coordCentro + (newLadoX/2), tmpUl.getY());
			tmpLr = new Point2D.Double(coordCentro - (newLadoX/2), e.getY());
		}
		if(redimActive[4]){//inclinado
			double distX = (e.getX() - pointInit.getX());
			double distY = (e.getY() - pointInit.getY());
			double incrLadoX = 0D, incrLadoY = 0D;
			
			if(longLadoY <= longLadoX){
				incrLadoX = distX;
				incrLadoY = (incrLadoX * longLadoY) / longLadoX;
			}else{
				incrLadoY = distY;
				incrLadoX = (incrLadoY * longLadoX) / longLadoY;
			}
			if(distX > 0 && e.getX() > lr.getX()){
				tmpUl = new Point2D.Double(rectInit.getMinX() - Math.abs(incrLadoX), rectInit.getMinY() + Math.abs(incrLadoY));
				tmpLr = new Point2D.Double(rectInit.getMaxX() + Math.abs(incrLadoX), rectInit.getMaxY() - Math.abs(incrLadoY));
			}else if(distX < 0 && e.getX() < ul.getX()){
				tmpUl = new Point2D.Double(rectInit.getMinX() - Math.abs(incrLadoX), rectInit.getMinY() + Math.abs(incrLadoY));
				tmpLr = new Point2D.Double(rectInit.getMaxX() + Math.abs(incrLadoX), rectInit.getMaxY() - Math.abs(incrLadoY));
			}else if(distX > 0 && e.getX() < lr.getX() && e.getX() > ul.getX()){
				tmpUl = new Point2D.Double(rectInit.getMinX() + Math.abs(incrLadoX), rectInit.getMinY() - Math.abs(incrLadoY));
				tmpLr = new Point2D.Double(rectInit.getMaxX() - Math.abs(incrLadoX), rectInit.getMaxY() + Math.abs(incrLadoY));
			}else if(distX < 0 && e.getX() < lr.getX() && e.getX() > ul.getX()){
				tmpUl = new Point2D.Double(rectInit.getMinX() + Math.abs(incrLadoX), rectInit.getMinY() - Math.abs(incrLadoY));
				tmpLr = new Point2D.Double(rectInit.getMaxX() - Math.abs(incrLadoX), rectInit.getMaxY() + Math.abs(incrLadoY));
			}
		}
		//A partir de un tamao mnimo no reducimos ms
		if((tmpLr.getX() - tmpUl.getX()) < 25 || (tmpUl.getY() - tmpLr.getY()) < 25){
			tmpUl = antUl;
			tmpLr = antLr;
		}
		
		getMapControl().repaint();
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Behavior.Behavior#setListener(org.gvsig.georeferencing.fmap.tools.ToolListener)
	 */
	public void setListener(ToolListener listener) {
		this.listener = (RectangleListener) listener;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Behavior.Behavior#getListener()
	 */
	public ToolListener getListener() {
		return listener;
	}
	
	/**
	 * Cuando movemos el ratn detecta si estamos en el marco de la 
	 * imagen y pone el icono del cursor del ratn adecuado.
	 */
	public void mouseMoved(MouseEvent e) throws BehaviorException {
		ViewPort vp = getMapControl().getMapContext().getViewPort();
		
		try{
			loadLayer();
		}catch(InstantiationException exc){
			JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
					PluginServices.getText(this, "error_capa_puntos"));
			return;
		}catch(ClassCastException exc){
			JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
					PluginServices.getText(this, "error_capa_puntos"));
			return;
		}
		
		double wcX = vp.toMapPoint(e.getX(), e.getY()).getX();
		double wcY = vp.toMapPoint(e.getX(), e.getY()).getY();
		
		double minX = lyrGeoRaster.getAssignExtent().getMin().getX();
		double minY = lyrGeoRaster.getAssignExtent().getMin().getY();
		double maxX = lyrGeoRaster.getAssignExtent().getMax().getX();
		double maxY = lyrGeoRaster.getAssignExtent().getMax().getY();
		
		//Calculamos el borde sobre el cual cambiar el puntero del ratn y la longitud de la esquina
		
		WIDTH_BORDER = (int)Math.round((maxX - minX) * 0.02);
		LONG_CORNER = (int)Math.round((maxX - minX) * 0.03);
		if(WIDTH_BORDER < 15)
			WIDTH_BORDER = 15;
		if(LONG_CORNER < 20)
			LONG_CORNER = 20;
				
		int iconPosActive = -1;
		
		//Flecha horizontal
		if(	((wcX > minX && wcX < minX + WIDTH_BORDER) ||
			 (wcX < maxX && wcX > maxX - WIDTH_BORDER )) &&
			(wcY < (maxY - LONG_CORNER)) &&
			(wcY > (minY + LONG_CORNER))){	
			getMapControl().setCursor(Toolkit.getDefaultToolkit().createCustomCursor(iconHoriz, new Point(16, 16), ""));
			iconPosActive = 0;
						
		}else{
			//Flecha Vertical
			if(	((wcY > minY && wcY < minY + WIDTH_BORDER) ||
				 (wcY < maxY && wcY > maxY - WIDTH_BORDER )) &&
				(wcX < (maxX - LONG_CORNER)) &&
				(wcX > (minX + LONG_CORNER))){	
					getMapControl().setCursor(Toolkit.getDefaultToolkit().createCustomCursor(iconVert, new Point(16, 16), ""));
					iconPosActive = 1;
			}else{
				//Flecha oblicua derecha
				if((wcX > (maxX - LONG_CORNER) && wcX < maxX &&
					wcY < (minY + LONG_CORNER) && wcY > minY) ||
				   (wcX > minX && wcX < (minX + LONG_CORNER) &&
					wcY < maxY && wcY > (maxY - LONG_CORNER))){
					getMapControl().setCursor(Toolkit.getDefaultToolkit().createCustomCursor(iconInclIzq, new Point(16, 16), ""));
					iconPosActive = 2;
				}else{
					//Flecha oblicua izquierda
					if((wcX > minX && wcX < (minX + LONG_CORNER) &&
						wcY < (minY + LONG_CORNER) && wcY > minY) ||
					   (wcX > (maxX - LONG_CORNER) && wcX < maxX &&
						wcY > (maxY - LONG_CORNER) && wcY < maxY)){
						getMapControl().setCursor(Toolkit.getDefaultToolkit().createCustomCursor(iconInclDer, new Point(16, 16), ""));
						iconPosActive = 3;
					}else{
						getMapControl().setCursor(Toolkit.getDefaultToolkit().createCustomCursor(defaultCursor, new Point(16, 16), ""));
						iconPosActive = -1;
					}
				}
			}
		}
		
		//Ponemos a true el incono activado y a false el resto
		for(int i=0;i<iconActive.length;i++){
			if(i==iconPosActive)
				iconActive[i] = true;
			else
				iconActive[i] = false;
		}			
		getMapControl().repaint();	
	}

}
