package org.gvsig.georeferencing.gui.panels;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.JPanel;

import org.gvsig.georeferencing.gui.dialogs.GeoreferencingDialog;


/**
 * Panel que contiene opciones de georreferenciacin a elegir por
 * el usuario. Esta clase no maneja eventos.
 * 
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */
public class ConectorPanel extends JPanel{

	private AdjustGeorefPanel adjustGeorefPanel = null; 
	private DataPointsTabPanel dataPointsTabPanel = null;
	private GeoreferencingDialog grd = null;
	private JPanel pNorth = null;
	private JPanel pSouth = null;
	private int heightConectorPanel = 385;
	private int heightSouthPanel = 175;
	private int heightNorthPanel = 210;
	
	/**
	 * This is the default constructor
	 */
	public ConectorPanel(GeoreferencingDialog grd) {
		super();
		this.grd = grd;
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {        
        GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.NONE;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.CENTER;
        gridBagConstraints1.gridy = 1;
        GridBagConstraints gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        this.setLayout(new GridBagLayout());
        this.setPreferredSize(new java.awt.Dimension(395,385));
        this.setSize(new java.awt.Dimension(395,385));
        this.setLocation(new java.awt.Point(0,0));
        this.add(getPNorth(), gridBagConstraints);
        this.add(getPSouth(), gridBagConstraints1);
	}

	
	public AdjustGeorefPanel getAdjustGeorefPanel() {
		if (adjustGeorefPanel == null) {
			adjustGeorefPanel = new AdjustGeorefPanel(grd);
			adjustGeorefPanel.setSize(400, 275);
			adjustGeorefPanel.setPreferredSize(new Dimension(400, 275));
			adjustGeorefPanel.setVisible(true);
		}
		return adjustGeorefPanel;
	}
	
	public DataPointsTabPanel getDataPointsTabPanel() {
		if (dataPointsTabPanel == null) {
			dataPointsTabPanel = new DataPointsTabPanel(grd);
			dataPointsTabPanel.setVisible(true);
		}
		return dataPointsTabPanel;
	}

	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPNorth() {
		if (pNorth == null) {
			BorderLayout borderLayout = new BorderLayout();
			pNorth = new JPanel();
			pNorth.setLayout(borderLayout);
			pNorth.setPreferredSize(new java.awt.Dimension(395,210));
			pNorth.add(getDataPointsTabPanel(), BorderLayout.CENTER);
		}
		return pNorth;
	}

	/**
	 * This method initializes jPanel1	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPSouth() {
		if (pSouth == null) {
			BorderLayout borderLayout = new BorderLayout();
			pSouth = new JPanel();
			pSouth.setLayout(borderLayout);
			pSouth.setPreferredSize(new java.awt.Dimension(395,175));
			pSouth.add(getAdjustGeorefPanel(), BorderLayout.CENTER);
		}
		return pSouth;
	}
	
	/**
	 * Asigna al canvas de los controles de zoom la propiedad de visibilidad a 
	 * verdadero o falso.
	 * @param visible true para mostrar los canvas y false para ocultarlos
	 */
	public void setCanvasVisible(boolean visible){
		this.getAdjustGeorefPanel().getZoomLeft().setVisible(visible);
		this.getAdjustGeorefPanel().getZoomRight().setVisible(visible);
	}
	
	/**
	 * Calculo del nuevo tamao a partir de un frame redimensionado
	 * @param w Ancho del frame
	 * @param h Alto del frame
	 */
	public void newFrameSize(int w, int h){	
		int newWidth = (int)(w * 0.98);
		this.setSize(new java.awt.Dimension(newWidth, heightConectorPanel));
		this.setPreferredSize(new java.awt.Dimension(newWidth, heightConectorPanel));
		
        this.getPNorth().setSize(new java.awt.Dimension(newWidth, heightNorthPanel));
        this.getPNorth().setPreferredSize(new java.awt.Dimension(newWidth, heightNorthPanel));
        this.getPSouth().setSize(new java.awt.Dimension(newWidth, heightSouthPanel));
        this.getPSouth().setPreferredSize(new java.awt.Dimension(newWidth, heightSouthPanel));
        
        this.getDataPointsTabPanel().newFrameSize(newWidth, h);
	}
}
