package org.gvsig.georeferencing.gui.panels;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.gvsig.georeferencing.gui.dialogs.GeoreferencingDialog;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLayers;
import com.iver.cit.gvsig.fmap.layers.FLyrGeoRaster;
import com.iver.cit.gvsig.fmap.layers.FLyrPoints;
import com.iver.cit.gvsig.gui.View;
import com.iver.utiles.swing.threads.Cancellable;

/**
 * Panel que contiene los controles de zoom de la vista para 
 * afinar en la georreferenciacin.
 * 
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */
public class ZoomControlPanel extends JPanel implements ActionListener{
	
	//**********************Vars**********************************
	private JPanel 					pImage = null;
	private JPanel 					pControls = null;
	private JButton 				bZoomMas = null;
	private JButton 				bZoomMenos = null;
	private int 					sX = 140, sY = 140; 
	private CanvasZone 				canvas = null;
	private ViewPort 				viewPort = null;
    public FLyrPoints 				lyrPoints = null;
    public FLyrGeoRaster 			lyrGeoRaster = null;
    public GeoreferencingDialog 	dialog = null;
    private boolean 				rightControl = false;
    /**
	 * Estado de los botones cuando se ejecuta disableAllControls
	 */
	private boolean[] 					buttonsState = new boolean[2];
	/**
	 * DisableAllControls ha sido ejecutada si est a true esta variabled
	 */
	private boolean 					disableAllControls = false;
	//**********************End Vars******************************
    
	//**********************Methods*******************************
	/**
	 * This is the default constructor
	 */
	public ZoomControlPanel(boolean rightControl, GeoreferencingDialog dialog) {
		super();
		this.rightControl = rightControl;
		this.dialog = dialog;
		initialize(rightControl);
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize(boolean rightControl) {
	        
        GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
        if(rightControl)
        	gridBagConstraints1.gridx = 1;
        else
        	gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        GridBagConstraints gridBagConstraints = new GridBagConstraints();
        if(rightControl)
        	gridBagConstraints.gridx = 0;
        else
        	gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        this.setLayout(new GridBagLayout());
        this.setPreferredSize(new java.awt.Dimension(175,142));
        this.setSize(new java.awt.Dimension(175,142));
        this.setLocation(new java.awt.Point(0,0));
        this.add(getPImage(), gridBagConstraints);
        this.add(getPControls(), gridBagConstraints1);
        //Desactivamos el control en la inicializacin
        this.setEnabled(false);
	}

	/**
	 * Carga las capas de puntos y de raster georreferenciado
	 *
	 */
	public void loadLayer() throws InstantiationException{
		//Cargamos las capas
		this.lyrGeoRaster = dialog.getLyrGeoRaster();
		this.lyrPoints = dialog.getLyrPoints();
		
		if(lyrGeoRaster == null || lyrPoints == null)
			throw new InstantiationException("No se ha podido instanciar la capa de puntos o georreferenciada");
		
	}
		
	/**
	 *Llama a la funcin de pintado del canvas con los parmetros que tenga en ese momento
	 */
	public void draw(){
		if(!getDialog().isEnlarge())
			getCanvas().loadBuffer();
		canvas.draw = true;
		getCanvas().repaint();
	}
		
	/**
	 * Manejo de los controles zoom ms y zoom menos
	 */
	public void actionPerformed(ActionEvent e) {
		if(e.getSource() == bZoomMas){
			canvas.calcZoom(0.6);
			canvas.draw = true;
			canvas.repaint();
		}
		
		if(e.getSource() == bZoomMenos){
			canvas.calcZoom(1.8);
	    	canvas.draw = true;
			canvas.repaint();
		}
		
	}
	    
    /**
     *Limpia el canvas
     */
    public void clear(){
    	canvas.setCleanCanvas(true);
    	canvas.repaint();
    }
    
    /**
	 * Esta funcin deshabilita todos los controles y guarda sus valores
	 * de habilitado o deshabilitado para que cuando se ejecute restoreControlsValue
	 * se vuelvan a quedar como estaba
	 */
	public void disableAllControls(){
		if(!disableAllControls){
			disableAllControls = true;
			
			//Salvamos los estados
			buttonsState[0] = getBZoomMas().isEnabled();
			buttonsState[1] = getBZoomMenos().isEnabled();
			
			//Desactivamos controles
			getBZoomMas().setEnabled(false);
			getBZoomMenos().setEnabled(false);
		}
	}
	
	/**
	 * Esta funcin deja los controles como estaban al ejecutar la funcin 
	 * disableAllControls
	 */
	public void restoreControlsValue(){
		if(disableAllControls){
			disableAllControls = false;
			getBZoomMas().setEnabled(buttonsState[0]);
			getBZoomMenos().setEnabled(buttonsState[1]);
		}
	}
	//**********************End Methods***************************
    
    //**********************Setters & Getters*********************
    /**
	 * @param viewPort The viewPort to set.
	 */
	public void setViewPort(ViewPort viewPort) {
		this.viewPort = viewPort;
		getCanvas().setViewPort(this.viewPort);
	}
	
	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPImage() {
		if (pImage == null) {
			pImage = new JPanel();
			BorderLayout borderLayout = new BorderLayout();
			pImage.setLayout(borderLayout);
			pImage.setPreferredSize(new java.awt.Dimension(140,140));
			pImage.setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED));
			pImage.add(getCanvas(), BorderLayout.CENTER);
		}
		return pImage;
	}

	/**
	 * This method initilizes canvas
	 * @return	CanvasZone
	 */
	public CanvasZone getCanvas(){
		if(canvas == null){
			canvas = new CanvasZone(sX, sY, this);
			canvas.setVisible(false);
			canvas.addMouseListener(canvas);
			canvas.addMouseMotionListener(canvas);
		}
		return canvas;
	}
	
	/**
	 * This method initializes jPanel1	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPControls() {
		if (pControls == null) {
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.gridx = 0;
			gridBagConstraints3.insets = new java.awt.Insets(5,0,0,0);
			gridBagConstraints3.gridy = 1;
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.gridx = 0;
			gridBagConstraints2.gridy = 0;
			pControls = new JPanel();
			pControls.setLayout(new GridBagLayout());
			pControls.setPreferredSize(new java.awt.Dimension(33,140));
			pControls.add(getBZoomMas(), gridBagConstraints2);
			pControls.add(getBZoomMenos(), gridBagConstraints3);
		}
		return pControls;
	}

	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public JButton getBZoomMas() {
		if (bZoomMas == null) {
			bZoomMas = new JButton();
			bZoomMas.setPreferredSize(new java.awt.Dimension(25,25));
			bZoomMas.setIcon(new ImageIcon(getClass().getClassLoader().getResource("images/aumentar.png")));
			bZoomMas.addActionListener(this);
		}
		return bZoomMas;
	}

	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public JButton getBZoomMenos() {
		if (bZoomMenos == null) {
			bZoomMenos = new JButton();
			bZoomMenos.setPreferredSize(new java.awt.Dimension(25,25));
			bZoomMenos.setIcon(new ImageIcon(getClass().getClassLoader().getResource("images/disminuir.png")));
			bZoomMenos.addActionListener(this);
		}
		return bZoomMenos;
	}
	
	/**
	 * Obtiene true si se trata de un control con los botones de zoom
	 * a la derecha y false si los tiene a la izquierda.
	 * @return Returns the rightControl.
	 */
	public boolean isRightControl() {
		return rightControl;
	}
	
	/**
	 * Obtiene la capa de imagen georreferenciada.
	 * @return Returns the lyrGeoRaster.
	 */
	public FLyrGeoRaster getLyrGeoRaster() {
		return lyrGeoRaster;
	}

	/**
	 * Obtiene la capa de puntos.
	 * @return Returns the lyrPoints.
	 */
	public FLyrPoints getLyrPoints() {
		try{
		if(lyrPoints == null)
			this.loadLayer();
		}catch(InstantiationException e){
			JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
					PluginServices.getText(this, "error_capa_puntos"));
		}
		return lyrPoints;
	}
	
	/**
	 * @return Returns the dialog.
	 */
	public GeoreferencingDialog getDialog() {
		try{
			if(dialog == null)
				this.loadLayer();
		}catch(InstantiationException e){
			JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
					PluginServices.getText(this, "error_capa_puntos"));
		}
		return dialog;
	}	
	
    /**
     * Asigna el viewPort recalculado
     * @param vp ViewPort
     */
    public void setNewViewPort(ViewPort vp){
    	canvas.setNewViewPort(vp);
    }
    
    /**
     * Obtiene el viewPort recalculado
     * @return vp ViewPort
     */
    public ViewPort getNewViewPort(){
    	return canvas.getNewViewPort();
    }
    
	/**
	 * Asigna al canvas la propiedad de visibilidad a verdadero o falso.
	 * @param visible true para mostrar el canvas y false para ocultarlo
	 */
	public void setVisible(boolean visible){
		canvas.setVisible(visible);
		canvas.draw = true;
		canvas.repaint();
	}
	
	/**
	 * Asigna el punto sobre el que se centrar el zoom de la minimagen
	 * @param pto
	 */
	public void setCenterPoint(Point2D pto){
		getCanvas().setCenterPoint(pto);
	}
	
	/**
	 * Recupera el nmero de punto que se est tratando
	 * @param n
	 */
	public int getNumberPoint(){
		try{
			if(getDialog().getConectorPanel().getDataPointsTabPanel().getTbPoints().getSelectedIndex() == 0)
				return Integer.valueOf(getDialog().getConectorPanel().getDataPointsTabPanel().getSelectPointsPanel().getTableControlerPanel().getCPoint().getSelectedItem().toString()).intValue() - 1;
			else
				return Integer.valueOf(getDialog().getConectorPanel().getDataPointsTabPanel().getTablePointsPanel().getTableControlerPanel().getCPoint().getSelectedItem().toString()).intValue() - 1;
		}catch(NullPointerException exc){
			return -1;
		}
	}
	
	/**
     * Activa o desactiva este panel y todos los que lo componen
     * @param enabled variable booleana para la activacin y/o desactivacin
     */
    public void setEnabled(boolean enabled){
    	this.getBZoomMas().setEnabled(enabled);
    	this.getBZoomMenos().setEnabled(enabled);
    	this.getCanvas().setEnableEvent(enabled);
    }
	
	//**********************End Setters & Getters*****************
	/**
	 * Control que representa la zona de dibujado de la ventana. Este se encarga de
	 * repintar el contenido sobre el Graphics2D a partir del viewPort que se le pase
	 * @author Nacho Brodin (brodin_ign@gva.es)
	 */
	public class CanvasZone extends JPanel implements MouseListener, MouseMotionListener{
		
		//**********************Params********************************
		/**
		 * Zoom inicial de la miniimagen
		 */
		private int 				initZoom = 20;
		//**********************End Params****************************
		
		//**********************Vars**********************************
		private ZoomControlPanel 	zoomControl = null;
	    private int 				ancho;
	    private int 				alto;
	    private FLayers 			flayers = null;
	    private ViewPort 			newViewPort = null;
	    private Point2D 			centerPoint = null;
	    private Image 				dobleBuffer = null;
	    private BufferedImage 		imgTmp = null;
	    private Graphics 			dbGraphics = null;
	    private boolean 			cleanCanvas = false;
	    /**
	     * Cuando el control est desactivado necesitamos controlar con esta variable que
	     * aunque se pinche sobre el canvas no se capture el evento 
	     */	    
	    private boolean				enableEvent = true;
	    /**
	     * Puntos inicial y final para el desplazamiento del punto de control
	     */
	    private Point2D 			ptoIni = null, ptoFin = null;
	    private Point2D 			movDragged = null, initCrux = null;
	    private boolean 			dragged = false;
	    private final Image 		redCursor = new ImageIcon(getClass().getClassLoader().getResource(
												"images/icoRed.png")).getImage();
	    private final Image 		blueCursor = new ImageIcon(getClass().getClassLoader().getResource(
												"images/icoBlue.png")).getImage();
	    /**
	     * Esta variable se usa para informar cuando debe recargarse el buffer de las miniimagenes. Si siempre
	     * est a true no para de llamar al draw de FLyers para que pinte sobre el minicanvas.
	     */
	    public	boolean				draw = true;
		//**********************End Vars******************************
	    
	    //**********************Classes*******************************
	    class CancellableClass implements Cancellable{
	    	private boolean cancel = false;
       		public void setCanceled(boolean canceled) {
       			this.cancel = canceled;
       		}
       		
			public boolean isCanceled() {
				return this.cancel;
			}
       	}
	    //**********************End Classes***************************
	    
	    //**********************Methods*******************************
	    /**
	     * 
	     */
	    public CanvasZone( int anc,int alt, ZoomControlPanel zc) {
	    	zoomControl = zc;
	        ancho = anc;
	        alto = alt;
	      
	        this.setSize(ancho, alto);	    
	        this.setBackground(Color.WHITE);
	        
	        imgTmp = new BufferedImage(this.ancho, this.alto, BufferedImage.TYPE_INT_ARGB);
	        dobleBuffer = new BufferedImage(this.ancho, this.alto, BufferedImage.TYPE_INT_ARGB);
	        dbGraphics = dobleBuffer.getGraphics();
	    }
	    	    
	    /**
	     * Calcula el zoom dependiendo del factor de escala pasado por
	     * parmetro.
	     */
	    public void calcZoom(double factor){
	    	Rectangle2D.Double r = new Rectangle2D.Double();
			double nuevoX = centerPoint.getX() -
				((newViewPort.getExtent().getWidth() * factor) / 2.0);
			double nuevoY = centerPoint.getY() -
				((newViewPort.getExtent().getHeight() * factor) / 2.0);
			r.x = nuevoX;
			r.y = nuevoY;
			r.width = newViewPort.getExtent().getWidth() * factor;
			r.height = newViewPort.getExtent().getHeight() * factor;
			newViewPort.setExtent(r);
			newViewPort.setScale();
		}
	
	    
	    /**
	     *Calculamos el viewPort para el zoom de la minimagen
	     */
	    public ViewPort initViewPort(ViewPort vp, Point2D center, ViewPort vpDest){
	    	if(center != null){
	    			    		
	        	//Hallamos la relacin entre el pixel y las WC a partir de la imagen de la capa
				double relacionPixelWcWidth =  ((vp.getExtent().getMaxX() - vp.getExtent().getMinX()) / initZoom) / ancho;
				double relacionPixelWcHeight = ((vp.getExtent().getMaxY() - vp.getExtent().getMinY()) / initZoom) / alto;
				
				double wcOriginX = center.getX() - ((vp.getImageWidth() * relacionPixelWcWidth) / 2);
				double wcOriginY = center.getY() - ((vp.getImageHeight() * relacionPixelWcHeight) / 2);
				
				double wcDstMinX = wcOriginX; 
				double wcDstMinY = wcOriginY; 
				double wcDstMaxX = wcDstMinX + (vp.getImageWidth()*relacionPixelWcWidth);
				double wcDstMaxY = wcDstMinY + (vp.getImageHeight()*relacionPixelWcHeight);
				
				double wcDstWidth = wcDstMaxX - wcDstMinX;
				double wcDstHeight = wcDstMaxY - wcDstMinY;
								
	        	Rectangle2D ext = new Rectangle2D.Double(wcDstMinX, wcDstMinY, wcDstWidth, wcDstHeight);
	        	
	        	if(vpDest == null){
	    			newViewPort = vp.cloneViewPort();
	        		newViewPort.setExtent(ext);
	        		newViewPort.setImageSize(new java.awt.Dimension(ancho, alto));	
	        		newViewPort.setScale();
	        		return newViewPort;
	    		}else{
	    			vpDest.setExtent(ext);
	    			vpDest.setScale();
	    			return vpDest;
	    		}
	    		
	    	}
	    	return null;
	    }
	    
	    /**
	     * Dibuja sobre el graphics de la mini imagen la imagen
	     * definida en el newViewPort.
	     */
	    public void loadBuffer(){		    
	    	Graphics2D gTmp = (Graphics2D)imgTmp.getGraphics();
	    	
		    dbGraphics.setColor( Color.WHITE );
		    dbGraphics.fillRect( 0,0,ancho,alto );
		    gTmp.setColor( Color.WHITE );
			gTmp.fillRect( 0,0,ancho,alto );
		        
		    try{     		       
			     if(newViewPort != null){
			       	if(zoomControl.getNumberPoint() != -1){//Salvamos el miniextent si hay puntos en la lista
			        	if(!zoomControl.isRightControl())
			        		zoomControl.getLyrPoints().setMiniExtent(zoomControl.getNumberPoint(), 
			        												centerPoint,
																	newViewPort, 
																	true);
			        	else
			        		zoomControl.getLyrPoints().setMiniExtent(zoomControl.getNumberPoint(), 
																	centerPoint,
																	newViewPort, 
																	false);
			        }				        	
			       	//Si no tenemos las capas las cargamos
			       	if(flayers == null){
			       		View  theView = null;
	            		try{
	            			theView = (View)PluginServices.getMDIManager().getActiveView();
	            		}catch(ClassCastException exc){
	            			return;
	            		}
			         	flayers = theView.getMapControl().getMapContext().getLayers();
			        }
			        	
			        /*Dibujamos sobre las miniimagenes
			       	Forzamos un draw de las capas para dibujar la vista sobre el control de las miniimagenes.
			       	Como la capa grfica est en FMap esta no aparece en el dibujado de FLayers por los que 
			       	forzaremos el pintado tambin de la capa de puntos sobre nuestro control*/
			       			       	
			       	Cancellable cancellable = new CancellableClass();
			        if(zoomControl.getDialog().isEnlarge()){
			        	if(zoomControl.getNumberPoint() != -1){
					       	draw(null, (Graphics2D) dbGraphics, newViewPort,cancellable ,flayers.getFMap().getScaleView());
					       	if(zoomControl.isRightControl())
					       		zoomControl.getLyrPoints().drawPixelCrux((Graphics2D) dbGraphics, newViewPort, cancellable);
					       	else
					       		zoomControl.getLyrPoints().drawMapCrux((Graphics2D) dbGraphics, newViewPort, cancellable);
			        	}
			        }
			        	
			        draw(null, gTmp, newViewPort, cancellable, flayers.getFMap().getScaleView());
			    	if(zoomControl.isRightControl())
			    		zoomControl.getLyrPoints().drawPixelCrux(gTmp, newViewPort, cancellable);
			    	else
			       		zoomControl.getLyrPoints().drawMapCrux(gTmp, newViewPort, cancellable);
			       	
			       
			       }
		     }catch (DriverException e) {
		       	 e.printStackTrace();
		     }
	    }
	    
	    /**
	     * Esta funcin es una copia de la de Flyers modificada para para dibujar sobre las miniimagenes. En
	     * la miniimagen con los controles a la derecha solo dibujar el raster a georreferenciar. En la miniimagen
	     * con los controles a la izquierda dibujar la cartografa de base.
	     * @param image 
	     * @param g
	     * @param viewPort
	     * @param cancel
	     * @param scale
	     * @throws DriverException
	     */
	    public void draw(BufferedImage image, Graphics2D g, ViewPort viewPort,
	    		Cancellable cancel,double scale) throws DriverException {	            
	    	for(int i=0;i<flayers.getLayersCount();i++){
	    		if (cancel.isCanceled())
	    			break; 
	    		FLayer lyr = flayers.getLayer(i);
	    		
		    		try{
		    			if(zoomControl.isRightControl()){
		    				if(lyr instanceof FLyrGeoRaster)
		    					lyr.draw(image, g, viewPort, cancel,scale);
		    			}else{
		    				if (lyr.isVisible()){
		    					if(!(lyr instanceof FLyrGeoRaster))
		    						lyr.draw(image, g, viewPort, cancel,scale);
		    				}
		    			}
		    			
		    		} catch (DriverException e){
		    		}
	    	}
	    }
	    
	    /**
	     * 
	     */
	    public void update(Graphics g){
	    	draw = true;
	    	this.paint(g);
	    }
	    
	    /**
	     * Dibuja sobre el Graphics2D pasado el viewPort solicitado a
	     * la lista de capas.
	     * @param g
	     */
	    public void paint( Graphics g ) {
	    	if(!cleanCanvas){
	    		if(draw){
	    			loadBuffer();
	    			draw = false;
	    		}
	    		g.drawImage(dobleBuffer, 0, 0, null);
	    		AdjustGeorefPanel p = zoomControl.getDialog().getConectorPanel().getAdjustGeorefPanel();
	    		try{
	    			if(!this.zoomControl.isRightControl())
	    				p.setLZoomLeft("1:"+((int)newViewPort.getScale()));
	    			else
	    				p.setLZoomRight("1:"+((int)newViewPort.getScale()));
	    		}catch(NullPointerException exc){
	    			//No ponemos la escala
	    		}
	    	}else{
	    		dbGraphics.setColor( Color.WHITE );
			    dbGraphics.fillRect( 0,0,ancho,alto );
	    		g.setColor( Color.WHITE );
	    		g.fillRect( 0,0,ancho,alto );
	    		cleanCanvas = false;
	    	}
	    }
	    	    
	    /**
	     * Desplaza el extent al nuevo centro 
	     * @param pto
	     */
	    public void shiftExtent(Point2D pto){
	    	this.setCenterPoint(pto);
	    	this.calcZoom(1);
	    }
	    
		/* (non-Javadoc)
		 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
		 */
		public void mouseClicked(MouseEvent ev) {
			
		}
		
		/**
		 * Cuando el ratn entra dentro del control fijamos el valor de las coordenadas
		 * que tiene el punto seleccionado. De esta forma cuando salga del control podr
		 * volver a restaurarlo en caso de que no haya sido cambiado.
		 */
		public void mouseEntered(MouseEvent arg0) {
			if(isEnableEvent()){
				if(!zoomControl.rightControl)
					zoomControl.getDialog().getPointManager().setCurrentWC();
				else
					zoomControl.getDialog().getPointManager().setCurrentPixelCoord();
			}
		}
		
		/**
		 * Al sacar el ratn de la zona de dibujado de la miniimagen restauramos los valores
		 * de los JTextField. 
		 * 
		 * Se actualizaran los correspondientes a las coordenadas del mundo real 
		 * en el caso de que el control tenga botones a la izquierda.
		 * 
		 * Se actualizaran los correspondientes a las coordenadas en pixels cuando se trata del control con
		 * botones a la derecha.
		 */
		public void mouseExited(MouseEvent arg0) {
			if(isEnableEvent()){
				if(!zoomControl.rightControl)
					zoomControl.getDialog().getPointManager().replaceWC();
				else
					zoomControl.getDialog().getPointManager().replacePixelCoord();
			}
		}
		
		/**
		 * Controla la actualizacin del valor de un punto en los JTextDialog. Cuando se mueve sobre
		 * el panel con controles a la izquierda actualizamos las coordenadas del mapa. Cuando se mueve sobre
		 * el panel con controles a la derecha actualizamos las coordenadas de la imagen.  
		 */
		public void mouseMoved(MouseEvent arg0) {
			if(	isEnableEvent() && 
				zoomControl.getLyrPoints() != null &&
				zoomControl.getLyrPoints().getCountPoints() > 0 &&
				this.newViewPort != null &&
				this.newViewPort.getExtent() != null){
				
				double newWCPointX = this.newViewPort.getExtent().getMinX() + ((arg0.getX() * this.newViewPort.getExtent().getWidth()) / this.getWidth());
				int ptoSelectY = (int)(this.getHeight() - arg0.getY());
				double newWCPointY = this.newViewPort.getExtent().getMinY() + ((ptoSelectY * this.newViewPort.getExtent().getHeight()) / this.getHeight());
				if(!zoomControl.rightControl){
					zoomControl.getDialog().getPointManager().setTempWC(newWCPointX, newWCPointY);
				}else{
					Point2D pixelImg = zoomControl.getLyrGeoRaster().world2Img(newWCPointX, newWCPointY);
					if(pixelImg != null)
						zoomControl.getDialog().getPointManager().setTempPixelCoord(pixelImg.getX(), pixelImg.getY());
					else 
						//Si pixelImg es null es que nos hemos salido de la imagen por lo que reponemos los valores originales hasta que vuelva a entrar.
						zoomControl.getDialog().getPointManager().replacePixelCoord();	
					
				}
			}
		}
		
		/**
		 * Al pulsar el ratn sobre la minimagen permitimos que se pueda desplazar la cruz
		 *
		 */
		public void mousePressed(MouseEvent ev) {
			int cte = 10;
			if( (ev.getX() > ((getWidth() >> 1) - cte)) && (ev.getX() < ((getWidth() >> 1) + cte)) &&
				(ev.getY() > ((getHeight() >> 1) - cte)) && (ev.getY() < ((getHeight() >> 1) + cte))){
				this.dragged = true;
				this.movDragged = ptoIni = ev.getPoint();
				try{
					if(zoomControl.isRightControl())
						setCursor(Toolkit.getDefaultToolkit().createCustomCursor(redCursor, new Point(16, 16), ""));
					else
						setCursor(Toolkit.getDefaultToolkit().createCustomCursor(blueCursor, new Point(16, 16), ""));
					
					zoomControl.loadLayer();
					initCrux = new Point2D.Double();
					initCrux.setLocation(this.getWidth() / 2, this.getHeight() / 2);
			    	draw = true;
					repaint();
				}catch(InstantiationException e){
					JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
							PluginServices.getText(this, "error_capa_puntos"));
				}
			}
		}

		/* (non-Javadoc)
		 * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
		 */
		public void mouseReleased(MouseEvent ev) {
			if(!dragged)
				return;
			dragged = false;
			View  theView = null;
			try{
				theView = (View) PluginServices.getMDIManager().getActiveView();
			}catch(ClassCastException exc){
				return;
			}
			ViewPort viewPort = theView.getMapControl().getMapContext().getViewPort();
			setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
			
			ptoFin = ev.getPoint();

			Point2D point = newViewPort.toMapPoint(ev.getX(), ev.getY());			
			int nPoint = Integer.valueOf(zoomControl.getDialog().getSelectPointsPanel().getTableControlerPanel().getCPoint().getSelectedItem().toString()).intValue();

			FLyrPoints lp = zoomControl.getLyrPoints();//(FLyrPoints)lyr;
        			 
        	if(zoomControl.isRightControl()){ //Control de actualizacin de pixel de imagen
        		Point2D pixelImg = zoomControl.getLyrGeoRaster().world2Img(point);
        		//Point2D pixelImg = zoomControl.getLyrGeoRaster().world2Img(newWCPointX, newWCPointY);
        		
	        	if(pixelImg == null){
	        		JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
	     							PluginServices.getText(this, "fuera_de_extent"));
	     			return;
	     		}
	        	zoomControl.getDialog().getPointManager().updateData(	nPoint,
	        	 														pixelImg,
	        	 														null, 
	        	 														zoomControl.getDialog(),
	        	 														null);

	        	//Actualizamos la otra miniimagen
	        	shiftExtent(zoomControl.getLyrGeoRaster().img2World(pixelImg));
	        	zoomControl.getDialog().getConectorPanel().getAdjustGeorefPanel().getZoomLeft().draw();
	        	zoomControl.getDialog().getConectorPanel().getAdjustGeorefPanel().getZoomRight().draw();
	        	zoomControl.getDialog().getPointManager().setCurrentPixelCoord();
        	}else{ //Control de actualizacin de coordenadas del mundo
        			Point2D mapPoint = new Point2D.Double();
        			mapPoint.setLocation(point);
        			zoomControl.getDialog().getPointManager().updateData(	nPoint,
																			null, 
																			mapPoint,
																			zoomControl.getDialog(),
																			null);
        			 	 
        			//Actualizamos la otra miniimagen
        			shiftExtent(mapPoint);
        			zoomControl.getDialog().getConectorPanel().getAdjustGeorefPanel().getZoomRight().draw();
        			zoomControl.getDialog().getConectorPanel().getAdjustGeorefPanel().getZoomLeft().draw();
        			zoomControl.getDialog().getPointManager().setCurrentWC();
        	 }
	    	draw = true;
			repaint();
			theView.getMapControl().getMapContext().invalidate();
		}
		
		/* (non-Javadoc)
		 * @see java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent)
		 */
		public void mouseDragged(MouseEvent ev) {
			this.mouseMoved(ev);
		}
	    //**********************End Methods***************************

		//**********************Setters & Getters*********************
		/**
		 * @param cleanCanvas The cleanCanvas to set.
		 */
		public void setCleanCanvas(boolean cleanCanvas) {
			this.cleanCanvas = cleanCanvas;
		}
		
		 /**
	     * Asigna el punto donde se centrar la vista de la minimagen
	     * @param pto Punto central.
	     */
	    public void setCenterPoint(Point2D pto){
	    	this.centerPoint = pto;
	    }
	    
	    /**
	     * Asigna el viewPort
	     * @param vp ViewPort
	     */
	    public void setViewPort(ViewPort vp){
	    	this.initViewPort(vp, this.centerPoint, null);
	    }

	    /**
	     * Asigna el viewPort recalculado
	     * @param vp ViewPort
	     */
	    public void setNewViewPort(ViewPort vp){
	    	this.newViewPort = vp;
	    }
	    
	    /**
	     * Obtiene el viewPort recalculado
	     * @return vp ViewPort
	     */
	    public ViewPort getNewViewPort(){
	    	return this.newViewPort;
	    }
	    
	    /**
	     * Obtiene el valor del desactivador de eventos sobre el canvas.
	     * @return
	     */
	    public boolean isEnableEvent() {
			return enableEvent;
		}

	    /**
	     * Asigna el desactivador de eventos sobre el canvas
	     * @param enableEvent
	     */
		public void setEnableEvent(boolean enableEvent) {
			this.enableEvent = enableEvent;
		}
		//**********************End Setters & Getters*****************
		
	}
 }
