package org.gvsig.georeferencing.gui.pointsTable;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;

import org.gvsig.georeferencing.gui.dialogs.GeoreferencingDialog;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.layers.FLyrPoints;

/**
 * Panel que contiene la tabla de puntos con sus controles. 
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class TablePointsPanel extends JPanel implements ActionListener{
	
	//**********************Vars**********************************
	private GeoreferencingDialog 			parent = null;
	private JPanel 							pTable = null;
	private JPanel 							pPointSelection = null;
	private TableControlerPanel 			tableControlerPanel = null;
	private int 							widthGeneralPanel = 380, heightGeneralPanel = 160;
	private int 							widthSupPanel = 370, heightSupPanel = 125;
	private int 							widthInfPanel = 375, heightInfPanel = 27;
	private int 							widthTable = 375, heightTable = 110;
	private FLyrPoints 						lyrPoints = null;
	private int 							lastIndexSelection = -1;
	private boolean 						disableEvent = false;
	final static String[] 					columnNames = {"N", "X", "Y", 
														"X'", "Y'", "Error X",  
														"Error Y",  "RMS"};
	private PointTable 						table = null;
	private ButtonsExportImportPanel	 	pButtonsExportImportPanel = null;
	/**
	 * Estado de los botones cuando se ejecuta disableAllControls
	 */
	private boolean[] 					buttonsState = new boolean[3];
	/**
	 * DisableAllControls ha sido ejecutada si est a true esta variabled
	 */
	private boolean 					disableAllControls = false;
	//**********************End Vars******************************
	
	//**********************Classes*******************************
	/**
	 * Clase que representa la tabla de puntos
	 * @author Nacho Brodin (brodin_ign@gva.es)
	 */
	public class PointTable extends JPanel {
	   
		private JTable table = null;
		private JScrollPane scrollPane = null;
		
		private TableModelPoint tabModel = null;
		
		public TableModelPoint getTableModel(){
			return tabModel;
		}
		
	    public PointTable() {
	        super(new GridLayout(1,0));
	       
	        tabModel = new TableModelPoint();
	        table = new JTable(tabModel);
	        table.setPreferredScrollableViewportSize(new Dimension(500, 70));

	       
	        table.addMouseListener(new MouseAdapter() {
	           public void mouseClicked(MouseEvent e) {
	                   
	           }
	        });

	        scrollPane = new JScrollPane(table);
	        add(scrollPane);
	    }

	    /**
	     * TableModel del JTable que contiene los puntos con sus errores
	     * @author Nacho Brodin (brodin_ign@gva.es)
	     */
	    public class TableModelPoint extends DefaultTableModel {
	    	 
	        public TableModelPoint(){
	        	super(new Object[0][8], columnNames);
	        }
	        
	        public int getColumnCount() {
	            return columnNames.length;
	        }

	        public String getColumnName(int col) {
	            return columnNames[col];
	        }

	        public Class getColumnClass(int c) {
	           return String.class;
	        }

	        public void addNew() {
	            super.addRow(new Object[] {"", "", "", "", "", "", "", ""});
	        }
	        
	        public void setValueAt(Object value, int row, int col) { 
	        	super.setValueAt(value, row, col);
	        }
	        
	        /**
	         * Devuelve true si la celda es editable. Solo son editables
	         * las celdas de los puntos.
	         */
	        public boolean isCellEditable(int row, int col){
	        	if(col >=1 && col <=4)
	        		return true;
	        	return false;
	        }

	 
	    }
		/**
		 * @return Returns the table.
		 */
		public JTable getTable() {
			return table;
		}
	}
	//**********************End Classes*****************************
	
	//**********************Methods*********************************
	/**
	 * This is the default constructor
	 */
	public TablePointsPanel(GeoreferencingDialog p) {
		super();
		this.parent = p;
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
		private void initialize() {
        GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
        GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
        GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
        this.setLayout(new GridBagLayout());
        this.setPreferredSize(new java.awt.Dimension(widthGeneralPanel, heightGeneralPanel));
        this.setSize(new java.awt.Dimension(widthGeneralPanel, heightGeneralPanel));
        this.setLocation(new java.awt.Point(0,0));
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.insets = new java.awt.Insets(2,0,2,0);
        gridBagConstraints2.gridx = 0;
        gridBagConstraints2.gridy = 1;
        gridBagConstraints2.insets = new java.awt.Insets(5,0,5,0);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.NORTH;
        gridBagConstraints5.gridx = 0;
        gridBagConstraints5.gridy = 1;
        gridBagConstraints5.insets = new java.awt.Insets(1,0,1,0);
        this.add(getPTable(), gridBagConstraints1);
        this.add(getPPointSelection(), gridBagConstraints5);
	}

	/**
	 * Actualiza el nmero de puntos del combo a partir de los contenidos en
	 * la capa de puntos. Esta funcin es usada desde otros paneles que cambien
	 * el punto seleccionado para comunicar a la tabla la nueva seleccin.
	 */
	public void updateComboPoint(int pointSelected){
		try{
			loadLyrPoint();
		}catch(InstantiationException exc){
    		JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
					PluginServices.getText(this, "error_capa_puntos"));
    		return;
    	}
		
		disableEvent = true;
		tableControlerPanel.setNItems(lyrPoints.getCountPoints());
		try{
			tableControlerPanel.getCPoint().setSelectedIndex(pointSelected);
		}catch(IllegalArgumentException ex){
			
		}
		getJTable().getTable().setRowSelectionInterval(pointSelected, pointSelected);
		tableControlerPanel.checkArrows();
		disableEvent = false;
	}
	
	/**
	 * Obtiene una capa de puntos. Si no existe crea una.
	 * @return
	 */
	private void loadLyrPoint() throws InstantiationException{
			
		if(lyrPoints == null){
			if(lyrPoints == null){
				lyrPoints = this.parent.getLyrPoints();
				if(lyrPoints == null)
					throw new InstantiationException();
			}
		}

	}
	
	/**
	 * Comprueba si los valores de un punto en los campos de la tabla estn
	 * vacios y lanza una excepcin si hay alguno sin informacin.
	 * @throws NoSuchFieldException
	 */
	private void checkInfoPointText(int nPunto)throws NoSuchFieldException{
		if(	getJTable().getTable().getValueAt(nPunto, 0).toString().equals("") ||
			getJTable().getTable().getValueAt(nPunto, 1).toString().equals("") ||
			getJTable().getTable().getValueAt(nPunto, 2).toString().equals("") ||
			getJTable().getTable().getValueAt(nPunto, 3).toString().equals("") ||
			getJTable().getTable().getValueAt(nPunto, 4).toString().equals("") ){
				disableEvent = true;
				tableControlerPanel.setSelectedIndex(lastIndexSelection);
				disableEvent = false;
				throw new NoSuchFieldException();
		}
	}
	/**
	 * Cuando pasamos de un punto a otro
	 * <UL>
	 * <LI>Cargamos la capa de puntos si no est cargada todavia</LI>
	 * <LI>Comprobamos que el punto desde el que partimos no tenga campos inicio o fin vacios</LI>
	 * <LI>Seleccionamos en el combo en nuevo punto</LI>
	 * <LI>Cargamos la informacin del nuevo punto en los JTextField, mini imagen y seleccionamos la fila de la tabla</LI>
	 * <LI>Chequeamos la flechas para activar o desactivar las que toquen.</LI>
	 * </UL>
	 * @param changeSelection	Cambia la posicin seleccionada del combo si es true
	 * @param pos	Posicin a la que cambia el combo si el parametro chageSelection es true
	 */
	private void changePoint(boolean changeSelection, int pos){
		disableEvent = true;
    	try{
    		loadLyrPoint();
    		checkInfoPointText(pos);			
        	if(changeSelection){
        		lastIndexSelection = pos;
        		tableControlerPanel.setSelectedIndex(pos);
        	}
        	lyrPoints.getPointManager().selectPoint(tableControlerPanel.getSelectedIndex(), parent);
        	tableControlerPanel.checkArrows();
    	}catch(NoSuchFieldException exc){
    		JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
					PluginServices.getText(this, "coordenadas_vacias"));
    	}catch(InstantiationException exc){
    		JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
					PluginServices.getText(this, "error_capa_puntos"));
    	}
    	disableEvent = false;
	}
	
	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		//Botn de ir al punto inicial
		if(e.getSource() == tableControlerPanel.getBFirst()){
			if(tableControlerPanel.getItemCount() >= 1)
				changePoint(true, 0);
		}
		
		//Botn de ir al punto anterior
		if(e.getSource() == tableControlerPanel.getBPrev()){
			if(tableControlerPanel.getSelectedIndex() != 0)		        	 
		       	changePoint(true, tableControlerPanel.getSelectedIndex() - 1);
		}
		
		//Botn de ir al punto siguiente
		if(e.getSource() == tableControlerPanel.getBNext()){
			if(tableControlerPanel.getSelectedIndex() != (tableControlerPanel.getItemCount() - 1))
	        	changePoint(true, tableControlerPanel.getSelectedIndex() + 1);
		
		}
		
		//Botn de ir al punto final
		if(e.getSource() == tableControlerPanel.getBLast()){
			if(tableControlerPanel.getItemCount() >= 1)
				changePoint(true, tableControlerPanel.getItemCount() - 1);
		}
		
		//Combo de seleccin de punto
		if(e.getSource() == tableControlerPanel.getCPoint()){
			if(tableControlerPanel.getSelectedIndex() != -1){
				if(!disableEvent){
					changePoint(true,tableControlerPanel.getSelectedIndex());	
				}
			}
		}
	
	}
	
	/**
	 * Esta funcin deshabilita todos los controles y guarda sus valores
	 * de habilitado o deshabilitado para que cuando se ejecute restoreControlsValue
	 * se vuelvan a quedar como estaba
	 */
	public void disableAllControls(){
		if(!disableAllControls){
			disableAllControls = true;
			
			//Salvamos los estados
			buttonsState[0] = getPButtonsExportImportPanel().getBSave().isEnabled();
			buttonsState[1] = getPButtonsExportImportPanel().getBLoad().isEnabled();
			buttonsState[2] = getPButtonsExportImportPanel().getBExportToAscii().isEnabled();
			
			//Desactivamos controles
			getPButtonsExportImportPanel().getBSave().setEnabled(false);
			getPButtonsExportImportPanel().getBLoad().setEnabled(false);
			getPButtonsExportImportPanel().getBExportToAscii().setEnabled(false);
		}
	}
	
	/**
	 * Esta funcin deja los controles como estaban al ejecutar la funcin 
	 * disableAllControls
	 */
	public void restoreControlsValue(){
		if(disableAllControls){
			disableAllControls = false;
			getPButtonsExportImportPanel().getBSave().setEnabled(buttonsState[0]);
			getPButtonsExportImportPanel().getBLoad().setEnabled(buttonsState[1]);
			getPButtonsExportImportPanel().getBExportToAscii().setEnabled(buttonsState[2]);
		}
	}
	//**********************End Methods*****************************
	
	//**********************Setters & Getters***********************
	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	public JPanel getPTable() {
		if (pTable == null) {
			BorderLayout borderLayout4 = new BorderLayout();
			pTable = new JPanel();
			pTable.setLayout(borderLayout4);
			pTable.setSize(new java.awt.Dimension(widthSupPanel,heightSupPanel));
			pTable.setPreferredSize(new java.awt.Dimension(widthSupPanel,heightSupPanel));
			pTable.add(getJTable(), BorderLayout.CENTER);
		}
		return pTable;
	}
	
	/**
	 * This method initializes jPanel1	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	public JPanel getPPointSelection() {
		if (pPointSelection == null) {
			pPointSelection = new JPanel();
			pPointSelection.setPreferredSize(new java.awt.Dimension(widthInfPanel, heightInfPanel));
			if(this.tableControlerPanel == null){
				FlowLayout flowLayout6 = new FlowLayout();
				pPointSelection.setLayout(flowLayout6);
				tableControlerPanel = new TableControlerPanel();
				flowLayout6.setAlignment(java.awt.FlowLayout.LEFT);
				flowLayout6.setHgap(0);
				flowLayout6.setVgap(2);
				pPointSelection.add(tableControlerPanel, null);
				pPointSelection.add(getPButtonsExportImportPanel(), null);
				pPointSelection.setBorder(javax.swing.BorderFactory.createLineBorder(java.awt.Color.gray,1));
				tableControlerPanel.getBFirst().addActionListener(this);
				tableControlerPanel.getBLast().addActionListener(this);
				tableControlerPanel.getBNext().addActionListener(this);
				tableControlerPanel.getBPrev().addActionListener(this);
				tableControlerPanel.getBNew().setVisible(false);
				tableControlerPanel.getCPoint().addActionListener(this);
			}
		}
		return pPointSelection;
	}
	
	/**
	 * This method initializes jTable	
	 * 	
	 * @return javax.swing.JTable	
	 */    
	public PointTable getJTable() {
		if (table == null) {
			table = new PointTable();
			table.setSize(new java.awt.Dimension(widthTable, heightTable));
			table.setPreferredSize(new java.awt.Dimension(widthTable, heightTable));
		}
		return table;
	}
	
	/**
	 * Elimina todas las filas de la tabla.
	 */
	public void clearTable(){
		int l = table.getTable().getRowCount();
		this.setSaveToDisk(false);
		for(int i=0;i<l;i++)
			((DefaultTableModel)table.getTable().getModel()).removeRow(0);
	}

	/**
	 * @return Returns the tableControlerPanel.
	 */
	public TableControlerPanel getTableControlerPanel() {
		return tableControlerPanel;
	}
	
	/**
	 * Asigna una tabla
	 * @param table
	 */
	public void setTable(PointTable table){
		this.table = table;
		table.setSize(new java.awt.Dimension(widthTable, heightTable));
		table.setPreferredSize(new java.awt.Dimension(widthTable, heightTable));
	}
	
	/**
	 * Calculo del nuevo tamao a partir de un frame redimensionado
	 * @param w Ancho del frame
	 * @param h Alto del frame
	 */
	public void newFrameSize(int w, int h){
		int newWidth = w;
		
        this.setSize(new java.awt.Dimension(newWidth, heightGeneralPanel));
        this.setPreferredSize(new java.awt.Dimension(newWidth, heightGeneralPanel));
        
        getPTable().setSize(new java.awt.Dimension((int)(0.98*newWidth),heightSupPanel));
        getPTable().setPreferredSize(new java.awt.Dimension((int)(0.98*newWidth),heightSupPanel));
                
        getJTable().setSize(new java.awt.Dimension((int)(0.98*newWidth), heightTable));
		getJTable().setPreferredSize(new java.awt.Dimension((int)(0.98*newWidth), heightTable));
		
		getPPointSelection().setSize(new java.awt.Dimension((int)(0.98*newWidth), heightInfPanel));
	}

	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	public ButtonsExportImportPanel getPButtonsExportImportPanel() {
		if (pButtonsExportImportPanel == null) {
			pButtonsExportImportPanel = new ButtonsExportImportPanel();
		}
		return pButtonsExportImportPanel;
	}
	
	/**
	 * Activa o desactiva los botones de exportar capa de puntos y salvar
	 * la tabla a Ascii.
	 *@param isPossible Si vale true activa los controles de salvar y exportacin
	 *y si vale false los desactiva
	 */
	public void setSaveToDisk(boolean isPossible){
		this.getPButtonsExportImportPanel().setSaveToDisk(isPossible);
	}
	//**********************End Setters & Getters*******************
}
