package org.gvsig.georeferencing.gui.selectPoints;

import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.geom.Point2D;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.gvsig.georeferencing.GeoreferencingToolsModule;
import org.gvsig.georeferencing.gui.dialogs.GeoreferencingDialog;
import org.gvsig.georeferencing.gui.listeners.GeorefPointSelectorListener;
import org.gvsig.georeferencing.gui.pointsTable.TableControlerPanel;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.layers.FLyrPoints;
import com.iver.cit.gvsig.gui.View;

public class SelectPointsPanel extends JPanel implements ActionListener, KeyListener, FocusListener{

	//**********************Vars**********************************
	private String					pathToImages = "images/"; //"/com/iver/cit/gvsig/gui/panels/images/";
	private JPanel 					pGeneral = null;
	private JPanel 					pPointSelection = null;
	private JPanel 					pControlPoint = null;
	private JButton 				bSelectFromView = null;
	private FLyrPoints 				lyrPoints = null;
	//private boolean 				disableEvent = false;
	private int						lastIndexSelection = -1;
	private JButton 				bExtended = null;
	private JPanel 					pExtend = null;
	private TableControlerPanel 	tableControlerPanel = null;
	private DataPointPanel 			dataPointPanel = null;
	private ErrorPointPanel 		errorPointPanel = null;
	private GeoreferencingDialog	grd = null;
	private boolean 				update = false;
	/**
	 * Valores ltimos que se han introducido en los JTextField para poder reponerlos.
	 */
	public String 					lastTx = "", lastTy = "", lastLong = "", lastLat = "";
	//**********************End Vars********************************
	
	//**********************Methods*********************************
	/**
	 * Contructor. Inicializa el panel.
	 */
	public SelectPointsPanel() {
		super();
		initialize();
	}
	
	/**
	 * This is the default constructor
	 */
	public SelectPointsPanel(GeoreferencingDialog grd) {
		super();
		initialize();
		this.grd = grd;
	}
	
	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
        this.setPreferredSize(new java.awt.Dimension(390,175));
        this.setSize(new java.awt.Dimension(390,175));
        this.setLocation(new java.awt.Point(0,0));
        this.add(getPGeneral(), null);
	}
		
	/**
	 * Actualiza el nmero de puntos del combo a partir de los contenidos en
	 * la capa de puntos. Esta funcin es usada desde otros paneles que cambien
	 * el punto seleccionado para comunicar al control la nueva seleccin.
	 */
	public void updateComboPoint(int pointSelected){
		//disableEvent = true;
		try{
			loadLyrPoint();
		}catch(InstantiationException exc){
    		JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
					PluginServices.getText(this, "error_capa_puntos"));
    		return;
    	}
		
		tableControlerPanel.setNItems(lyrPoints.getCountPoints());
		try{
			tableControlerPanel.getCPoint().setSelectedIndex(pointSelected);
		}catch(IllegalArgumentException ex){
			
		}
		tableControlerPanel.checkArrows();
		lyrPoints.getPointManager().selectPoint(pointSelected, this.grd);
		//disableEvent = false;
	}
	
	/**
	 * Comprueba si los valores de un punto en los campos de texto estn
	 * vacios y lanza una excepcin si hay alguno sin informacin.
	 * @throws NoSuchFieldException
	 */
	public void checkInfoPointText()throws NoSuchFieldException{
		if(	dataPointPanel.getTX().getText().equals("") ||
				dataPointPanel.getTY().getText().equals("") ||
				dataPointPanel.getLatitud().getText().equals("") ||
				dataPointPanel.getLongitud().getText().equals("")){
				//disableEvent = true;
				try{
					tableControlerPanel.getCPoint().setSelectedIndex(lastIndexSelection);
					//disableEvent = false;
				}catch(IllegalArgumentException ex){
					//Si el lastIndex tiene un valor erroneo no hacemos ninguna seleccin
				}
				throw new NoSuchFieldException();
		}
	}
		
	/**
	 * Esta funcin resetea los controles del panel de info de un punto.
	 */
	public void resetControls(boolean active){
		dataPointPanel.resetControls(null);
		errorPointPanel.resetControls(null);
		getBSelectFromView().setEnabled(active);
		getTableControlerPanel().getBClear().setEnabled(active);
		getTableControlerPanel().getBDelPoint().setEnabled(active);
		if(!active)
			getTableControlerPanel().resetControls();
	}
	
	/**
	 * Cuando pasamos de un punto a otro o se va a crear uno nuevo
	 * <UL>
	 * <LI>Cargamos la capa FLyrPoint si no est ya cargada.</LI>
	 * <LI>Comprobamos que los campos no esten vacios y lanzamos una excepcin si hay algn campo vacio si lo estn.</LI>
	 * <LI>Insertamos el nuevo punto seleccionado en los campos de informacin.</LI>
	 * <LI>Checkeamos las flechas para ver si tenemos que activarlas o desactivarlas.</LI>
	 * </UL>
	 * @param changeSelection	Cambia la posicin seleccionada del combo si es true
	 * @param pos	Posicin a la que cambia el combo si el parametro chageSelection es true
	 */
	public void changePoint(boolean changeSelection, int pos){
    	try{
    		loadLyrPoint();
    		checkInfoPointText();			
        	if(changeSelection){
        		lastIndexSelection = pos;
        		tableControlerPanel.setSelectedIndex(pos);
        	}
        	lyrPoints.getPointManager().selectPoint(tableControlerPanel.getSelectedIndex(), this.grd);
        	tableControlerPanel.checkArrows();
    	}catch(NoSuchFieldException exc){
    		JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
					PluginServices.getText(this, "coordenadas_vacias"));
    	}catch(InstantiationException e){
			JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
					PluginServices.getText(this, "error_capa_puntos"));
		}
	}
	
	/**
	 * Obtiene una capa de puntos. Si no existe crea una.
	 * @return
	 */
	private void loadLyrPoint() throws InstantiationException{
		if(lyrPoints == null){
			lyrPoints = this.grd.getLyrPoints();
			if(lyrPoints == null)
				throw new InstantiationException();
		}
	}
	
	/**
	 * Este mtodo inicializa el botn seleccionar desde la vista. Cuando lo 
	 * activamos se carga la herramienta de seleccin de puntos desde la vista. 
	 * 	
	 * @return javax.swing.JButton	
	 */    
	public JButton getBSelectFromView() {
		if (bSelectFromView == null) {
			bSelectFromView = new JButton();
			bSelectFromView.setText("");
			bSelectFromView.setPreferredSize(new java.awt.Dimension(22,22));
			bSelectFromView.setIcon(new ImageIcon(getClass().getClassLoader().getResource(pathToImages + "selectfromview.png")));
			bSelectFromView.setEnabled(false);
			bSelectFromView.setToolTipText(PluginServices.getText(this, "selectfromview"));
			bSelectFromView.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {
					View theView = (View) PluginServices.getMDIManager().getActiveView();
					try{
						loadLyrPoint();
						lyrPoints.setLastTool(theView.getMapControl().getTool());
						GeorefPointSelectorListener.posPoint = tableControlerPanel.getSelectedIndex();
						GeorefPointSelectorListener.firstPoint = true;
						theView.getMapControl().setTool("pointLyrSelection");
					}catch(InstantiationException exc){
						JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
								PluginServices.getText(this, "error_capa_puntos"));
					}
				}
			});
		}
		return bSelectFromView;
	}
	
	/**
	 * Checkea un textField para que contenga caracteres validos. Consideramos validos
	 * los caracteres del 0 al 9, el punto, DEL, Supr, flechas y Enter teniendo en cuenta que el 
	 * cdigo ascii del teclado numerico es distinto.
	 */
	public boolean checkTextField(JTextField tf, int code, String lastText){
		
		//Si el caracter es erroneo pasamos de el y reponemos el texto que habia
		if(	(code >= 0 && code <= 7) || 
			code == 9 || 
			(code >= 11 && code <= 36) ||
			(code >= 41 && code <= 45) ||
			code == 47 ||
			(code >= 58 && code <= 95) ||
			(code >= 106 && code <= 109) ||
			(code >= 111 && code <= 126) ||
			code >= 128){
			tf.setText(lastText);
			return false;
		}
		
		return true;
	}
		
	/**
	 * Control de eventos de TableControlerPanel
	 */
	public void actionPerformed(ActionEvent e) {
		
		//Botn de ir al punto inicial
		if(e.getSource() == tableControlerPanel.getBFirst()){
			if(tableControlerPanel.getItemCount() >= 1)
				changePoint(true, 0);
		}
		
		//Botn de ir al punto anterior
		if(e.getSource() == tableControlerPanel.getBPrev()){
			if(tableControlerPanel.getSelectedIndex() != 0)		        	 
		       	changePoint(true, tableControlerPanel.getSelectedIndex() - 1);
		}
		
		//Botn de ir al punto siguiente
		if(e.getSource() == tableControlerPanel.getBNext()){
			if(tableControlerPanel.getSelectedIndex() != (tableControlerPanel.getItemCount() - 1))
	        	changePoint(true, tableControlerPanel.getSelectedIndex() + 1);
		
		}
		
		//Botn de ir al punto final
		if(e.getSource() == tableControlerPanel.getBLast()){
			if(tableControlerPanel.getItemCount() >= 1)
				changePoint(true, tableControlerPanel.getItemCount() - 1);
		}
		
		//Combo de seleccin de punto
		if(e.getSource() == tableControlerPanel.getCPoint()){
			if(tableControlerPanel.getSelectedIndex() != -1){
				try{ 
					loadLyrPoint();
				}catch(InstantiationException exc){
					return;
				}
				lyrPoints.getPointManager().selectPoint(tableControlerPanel.getSelectedIndex(), grd);
				tableControlerPanel.checkArrows();
			}
		}
		
		//Botn de nuevo punto
		if(e.getSource() == tableControlerPanel.getBNew()){
			grd.getPointManager().newEmptyPoint();
			//Desactivamos todos los controles para centrar la visin del usuario
			//sobre la introduccin de puntos
			grd.getPointManager().disableAllControls();
		}
				
		TableControlerPanel ctrlPanel = grd.getConectorPanel().getDataPointsTabPanel().getTablePointsPanel().getTableControlerPanel();
		
		//Botn de eliminado de todos los puntos
		if(	e.getSource() == getTableControlerPanel().getBClear() ||
			e.getSource() == ctrlPanel.getBClear()){
			grd.getPointManager().deleteAllPoints();
		}
		
		//Botn de eliminado de un punto
		if(e.getSource() == getTableControlerPanel().getBDelPoint() ||
				e.getSource() == ctrlPanel.getBDelPoint()){
			grd.getPointManager().deletePoint();
		}
		
		//Botn de extensin del cuadro de dialogo
		if(e.getSource() == this.getBExtended())
			this.setEnlarge(grd.isEnlarge());
		
	}

	/* (non-Javadoc)
	 * @see java.awt.event.KeyListener#keyPressed(java.awt.event.KeyEvent)
	 */
	public void keyPressed(KeyEvent e) {
		// TODO Auto-generated method stu
	}

	/**
	 * Actualiza los valores del punto nuevo introducido desde el teclado llamando a la funcin
	 * de la capa updataData que actualizar los controles necesarios para que se refresquen la vista
	 * mini imagenes, textfields, tabla y capa de puntos. 
	 * @param pixel Pixel de la imagen
	 * @param map Coordenadas de la vista.
	 */
	private void updatePointsValue(Point2D pixel, Point2D map){
		
		//Si se modifica el pixel tenemos que controlar que no se salga de la imagen.
		//Si se sale avisamos y reponemos su valor.
		if( pixel != null &&
			(pixel.getX() < 0 || 
			pixel.getX() > grd.getLyrGeoRaster().getImageWidth() ||
			pixel.getY() < 0 || 
			pixel.getY() > grd.getLyrGeoRaster().getImageHeight())){
				JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
						PluginServices.getText(this, "fuera_de_extent"));
				dataPointPanel.getTX().setText(lastTx);
				dataPointPanel.getTY().setText(lastTy);
				dataPointPanel.getLatitud().setText(lastLat);
				dataPointPanel.getLongitud().setText(lastLong);
				update = false;
				return;
			}
		
		//Capturamos los ltimos valores de los JtextField por si hay que reponerlos
		if(pixel != null){
			if( !lastTx.equals(dataPointPanel.getTX().getText()) )
				lastTx = dataPointPanel.getTX().getText();
			if( !lastTy.equals(dataPointPanel.getTY().getText()) )
				lastTy = dataPointPanel.getTY().getText();
		}
		
		if(map != null){
			if( !lastLat.equals(dataPointPanel.getLatitud().getText()) )
				lastLat = dataPointPanel.getLatitud().getText();
			if( !lastLong.equals(dataPointPanel.getLongitud().getText()) )
				lastLong = dataPointPanel.getLongitud().getText();
		}
		
		View theView = null;
		try{
			theView = (View)PluginServices.getMDIManager().getActiveView();
		}catch(ClassCastException exc){
			return;
		}		
		
		//Actualizamos los puntos
		grd.getPointManager().updateData(Integer.valueOf(this.grd.getSelectPointsPanel().getTableControlerPanel().getSelectedItem()).intValue(),
														pixel, 
														map, 
														this.grd,
														theView);
		update = false;
		
	}
	
	/**
	 * Control de eventos de las entradas de texto de los puntos
	 */
	public void keyReleased(KeyEvent e) {
		//Campo de texto de la coordenada X en pixeles
		Point2D pixel = null;
		Point2D map = null;
		
		try{
			if(e.getSource() == dataPointPanel.getTX()){
				if(checkTextField(dataPointPanel.getTX(), e.getKeyCode(), lastTx)){
					pixel = new Point2D.Double();
					pixel.setLocation(	Double.valueOf(dataPointPanel.getTX().getText()).doubleValue(), 
										Double.valueOf(dataPointPanel.getTY().getText()).doubleValue());
					
					if(e.getKeyCode() == 10)
						update = true;
				}
			}
			
			//Campo de texto de la coordenada Y en pixeles
			if(e.getSource() == dataPointPanel.getTY()){
				if(checkTextField(dataPointPanel.getTY(), e.getKeyCode(), lastTy)){
					pixel = new Point2D.Double();
					pixel.setLocation(	Double.valueOf(dataPointPanel.getTX().getText()).doubleValue(), 
										Double.valueOf(dataPointPanel.getTY().getText()).doubleValue());
		
					if(e.getKeyCode() == 10)
						update = true;
				}
			}
	
			//Campo de texto de la coordenada X en coordenadas del mundo
			if(e.getSource() == dataPointPanel.getLatitud()){
				if(checkTextField(dataPointPanel.getLatitud(), e.getKeyCode(), lastLat)){
					map = new Point2D.Double();
					map.setLocation(	Double.valueOf(dataPointPanel.getLongitud().getText()).doubleValue(), 
										Double.valueOf(dataPointPanel.getLatitud().getText()).doubleValue());
					if(e.getKeyCode() == 10)
						update = true;
				}
			}
	
			//Campo de texto de la coordenada Y en coordenadas del mundo
			if(e.getSource() == dataPointPanel.getLongitud()){
				if(checkTextField(dataPointPanel.getLongitud(), e.getKeyCode(), lastLong)){
					map = new Point2D.Double();
					map.setLocation(	Double.valueOf(dataPointPanel.getLongitud().getText()).doubleValue(), 
										Double.valueOf(dataPointPanel.getLatitud().getText()).doubleValue());
					if(e.getKeyCode() == 10)
						update = true;
				}
			}
			
			if(update){
				this.updatePointsValue(pixel, map);
				//Si estn rellenos todos los campos restauramos los controles
				if(	!dataPointPanel.getTX().equals("") && 
					!dataPointPanel.getTY().equals("") &&
					!dataPointPanel.getLongitud().equals("") && 
					!dataPointPanel.getLatitud().equals("")){
					grd.getPointManager().restoreControlsValue();
				}
			}
			
			this.abortSelectPoints();
			
		}catch(NumberFormatException exc){
			System.out.println("Formato numerico erroneo");
		}
		
	}

	/* (non-Javadoc)
	 * @see java.awt.event.KeyListener#keyTyped(java.awt.event.KeyEvent)
	 */
	public void keyTyped(KeyEvent e) {
		// TODO Auto-generated method stub
		
	}
	
	/* (non-Javadoc)
	 * @see java.awt.event.FocusListener#focusGained(java.awt.event.FocusEvent)
	 */
	public void focusGained(FocusEvent arg0) {
		// TODO Auto-generated method stub

	}
	
	/**
	 * Cancela la seleccin de puntos sobre la vista de forma grfica. Esta accin
	 * conllevar seleccionar la ltima herramienta del mapCtrl
	 */
	private void abortSelectPoints(){
		String tx = dataPointPanel.getTX().getText();
		String ty = dataPointPanel.getTY().getText();
		String lon = dataPointPanel.getLongitud().getText();
		String lat = dataPointPanel.getLatitud().getText();
		
		if(!tx.equals("") && !ty.equals("") && !lon.equals("") && !lat.equals("")){
			//Si todos los puntos estn rellenos deseleccionamos la herramienta de seleccin de puntos
			GeorefPointSelectorListener.firstPoint = true;
			
			//Activamos los controles de esta ya que habr sido desactivados al llamar a newEmptyPoint
			GeoreferencingToolsModule.setEnabled(true);
			
			com.iver.cit.gvsig.gui.View theView = null;
			try{
				theView = (com.iver.cit.gvsig.gui.View)PluginServices.getMDIManager().getActiveView();
				theView.getMapControl().setTool(grd.getLyrPoints().getLastTool());
			}catch(ClassCastException exc){
				//No se selecciona la herramienta
			}
		}
	}
	
	/**
	 * Los campos de texto de las coordenadas cuando pierden el foco actualizan el valor del pixel
	 * en la cap, vista, mini imagen y tabla
	 */
	public void focusLost(FocusEvent arg0) {
		String tx = dataPointPanel.getTX().getText();
		String ty = dataPointPanel.getTY().getText();
		String lon = dataPointPanel.getLongitud().getText();
		String lat = dataPointPanel.getLatitud().getText();
		
		if(!tx.equals("") && !ty.equals("") && !lon.equals("") && !lat.equals("")){
			Point2D pixel = new Point2D.Double();
			Point2D map = new Point2D.Double();
			pixel.setLocation(Double.valueOf(tx).doubleValue(),
							  Double.valueOf(ty).doubleValue());
			map.setLocation(Double.valueOf(lon).doubleValue(),
							Double.valueOf(lat).doubleValue());
			//Si update es true significa q se ha pulsado Enter por lo q ya se est actualizando
			//y no hace falta que se haga de nuevo pq podra salir un mensaje de error dos veces
			if(!update){
				this.updatePointsValue(pixel, map);
				//Si estn rellenos todos los campos restauramos los controles
				if(	!dataPointPanel.getTX().equals("") && 
					!dataPointPanel.getTY().equals("") &&
					!dataPointPanel.getLongitud().equals("") && 
					!dataPointPanel.getLatitud().equals("")){
					grd.getPointManager().restoreControlsValue();
				}
			}
			this.abortSelectPoints();			
		}
	}

	//**********************End Methods*******************************
	
	//**********************Setters & Getters*************************
	/**
	 * Extiende o pliega el panel inferior controlando el icono del botn. Si se
	 * asigna true quiere decir que el panel est desplegado, en este caso lo plegar.
	 * Si se asigna false quiere decir que el panel est plegado, en este caso lo 
	 * desplegar.
	 * @param enlarge True para recogerlo y false para desplegarlo
	 */
	public void setEnlarge(boolean enlarge){
		if(enlarge){
			grd.setEnlarge(!enlarge);
			bExtended.setIcon(new ImageIcon(getClass().getClassLoader().getResource(pathToImages + "down.png")));
		}else{
			grd.setEnlarge(!enlarge);
			bExtended.setIcon(new ImageIcon(getClass().getClassLoader().getResource(pathToImages + "up.png")));
			if(lyrPoints != null){
				//lyrPoints.getPointManager().selectPoint(tableControlerPanel.getSelectedIndex(), this.grd);
				if(lyrPoints.isNewPoint(grd.getSelectedPoint())){
					grd.getConectorPanel().getAdjustGeorefPanel().getZoomLeft().setEnabled(false);
					grd.getConectorPanel().getAdjustGeorefPanel().getZoomRight().setEnabled(false);
				}
				grd.getZoomControlRight().draw();
				grd.getZoomControlLeft().draw();
			}
		}
	}
	
	/**
	 * @return Returns the errorPointPanel.
	 */
	public ErrorPointPanel getErrorPointPanel() {
		if(this.errorPointPanel == null)
			errorPointPanel = new ErrorPointPanel();
			errorPointPanel.setPreferredSize(new java.awt.Dimension(175,130));
		return errorPointPanel;
	}
	/**
	 * @return Returns the lyrPoints.
	 */
	public FLyrPoints getLyrPoints() {
		if(lyrPoints == null){
			try{
				loadLyrPoint();
					}catch(InstantiationException exc){
				JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
						PluginServices.getText(this, "error_capa_puntos"));
			}
		}
		return lyrPoints;
	}
	
	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	public JButton getBExtended() {
		if (bExtended == null) {
			bExtended = new JButton();
			bExtended.setPreferredSize(new java.awt.Dimension(42,22));
			bExtended.setIcon(new ImageIcon(getClass().getClassLoader().getResource(pathToImages + "down.png")));
			bExtended.setToolTipText(PluginServices.getText(this, "plegar_despl"));
			bExtended.addActionListener(this);
		}
		return bExtended;
	}
	
	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getPExtend() {
		if (pExtend == null) {
			FlowLayout flowLayout21 = new FlowLayout();
			pExtend = new JPanel();
			pExtend.setLayout(flowLayout21);
			flowLayout21.setHgap(0);
			flowLayout21.setVgap(1);
			flowLayout21.setAlignment(java.awt.FlowLayout.LEFT);
			pExtend.setPreferredSize(new java.awt.Dimension(70,22));
			pExtend.add(getBSelectFromView(), null);
			pExtend.add(getBExtended(), null);
		}
		return pExtend;
	}

	/**
	 * @return Returns the dataPointPanel.
	 */
	public DataPointPanel getDataPointPanel() {
		return dataPointPanel;
	}

	/**
	 * @param dataPointPanel The dataPointPanel to set.
	 */
	public void setDataPointPanel(DataPointPanel dataPointPanel) {
		this.dataPointPanel = dataPointPanel;
	}
	
	/**
	 * @return Returns the tableControlerPanel.
	 */
	public TableControlerPanel getTableControlerPanel() {
		return tableControlerPanel;
	}

	/**
	 * @param tableControlerPanel The tableControlerPanel to set.
	 */
	public void setTableControlerPanel(TableControlerPanel tableControlerPanel) {
		this.tableControlerPanel = tableControlerPanel;
	}
	
	/**
	 * This method initializes pGeneral	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPGeneral() {
		if (pGeneral == null) {
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.insets = new java.awt.Insets(0,0,0,0);
			gridBagConstraints1.gridy = 0;
			gridBagConstraints1.gridx = 0;
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.insets = new java.awt.Insets(5,0,5,0);
			gridBagConstraints.gridy = 1;
			gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTH;
			gridBagConstraints.gridheight = 1;
			gridBagConstraints.gridx = 0;
			pGeneral = new JPanel();
			pGeneral.setLayout(new GridBagLayout());
			pGeneral.setPreferredSize(new java.awt.Dimension(390,175));
			gridBagConstraints.gridwidth = 1;
			pGeneral.add(getPPointSelection(), gridBagConstraints);
			pGeneral.add(getPControlPoint(), gridBagConstraints1);
		}
		return pGeneral;
	}

	/**
	 * This method initializes pFileSelection	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPPointSelection() {
		if (pPointSelection == null) {
			//jLabel = new JLabel();
			FlowLayout flowLayout1 = new FlowLayout();
			FlowLayout flowLayout = new FlowLayout();
			flowLayout.setVgap(-3);
			flowLayout.setAlignment(java.awt.FlowLayout.LEFT);
			pPointSelection = new JPanel();
			pPointSelection.setBorder(javax.swing.BorderFactory.createTitledBorder(null, "", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Dialog", java.awt.Font.BOLD, 12), java.awt.Color.black));
			pPointSelection.setName("");
			pPointSelection.setLayout(flowLayout);
			pPointSelection.setPreferredSize(new java.awt.Dimension(375,27));
			flowLayout.setHgap(0);
			if(this.tableControlerPanel == null){
				tableControlerPanel = new TableControlerPanel();
				tableControlerPanel.setLayout(flowLayout1);
				flowLayout1.setVgap(1);
				flowLayout1.setHgap(0);
				//jLabel.setVerticalAlignment(javax.swing.SwingConstants.CENTER);
				//tableControlerPanel.setLNumberOfPoints(jLabel);
				tableControlerPanel.getBFirst().addActionListener(this);
				tableControlerPanel.getBLast().addActionListener(this);
				tableControlerPanel.getBNext().addActionListener(this);
				tableControlerPanel.getBPrev().addActionListener(this);
				tableControlerPanel.getBNew().addActionListener(this);
				tableControlerPanel.getCPoint().addActionListener(this);
				tableControlerPanel.getBClear().addActionListener(this);
				tableControlerPanel.getBDelPoint().addActionListener(this);
			}
			//pPointSelection.add(getPTableControl(), null);
			pPointSelection.add(tableControlerPanel, null);
			pPointSelection.add(getPExtend(), null);
		}
		return pPointSelection;
	}

	/**
	 * This method initializes pControls	
	 * 		
	 */
	private JPanel getPControlPoint() {
		if (pControlPoint == null) {
			GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.gridx = 0;
			gridBagConstraints2.insets = new java.awt.Insets(0,0,0,0);
			gridBagConstraints2.gridy = 0;
			pControlPoint = new JPanel();
			pControlPoint.setLayout(new GridBagLayout());
			pControlPoint.setPreferredSize(new java.awt.Dimension(380,130));
			pControlPoint.setBorder(javax.swing.BorderFactory.createEmptyBorder(0,0,0,0));
			gridBagConstraints11.gridy = 0;
			gridBagConstraints11.gridx = 1;
			if(this.dataPointPanel == null){
				dataPointPanel = new DataPointPanel();
				dataPointPanel.setPreferredSize(new java.awt.Dimension(200,130));
				dataPointPanel.getTX().addKeyListener(this);
				dataPointPanel.getTY().addKeyListener(this);
				dataPointPanel.getLatitud().addKeyListener(this);
				dataPointPanel.getLongitud().addKeyListener(this);
				dataPointPanel.getTX().addFocusListener(this);
				dataPointPanel.getTY().addFocusListener(this);
				dataPointPanel.getLatitud().addFocusListener(this);
				dataPointPanel.getLongitud().addFocusListener(this);
			}
			pControlPoint.add(dataPointPanel, gridBagConstraints2);
		
			getErrorPointPanel();
			pControlPoint.add(errorPointPanel, gridBagConstraints11);
		}
		return pControlPoint;
	}
	
	/**
	 * Asigna la ruta donde estn las imagenes
	 * @param pathToImages
	 */
	public void setPathToImages(String pathToImages) {
		this.pathToImages = pathToImages;
	}
	
	/**
	 * Obtiene el ltimo indice seleccionado de la lista de puntos
	 * @return entero con el ltimo indice seleccionado de la lista de puntos
	 */
	public int getLastIndexSelection() {
		return lastIndexSelection;
	}

	/**
	 * Asigna el ltimo indice seleccionado de la lista de puntos
	 * @param entero con el ltimo indice seleccionado de la lista de puntos
	 */
	public void setLastIndexSelection(int lastIndexSelection) {
		this.lastIndexSelection = lastIndexSelection;
	}
	
	/**
     * Activa o desactiva este panel y todos los que lo componen
     * @param enabled variable booleana para la activacin y/o desactivacin
     */
    public void setEnabled(boolean enabled){
    	this.dataPointPanel.setEnabled(enabled);
    	this.errorPointPanel.setEnabled(enabled);
    	this.tableControlerPanel.setEnabled(enabled);
    	this.getBSelectFromView().setEnabled(enabled);
    }
	//**********************End Setters & Getters*********************
 }
