/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2005 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/
package org.gvsig.georeferencing.toc;

import java.awt.event.ActionEvent;
import java.awt.geom.Point2D;

import javax.swing.JDesktopPane;
import javax.swing.JInternalFrame;
import javax.swing.JLayeredPane;
import javax.swing.JMenuItem;

import org.gvsig.georeferencing.gui.dialogs.GeoreferencingDialog;
import org.gvsig.georeferencing.utils.GeoPointPersistence;
import org.gvsig.georeferencing.utils.MathUtils;
import org.gvsig.georeferencing.utils.PointManager;
import org.gvsig.georeferencing.utils.StackZoom;

import com.iver.andami.PluginServices;
import com.iver.andami.ui.mdiFrame.MDIFrame;
import com.iver.cit.gvsig.fmap.FMap;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrGeoRaster;
import com.iver.cit.gvsig.gui.View;
import com.iver.cit.gvsig.gui.toc.FPopupMenu;
import com.iver.cit.gvsig.gui.toc.TocMenuEntry;

/**
 * Entrada del men del TOC que obtiene el cuadro de propiedades de capa 
 * georreferenciada.
 * 
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class GeoRasterTocMenuEntry  extends TocMenuEntry{
	private JMenuItem 			propsMenuItem;
	private FLayer 				lyr = null;
	public static boolean 		isOpen = false;
		
	public void initialize(FPopupMenu m) {
		super.initialize(m);
		
		if (isTocItemBranch()) {
			lyr = getNodeLayer();
			
    		// Opcciones para capas FLyrGeoRaster
    		if ((lyr instanceof FLyrGeoRaster)) {
    			propsMenuItem = new JMenuItem(PluginServices.getText(this, "georaster"));
    			getMenu().add( propsMenuItem );
    			propsMenuItem.setFont(FPopupMenu.theFont);
    			getMenu().setEnabled(true);
    			propsMenuItem.addActionListener(this);
     		}
		}
	}
		
	/**
	 * Evento producido al pulsar la entrada de men del toc
	 * que muestra el cuadro con el dialogo de las propiedades 
	 * de la capa de raster georreferenciada.
	 */
	public void actionPerformed(ActionEvent e) {
		FLyrGeoRaster lyrGeoRaster = null;
		GeoreferencingDialog geoDialog = null;
		
		//Caso en el que se crea una capa de georreferenciacin al abrir un proyecto
		//hay que crear el dialogo, la persistencia y la pila de zooms y asignarlo a 
		//la capa de puntos. Todo esto se hace aqu porque al levantar la capa desde el
		//setXMLEntity no podemos acceder al cdigo de gvSIG
		if(((FLyrGeoRaster)lyr).getPersistence() == null && ((FLyrGeoRaster)lyr).getZoom() == null){
			JInternalFrame panel = PointManager.getInternalFrame(this);
	        geoDialog = new GeoreferencingDialog(panel, ((FLyrGeoRaster)lyr));
	        ((FLyrGeoRaster)lyr).getStackZoom().setInitExtent(((FLyrGeoRaster)lyr).getAssignExtent());
	        ((FLyrGeoRaster)lyr).getFLyrPoints().saveState();
	        ((FLyrGeoRaster)lyr).setGeoDialog(geoDialog);
	        GeoPointPersistence	geoPointPersistence = new GeoPointPersistence();
			StackZoom zoom = new StackZoom();
			((FLyrGeoRaster)lyr).setPersistence(geoPointPersistence);
			((FLyrGeoRaster)lyr).setZoom(zoom);
			GeoRasterTocMenuEntry.isOpen = true;
		}
			
		if(isOpen){
			GeoRasterTocMenuEntry.isOpen = false;
	       	lyr = getNodeLayer();
	     
			PluginServices.getMainFrame().enableControls();
					
	       	//Modificamos el nombre de la capa
			if (lyr instanceof FLyrGeoRaster){
				lyrGeoRaster = (FLyrGeoRaster)lyr;
				lyrGeoRaster.setName("*"+lyrGeoRaster.getName());
				lyrGeoRaster.setActive(true);
				geoDialog = (GeoreferencingDialog)lyrGeoRaster.getGeoDialog();
				geoDialog.getSelectPointsPanel().resetControls(false);
				geoDialog.getConectorPanel().getDataPointsTabPanel().getTablePointsPanel().clearTable();
				geoDialog.getSelectPointsPanel().setEnlarge(true);
				geoDialog.getLyrPoints().setVisible(true);
			
		        if(lyrGeoRaster.getFLyrPoints() != null){
		        	//Salvamos el estado actual de la capa por si se cancela.
		        	lyrGeoRaster.getFLyrPoints().saveState();
		        	
		        	View theView = null;
		        	try{
		        		theView = (View) PluginServices.getMDIManager().getActiveView();
		        	}catch(ClassCastException exc){
		        		return ;
		        	}
		        	
		        	//Asignamos la capa de puntos como capa grafica
		        	FMap fmap = theView.getMapControl().getMapContext();
		        	if(fmap != null)
		        		fmap.setGraphicsLayer(lyrGeoRaster.getFLyrPoints());
		       		        	
		        	//Hacemos visible la capa de puntos
		        	lyrGeoRaster.getFLyrPoints().setVisible(true);
		        	theView.getMapControl().getMapContext().invalidate();
		        
					geoDialog.getConectorPanel().getDataPointsTabPanel().
						getSelectPointsPanel().getTableControlerPanel().setNItems(lyrGeoRaster.getFLyrPoints().getCountPoints());
				
		        	if(lyrGeoRaster.getFLyrPoints().getCountPoints() > 0){
		        		//Cargamos el primer punto en el panel 
		        		lyrGeoRaster.getFLyrPoints().getPointManager().selectPoint(0, geoDialog);
		        		
		        		//Activamos controles
		        		if(lyrGeoRaster.getFLyrPoints().getCountPoints() > 0){
		        			geoDialog.getConectorPanel().getDataPointsTabPanel().getSelectPointsPanel().getTableControlerPanel().getBDelPoint().setEnabled(true);
		        			geoDialog.getConectorPanel().getDataPointsTabPanel().getSelectPointsPanel().getTableControlerPanel().getBClear().setEnabled(true);
		        			geoDialog.getConectorPanel().getDataPointsTabPanel().getTablePointsPanel().getPButtonsExportImportPanel().getBSave().setEnabled(true);
		        			geoDialog.getConectorPanel().getDataPointsTabPanel().getTablePointsPanel().getPButtonsExportImportPanel().getBExportToAscii().setEnabled(true);
		        		}
		        		
		        		//Cargamos la tabla de puntos al abrir el dialogo con el valor actual de estos
		        		for(int i=0;i<lyrGeoRaster.getFLyrPoints().getCountPoints();i++){
		        			geoDialog.addTableNew();
			        		geoDialog.setTableValueAt(String.valueOf(i + 1), i, 0);
			        		Point2D pixel = lyrGeoRaster.getFLyrPoints().getPoint(i).pixelPoint;
			        		if(pixel != null){	
			        			geoDialog.setTableValueAt(MathUtils.tailDecimals(String.valueOf(pixel.getX()), 5), i, 1);
			        			geoDialog.setTableValueAt(MathUtils.tailDecimals(String.valueOf(pixel.getY()), 5), i, 2);
			        		}
			        		Point2D map = lyrGeoRaster.getFLyrPoints().getPoint(i).mapPoint;
			        		if(map != null){
			        			geoDialog.setTableValueAt(MathUtils.tailDecimals(String.valueOf(map.getX()), 5), i, 3);
			        			geoDialog.setTableValueAt(MathUtils.tailDecimals(String.valueOf(map.getY()), 5), i, 4);
			        		}
			        		geoDialog.getPointManager().setErrors(i);
		        		}
		        		
		        	}
		        }
			}
					
			//Lanzamos el dialogo 
			JInternalFrame panel = PointManager.getInternalFrame(this);
	        panel.getContentPane().add(geoDialog);
	        geoDialog.setEnlarge(false);
	        geoDialog.setSelectedTab(0);
	        geoDialog.setFrame(panel);
	        
	        MDIFrame mainFrame = (MDIFrame) PluginServices.getMainFrame();
	        JLayeredPane lyrPane = mainFrame.getLayeredPane();
	        lyrPane.add(panel, JDesktopPane.PALETTE_LAYER);
	        panel.pack();
	        panel.setSize(geoDialog.getWidth() + 12, geoDialog.getHeight() + 10);
	        panel.show();
	        
	        PluginServices.getMainFrame().enableControls();
		}
	}
	
	
}