/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.gvsig.georeferencing.utils;

import java.awt.geom.Point2D;
import java.io.File;

import org.cresques.cts.IProjection;
import org.gvsig.georeferencing.GeoreferencingToolsModule;
import org.gvsig.georeferencing.gui.dialogs.GeoreferencingDialog;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.drivers.RasterDriver;
import com.iver.cit.gvsig.fmap.layers.FLyrGeoRaster;
import com.iver.cit.gvsig.fmap.layers.FLyrPoints;
import com.iver.cit.gvsig.fmap.layers.FLyrRaster;
import com.iver.cit.gvsig.fmap.layers.RasterAdapter;
import com.iver.cit.gvsig.fmap.layers.RasterFileAdapter;
import com.iver.cit.gvsig.gui.View;


/**
 * Clase para la creacin de capas de georreferenciacin y capas de puntos
 *
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class GeoLayerFactory{
	
	//**********************Params********************************
	
	//**********************End Params****************************
	
	//**********************Vars**********************************
	
	//**********************End Vars**********************************
	
	//**********************Classes***********************************
	
	//**********************End Classes*******************************

	//**********************Methods***********************************
	
	
	/**
	 * Crea una capa FLyrPoints a partir de los parmetros de entrada.
	 * @param lyGeoRaster	Capa a georreferenciar
	 * @param pixel			Coordenadas relativas a la imagen a georreferenciar en pixels
	 * @param map			Coordenadas de destino
	 * @param active		True o false si el punto est activo o no
	 * @param leftCenter	Punto central del viewport de la miniimagen de coordenadas reales
	 * @param rightCenter	Punto central del viewport de la miniimagen de coordenadas en pixels
	 * @param leftViewPort	ViewPort de la miniimagen de coordenadas reales
	 * @param rightViewPort	ViewPort de la miniimagen de coordenadas en pixels
	 * @return Capa de puntos creada.
	 */
	public static FLyrPoints createFLyrPoints(	FLyrGeoRaster lyGeoRaster, 
												Point2D[] pixel, 
												Point2D[] map, 
												boolean[] active,
												Point2D[] leftCenter, 
												Point2D[] rightCenter,
												ViewPort[] leftViewPort,
												ViewPort[] rightViewPort){
		GeoPointPersistence	geoPointPersistence = new GeoPointPersistence();
		FLyrPoints lyrPoints = new FLyrPoints(lyGeoRaster, geoPointPersistence);
		geoPointPersistence.setLyrPoints(lyrPoints);
		
		lyrPoints.getPointManager().newEmptyPoint();
		GeoreferencingToolsModule.setEnabled(true);
		View theView = null;
		try{
			theView = (View)PluginServices.getMDIManager().getActiveView();
		}catch(ClassCastException exc){
			return null;
		}
		int pos = lyrPoints.getCountPoints() -1;
		for(int i = 0;i < pixel.length;i++){
			lyrPoints.setPointActive(pos, active[i]);
			lyrPoints.setLeftCenterPoint(pos,leftCenter[i]);
			lyrPoints.setRightCenterPoint(pos, rightCenter[i]);
			lyrPoints.setLeftViewPort(pos, leftViewPort[i]);
			lyrPoints.setRightViewPort(pos, rightViewPort[i]);
			lyrPoints.getPointManager().updateData(pos + 1, pixel[i], map[i], lyrPoints.getPointManager().getDialog(), theView);
		}
		
		if(lyrPoints.getCountPoints() > 0){
			GeoreferencingDialog grd = lyrPoints.getPointManager().getDialog();
			grd.getConectorPanel().getDataPointsTabPanel().
				getSelectPointsPanel().getTableControlerPanel().setNItems(lyrPoints.getCountPoints());
			grd.getConectorPanel().getDataPointsTabPanel().
				getTablePointsPanel().getTableControlerPanel().setNItems(lyrPoints.getCountPoints());
			lyrPoints.getPointManager().selectPoint(0, grd);
		}
		
		return lyrPoints;
	}

	/**
	 * Crea una capa Raster de georreferenciacin a partir del nombre driver, fichero y proyeccin y 
	 * coordenadas de georeferenciacin. Esta funcin es para georeferenciar 
	 * capas raster. Para imagenes que no tienen georeferenciacin hay que asignarle
	 * una temporal, normalmente a partir de la vista activa.
	 *
	 * @param layerName Nombre de la capa.
	 * @param d RasterDriver.
	 * @param f Fichero.
	 * @param proj Proyeccin.
	 * @param widthPxImg Ancho en pixeles de la imagen
	 * @param heightPxImg Alto en pixeles de la imagen
	 *
	 * @return Nueva capa de tipo raster.
	 *
	 * @throws DriverIOException
	 */
	public static FLyrGeoRaster createLayer(String layerName, RasterDriver d,
		File f, IProjection proj, double widthPxImg, double heightPxImg) throws DriverException {
		RasterAdapter adapter = new RasterFileAdapter(f);
		adapter.setDriver(d);
		GeoPointPersistence	geoPointPersistence = new GeoPointPersistence();
		StackZoom zoom = new StackZoom();
		FLyrGeoRaster lyrGeoRaster = new FLyrGeoRaster(geoPointPersistence, zoom);
		geoPointPersistence.setLyrPoints(lyrGeoRaster.getFLyrPoints());
		
		if (lyrGeoRaster != null) {
			lyrGeoRaster.setName(layerName);
			lyrGeoRaster.setSource(adapter);
			lyrGeoRaster.setProjection(proj);
			lyrGeoRaster.setImageDimension(widthPxImg, heightPxImg);
			try {
				lyrGeoRaster.load();
			} catch (DriverIOException e) {
				throw new DriverException(e);
			}
	        
			lyrGeoRaster.setVisible(true);
		}else 
			return null;
		return lyrGeoRaster;
	}
	
	/**
	 * Crea una capa Raster a partir del nombre driver, fichero y proyeccin.
	 *
	 * @param layerName Nombre de la capa.
	 * @param d RasterDriver.
	 * @param f Fichero.
	 * @param proj Proyeccin.
	 *
	 * @return Nueva capa de tipo raster.
	 *
	 * @throws DriverIOException
	 */
	public static FLyrRaster createLayer(String layerName, RasterDriver d,
		File f, IProjection proj) throws DriverException {
		RasterAdapter adapter = new RasterFileAdapter(f);
		adapter.setDriver(d);

		FLyrRaster capa = new FLyrRaster();
		capa.setName(layerName);

		capa.setSource(adapter);
		capa.setProjection(proj);
		try {
			capa.load();
		} catch (DriverIOException e) {
			throw new DriverException(e);
		}
		
		return capa;
	}
	//**********************End Methods********************************
	
	//**********************Setters & Getters**************************
	
	//**********************End Setters & Getters*********************
}
