package org.gvsig.georeferencing.utils;
import java.awt.Dimension;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.StringWriter;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.crimson.jaxp.DocumentBuilderFactoryImpl;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.cresques.cts.ProjectionPool;
import org.cresques.io.RasterMetaFileTags;
import org.gvsig.georeferencing.GeoreferencingToolsModule;
import org.gvsig.georeferencing.gui.dialogs.GeoreferencingDialog;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.layers.FLyrGeoRaster;
import com.iver.cit.gvsig.fmap.layers.FLyrPoints;
import com.iver.cit.gvsig.fmap.layers.GeoPoint;
import com.iver.cit.gvsig.fmap.layers.IPersistence;
import com.iver.cit.gvsig.gui.View;

/**
 * Clase que controla la persistencia de la capa de puntos. 
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class GeoPointPersistence implements IPersistence{
	//**********************Vars**********************************
	private ArrayList 		geoPointList = null;
	private Document 		xmlDoc = null;
	private Element 		generalTag = null;
	private FLyrPoints 		lyrPoints = null;
	private double[]		lastViewPort = null;
	//**********************End Vars******************************
	
	//**********************Methods*******************************
	/**
	 * Constructor
	 */
	public GeoPointPersistence(){}
	
	/**
	 * Constructor
	 * @param geoPointList	Lista de geopuntos
	 */
	public GeoPointPersistence(FLyrPoints lyrPoints){
		this.geoPointList = lyrPoints.getListPoint();
		this.lyrPoints = lyrPoints;
	}
		
	/**
	 * Carga la lista de puntos desde un fichero
	 * @param file	Nombre del fichero
	 * @return	Array con la lista de puntos
	 */
	public void loadPointList(String file){
		try {
	        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
	        DocumentBuilder db = dbf.newDocumentBuilder();
	        File f = new File(file);
	        Document doc = db.parse(f);
		      
	        if(doc.getDocumentElement().getTagName().equals(RasterMetaFileTags.MAIN_TAG)){
	        	NodeList nodeListMain = doc.getDocumentElement().getChildNodes();
	        	for(int j=0;j<nodeListMain.getLength();j++){
	        		Node nodeMain = nodeListMain.item(j);
	        		if(nodeMain.getNodeName().equals(RasterMetaFileTags.GEOPOINTS)){
	        	
	        			NodeList nodeListGeoPoints = nodeMain.getChildNodes();
	    	        	lyrPoints.clear();
	    	        	for(int i=0;i<nodeListGeoPoints.getLength();i++){
	    	        		Node node = nodeListGeoPoints.item(i);
	    	        		if(node.getNodeName().equals(RasterMetaFileTags.POINTS_LYR)){
	    	        			NodeList nl = node.getChildNodes();
	    	        			lastViewPort = processFlyrGeoRasterNodeValue(nl);	
	    	        		}
	    	        		if(node.getNodeName().equals(RasterMetaFileTags.GEOPOINT)){
	    	        			NodeList nl = node.getChildNodes();
	    	        			processGeoPointNodeValue(nl);	
	    	        		}
	    	        	}
	    	        	this.geoPointList = lyrPoints.getListPoint();
	    	        	
	        		}
	        	}
	        }
	    } catch (ParserConfigurationException e) {
	            return;
	    } catch (SAXException e1) {
	            return;
	    } catch (IOException e1) {
	            return;
	    }
	}
		
	/**
	 * Crea el XML con la lista de puntos y la salva a disco
	 * @param file
	 */
	public void savePointList(String file){
		this.geoPointList = lyrPoints.getListPoint();
		try {
			DocumentBuilderFactory dbFactory = DocumentBuilderFactoryImpl.newInstance();
		    DocumentBuilder docBuilder = dbFactory.newDocumentBuilder();
		    xmlDoc = docBuilder.newDocument();
		} catch(Exception e) {
		      System.out.println(e);
		}
		    
		generalTag = xmlDoc.createElement(RasterMetaFileTags.GEOPOINTS);
		xmlDoc.appendChild(generalTag);
		
		createXMLFLyrGeoRasterNode(lyrPoints.getLyrGeoRaster());
		
		for(int i=0;i<geoPointList.size();i++)
			createXMLGeoPointNode((GeoPoint)(geoPointList.get(i)), i);
		  
		XML2File(Document2Text(), file);
	} 
	
	/**
	 * Crea el fichero Ascii con la lista de puntos y la salva a disco
	 * @param file
	 */
	public void saveAsciiPointList(String file){
		this.geoPointList = lyrPoints.getListPoint();
		try{
			BufferedWriter out = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file)));
			if(lyrPoints.getLyrGeoRaster().getGeoDialog().isErrorCSV())
				out.write("\"Pt\",\"X\",\"Y\",\"X'\",\"Y'\",\"ErrX\",\"ErrY\",\"RMS\"\n");
			else
				out.write("\"Pt\",\"X\",\"Y\",\"X'\",\"Y'\"\n");
				
			for(int i=0;i<geoPointList.size();i++){
				GeoPoint geoPoint =  (GeoPoint)geoPointList.get(i);
				String point = (i+1)+","+
								geoPoint.pixelPoint.getX()+","+
								geoPoint.pixelPoint.getY()+","+
								geoPoint.mapPoint.getX()+","+
								geoPoint.mapPoint.getY();
				double[][] err = lyrPoints.getLyrGeoRaster().getGeoDialog().getErrors();
				String error = "";
				try{
					error = err[i][0]+","+err[i][1]+","+err[i][2];
					if(lyrPoints.getLyrGeoRaster().getGeoDialog().isErrorCSV())
						out.write(point+","+error+"\n");
					else
						out.write(point+"\n");
				}catch(ArrayIndexOutOfBoundsException ex){
					out.write(point+"\n");
				}
			}   
			out.close();
		}catch(FileNotFoundException ex){
			//No salvamos el csv
		}catch(IOException ex){
			//No salvamos el csv
		}
	} 
	//****************************************************
	
	/**
	 * Aplica una transformacin al extent dependiendo de si el extent de la imagen 
	 * original ha cambiado o no. Si la capa FLyrGeoRaster cargada tiene un extent
	 * distinto al que se ha salvado en el fichero de puntos esto significa que el 
	 * extent de la miniimagen que tiene los puntos relativos a las coordenadas de la
	 * imagen en pixels ha de sufrir un desplazamiento.
	 * @param imgExtent Extent de la imagen 
	 * @param miniExtent ViewPor del miniextent a transformar
	 * @return ViewPort	ViewPort del miniextent transformado 
	 */
	private ViewPort transformMiniExtent(Rectangle2D imgExtent, ViewPort miniExtent){
		if(	lastViewPort[0] != imgExtent.getMinX() ||
			lastViewPort[1] != imgExtent.getMinY() ||
			lastViewPort[2] != imgExtent.getWidth() ||
			lastViewPort[3] != imgExtent.getHeight()){
			Rectangle2D r = new Rectangle2D.Double();
			r.setRect( 	miniExtent.getExtent().getMinX() - (lastViewPort[0] - imgExtent.getMinX()), 
						miniExtent.getExtent().getMinY() - (lastViewPort[1] - imgExtent.getMinY()),
						miniExtent.getExtent().getWidth(),
						miniExtent.getExtent().getHeight());
			miniExtent.setExtent(r);
		}
		return miniExtent;
	}
	
	/**
	 * Aplica una transformacin al centro del extent dependiendo de si el extent de la imagen 
	 * original ha cambiado o no. 
	 */
	private Point2D transformCenter(Rectangle2D imgExtent, Point2D center){
		if(	lastViewPort[0] != imgExtent.getMinX() ||
			lastViewPort[1] != imgExtent.getMinY() ||
			lastViewPort[2] != imgExtent.getWidth() ||
			lastViewPort[3] != imgExtent.getHeight()){
			Point2D c = new Point2D.Double();
			c.setLocation(center.getX() - (lastViewPort[0] - imgExtent.getMinX()), 
						  center.getY() - (lastViewPort[1] - imgExtent.getMinY()));
			return c;
		}
		return center;
	}
	
	/**
	 * Obtiene el valor del extent de la imagen obtenido desde un nodo
	 * XML. A partir de este valor obtenido crea un punto en la capa.
	 */
	private double[] processFlyrGeoRasterNodeValue(NodeList nl){
		double[] res = null;
		for(int j=0;j<nl.getLength();j++){
			Node geoNode = nl.item(j);
			if(geoNode.getNodeName().equals(RasterMetaFileTags.POINTS_VP)){
				res = new double[6];
				NodeList vpChildNodes = geoNode.getChildNodes();
				for(int i=0;i<vpChildNodes.getLength();i++){
					Node vpNode = vpChildNodes.item(i);
					if(vpNode.getNodeName().equals(RasterMetaFileTags.POINTS_BBOX)){
						NodeList extentChildNodes = vpNode.getChildNodes();
						for(int k=0;k<extentChildNodes.getLength();k++){
							Node extentNode = extentChildNodes.item(k);
							if(extentNode.getNodeName().equals(RasterMetaFileTags.POINTS_PX))
								res[0] = Double.valueOf(extentNode.getFirstChild().getNodeValue()).doubleValue();
							if(extentNode.getNodeName().equals(RasterMetaFileTags.POINTS_PY))
								res[1] = Double.valueOf(extentNode.getFirstChild().getNodeValue()).doubleValue();
							if(extentNode.getNodeName().equals(RasterMetaFileTags.POINTS_W))
								res[2] = Double.valueOf(extentNode.getFirstChild().getNodeValue()).doubleValue();
							if(extentNode.getNodeName().equals(RasterMetaFileTags.POINTS_H))
								res[3] = Double.valueOf(extentNode.getFirstChild().getNodeValue()).doubleValue();
						}
					}
					if(vpNode.getNodeName().equals(RasterMetaFileTags.POINTS_DIM)){
						NodeList dimChildNodes = vpNode.getChildNodes();
						for(int k=0; k<dimChildNodes.getLength();k++){
							Node dimNode = dimChildNodes.item(k);
							if(dimNode.getNodeName().equals(RasterMetaFileTags.WIDTH))
								res[4] = Double.valueOf(dimNode.getFirstChild().getNodeValue()).doubleValue();
							if(dimNode.getNodeName().equals(RasterMetaFileTags.HEIGHT))
								res[5] = Double.valueOf(dimNode.getFirstChild().getNodeValue()).doubleValue();
						}
						lyrPoints.getLyrGeoRaster().setImageDimension(res[4], res[5]);
					}
				}
			}
		}
		return res;
	}
	
	/**
	 * Obtiene el valor de un punto georeferenciado obtenido desde un nodo
	 * XML. A partir de este valor obtenido crea un punto en la capa.
	 */
	private void processGeoPointNodeValue(NodeList nl){
		Point2D leftCenter = null, rightCenter = null;
		ViewPort leftViewPort = null, rightViewPort = null;
		double pX = 0D, pY = 0D, mX = 0D, mY = 0D;
		boolean active = false;
		for(int j=0;j<nl.getLength();j++){
			Node geoNode = nl.item(j);
			if(geoNode.getNodeName().equals(RasterMetaFileTags.GEOPOINT_PX))
				pX = Double.valueOf(geoNode.getFirstChild().getNodeValue()).doubleValue();
			if(geoNode.getNodeName().equals(RasterMetaFileTags.GEOPOINT_PY))
				pY = Double.valueOf(geoNode.getFirstChild().getNodeValue()).doubleValue();
			if(geoNode.getNodeName().equals(RasterMetaFileTags.GEOPOINT_MAPX))
				mX = Double.valueOf(geoNode.getFirstChild().getNodeValue()).doubleValue();
			if(geoNode.getNodeName().equals(RasterMetaFileTags.GEOPOINT_MAPY))
				mY = Double.valueOf(geoNode.getFirstChild().getNodeValue()).doubleValue();
			if(geoNode.getNodeName().equals(RasterMetaFileTags.GEOPOINT_ACTIVE))
				active =  Boolean.valueOf(geoNode.getFirstChild().getNodeValue()).booleanValue();
			if(geoNode.getNodeName().equals(RasterMetaFileTags.GEOPOINT_LCENTER))
				leftCenter = processCenterPoint(geoNode.getChildNodes());
			if(geoNode.getNodeName().equals(RasterMetaFileTags.GEOPOINT_RCENTER))
				rightCenter = processCenterPoint(geoNode.getChildNodes());
			if(geoNode.getNodeName().equals(RasterMetaFileTags.GEOPOINT_LVP))
				leftViewPort = processViewPort(geoNode.getChildNodes());
			if(geoNode.getNodeName().equals(RasterMetaFileTags.GEOPOINT_RVP))
				rightViewPort = processViewPort(geoNode.getChildNodes());
		}
				
		Point2D p = new Point2D.Double();
		p.setLocation(pX, pY);
		Point2D m = new Point2D.Double();
		m.setLocation(mX, mY);
	
		lyrPoints.getPointManager().newEmptyPoint();
		GeoreferencingToolsModule.setEnabled(true);
		View theView = null;
		try{
			theView = (View)PluginServices.getMDIManager().getActiveView();
		}catch(ClassCastException exc){
			return;
		}
		int pos = lyrPoints.getCountPoints() -1;
		try{
			leftViewPort = transformMiniExtent(lyrPoints.getLyrGeoRaster().getFullExtent(), leftViewPort);
			leftCenter = transformCenter(lyrPoints.getLyrGeoRaster().getFullExtent(), leftCenter);
		}catch(DriverException ex){}
		lyrPoints.setPointActive(pos, active);
		lyrPoints.setLeftCenterPoint(pos,leftCenter);
		lyrPoints.setRightCenterPoint(pos, rightCenter);
		lyrPoints.setLeftViewPort(pos, leftViewPort);
		lyrPoints.setRightViewPort(pos, rightViewPort);
		lyrPoints.getPointManager().updateData(pos + 1, p, m, lyrPoints.getPointManager().getDialog(), theView);
		
		if(lyrPoints.getCountPoints() > 0){
			GeoreferencingDialog grd = lyrPoints.getPointManager().getDialog();
			grd.getConectorPanel().getDataPointsTabPanel().
				getSelectPointsPanel().getTableControlerPanel().setNItems(lyrPoints.getCountPoints());
			grd.getConectorPanel().getDataPointsTabPanel().
				getTablePointsPanel().getTableControlerPanel().setNItems(lyrPoints.getCountPoints());
			lyrPoints.getPointManager().selectPoint(0, grd);
		}
		//lyrPoints.addPoint(p, m);
	}
	
	/**
	 * Procesa un nodo correspondiente a un extent
	 * @param nodeList Lista de nodos
	 * @return Rectangle2D
	 */
	private Rectangle2D processExtent(NodeList nodeList){
		Rectangle2D r = new Rectangle2D.Double();
		double x = 0D, y = 0D, width = 0D, height = 0D;
		for(int j=0;j<nodeList.getLength();j++){
			Node node = nodeList.item(j);
			if(node.getNodeName().equals(RasterMetaFileTags.POINTS_PX))
				x = Double.valueOf(node.getFirstChild().getNodeValue()).doubleValue();
			if(node.getNodeName().equals(RasterMetaFileTags.POINTS_PY))
				y = Double.valueOf(node.getFirstChild().getNodeValue()).doubleValue();
			if(node.getNodeName().equals(RasterMetaFileTags.POINTS_W))
				width = Double.valueOf(node.getFirstChild().getNodeValue()).doubleValue();
			if(node.getNodeName().equals(RasterMetaFileTags.POINTS_H))
				height = Double.valueOf(node.getFirstChild().getNodeValue()).doubleValue();
		}
		r.setRect(x, y, width, height);
		return r;
	}
	
	/**
	 * Procesa un nodo correspondiente a la dimension de un viewPort
	 * @param nodeList Lista de nodos
	 * @return Rectangle2D
	 */
	private Dimension processImageSize(NodeList nodeList){
		Dimension d = new Dimension();
		double x = 0D, y = 0D;
		for(int j=0;j<nodeList.getLength();j++){
			Node node = nodeList.item(j);
			if(node.getNodeName().equals(RasterMetaFileTags.WIDTH))
				x = Double.valueOf(node.getFirstChild().getNodeValue()).doubleValue();
			if(node.getNodeName().equals(RasterMetaFileTags.HEIGHT))
				y = Double.valueOf(node.getFirstChild().getNodeValue()).doubleValue();
		}
		d.setSize(x, y);
		return d;
	}
	
	/**
	 * Procesa el nodo correspondiente a un viewport de una mini imagen
	 * @param nodeList Lista de nodos
	 * @return ViewPort obtenido
	 */
	private ViewPort processViewPort(NodeList nodeList){
		ViewPort vp = new ViewPort(null);
		for(int j=0;j<nodeList.getLength();j++){
			Node node = nodeList.item(j);
			if(node.getNodeName().equals(RasterMetaFileTags.POINTS_PROJ))
				vp.setProjection(ProjectionPool.get(node.getFirstChild().getNodeValue()));
			if(node.getNodeName().equals(RasterMetaFileTags.POINTS_BBOX))
				vp.setExtent(processExtent(node.getChildNodes()));
			if(node.getNodeName().equals(RasterMetaFileTags.POINTS_DIM))
				vp.setImageSize(processImageSize(node.getChildNodes()));
		}
		vp.setScale();
		return vp;
	}
	
	/**
	 * Procesa un node correspondiente a un punto central de una
	 * mini imagen
	 * @param nodeList Lista de elementos del nodo centro
	 * @return Punto obtenido
	 */
	private Point2D processCenterPoint(NodeList nodeList){
		Point2D p = new Point2D.Double();
		double x = 0D, y = 0D;
		for(int j=0;j<nodeList.getLength();j++){
			Node node = nodeList.item(j);
			if(node.getNodeName().equals(RasterMetaFileTags.POSX))
				x = Double.valueOf(node.getFirstChild().getNodeValue()).doubleValue();
			if(node.getNodeName().equals(RasterMetaFileTags.POSY))
				y = Double.valueOf(node.getFirstChild().getNodeValue()).doubleValue();
		}
		p.setLocation(x, y);
		return p;
	}

	//****************************************************
	
	/**
	 * Crea un elemento simple xml de la forma <elem>texNode</elem>
	 * aadiendoselo al elemento padre pasado por parmetro
	 * @param parent Elemento padre del XML
	 * @param elem	tag
	 * @param textNode	valor
	 */
	private void createSimpleElement(Element parent, String elem, String textNode){
		Element itemlevel1 = null;
		itemlevel1 = xmlDoc.createElement(elem);
		itemlevel1.appendChild(xmlDoc.createTextNode(textNode));
		parent.appendChild(itemlevel1);    
	}
	
	/**
	 * Crear un bloque que representa un viewport
	 * @param parent Elemento padre del XML
	 * @param elem tag
	 * @param values lista de valores
	 */
	private void createViewPortNode(Element parent, String elem, String[] values){
		Element itemlevel1 = null;
		Element itemlevel2 = null;
		itemlevel1 = xmlDoc.createElement(elem);
		createSimpleElement(itemlevel1, RasterMetaFileTags.POINTS_PROJ, values[6]);
		itemlevel2 = xmlDoc.createElement(RasterMetaFileTags.POINTS_BBOX);
		createSimpleElement(itemlevel2, RasterMetaFileTags.POINTS_PX, values[0]);
		createSimpleElement(itemlevel2, RasterMetaFileTags.POINTS_PY, values[1]);
		createSimpleElement(itemlevel2, RasterMetaFileTags.POINTS_W, values[2]);
		createSimpleElement(itemlevel2, RasterMetaFileTags.POINTS_H, values[3]);
		itemlevel1.appendChild(itemlevel2);
		itemlevel2 = xmlDoc.createElement(RasterMetaFileTags.POINTS_DIM);
		createSimpleElement(itemlevel2, RasterMetaFileTags.WIDTH, values[4]);
		createSimpleElement(itemlevel2, RasterMetaFileTags.HEIGHT, values[5]);
		itemlevel1.appendChild(itemlevel2);		
		parent.appendChild(itemlevel1);
	}
	
	/**
	 * Convierte la capa a georreferenciar a un elemento XML que ser aadidoo al
	 * documento. 
	 * @param lyrGeo Capa a georreferenciar 
	 */
	private void createXMLFLyrGeoRasterNode(FLyrGeoRaster lyrGeo) {
		Element xmlGeoPoint;
		xmlGeoPoint = xmlDoc.createElement(RasterMetaFileTags.POINTS_LYR);
		generalTag.appendChild(xmlGeoPoint);
		xmlGeoPoint.setAttribute("Name", lyrGeo.getName());
		String[] vpData = {String.valueOf(lyrGeo.getMinX()),
							String.valueOf(lyrGeo.getMinY()),
							String.valueOf(lyrGeo.getWidth()),
							String.valueOf(lyrGeo.getHeight()),
							String.valueOf(lyrGeo.getImageWidth()),
							String.valueOf(lyrGeo.getImageHeight()),
							lyrGeo.getProjection().getAbrev()};
		createViewPortNode(xmlGeoPoint, RasterMetaFileTags.POINTS_VP, vpData);
	}
		
	/**
	 * Convierte un geopunto a un elemento XML que ser aadido al 
	 * documento. 
	 * @param geoPoint GeoPoint
	 * @param i Posicin del punto en la lista
	 */
	private void createXMLGeoPointNode(GeoPoint geoPoint, int i) {
		Element xmlGeoPoint;
		Element itemlevel1;

		xmlGeoPoint = xmlDoc.createElement(RasterMetaFileTags.GEOPOINT);
		xmlGeoPoint.setAttribute("n", String.valueOf(i));
		generalTag.appendChild(xmlGeoPoint);
		
		createSimpleElement(xmlGeoPoint, RasterMetaFileTags.GEOPOINT_PX, String.valueOf(geoPoint.pixelPoint.getX()));
		createSimpleElement(xmlGeoPoint, RasterMetaFileTags.GEOPOINT_PY, String.valueOf(geoPoint.pixelPoint.getY()));
		createSimpleElement(xmlGeoPoint, RasterMetaFileTags.GEOPOINT_MAPX, String.valueOf(geoPoint.mapPoint.getX()));
		createSimpleElement(xmlGeoPoint, RasterMetaFileTags.GEOPOINT_MAPY, String.valueOf(geoPoint.mapPoint.getY()));
		createSimpleElement(xmlGeoPoint, RasterMetaFileTags.GEOPOINT_ACTIVE, String.valueOf(geoPoint.active));
				
		itemlevel1 = xmlDoc.createElement(RasterMetaFileTags.GEOPOINT_LCENTER);
		createSimpleElement(itemlevel1, RasterMetaFileTags.POSX, String.valueOf(geoPoint.leftCenterPoint.getX()));
		createSimpleElement(itemlevel1, RasterMetaFileTags.POSY, String.valueOf(geoPoint.leftCenterPoint.getY()));
		xmlGeoPoint.appendChild(itemlevel1);
		
		itemlevel1 = xmlDoc.createElement(RasterMetaFileTags.GEOPOINT_RCENTER);
		createSimpleElement(itemlevel1, RasterMetaFileTags.POSX, String.valueOf(geoPoint.rightCenterPoint.getX()));
		createSimpleElement(itemlevel1, RasterMetaFileTags.POSY, String.valueOf(geoPoint.rightCenterPoint.getY()));
		xmlGeoPoint.appendChild(itemlevel1);
		
		String[] values1 = {String.valueOf(geoPoint.leftViewPort.getExtent().getX()),
							String.valueOf(geoPoint.leftViewPort.getExtent().getY()),
							String.valueOf(geoPoint.leftViewPort.getExtent().getWidth()),
							String.valueOf(geoPoint.leftViewPort.getExtent().getHeight()),
							String.valueOf(geoPoint.leftViewPort.getImageWidth()),
							String.valueOf(geoPoint.leftViewPort.getImageHeight()),
							geoPoint.leftViewPort.getProjection().getAbrev()};
		createViewPortNode(xmlGeoPoint, RasterMetaFileTags.GEOPOINT_LVP, values1);
		
		String[] values2 = {String.valueOf(geoPoint.rightViewPort.getExtent().getX()),
							String.valueOf(geoPoint.rightViewPort.getExtent().getY()),
							String.valueOf(geoPoint.rightViewPort.getExtent().getWidth()),
							String.valueOf(geoPoint.rightViewPort.getExtent().getHeight()),
							String.valueOf(geoPoint.rightViewPort.getImageWidth()),
							String.valueOf(geoPoint.rightViewPort.getImageHeight()),
							geoPoint.rightViewPort.getProjection().getAbrev()};

		createViewPortNode(xmlGeoPoint, RasterMetaFileTags.GEOPOINT_RVP, values2);		
    }
		
	//****************************************************
	
	/**
	 * escribe a disco el texto XML
	 * @param textXML	Texto XML
	 * @param fileName	Nombre del fichero
	 */
	private void XML2File(String textXML, String fileName) {
	    try {
	      OutputStream fout= new FileOutputStream(fileName);
	      OutputStream bout= new BufferedOutputStream(fout);
	      OutputStreamWriter out = new OutputStreamWriter(bout);
	      out.write(textXML);
	      out.flush();
	      out.close();
	      File rmfFile = new File(fileName.substring(0, fileName.lastIndexOf("."))+".rmf");
	      if(rmfFile.exists())
	    	  mergeFiles(fileName);
	      else
	    	 newRmfFile(fileName, rmfFile); 
	    	 
	    }catch (IOException e) {
	      System.out.println(e.getMessage());
	    }
	}
	
	/**
	 * En el caso de que no exista ningn fichero .rmf asociado a la imagen a georreferenciar lo creamos
	 * con la cabecera rmf y los puntos que hemos salvado en el fichero temporal (.grf) 
	 * @param grfStr Nombre del fichero temporal .grf
	 * @param rmf Fichero donde se salvaran los puntos .rmf
	 */
	private void newRmfFile(String grfStr, File rmf){
		try{
			File grf = new File(grfStr);
			BufferedReader inGrf = new BufferedReader(new InputStreamReader(new FileInputStream(grf)));
			BufferedWriter outTmp = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(rmf)));
			
			outTmp.write("<?xml version=\"1.0\" encoding=\""+RasterMetaFileTags.ENCODING+"\"?>\n");
			outTmp.write("<"+RasterMetaFileTags.MAIN_TAG+" "+"xmlns=\""+RasterMetaFileTags.NAMESPACE+"\">\n");
			String str = inGrf.readLine();
			str = inGrf.readLine();
			while(str != null){
				outTmp.write(str+"\n");
		       	str = inGrf.readLine();
		    }
			outTmp.write("</"+RasterMetaFileTags.MAIN_TAG+">\n");
			grf.delete();
			inGrf.close();
			outTmp.close();
		}catch(FileNotFoundException exc){
			
		}catch(IOException exc){
			
		}
	}
	
	/**
	 * En el caso de que ya exista un fichero .rmf asociado a la imagen habr que mezclar este
	 * con los puntos que hemos salvado en el fichero temporal .grf
	 * @param fileName Nombre del fichero temporal
	 */	
	private void mergeFiles(String grfName){
		try{
			File rmfFile = new File(grfName.substring(0, grfName.lastIndexOf("."))+".rmf");
			File grfFile = new File(grfName);
			File out = new File(rmfFile+"_tmpOutput");
			
			BufferedReader inRmf = new BufferedReader(new InputStreamReader(new FileInputStream(rmfFile)));
			BufferedReader inGrf = new BufferedReader(new InputStreamReader(new FileInputStream(grfFile)));
			BufferedWriter outTmp = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(out)));
			
			//Leemos el principio del .rmf
	        String str = inRmf.readLine();
	        while(!str.startsWith("</"+RasterMetaFileTags.LAYER) && 
	        	  !str.startsWith("<"+RasterMetaFileTags.GEOPOINTS)){
	        	outTmp.write(str+"\n");
	        	str = inRmf.readLine();
	        }
	        if(str.startsWith("</"+RasterMetaFileTags.LAYER))
	        	outTmp.write(str+"\n");
	        
	        //Leemos la lista de puntos
	        str = inGrf.readLine();
	        while(str != null){
	        	if(!str.startsWith("<?xml"))
	        		outTmp.write(str+"\n");
	        	str = inGrf.readLine();
	        }
	        	
	        //Saltamos los puntos que ya existian en el .rmf
	        try{
	        	str = inRmf.readLine();
		        while(str != null && 
		        	  !str.startsWith("</"+RasterMetaFileTags.GEOPOINTS) && 
		        	  !str.startsWith("</"+RasterMetaFileTags.MAIN_TAG))
		        	str = inRmf.readLine();
	        }catch(Exception exc){
	        	
	        }
	        
	        //Leemos el resto del fichero .rmf
	        if( !str.startsWith("</"+RasterMetaFileTags.MAIN_TAG))
	        	str = inRmf.readLine();
	        while(str != null){
	        	if(!str.startsWith("<?xml"))
	        		outTmp.write(str+"\n");
	        	str = inRmf.readLine();
	        }
	        
	        outTmp.close();
	        inRmf.close();
	        inGrf.close();
	        
	        //Eliminamos el antiguo .rmf y lo sustituimos
	        rmfFile.delete();
	        out.renameTo(rmfFile);
	        grfFile.delete();
	        
		}catch(FileNotFoundException exc){
			
		}catch(IOException exc){
			
		}
	}
	
	/**
	 * Convierte el objeto Document que contiene el XML construido a 
	 * texto XML para que podamos salvarlo a disco.
	 * @return Cadena de texto XML
	 */
	private String Document2Text() {
	    StringWriter strWriter = null;
	    XMLSerializer xmlSerializer = null;
	    OutputFormat outFormat = null;
	    try {
	    	xmlSerializer = new XMLSerializer();
		    strWriter = new StringWriter();
		    outFormat = new OutputFormat();

		    outFormat.setEncoding(RasterMetaFileTags.ENCODING);
		    outFormat.setIndenting(true);
		    outFormat.setIndent(4);

		    xmlSerializer.setOutputCharStream(strWriter);
		    xmlSerializer.setOutputFormat(outFormat);
		    xmlSerializer.serialize(xmlDoc);
		    strWriter.close();
		   } catch (IOException ioEx) {
		      System.out.println("Error : " + ioEx);
		   }
		    return strWriter.toString();
	}	
	//**********************End Methods***************************
	
	//**********************Getters & Setters*********************
	/**
	 * Asigna la capa de puntos
	 * @param lyrPoints
	 */
	public void setLyrPoints(FLyrPoints lyrPoints) {
		this.geoPointList = lyrPoints.getListPoint();
		this.lyrPoints = lyrPoints;
	}
	//**********************End Getters & Setters*****************
}