/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.gvsig.georeferencing.utils;

import java.awt.geom.Rectangle2D;

import org.cresques.cts.IProjection;
import org.cresques.io.GeoRasterFile;
import org.cresques.px.Extent;

import com.iver.cit.gvsig.fmap.ViewPort;

public class MathUtils {
	public static double INCHESMTR = 39.73007874D;
	public static double INCHESCM = 0.3973007874D;
	public static double INCHESMM = 0.03973007874D;
	public static double MTRSINCH = 0.0254D;
	public static double MMSINCH = 25.4D;
	public static double CMSINCH = 2.54D;
	
	
	/**
     * Recorta el nmero de decimales a n del nmero pasado por parmetro
     * @param num Valor a recortar el nmero de decimales
     * @param n Nmero de decimales a recortar
     * @return Nmero recortado
     */
    public static double tailDecimals(double num, int n){
    	long m = (long)Math.pow(10, n);
        num *= m;
        long aux = ((long)num);
        num = (double)((double)aux / (double)m);
        return num;
    }
    
    /**
     * Recorta el nmero de decimales a n del nmero pasado por parmetro
     * @param num Valor a recortar el nmero de decimales
     * @param n Nmero de decimales a recortar
     * @return Nmero recortado
     */
    public static String tailDecimals(String num, int n){
    	int index = num.indexOf(".");
    	if(index != -1){
    		try{
    			num = num.substring(0, index + n);
    		}catch(IndexOutOfBoundsException e){
    			//No hacemos nada pq retornar num
    		}
    	}
    	return num;
    }
    
	/**
	 * Convierte pixels en milimetros dependiendo del los puntos por pulgada
	 * @param pixels Nmero de pixels
	 * @param ppp Puntos por pulgada
	 * @return Nmero de milimetros
	 */
	public static double convertPixelsToMms(int pixels, int ppp){
		return convertPixelsToInches(pixels, ppp) * MMSINCH;
	}
	
	/**
	 * Convierte pixels en centimetros dependiendo del los metros por pixel
	 * @param pixels Nmero de pixels
	 * @param mtsPixel Metros por pixel
	 * @return Nmero de centimetros
	 */
	public static double convertPixelsToCms(int pixels, int ppp){
		return convertPixelsToInches(pixels, ppp) * CMSINCH;
	}
	
	/**
	 * Convierte pixels en metros dependiendo del los metros por pixel
	 * @param pixels Nmero de pixels
	 * @param mtsPixel Metros por pixel
	 * @return Nmero de metros
	 */
	public static double convertPixelsToMts(int pixels, int ppp){
		return convertPixelsToInches(pixels, ppp) * MTRSINCH;
	}
	
	/**
	 * Convierte pixels en pulgadas dependiendo del los metros por pixel
	 * @param pixels Nmero de pixels
	 * @param mtsPixel Metros por pixel
	 * @return Nmero de pulgadas
	 */
	public static double convertPixelsToInches(int pixels, int ppp){
		return (double)((double)pixels / (double)ppp);
	}
	
	/**
	 * Convierte metros en pixels dependiendo de los puntos por pulgada
	 * @param mts Nmero de metros
	 * @param ppp Puntos por pulgada
	 * @return Nmero de pixeles
	 */
	public static int convertMtsToPixels(double mts, int ppp){
		return (int)(mts * MathUtils.INCHESMTR * ppp);
	}
	
	/**
	 * Convierte pulgadas en pixels dependiendo de los puntos por pulgada
	 * @param mts Nmero de pulgadas
	 * @param ppp Puntos por pulgada
	 * @return Nmero de pixeles
	 */
	public static int convertInchesToPixels(double inches, int ppp){
		return (int)(inches * ppp);
	}
	
	/**
	 * Convierte centimetros en pixels dependiendo de los puntos por pulgada
	 * @param mts Nmero de centmetros
	 * @param ppp Puntos por pulgada
	 * @return Nmero de pixeles
	 */
	public static int convertCmsToPixels(double cms, int ppp){
		return (int)(cms * MathUtils.INCHESCM * ppp);
	}
	
	/**
	 * Convierte milimetros en pixels dependiendo de los puntos por pulgada
	 * @param mts Nmero de milimetros
	 * @param ppp Puntos por pulgada
	 * @return Nmero de pixeles
	 */
	public static int convertMmsToPixels(double mms, int ppp){
		return (int)(mms * MathUtils.INCHESMM * ppp);
	}
	
	/**
	 * Devuelve la traspuesta de una matriz
	 * @param M Matriz
	 * @param m Nmero de filas
	 * @param n Nmero de columnas
	 * @return Matriz traspuesta
	 */
	public static double[][] transpose(double[][] M, int m, int n){
	  double[][]	a = new double[n][m];
	  int			i, j;
	
	  for (i = 0; i < n; i++)
	    for (j = 0; j < m; j++)
	      a[i][j] = M[j][i];
	  return a;
	}
	
	/**
	 * Producto de matrices (AxB)
	 * @param a	Matriz A
	 * @param b Matriz B
	 * @param M 
	 * @param N	
	 * @param O 
	 * @return
	 */
	public static double[][] multmatrix(double[][] a, double[][] b, int M, int N, int O){
	  double[][]	c = new double[M][O]; 
	  double		sum;
	  int			i, j, k;
	  
	  for (i = 0; i < M; i++) {
	    for (j = 0; j < O; j++) {
	      sum = 0;
	      for (k = 0; k < N; k++)
	        sum += b[k][j] * a[i][k];
	      c[i][j] = sum;
	    }
	  }
	  return (c);
	}
}
