package org.gvsig.georeferencing.utils;

import java.awt.Component;
import java.awt.geom.Point2D;

import javax.swing.JDesktopPane;
import javax.swing.JInternalFrame;
import javax.swing.JLayeredPane;
import javax.swing.JOptionPane;

import org.gvsig.georeferencing.GeoOperations;
import org.gvsig.georeferencing.GeoreferencingToolsModule;
import org.gvsig.georeferencing.gui.dialogs.GeoreferencingDialog;
import org.gvsig.georeferencing.gui.listeners.GeoRasterFrameListener;
import org.gvsig.georeferencing.gui.panels.AdjustGeorefPanel;
import org.gvsig.georeferencing.gui.panels.OptionsPanel;
import org.gvsig.georeferencing.gui.panels.ZoomControlPanel;
import org.gvsig.georeferencing.gui.pointsTable.ButtonsExportImportPanel;
import org.gvsig.georeferencing.gui.pointsTable.TableControlerPanel;
import org.gvsig.georeferencing.gui.pointsTable.TablePointsPanel;
import org.gvsig.georeferencing.gui.pointsTable.TablePointsPanel.PointTable;
import org.gvsig.georeferencing.gui.selectPoints.DataPointPanel;
import org.gvsig.georeferencing.gui.selectPoints.ErrorPointPanel;
import org.gvsig.georeferencing.gui.selectPoints.SelectPointsPanel;

import com.iver.andami.PluginServices;
import com.iver.andami.ui.mdiFrame.MDIFrame;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.layers.FLyrGeoRaster;
import com.iver.cit.gvsig.fmap.layers.FLyrPoints;
import com.iver.cit.gvsig.fmap.layers.GeoPoint;
import com.iver.cit.gvsig.fmap.layers.IGeoUi;
import com.iver.cit.gvsig.gui.View;

/**
 * Gestor de puntos. Contiene metodos para aadir. borrar, seleccionar,..
 * puntos coordinando la capa de puntos y la visualizacin del interfaz
 * grfico.
 * 
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */
public class PointManager{

	//**********************Vars**********************************
	private	GeoreferencingDialog 		dialog = null;
	
	private SelectPointsPanel 			selectPointsPanel = null;
	private ErrorPointPanel 			errorPointPanel = null;
	private DataPointPanel 				dataPointPanel = null;
	private TableControlerPanel 		tableControlerPanel1 = null;
	
	private TablePointsPanel 			tablePointsPanel = null;
	private TableControlerPanel 		tableControlerPanel2 = null;
	private ButtonsExportImportPanel	buttonsExportImportPanel = null;
	
	private OptionsPanel				optionsPanel = null;
	private ZoomControlPanel			zoomControlPanelLeft = null;
	private ZoomControlPanel			zoomControlPanelRight = null;
	
	private FLyrPoints	 				lyrPoints = null;
	private	double						currentMapX = 0D;
	private	double						currentMapY = 0D;
	private	double						currentPixelX = 0D;
	private	double						currentPixelY = 0D;
	private boolean 					disableAllControls = false;
	//**********************End Vars******************************
	
	//**********************Methods*******************************
	/**
	 * Constructor
	 */
	public PointManager(GeoreferencingDialog dialog, FLyrPoints flyrPoints){
		this.dialog = dialog;
		this.selectPointsPanel = dialog.getSelectPointsPanel();
		this.errorPointPanel = dialog.getSelectPointsPanel().getErrorPointPanel();
		this.dataPointPanel = dialog.getSelectPointsPanel().getDataPointPanel();
		this.tableControlerPanel1 = dialog.getSelectPointsPanel().getTableControlerPanel();
		this.tablePointsPanel = dialog.getConectorPanel().getDataPointsTabPanel().getTablePointsPanel();
		this.tableControlerPanel2 = dialog.getConectorPanel().getDataPointsTabPanel().getTablePointsPanel().getTableControlerPanel();
		this.buttonsExportImportPanel = dialog.getConectorPanel().getDataPointsTabPanel().getTablePointsPanel().getPButtonsExportImportPanel();
		this.optionsPanel = dialog.getConectorPanel().getAdjustGeorefPanel().getOptionsPanel();
		this.zoomControlPanelLeft = dialog.getConectorPanel().getAdjustGeorefPanel().getZoomLeft();
		this.zoomControlPanelRight = dialog.getConectorPanel().getAdjustGeorefPanel().getZoomRight();
		this.lyrPoints = flyrPoints;
	}
	
	/**
	 * Crea un nuevo punto vacio en la capa y lo aade al interfaz grfico para
	 * que puedan ser introducidas las coordenadas por el usuario
	 */
	public void newEmptyPoint(){
		try{
			//Antes de aadir un nuevo pto compruba que los textField del anterior no estn vacios
			if(tableControlerPanel1.getItemCount() > 0)
				selectPointsPanel.checkInfoPointText();
			
			//Aade un punto vacio a la capa
			lyrPoints.addPoint(null, null);
			
			//Aade un punto al control de tabla y checkea la activacin de los botones
			tableControlerPanel1.addPointToTable(lyrPoints.getCountPoints());
							
			//Guardamos el ltimo punto seleccionado por si hay que recuperarlo
			selectPointsPanel.setLastIndexSelection(lyrPoints.getCountPoints() - 1);
									
			//resetea los controles de (del, delall, selectFromView y miniimagenes)
			dialog.resetControls(true);
			
			//Desactivamos los controles de las miniimagenes para que no capture los
			//eventos hasta que no se hayan introducido los puntos
			dialog.getConectorPanel().getAdjustGeorefPanel().setEnabled(false);
			dialog.getConectorPanel().getAdjustGeorefPanel().setLZoomRight("1:");
			dialog.getConectorPanel().getAdjustGeorefPanel().setLZoomLeft("1:");
									
		}catch(NoSuchFieldException exc){
			JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
				PluginServices.getText(this, "coordenadas_vacias"));
		}
	}
	
	/**
	 * A partir de nuevas coordenadas actualiza la vista, minimagen, capa de puntos el
	 * dialogo y la tabla.
	 *
	 */
	public void updateData(int nPunto, Point2D pixel, Point2D map, IGeoUi geoUI, View view){
		dialog = (GeoreferencingDialog)geoUI;
		if(dialog == null)
			return;
		if(pixel == null && map == null)
			return;
		
		//Actualizamos los textBox
		if(pixel != null){
			dialog.setPixelText(pixel.getX(), pixel.getY());
			dialog.getSelectPointsPanel().lastTx = String.valueOf(pixel.getX());
			dialog.getSelectPointsPanel().lastTy = String.valueOf(pixel.getY());
		}
		if(map != null){
			dialog.setMapCoordText(map.getX(), map.getY());
			dialog.getSelectPointsPanel().lastLong = String.valueOf(map.getX());
			dialog.getSelectPointsPanel().lastLat = String.valueOf(map.getY());
		}
		
		//El Checkbox de activacin
		GeoPoint gp = ((GeoPoint)lyrPoints.getPoint(nPunto - 1));
		if(gp != null){
			dataPointPanel.getCbActive().setSelected(gp.active);
			dialog.setEnabled(gp.active);
		}
    	
		//Actualizamos la tabla
		
		while(dialog.getTableRowCount() < nPunto)
			dialog.addTableNew();
		dialog.setTableValueAt(String.valueOf(nPunto), nPunto - 1, 0);
		
		if(pixel != null){	
			dialog.setTableValueAt(MathUtils.tailDecimals(String.valueOf(pixel.getX()), 5), nPunto - 1, 1);
			dialog.setTableValueAt(MathUtils.tailDecimals(String.valueOf(pixel.getY()), 5), nPunto - 1, 2);
		}
		if(map != null){
			dialog.setTableValueAt(MathUtils.tailDecimals(String.valueOf(map.getX()), 5), nPunto - 1, 3);
			dialog.setTableValueAt(MathUtils.tailDecimals(String.valueOf(map.getY()), 5), nPunto - 1, 4);
		}
		
		//Actualizamos la capa de puntos
		if(pixel != null || map != null)
			lyrPoints.updatePoint(pixel, map, nPunto - 1);
		
		//Actualizamos la miniimagen
		if(view != null){
			ViewPort viewPort = view.getMapControl().getMapContext().getViewPort();

			if(pixel != null){
				dialog.getZoomControlLeft().setCenterPoint(lyrPoints.getLyrGeoRaster().img2World(pixel));
				dialog.getConectorPanel().getAdjustGeorefPanel().setViewPort(viewPort);
				//Si est extendido hacemos un draw corriente sino es q est plegado y tocar asignar el extent de la miniimagen
				if(dialog.isEnlarge())
					dialog.getConectorPanel().getAdjustGeorefPanel().getZoomLeft().draw();
				else
					lyrPoints.setMiniExtent(nPunto - 1, 
											lyrPoints.getLyrGeoRaster().img2World(pixel), 
											dialog.getConectorPanel().getAdjustGeorefPanel().getZoomLeft().getCanvas().initViewPort(viewPort, lyrPoints.getLyrGeoRaster().img2World(pixel), null),
											false);
			}
			
			if(map != null){
				dialog.getZoomControlRight().setCenterPoint(map);
				dialog.getConectorPanel().getAdjustGeorefPanel().setViewPort(viewPort);
				//Si est extendido hacemos un draw corriente sino es q est plegado y tocar asignar el extent de la miniimagen
				if(dialog.isEnlarge())
					dialog.getConectorPanel().getAdjustGeorefPanel().getZoomRight().draw();
				else
					lyrPoints.setMiniExtent(nPunto - 1, 
											map, 
											dialog.getConectorPanel().getAdjustGeorefPanel().getZoomRight().getCanvas().initViewPort(viewPort, map, null),
											true);
			}
									
			//Actualizamos la vista
			view.getMapControl().getMapContext().invalidate();
		}
		
		//Calculamos los errores
    	if(pixel != null && map != null)
			setErrors(nPunto - 1);
    	
	}
	
	/**
	 * Al actualizar un punto debemos recalcular los errores RMS y Total para ello tenemos que
	 * pedir la transformacin de toda la lista de puntos. Esta funcin calcula esta lista
	 * transformada y la asigna a un array en la capa de puntos. Esta funcin tiene en cuenta
	 * que un punto puede estar inactivo.
	 */
	private void transformPointList(){
		try{
			GeoOperations go = new GeoOperations(lyrPoints);
			Point2D[] pointList = new Point2D[lyrPoints.getCountActivePoints()];
			int pActive = 0;
			for(int i=0; i< lyrPoints.getCountPoints(); i++){
				if(lyrPoints.getPoint(i).active){
					pointList[pActive] = lyrPoints.getPoint(i).pixelPoint;
					pActive ++;
				}
			}
			
			Point2D[] transformActivePointList = go.transformPoints(pointList);
			Point2D[] transformPointList = new Point2D[lyrPoints.getCountPoints()];
			
			pActive = 0;
			for(int i=0; i< lyrPoints.getCountPoints(); i++){
				if(lyrPoints.getPoint(i).active){
					transformPointList[i] = transformActivePointList[pActive];
					pActive ++;
				}else
					transformPointList[i] = null;
			}
			
			lyrPoints.setTransformPointList(transformPointList);
		}catch(NullPointerException e){
			//Si ocurre esta excepcin es que se est intentando incluir un punto que se acaba de crear
			//y por lo tanto no tiene el valor asignado todavia por lo que es mejor no hacer nada 
			//y seguir con el proceso
		}
	}
	
	/**
	 * Busca el punto en la capa FLyrPoint que corresponde con la posicion pos
	 * y carga la informacin de este en el panel de informacin pasado por parmetro.
	 * @param pos Posicin del punto
	 * @param panel donde se carga la informacin
	 */
	public void selectPoint(int pos, GeoreferencingDialog dialog){
		GeoPoint pto = null;
		try{
			pto = lyrPoints.getPoint(pos);
			if(pto == null)
				return;
		}catch(ArrayIndexOutOfBoundsException exc){
			//Todavia no hay puntos por lo que no hacemos nada
			return;
		}
		DataPointPanel dpp = dialog.getConectorPanel().getDataPointsTabPanel().getSelectPointsPanel().getDataPointPanel();
		
		//Llenamos los JTextField con los valores del punto
		if(pto.pixelPoint != null){
			dpp.getTX().setText(""+pto.pixelPoint.getX());
			dpp.getTY().setText(""+pto.pixelPoint.getY());
    	}
    	if(pto.mapPoint != null){
    		dpp.getLatitud().setText(MathUtils.tailDecimals(String.valueOf(pto.mapPoint.getY()), 5));
    		dpp.getLongitud().setText(MathUtils.tailDecimals(String.valueOf(pto.mapPoint.getX()), 5));
    	}
    	
    	//El Checkbox de activacin
    	dataPointPanel.getCbActive().setSelected(pto.active);
    	dialog.setEnabled(pto.active);
    	    	
    	//Seleccionamos la fila de la tabla
    	PointTable table = dialog.getConectorPanel().getDataPointsTabPanel().getTablePointsPanel().getJTable();
    	try{
    		table.getTable().setRowSelectionInterval(pos, pos);
    	}catch(IllegalArgumentException exc){
    		System.err.println("Intervalo erroneo "+pos);
    	}
    	
    	//Cargamos las mini imagenes
    	dialog.getZoomControlLeft().setCenterPoint(lyrPoints.getCenterPoint(pos, false));
    	dialog.getZoomControlLeft().setNewViewPort(lyrPoints.getMiniExtent(pos, false));
    	dialog.getZoomControlLeft().draw();
    	    	
    	dialog.getZoomControlRight().setCenterPoint(lyrPoints.getCenterPoint(pos, true));
    	dialog.getZoomControlRight().setNewViewPort(lyrPoints.getMiniExtent(pos, true));
    	dialog.getZoomControlRight().draw();
    	    	
    	//Calculamos los errores
    	if(pto.pixelPoint != null && pto.mapPoint != null)
	    	setErrors(pos);
	}
	
	/**
	 * Obtiene los errores en una matriz de Nx3 en la que cada fila es un punto y la
	 * columna 0 es el error en X, la columna 1 es el error en Y y la columna 2 es el error RMS. 
	 * @return matriz de errores
	 */
	public double[][] getErrors(){
		
		transformPointList();
		
		//Al actualizar un punto debemos recalcular los errores RMS y Total para ello tenemos que
		//pedir la transformacin de toda la lista de puntos
		try{
			GeoOperations go = new GeoOperations(lyrPoints);
			Point2D[] pointList = new Point2D[lyrPoints.length()]; 
			for(int i=0; i< lyrPoints.length(); i++)
				pointList[i] = lyrPoints.getPoint(i).pixelPoint;
			
			Point2D[] transformPointList = go.transformPoints(pointList);
			lyrPoints.setTransformPointList(transformPointList);
		}catch(NullPointerException e){
			//Si ocurre esta excepcin es que se est intentando incluir un punto que se acaba de crear
			//y por lo tanto no tiene el valor asignado todavia por lo que es mejor no hacer nada 
			//y seguir con el proceso
		}
		
		double[][] err =  new double[lyrPoints.getCountPoints()][3];
		for(int i = 0; i<lyrPoints.getCountPoints(); i++){
			err[i][0] = lyrPoints.calcDoubleResX(i);
			err[i][1] = lyrPoints.calcDoubleResY(i);
			err[i][2] = lyrPoints.calcDoubleRMS(i);
		}
		return err;
	}
	
	/**
	 * Calcula los errores y los asigna al dialogo y tabla. Hay que tener en cuenta que 
	 * hay que tener en la capa de puntos la lista de puntos transformados
	 * @param pos
	 */
	public void setErrors(int pos){
		
		if(lyrPoints != null){
			transformPointList();
			
			//Calculamos los errores
			double d = lyrPoints.calcDoubleRMS(pos);
			//System.out.println("-->RMS:"+d);
	    	dialog.setRMS(String.valueOf(MathUtils.tailDecimals(d, 5)));
	    	try{
	    		dialog.setTableValueAt(String.valueOf(MathUtils.tailDecimals(d, 5)), pos, 7);
	    	}catch(ArrayIndexOutOfBoundsException e){
	    		//No asigna el valor a la tabla
	    	}
	    	
	    	d = lyrPoints.calcDoubleResX(pos);
	    	//System.out.println("-->ResX:"+d);
	    	dialog.setResX(String.valueOf(MathUtils.tailDecimals(d, 5)));
	    	try{
	    		dialog.setTableValueAt(String.valueOf(MathUtils.tailDecimals(d, 5)), pos, 5);
	    	}catch(ArrayIndexOutOfBoundsException e){
	    		//No asigna el valor a la tabla
	    	}
	    	
	    	d = lyrPoints.calcDoubleResY(pos);
	    	//System.out.println("-->ResY:"+d);
	    	dialog.setResY(String.valueOf(MathUtils.tailDecimals(d, 5)));
	    	try{
	    		dialog.setTableValueAt(String.valueOf(MathUtils.tailDecimals(d, 5)), pos, 6);
	    	}catch(ArrayIndexOutOfBoundsException e){
	    		//No asigna el valor a la tabla
	    	}
	    	
	    	d = lyrPoints.calcDoubleTotal();
	    	//System.out.println("-->Total:"+d);
	    	dialog.setTotal(String.valueOf(MathUtils.tailDecimals(d, 5)));
		}
    	
	}
	
	/**
	 * Elimina el punto seleccionado de la lista de puntos. Obtiene el punto seleccionado
	 * del combobox del tableControler
	 */
	public void deletePoint(){
		int op = JOptionPane.showConfirmDialog((Component)PluginServices.getMainFrame(),
				PluginServices.getText(this, "eliminar_punto"));
		if(op == JOptionPane.OK_OPTION){
			if(lyrPoints.length() == 1) //Si solo hay un punto reseteamos
				clearVisualList();
			else{
				int posSelect = tableControlerPanel1.getSelectedIndex();
				if(posSelect == -1)
					return;
				deleteVisualPoint(posSelect);	
											
				if(posSelect == 0){
					selectPointsPanel.setLastIndexSelection(posSelect);
				}else{
					selectPointsPanel.setLastIndexSelection(posSelect - 1);
				}					
			}
		}
	}
	
	/**
	 * Elimina todos los puntos de la capa y resetea el dialogo
	 */
	public void deleteAllPoints(){
		int op = JOptionPane.showConfirmDialog((Component)PluginServices.getMainFrame(),
				PluginServices.getText(this, "eliminar_todos_puntos"));
		if(op == JOptionPane.OK_OPTION){
			clearVisualList();
		}
			
	}
	
	/**
	 * Elimina un punto de la lista pasado por parmetro y lo elimina del panel 
	 * @param pos	Posicin del punto
	 * @param grd	Panel a limpiar
	 */
	private void deleteVisualPoint(int pos){
		lyrPoints.remove(pos);
		
		//Actualiza los controles de tabla y selector
		tableControlerPanel1.setNItems(lyrPoints.length());
		tableControlerPanel2.setNItems(lyrPoints.length());
		
		//Elimina la fila de la tabla y renumera los puntos
		try{
			dialog.removeTableValueAt(pos);
		
			for(int i=0;i<lyrPoints.length();i++)
				dialog.setTableValueAt(String.valueOf(i + 1), i, 0);
		}catch(ArrayIndexOutOfBoundsException exc){
			//Si intentamos eliminar una fila que no existe no hacemos nada, simplemente no la eliminamos
		}
				
		//Carga los puntos
		if(pos == 0){
			dialog.getSelectPointsPanel().getTableControlerPanel().setSelectedIndex(pos);
			selectPoint(pos, dialog);
		}else{
			dialog.getConectorPanel().getDataPointsTabPanel().getTablePointsPanel().getTableControlerPanel().setSelectedIndex(pos - 1);
			selectPoint(pos - 1, dialog);
		}
		
		//Redibujamos la vista
		View theView = null;
		try{
			theView = (View) PluginServices.getMDIManager().getActiveView();
			theView.getMapControl().getMapContext().invalidate();
		}catch(ClassCastException exc){
			return;
		}
	}
	
	/**
	 * Limpia la lista de puntos .
	 */
	public void clearVisualList(){
		AdjustGeorefPanel agp = dialog.getConectorPanel().getAdjustGeorefPanel();
		
		//Limpiamos la lista de puntos
		lyrPoints.clear();
		//Limpiamos los controles del Selector de puntos
		selectPointsPanel.resetControls(false);
		selectPointsPanel.getTableControlerPanel().resetControls();
		
		//Limpiamos la tabla y sus controles
		tablePointsPanel.clearTable();
		tablePointsPanel.getTableControlerPanel().resetControls();
				
		//Limpiamos las miniimagenes bloqueamos los controles
		agp.getZoomLeft().clear();
		agp.getZoomRight().clear();
		dialog.getConectorPanel().getAdjustGeorefPanel().setEnabled(false);
		
		//Actualizamos la vista
		View theView = null;
		try{
			theView = (View) PluginServices.getMDIManager().getActiveView();
			theView.getMapControl().getMapContext().invalidate();
		}catch(ClassCastException exc){
			return;
		}
	
	}
	
	/**
	 * Obtiene la ventana flotante sobre la que ir el dialogo (GeoreferencingDialog).
	 * @param obj
	 * @return JInternalFrame
	 */
	public static JInternalFrame getInternalFrame(Object obj){
		JInternalFrame panel = new JInternalFrame();
		panel.addInternalFrameListener(new GeoRasterFrameListener());
        panel.setClosable(true);
        panel.setResizable(false);
        panel.setTitle(PluginServices.getText(obj,"georreferenciar"));
        return panel;
	}
	
	
	public static GeoreferencingDialog initWindow(FLyrGeoRaster lyrGeoRaster, boolean show, Object obj){
		GeoreferencingDialog geoDialog = null;
		
		JInternalFrame panel = PointManager.getInternalFrame(obj);
        geoDialog = new GeoreferencingDialog(panel, lyrGeoRaster);	     
		lyrGeoRaster.getStackZoom().setInitExtent(lyrGeoRaster.getAssignExtent());
    	lyrGeoRaster.getFLyrPoints().saveState();
        lyrGeoRaster.setGeoDialog(geoDialog);
        panel.setSize(geoDialog.getWidth() + 12, geoDialog.getHeight() + 10);
        panel.getContentPane().add(geoDialog);
       
        MDIFrame mainFrame = (MDIFrame) PluginServices.getMainFrame();
        
        JDesktopPane desk = new JDesktopPane();
        desk.setDragMode(JDesktopPane.OUTLINE_DRAG_MODE);
              
        JLayeredPane lyrPane = mainFrame.getLayeredPane();
        lyrPane.add(panel, JDesktopPane.PALETTE_LAYER);    
        lyrPane.add(desk, JDesktopPane.OUTLINE_DRAG_MODE);
        if(show)
        	panel.show();
        
        return geoDialog;
	}
			
	/**
	 * Esta funcin deshabilita todos los controles y guarda sus valores
	 * de habilitado o deshabilitado para que cuando se ejecute restoreControlsValue
	 * se vuelvan a quedar como estaba
	 */
	public void disableAllControls(){
		if(!disableAllControls){
			disableAllControls = true;
			//Salvamos los estados y desactivamos controles 
			tableControlerPanel1.disableAllControls();
			tablePointsPanel.disableAllControls();
			tableControlerPanel2.disableAllControls();
			dialog.disableAllControls();
			dataPointPanel.getCbActive().setEnabled(false);
			selectPointsPanel.getBExtended().setEnabled(false);
			dialog.getFrame().setClosable(false);
			
			//Desactivamos las herramientas de georreferenciacin
			GeoreferencingToolsModule.setEnabled(false);
			dialog.getConectorPanel().getDataPointsTabPanel().getTbPoints().setEnabled(false);
		}
	}
	
	/**
	 * Esta funcin deja los controles como estaban al ejecutar la funcin 
	 * disableAllControls
	 */
	public void restoreControlsValue(){
		if(disableAllControls){
			disableAllControls = false;
			//Activamos controles
			tableControlerPanel1.restoreControlsValue();
			tablePointsPanel.restoreControlsValue();
			tableControlerPanel2.restoreControlsValue();
			dialog.restoreControlsValue();
			dataPointPanel.getCbActive().setEnabled(true);
			selectPointsPanel.getBExtended().setEnabled(true);
			dialog.getFrame().setClosable(true);
			
			//Activamos las herramientas de georreferenciacin
			GeoreferencingToolsModule.setEnabled(true);
			dialog.getConectorPanel().getDataPointsTabPanel().getTbPoints().setEnabled(true);
		}
	}
	//**********************End Methods***************************

	//**********************Getters & Setters*********************
	public FLyrPoints getFlyrPoints() {
		return lyrPoints;
	}

	public void setFlyrPoints(FLyrPoints flyrPoints) {
		this.lyrPoints = flyrPoints;
	}
	
	public GeoreferencingDialog getDialog() {
		return dialog;
	}
	
	/**
	 * Salva en 2 variables de instancia el valor actual en coordenadas del 
	 * mundo del punto seleccionado. Esto es util para su posterior restauracin.
	 */
	public void setCurrentWC() {
		int posSelect = 0;
		try{
			if(tableControlerPanel1 != null)
				posSelect = tableControlerPanel1.getSelectedIndex();
			if(lyrPoints != null){
				this.currentMapX = lyrPoints.getPoint(posSelect).mapPoint.getX();
				this.currentMapY = lyrPoints.getPoint(posSelect).mapPoint.getY();
			}
		}catch(NullPointerException exc){
			//Si tableControler o la capa de puntos son null no salvamos el punto
		}
	}
	
	/**
	 * Asigna nuevos valores al texto de la latitud y longitud.
	 * @param x	Longitud
	 * @param y Latitud
	 */
	public void setTempWC(double x, double y){
		dataPointPanel.getLongitud().setText(MathUtils.tailDecimals(String.valueOf(x), 5) + "");
		dataPointPanel.getLatitud().setText(MathUtils.tailDecimals(String.valueOf(y), 5) + "");
	}
	
	/**
	 * Restaura el ltimo valor seleccionado por el usuario de la latitud
	 * y longitud.
	 */
	public void replaceWC(){
		dataPointPanel.getLongitud().setText(MathUtils.tailDecimals(String.valueOf(currentMapX), 5) + "");
		dataPointPanel.getLatitud().setText(MathUtils.tailDecimals(String.valueOf(currentMapY), 5) + "");
	}
	
	/**
	 * Salva en 2 variables de instancia el valor actual en coordenadas del 
	 * mundo del punto seleccionado. Esto es util para su posterior restauracin.
	 */
	public void setCurrentPixelCoord() {
		int posSelect = 0;
		try{
			if(tableControlerPanel1 != null)
				posSelect = tableControlerPanel1.getSelectedIndex();
			if(lyrPoints != null){
				this.currentPixelX = lyrPoints.getPoint(posSelect).pixelPoint.getX();
				this.currentPixelY = lyrPoints.getPoint(posSelect).pixelPoint.getY();
			}
		}catch(NullPointerException exc){
			//Si tableControler o la capa de puntos son null no salvamos el punto
		}
	}
	
	/**
	 * Asigna nuevos valores al texto de la latitud y longitud.
	 * @param x	Longitud
	 * @param y Latitud
	 */
	public void setTempPixelCoord(double x, double y){
		dataPointPanel.getTX().setText(MathUtils.tailDecimals(String.valueOf(x), 5) + "");
		dataPointPanel.getTY().setText(MathUtils.tailDecimals(String.valueOf(y), 5) + "");
	}
	
	/**
	 * Restaura el ltimo valor seleccionado por el usuario de la latitud
	 * y longitud.
	 */
	public void replacePixelCoord(){
		dataPointPanel.getTX().setText(currentPixelX + "");
		dataPointPanel.getTY().setText(currentPixelY + "");
	}
	
	/**
	 * Consulta si se han desactivado los controles salvando sus valores
	 * @return true si se han desactivado los controles salvando previamente
	 * sus valores y false si estn en estado normal.
	 */
	public boolean isDisableAllControls() {
		return disableAllControls;
	}
	//**********************End Getters & Setters*****************
}