package org.gvsig.rasterTools.histogram.filter;

import java.awt.Image;
import java.awt.image.BufferedImage;

import org.cresques.io.raster.RasterBuf;
/**
 * Filtro para crear histogramas de brillo para cada una de las bandas de la imagen.
 * Toma como entrada la imagen y da como salida un histograma para cada una de las
 * bandas representado mediante una matriz.
 * @author Miguel ngel Querol Carratal  <querol_mig@gva.es>
 *
 */

public class HistogramImageFilter extends HistogramFilter {

	
	/**
	 * Matriz donde se van a almacenar los histogramas.
	 */
	int histoRed[];
	int histoGreen[];
	int histoBlue[];
	
	/**
	 * Constructor
	 *
	 */
	
	
	public HistogramImageFilter(){
		super();
	}
	
	public void pre(){
		this.image = (Image) params.get("raster");
		this.imgWidth = image.getWidth(null);
		this.imgHeight = image.getHeight(null);
		
		super.pre();
	}
	
	
	public void post() {
		// TODO Auto-generated method stub

	}

	public void process(int x, int y) {
		int px = ((BufferedImage) image).getRGB(x, y);
		int px3[] = {(px & 0xff0000) >> 16, (px & 0xff00) >> 8, (px & 0xff)};
		
		this.histoRed[px3[0]]++;  		//Histograma para la banda R
		this.histoGreen[px3[1]]++;  	//Histograma para la banda G
		this.histoBlue[px3[2]]++;  		//Histograma para la banda B
		

	}

	public void processLine(int y) {
		// TODO Auto-generated method stub

	}

	public int getInRasterDataType() {
		return RasterBuf.TYPE_IMAGE;
	}

	public int getOutRasterDataType() {
		return RasterBuf.TYPE_IMAGE;
	}

	public Object getResult(String name) {
		if (name.equals("raster")) {
            return (Object) this.image;
        } else {
            return null;
        }
	}
	
	
	/**
	 * Metodos para acceder desde fuera a los histogramas calculados por el filtro
	 * @return Histogramas de cada una de las bandas de color.
	 */
	public Object getRedHistogram(){
		return (Object) this.histoRed;
	}
	
	public Object getGreenHistogram(){
		return (Object) this.histoGreen;
	}
	
	public Object getBlueHistogram(){
		return (Object) this.histoBlue;
	}

}
