package org.gvsig.rasterTools.histogram.filter;

import java.util.ArrayList;

import org.cresques.io.GeoRasterFile;
import org.cresques.io.raster.IStackManager;
import org.cresques.io.raster.RasterBuf;
import org.cresques.io.raster.RasterFilter;
import org.cresques.io.raster.RasterFilterStack;
import org.cresques.io.raster.RasterFilterStackManager;

/**
 * Gestor de la pila de filtros para el filtro de histogramas.
 * @author Miguel ngel Querol Carratal  <querol_mig@gva.es>
 *
 */


public class HistogramStackManager implements IStackManager{
	
	public final static int				histogram = 6;
	protected RasterFilterStack 		filterStack = null;
	private RasterFilterStackManager	filterStackManager = null;
	
	public HistogramStackManager(RasterFilterStackManager filterStackManager){
		this.filterStackManager = filterStackManager;
		this.filterStack = filterStackManager.getFilterStack();
		filterStackManager.addTypeFilter("histogram", HistogramStackManager.histogram, 6);
	}
	
	/**
	 * Obtiene el valor entero que corresponde al tipo de filtro.
	 * @param rasterFilter
	 * @return
	 */
	public int getType(RasterFilter rasterFilter){
		if(rasterFilter instanceof HistogramFilter)
			return HistogramStackManager.histogram;
					
		return filterStackManager.getType(rasterFilter);
	}
	
	/**
	 * Aade un filtro de histograma a la pila de filtros
	 *
	 */
	public void addHistogramFilter(){
		
		RasterFilter filtro =  null;
		
		//Si ya hay un filtro de histrograma en la pila lo sobreescribimos
		
		if(filterStack.isActive(((Integer) filterStackManager.getTypeFilters().get("histogram")).intValue()))
			filterStack.removeFilter(((Integer) filterStackManager.getTypeFilters().get("histogram")).intValue());
		
		//Crea el filtro a partir del dato de entrada que tenemos
		switch(filterStack.getDataTypeInFilter(((Integer) filterStackManager.getTypeFilters().get("histogram")).intValue())){
		case RasterBuf.TYPE_IMAGE:
			filtro = new HistogramImageFilter();
		case RasterBuf.TYPE_SHORT:
		case RasterBuf.TYPE_USHORT:
		case RasterBuf.TYPE_INT:
			break;
		}
		
		if(filtro != null){
			filterStack.addFilter(((Integer) filterStackManager.getTypeFilters().get("histogram")).intValue(),filtro);
	        
	        filterStackManager.controlTypes();
		}
	}
	

	public void createStackFromStrings(ArrayList f, Integer pos) {
		// TODO Auto-generated method stub
		
	}
	
	/**
     * Crea una pila de filtros a partir de un Array de Strings. Cada elemento del array debe
     * tener la forma elemento=valor.
     * @param filters
     */
	
	public int createStackFromStrings(ArrayList filters, String fil, GeoRasterFile[] grfList, int filteri) {
		
		if(fil.startsWith("filter.histogram.active") && 
				RasterFilterStackManager.getValue(fil).equals("true") &&
				grfList != null){
			
			addHistogramFilter();
			filteri = -1;
		
		}
				
		return filteri;
	}

	
	
	public ArrayList getStringsFromStack() {
		// TODO Auto-generated method stub
		return null;
	}

	
	public ArrayList getStringsFromStack(ArrayList filterList, RasterFilter rf) {
		if(rf instanceof HistogramFilter)
			filterList.add("filter.histogram.active=true");
		
		return filterList;
	}

}
