/*
 * Created on 23-mar-2006
 * 
 * gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 */
package org.gvsig.rasterTools.pansharpening.toc;

import org.cresques.io.raster.IStackManager;
import org.cresques.io.raster.RasterFilter;
import org.cresques.io.raster.RasterFilterStackManager;
import org.cresques.ui.raster.FilterRasterDialogPanel;
import org.gvsig.rasterTools.pansharpening.filter.PanSharpeningStackManager;
import org.gvsig.rasterTools.pansharpening.ui.PanSharpeningPanel;

import com.iver.cit.gvsig.fmap.layers.FLyrRaster;
import com.iver.cit.gvsig.gui.Panels.IRasterPropertiesRegistrable;
import com.iver.cit.gvsig.gui.Panels.PropertiesRasterDialog;
import com.iver.cit.gvsig.gui.toc.FLyrRasterAdjustPropertiesTocMenuEntry;

/**
 * Entrada del menu contextual del TOC que contiene la parte del listener que
 * corresponde al PanSharpening. La funcin actionPerformed de este objeto ser
 * llamada por la clase  FLyrRasterAdjustPropertiesTocMenuEntry. Para esto debe
 * registrarse en ella a travs del mtodo "register".
 * 
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class PanSharpeningTocMenuEntry implements IRasterPropertiesRegistrable{
	
	private PanSharpeningPanel 			panel = null;
	private PanSharpeningStackManager 	manager = null;  
	
	/**
	 * Contructor. 
	 * Se registra a si mismo en las propiedades de raster del TOC 
	 * @param menu
	 */
	public PanSharpeningTocMenuEntry(FLyrRasterAdjustPropertiesTocMenuEntry menu){
		menu.register(this);	
	}
	
	/**
	 * Asigna el gestor de pila de filtros generico al gestor de pila de pansharpening 
	 * @param stackManager RasterFilterStackManager
	 */
	public void setStackManager(RasterFilterStackManager stackManager){
		panel = new PanSharpeningPanel();
		manager = (PanSharpeningStackManager)stackManager.getManagerByClass(PanSharpeningStackManager.class);
    	panel.setPanSharpeningStackManager(manager);
	}
	
	/**
	 * Obtiene el stackManger para este filtro
	 * @return PanSharpeningStackManager
	 */
	public IStackManager getStackManager(){
		return manager;
	}
	
	/**
	 * Aade el panel al cuadro de dialogo de propiedades de raster
	 * @param menu FLyrRasterAdjustPropertiesTocMenuEntry
	 */
	public void addTabPanel(FLyrRasterAdjustPropertiesTocMenuEntry menu){
		if(panel == null)
			panel = new PanSharpeningPanel();
		menu.getPropertiesDialog().addTab(panel.getName(), panel);			
	}
	
	public void actionPerformed(RasterFilterStackManager stackManager, PropertiesRasterDialog propsDialog, FLyrRaster fLayer) {
		//Modificacin del estado de los controles del pansharpening
		if(stackManager.isActive(stackManager.getTypeFilter("sharpening"))){
			PanSharpeningPanel pSharp = (PanSharpeningPanel)((FilterRasterDialogPanel)propsDialog.getContentPane()).getPanelByClassName("PanSharpeningPanel");
			pSharp.getCbActiveSharpening().setSelected(true);
			pSharp.setTableEnabled(true);
			
			RasterFilter sharp = stackManager.getFilter("sharpening");
			String pancrName = (String)sharp.getParam("pancrName");
			for(int i=0;i<pSharp.getNBands();i++){
				if(pSharp.getBandName(i).endsWith(pancrName))
					pSharp.selectRow(i);
			}
			if(((String)sharp.getParam("method")).equals("hsl")){
				pSharp.getRbHSL().setSelected(true);
				Double coef = ((Double)sharp.getParam("coef"));
				pSharp.getJTextField().setText(coef.toString());
				pSharp.getJSlider().setValue((int)((coef.doubleValue() * 100)/0.5));
			}
			
			if(((String)sharp.getParam("method")).equals("brovey")){
				pSharp.getRbBrovey().setSelected(true);
				Integer coefBrovey = ((Integer)sharp.getParam("coefBrovey"));
				pSharp.getJTextField1().setText(coefBrovey.toString());
				pSharp.getJSlider1().setValue((int)((coefBrovey.intValue() * 100)/50));
			}
		}
	}
}