/*
 * Created on 17-feb-2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.rasterTools.saveRaster.operations;

import java.awt.Dimension;
import java.io.File;
import java.io.IOException;

import org.cresques.geo.ViewPortData;
import org.cresques.io.GeoRasterWriter;
import org.cresques.io.IDataWriter;
import org.cresques.px.Extent;
import org.gvsig.rasterTools.saveRaster.ui.info.EndInfoDialog;
import org.gvsig.rasterTools.saveRaster.ui.properties.ProgressSaveRasterDialog;
import org.gvsig.rasterTools.saveRaster.util.DriverProperties;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.MapControl;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.layers.FLayers;

/**
 * Thread que se encarga de llamar a los writer para realizar la tarea
 * de salvado y/p compresin
 * @author Nacho Brodin <brodin_ign@gva.es>
 */
public class SaveRasterThread extends Thread{
		
	private GeoRasterWriter				writer = null;
	
	private ProgressSaveRasterDialog 	window = null;
	private FLayers 					layers = null;
	private ViewPort					vp = null;
	private MapControl					mapCtrl = null;
	private Dimension 					dimension = null;
	
	private String 						fName = "";
	private int 						sizeBlock = 64;
	private double[]					maxPercentBarList = new double[1];;
	private int 						compresion = 1;
	private DriverProperties 			driverProps = null;

		
	public SaveRasterThread(MapControl mapCtrl, 
							FLayers layers, 
							Dimension dimension, 
							ViewPort vp, 
							ProgressSaveRasterDialog window){
		this.window = window;
		this.layers = layers;
		this.mapCtrl = mapCtrl;
		this.dimension = dimension;
		this.vp = vp;
	}
		
	public void run(){
		long t2;
        long t1 = new java.util.Date().getTime();
        
		//Creamos el servidor de datos de la vista
		RasterizerLayer p = new RasterizerLayer(layers, vp, sizeBlock, mapCtrl);
		
		//Asignamos la barra de progreso
		p.setProgressBar(window, maxPercentBarList[0]);
		
		//Creamos el driver
		Extent ex = new Extent(vp.getAdjustedExtent());
		Dimension imgSz = vp.getImageSize();
		ViewPortData vpData = new ViewPortData(vp.getProjection(), ex, imgSz );
		writer = GeoRasterWriter.getWriter((IDataWriter)p, fName, sizeBlock, 
											3, vpData, compresion, dimension.width, dimension.height);
		
		//Si han sido modificadas las propiedades se asignan los valores modificados
		String[] prp = driverProps.getProperties(fName.toLowerCase().substring(fName.lastIndexOf(".") + 1)); 
		if(prp!=null)
			writer.setProps(prp);
		
		//Ejecutamos el driver con los datos pasados
		try{
			writer.dataWrite();
			writer.writeClose(); 
		}catch(IOException ev){
			return;
		}
		
		t2 = new java.util.Date().getTime();
		
		try{
			this.setEndInfo(fName, (t2 - t1));
			PluginServices.getMDIManager().closeView(this.window);
		}catch(ArrayIndexOutOfBoundsException exc){
			//Si la ventana se ha cerrado ya es porque ha sido cancelada por lo que
			//producir esta excepcin. En este caso no se lanza la ventana de informacin
			//de finalizacin.
		}
	}
	
	private void setEndInfo(String fName, long milis){
		EndInfoDialog info = new EndInfoDialog();
		info.setFileName(" "+fName);
		File f = new File(fName);
		info.setSize(f.length());
		info.setTime(milis);
		if(	fName.endsWith("ecw") || 
			fName.endsWith("jp2") || 
			fName.endsWith("jpg") ||
			fName.endsWith("jpeg"))
			info.setCompression(true);
		else
			info.setCompression(false);
		PluginServices.getMDIManager().addView(info);
	}
	
	/**
	 *Detiene la compresin
	 */
	synchronized public void stopThread(){
		writer.writeCancel();
	}
	
	/**
	 * Asigna el objeto que contiene las propiedades
	 * @param driverProps
	 */
	public void setProps(DriverProperties driverProps){
		this.driverProps = driverProps;
	}
	
	/**
	 * Asigna el nombre del fichero
	 * @param fName	Nombre del fichero
	 */
	public void setFName(String fName){
		this.fName = fName;
	}
	
	/**
	 * Asigna el GeoRasterWriter que se usar para salvar
	 * @param writer
	 */
	public void setWriter(GeoRasterWriter writer){
		this.writer = writer;
	}
	
	/**
	 * Asigna el tamao de bloque que se lee de una sola vez desde el origen
	 * Esto influye en la velocidad de escritura y depende de la memoria que 
	 * tengamos disponible. Si disponemos de poca corremos el riesgo de que al
	 * asignar un bloque muy grande nos de un error
	 * @param sizeBlock Tamao de bloque
	 */
	public void setBlockSize(int sizeBlock){
		this.sizeBlock = sizeBlock;
	}
	
	/**
	 * Asigna la compresin si el formato dispone de este parmetro
	 * @param compresion Nivel de compresin
	 */
	public void setCompresion(int compresion){
		this.compresion = compresion;
	}
}
	
	



