/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.gvsig.rasterTools.saveRaster.ui.listener;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.JTextField;
import javax.swing.filechooser.FileFilter;

import org.cresques.io.GeoRasterWriter;
import org.gvsig.rasterTools.saveRaster.ui.main.SaveRasterControlsPanel;
import org.gvsig.rasterTools.saveRaster.ui.main.SaveRasterDefaultPanel;
import org.gvsig.rasterTools.saveRaster.util.MathUtils;

import com.iver.andami.PluginServices;


/**
 * Panel encargado de manejar los eventos del los controles de Salvar a Raster
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class DataInputListener implements ActionListener,
                                                      MouseListener,
                                                      FocusListener,
                                                      KeyListener {
    final private static long 		serialVersionUID = -3370601314380922368L;
    private SaveRasterControlsPanel	controlPanel = null;
    private SaveRasterDefaultPanel 	dialog = null;
    private String 					fName = null;
    private int						widthInPixels = 0;
    private int						heightInPixels = 0;
    private double					mtsPerPixel = 0D;
    private Object					obj = null;
    private double					widthMts = 0D, heightMts = 0D;

    /**
     * This method initializes
     *
     */
    public DataInputListener(SaveRasterControlsPanel controlPanel) {
        super();
        this.controlPanel = controlPanel;
        initialize();
    }

    /**
     * This method initializes this
     *
     * @return void
     */
    void initialize() {

        //Aadimos gestin de eventos
    	controlPanel.getTScale().addActionListener(this);
    	controlPanel.getTScale().addMouseListener(this);
    	controlPanel.getTScale().addFocusListener(this);
    	controlPanel.getTScale().addKeyListener(this);
    	
    	controlPanel.getTMtsPixel().addActionListener(this);
    	controlPanel.getTMtsPixel().addMouseListener(this);
    	controlPanel.getTMtsPixel().addFocusListener(this);
    	controlPanel.getTMtsPixel().addKeyListener(this);

    	controlPanel.getTWidth().addActionListener(this);
    	controlPanel.getTWidth().addMouseListener(this);
    	controlPanel.getTWidth().addFocusListener(this);
    	controlPanel.getTWidth().addKeyListener(this);
    	controlPanel.getTHeight().addActionListener(this);
    	controlPanel.getTHeight().addMouseListener(this);
    	controlPanel.getTHeight().addFocusListener(this);
    	controlPanel.getTHeight().addKeyListener(this);
    	
    	controlPanel.getCbMeasureType().addActionListener(this);
    	
    	controlPanel.getTInfDerX().addActionListener(this);
    	controlPanel.getTInfDerY().addActionListener(this);
    	controlPanel.getTSupIzqX().addActionListener(this);
    	controlPanel.getTSupIzqY().addActionListener(this);
    	controlPanel.getTInfDerX().addMouseListener(this);
    	controlPanel.getTInfDerY().addMouseListener(this);
    	controlPanel.getTSupIzqX().addMouseListener(this);
    	controlPanel.getTSupIzqY().addMouseListener(this);
        controlPanel.getTInfDerX().addFocusListener(this);
        controlPanel.getTInfDerY().addFocusListener(this);
        controlPanel.getTSupIzqX().addFocusListener(this);
        controlPanel.getTSupIzqY().addFocusListener(this);
        
        controlPanel.getBSelect().addActionListener(this);
        controlPanel.getBProperties().addActionListener(this);
                
    	controlPanel.getCbResolution().addActionListener(this);
        controlPanel.getCbResolution().addKeyListener(this);
    }

    /**
     * Asigna un valor al panel padre
     * @param dialogPanel
     */
    public void setDialogPanel(SaveRasterDefaultPanel dialog) {
        this.dialog = dialog;
    }

    /**
     * Obtiene el Panel de Controles interior
     * @return
     */
    public SaveRasterControlsPanel getSaveParameters() {
        if (controlPanel == null) {
        	controlPanel = new SaveRasterControlsPanel();
            //saveRaster.setBounds(9, 9, 328, 75);
        }
        return controlPanel;
    }
    
    /**
	 * A partir del valor de tamao de imagen de salida, comprueba el tipo de 
	 * dato seleccionado por el usuario (Pixels, mms, cms, mts, pulgadas) y devuelve
	 * el valor en pixeles.
	 * @param value Cadena introducida por el usuario
	 */
	private int getCorrectMeasure(String value){			
		if(controlPanel.getCbMeasureType().getSelectedItem().equals("Pixels"))
			return Integer.parseInt(value);
		
		int ppp = Integer.parseInt((String)controlPanel.getCbResolution().getSelectedItem());
		if(controlPanel.getCbMeasureType().getSelectedItem().equals("Mts"))
			return MathUtils.convertMtsToPixels(Double.parseDouble(value), ppp); 
		if(controlPanel.getCbMeasureType().getSelectedItem().equals("Cms"))
			return MathUtils.convertCmsToPixels(Double.parseDouble(value), ppp);
		if(controlPanel.getCbMeasureType().getSelectedItem().equals("Mms"))
			return MathUtils.convertMmsToPixels(Double.parseDouble(value), ppp);
		if(controlPanel.getCbMeasureType().getSelectedItem().equals("Inches"))
			return MathUtils.convertInchesToPixels(Double.parseDouble(value), ppp);
		
		return 0;
	}
	
	/**
	 * Asigna al JTextField pasado como parmetro el valor en pixels, cms, mms ,mtrs, pulgadas dependiendo
	 * de la seleccin del combo
	 * @param pixel	Valor en pixels	
	 * @param field Campo donde se escribe el valor
	 */
	private void setCorrectMeasure(int pixel, JTextField field){			
		if(controlPanel.getCbMeasureType().getSelectedItem().equals("Pixels"))
			field.setText(String.valueOf(pixel));
		
		int ppp = Integer.parseInt((String)controlPanel.getCbResolution().getSelectedItem());
		if(controlPanel.getCbMeasureType().getSelectedItem().equals("Mts"))
			field.setText(String.valueOf(MathUtils.tailDecimals(MathUtils.convertPixelsToMts(pixel, ppp),5)));
		
		if(controlPanel.getCbMeasureType().getSelectedItem().equals("Cms"))
			field.setText(String.valueOf(MathUtils.tailDecimals(MathUtils.convertPixelsToCms(pixel, ppp),5)));
			
		if(controlPanel.getCbMeasureType().getSelectedItem().equals("Mms"))
			field.setText(String.valueOf(MathUtils.tailDecimals(MathUtils.convertPixelsToMms(pixel, ppp),5)));
			
		if(controlPanel.getCbMeasureType().getSelectedItem().equals("Inches"))
			field.setText(String.valueOf(MathUtils.tailDecimals(MathUtils.convertPixelsToInches(pixel, ppp),5)));
			
	}
		
    /**
     * Calculo del tamao en pixels de la imagen a partir de las coordenadas del
     * mundo real, la escala y los puntos por pulgada
     */
    private void recalcParams() {
    	validateFields();
    	double[] size = null;
     	try{
 	    	size = calcSizeInMts();
     	} catch (NumberFormatException e) {
 	       	return;
 	    }
     	widthMts = size[0];
        heightMts = size[1];
    	
    	int resolution = Integer.parseInt((String)controlPanel.getCbResolution().getSelectedItem());
    	 
    	//Al variar la resolucin independientemente del mtodo seleccionado recalculamos el ancho y el alto
        if (obj.equals(controlPanel.getCbResolution())) {
        	String escala = controlPanel.getTScale().getText();
        	if(controlPanel.getProjection().isProjected())
        		calcSizeMtsPixel(Integer.parseInt(escala), resolution);
        	return;
        }
        
    	//Mtodo por escala seleccionado
    	if(	controlPanel.getRbScale().isSelected() && 
    		!controlPanel.getTScale().getText().equals("")){
    		int scale = Integer.parseInt(controlPanel.getTScale().getText());
    		if(controlPanel.getProjection().isProjected())
    			calcSizeMtsPixel(scale, resolution);
    		else
    			calcSizeMtsPixelGeodesicas(scale, resolution);
    	}
    	
    	//Mtodo por tamao seleccionado
    	if(controlPanel.getRbSize().isSelected()){
        	double rel = (widthMts / heightMts);
                        
    		if(	obj != null && obj.equals(controlPanel.getTWidth()) &&
    			!controlPanel.getTWidth().getText().equals("")){
    			this.widthInPixels = this.getCorrectMeasure(controlPanel.getTWidth().getText());
    			this.heightInPixels = (int)Math.floor(this.widthInPixels / rel);
    			if(controlPanel.getProjection().isProjected())
    				calcScaleMtsPixel(this.widthInPixels, this.heightInPixels, resolution);
    		}
    		if(	obj != null && obj.equals(controlPanel.getTHeight()) &&
        			!controlPanel.getTHeight().getText().equals("")){
    			this.heightInPixels = this.getCorrectMeasure(controlPanel.getTHeight().getText());
    			this.widthInPixels = (int)Math.ceil(this.heightInPixels * rel);
    			if(controlPanel.getProjection().isProjected())
    				calcScaleMtsPixel(this.widthInPixels, this.heightInPixels, resolution);
        	}
    		if(	obj != null && 
    			obj.equals(controlPanel.getCbMeasureType())){
    				if(controlPanel.getProjection().isProjected())
    					calcScaleMtsPixel(this.widthInPixels, this.heightInPixels, resolution);
    		}
    	}
    	
    	//Mtodo metros por pixel seleccionado
    	if(	controlPanel.getRbMtsPixel().isSelected() && 
    		!controlPanel.getTMtsPixel().getText().equals("")){
    		double mtsPixel = Double.parseDouble(controlPanel.getTMtsPixel().getText());
    		if(controlPanel.getProjection().isProjected())
    			calcSizeScale(mtsPixel, resolution);
    	}
    }

    /**
     * Comprueba si un campo de texto tiene el tipo de dato entero o double y si no lo
     * tiene lo borra ya que su contenido es invalido para operar con el
     * @param field	Campo de texto a validar
     * @param isInt true si el valor a validar es entero y false si es double
     */
    private void validateTextField(JTextField field, boolean isInt){
    	try {
    		if(isInt)
	           Integer.parseInt(field.getText());
    		else
    			Double.parseDouble(field.getText());
	    } catch (NumberFormatException e) {
	       	field.setText("");
	    }
    }
    
    /**
     * Valida los campos de texto
     */
    private void validateFields(){
    	    	
    	//Validamos la escala si se ha introducido algo
    	if(!controlPanel.getTScale().getText().equals(""))
    		validateTextField(controlPanel.getTScale(), true);
    	    	
    	//Validamos los mts por pixel si se ha introducido algo
    	if(!controlPanel.getTMtsPixel().getText().equals(""))
    		validateTextField(controlPanel.getTMtsPixel(), false);
    	
    	//Validamos el ancho si se ha introducido algo
    	if(!controlPanel.getTWidth().getText().equals("")){
    		if(((String)controlPanel.getCbMeasureType().getSelectedItem()).equals("Pixels"))
    			validateTextField(controlPanel.getTWidth(), true);
    		else
    			validateTextField(controlPanel.getTWidth(), false);
    	}
    	
    	//Validamos el alto si se ha introducido algo
    	if(!controlPanel.getTHeight().getText().equals("")){
    		if(((String)controlPanel.getCbMeasureType().getSelectedItem()).equals("Pixels"))
    			validateTextField(controlPanel.getTHeight(), true);
    		else
    			validateTextField(controlPanel.getTHeight(), false);
    	}
    }
    
    /**
     * Calcula el tamao en mtrs a partir de las coordenadas.
     * TODO: Esto solo es valido para UTM. El ancho y alto debe obtenerse desde la vista
     * @return
     * @throws NumberFormatException
     */
    private double[] calcSizeInMts()throws NumberFormatException{
    	double[] size = new double[2];
      
    	double lrX = Double.parseDouble(controlPanel.getTInfDerX().getText());
        double lrY = Double.parseDouble(controlPanel.getTInfDerY().getText());
        double ulX = Double.parseDouble(controlPanel.getTSupIzqX().getText());
        double ulY = Double.parseDouble(controlPanel.getTSupIzqY().getText());

        if (ulX > lrX) {
            size[0] = ulX - lrX;
        } else {
        	size[0] = lrX - ulX;
        }
	
        if (ulY > lrY) {
        	size[1] = ulY - lrY;
        } else {
        	size[1] = lrY - ulY;
        }
        return size;
    }
    
    /**
     * A partir de la escala y la resolucin calcula el tamao en pixels y los metros por pixel
     * si la imagen est en coordenadas geograficas.
     * @param scale Escala
     * @param resolution Resolucin
     */
    private void calcSizeMtsPixelGeodesicas(int scaleIntro, int resolution){    
    	//TODO: Calculos para imagenes en coordenadas geograficas
    	
    	/*double lrX = Double.parseDouble(controlPanel.getTInfDerX().getText());
    	double ulX = Double.parseDouble(controlPanel.getTSupIzqX().getText());
    	
    	//Nos aseguramos de que la escala sea un entero. Si no lo es ponemos un 0
        try {
            Integer.parseInt(controlPanel.getTScale().getText());
        } catch (NumberFormatException e) {
        	controlPanel.getTScale().setText("0");
        }
        
    	double scale = controlPanel.getProjection().getScale(ulX, lrX, 
    			controlPanel.getWidthInPixelsGeodesicas(), 
    			Integer.parseInt((String)controlPanel.getCbResolution().getSelectedItem()));

    	this.widthInPixels = (int)((scaleIntro * controlPanel.getWidthInPixelsGeodesicas()) / scale);
    	this.heightInPixels = (int)((scaleIntro * controlPanel.getHeightInPixelsGeodesicas()) / scale);
    	    
        mtsPerPixel = MathUtils.tailDecimals(((double)(widthMts / this.widthInPixels)), 5);
        
        controlPanel.getTMtsPixel().setText(String.valueOf(mtsPerPixel));
        this.setCorrectMeasure(this.widthInPixels, controlPanel.getTWidth());
        this.setCorrectMeasure(this.heightInPixels, controlPanel.getTHeight());*/    	
    }
    
    /**
     * A partir de la escala y la resolucin calcula el tamao en pixels y los metros por pixel
     * @param scale Escala
     * @param resolution Resolucin
     */
    private void calcSizeMtsPixel(int scale, int resolution){        
    	if ((widthMts <= 0) || (heightMts <= 0) ||
                 (Integer.parseInt(controlPanel.getTScale().getText()) == 0)) {
         	controlPanel.getTWidth().setText("");
         	controlPanel.getTHeight().setText("");
             return;
        }

        //Calculo del tamao de la imagen definitiva en pulgadas
        double widthInches = (widthMts / Double.parseDouble(controlPanel.getTScale().getText())) * MathUtils.INCHESMTR;
        double heightInches = (heightMts / Double.parseDouble(controlPanel.getTScale().getText())) * MathUtils.INCHESMTR;

        //Ancho en pixeles = ppp * sizepulgadas
        this.widthInPixels = (int) (Integer.parseInt(controlPanel.getCbResolution().getSelectedItem().toString()) * widthInches);
        this.heightInPixels = (int) (Integer.parseInt(controlPanel.getCbResolution().getSelectedItem().toString()) * heightInches);
                 
        mtsPerPixel = (double)(widthMts / this.widthInPixels);        
        //double mtsPixelH = wc_altomts/altoPixels;
       
        //recortamos a 5 decimales
        mtsPerPixel = MathUtils.tailDecimals(mtsPerPixel, 5);
        
        controlPanel.getTMtsPixel().setText(String.valueOf(mtsPerPixel));
        this.setCorrectMeasure(this.widthInPixels, controlPanel.getTWidth());
        this.setCorrectMeasure(this.heightInPixels, controlPanel.getTHeight());
                
        //int anchopixels =(int) (Toolkit.getDefaultToolkit().getScreenResolution() * anchopulgadas);
        //int altopixels = (int) (Toolkit.getDefaultToolkit().getScreenResolution() * altopulgadas);
    }
    
    /**
     * A partir de los metros por pixel y la resolucin calcula el tamao en pixels y la escala
     * @param mtsPixel Metros por pixel 
     * @param resolution Resolucin
     */
    private void calcSizeScale(double mtsPixel, int resolution){        
        //Nmero de pxeles de ancho y alto 
    	this.widthInPixels = (int)(widthMts / mtsPixel);
    	this.heightInPixels = (int)(heightMts / mtsPixel);
        
        //Obtenemos los mts/pixel reales ya que el nmero de pixeles es entero deben redondearse
        mtsPerPixel = (double)(widthMts / widthInPixels);
        
        //recortamos a 5 decimales
        mtsPerPixel = MathUtils.tailDecimals(mtsPerPixel, 5);
        
        //Obtenemos el ancho y alto en pulgadas
        double widthInches = (double)(widthInPixels / Integer.parseInt(controlPanel.getCbResolution().getSelectedItem().toString()));
        double heightInches = (double)(heightInPixels / Integer.parseInt(controlPanel.getCbResolution().getSelectedItem().toString()));
        
        //Calculo de la escala
        int scale = (int)((widthMts * MathUtils.INCHESMTR) / widthInches);
        
        controlPanel.getTScale().setText(String.valueOf(scale));
        controlPanel.getTMtsPixel().setText(String.valueOf(mtsPerPixel));
        this.setCorrectMeasure(widthInPixels, controlPanel.getTWidth());
        this.setCorrectMeasure(heightInPixels, controlPanel.getTHeight());
    }

    /**
     * A partir del tamao en pixels de la imagen y la resolucin calcula los metros por pixel y la escala
     * @param widthPixels	Ancho de la imagen en pixels
     * @param heightPixels	Alto de la imagen en pixels
     * @param resolution Resolucin
     */
	private void calcScaleMtsPixel(int widthPixels, int heightPixels, int resolution){
        this.widthInPixels = widthPixels;
        this.heightInPixels = heightPixels;
        
        //Obtenemos los mts/pixel reales ya que el nmero de pixeles es entero deben redondearse
        mtsPerPixel = (double)(widthMts / widthPixels);
        
        //recortamos a 5 decimales
        mtsPerPixel = MathUtils.tailDecimals(mtsPerPixel, 5);
        
        //Obtenemos el ancho y alto en pulgadas
        double widthInches = (double)(widthPixels / Integer.parseInt(controlPanel.getCbResolution().getSelectedItem().toString()));
        double heightInches = (double)(heightPixels / Integer.parseInt(controlPanel.getCbResolution().getSelectedItem().toString()));
        
        //Calculo de la escala
        int scale = (int)((widthMts * MathUtils.INCHESMTR) / widthInches);
        
        controlPanel.getTScale().setText(String.valueOf(scale));
        controlPanel.getTMtsPixel().setText(String.valueOf(mtsPerPixel));
        this.setCorrectMeasure(widthPixels, controlPanel.getTWidth());
        this.setCorrectMeasure(heightPixels, controlPanel.getTHeight());
	}
    		
    /**
     * Controla cuando se cumplen todos los requisitos en el formulario para
     * poder activar el botn de aceptar.
     */
    private void enableButtons() {
        if(	controlPanel.getTWidth().getText().equals("0") ||
        	controlPanel.getTHeight().getText().equals("0") ||
        	controlPanel.getTWidth().getText().equals("") ||
        	controlPanel.getTHeight().getText().equals("") ||
        	this.fName == null || this.fName.equals("")){
            if (dialog != null)
                dialog.getAcceptButton().setEnabled(false);
            return;
        }

        if (dialog != null) 
            dialog.getAcceptButton().setEnabled(true);
    }

    /**
     * Gestiona los eventos del cambio de escala de la imagen y lla apertura del
     * dialogo de seleccin del nombre del fichero.
     */
    public void actionPerformed(ActionEvent e) {
    	obj = e.getSource();
    	
        //Selector de Fichero sobre el cual se va a salvar a raster
        if (obj.equals(controlPanel.getBSelect())) {
            JFileChooser chooser = new JFileChooser("./");
            chooser.setDialogTitle(PluginServices.getText(this, "seleccionar_fichero"));

            //Aadimos las extensiones que hayan sido registradas en el driver
            String[] extList = GeoRasterWriter.getDriversExtensions();
            for(int i=0;i<extList.length;i++)
            	chooser.addChoosableFileFilter(new WriterFilter(chooser, extList[i]));
            	         
            int returnVal = chooser.showOpenDialog(controlPanel);

            if (returnVal == JFileChooser.APPROVE_OPTION) {
                this.fName = chooser.getSelectedFile().toString();
                String ext = ((WriterFilter)chooser.getFileFilter()).getDescription();
                
                FileFilter filter = chooser.getFileFilter();
                ext = ext.toLowerCase().substring(ext.lastIndexOf(".") + 1, ext.length());

                if ((fName != null) && !fName.equals("")) {
                	if(!fName.endsWith(ext))
                		fName = fName + "." + ext;
                	
                	controlPanel.getBProperties().setText(PluginServices.getText(this, "props")+" "+GeoRasterWriter.getDriverType(ext));
                }
                controlPanel.getBProperties().setEnabled(true);
                controlPanel.getLFileName().setText(fName.substring(fName.lastIndexOf(File.separator) + 1, fName.length()));
             
                enableButtons();
            }
        }
        
        //Al variar las unidades recalculamos el ancho y el alto
        if (e.getSource().equals(controlPanel.getCbMeasureType())) {
        	this.setCorrectMeasure(this.widthInPixels, controlPanel.getTWidth());
        	this.setCorrectMeasure(this.heightInPixels, controlPanel.getTHeight());
        }
                
        if (!obj.equals(controlPanel.getBSelect()))
        	this.recalcParams();
        
        this.enableButtons();
        
    }

    /**
     * Devuelve el nombre del fichero seleccionado
     * @return Nombre del fichero seleccionado
     */
    public String getFileName() {
        return fName;
    }

    /**
     * Unused
     */
    public void mouseExited(MouseEvent e) {
    }

    /**
     * Unused. Only for compilant purposes
     */
    public void mouseReleased(MouseEvent e) {
    }

    /**
     * Unused. Only for compilant purposes
     */
    public void mouseEntered(MouseEvent e) {
    }

    public void mouseClicked(MouseEvent e) {
    	if( (obj.equals(controlPanel.getTHeight()) && e.getSource().equals(controlPanel.getTWidth())) ||
    		(obj.equals(controlPanel.getTWidth()) && e.getSource().equals(controlPanel.getTHeight())) )
    		return;
    	obj = e.getSource();
        this.recalcParams();
        enableButtons();
    }

    /**
     * Unused. Only for compilant purposes
     */
    public void mousePressed(MouseEvent e) {
    }

    /**
     * Unused. Only for compilant purposes
     */
    public void focusGained(FocusEvent e) {
    }

    /**
     * Recalcula el tamao de la imagen con los datos existentes y
     * activa o desactiva los botones.
     */
    public void focusLost(FocusEvent e) {
    	obj = e.getSource();
        this.recalcParams();
        enableButtons();
    }

    /**
     * Activa o Desactiva los botones a travs de la funcin que valida los campos
     */
    public void keyTyped(KeyEvent e) {
        enableButtons();
    }

    /**
     * Activa o Desactiva los botones a travs de la funcin que valida los campos
     */
    public void keyPressed(KeyEvent e) {
        enableButtons();
    }

    /**
     * Activa o Desactiva los botones a travs de la funcin que valida los campos
     */
    public void keyReleased(KeyEvent e) {
        enableButtons();
    }

    /**
     * Obtiene la altura en pixels de la imagen de salida
     * @return entero con la altura en pixels
     */
	public int getHeightInPixels() {
		return heightInPixels;
	}
	
	/**
     * Obtiene la anchura en pixels de la imagen de salida
     * @return entero con la anchura en pixels
     */
	public int getWidthInPixels() {
		return widthInPixels;
	}
}

/**
 * @author Nacho Brodin <brodin_ign@gva.es>
 * Filtro para el selector de formatos de escritura
 */
class WriterFilter extends javax.swing.filechooser.FileFilter {
    private JFileChooser chooser = null;
    private String filter;

    public WriterFilter(JFileChooser ch, String fil) {
        this.chooser = ch;
        this.filter = fil;
    }

    public boolean accept(File f) {
        return f.isDirectory() || f.getName().toLowerCase().endsWith("."+filter);
    }

    public String getDescription() {
        return "."+filter;
    }
}

