/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/
package com.iver.cit.gvsig.gui.toc;

import java.awt.event.ActionEvent;

import javax.swing.JMenuItem;

import org.cresques.io.GeoRasterFile;
import org.cresques.io.raster.RasterFilterStackManager;
import org.cresques.px.PxRaster;
import org.cresques.ui.raster.BandSetupPanel;
import org.cresques.ui.raster.EnhancedPanel;
import org.cresques.ui.raster.FilterRasterDialogPanel;
import org.cresques.ui.raster.RasterTransparencyPanel;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrWCS;
import com.iver.cit.gvsig.gui.dialog.WCSRasterPropsDialog;

/**
 * 
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class WCSPropsRasterTocMenuEntry  extends TocMenuEntry {
	private JMenuItem propsMenuItem;
	FLayer lyr = null;
	private WCSRasterPropsDialog	propsDialog = null;
	private BandSetupPanel 			bandSetup = null;
	
	public void initialize(FPopupMenu m) {
		super.initialize(m);
		
		if (isTocItemBranch()) {
			lyr = getNodeLayer();
    		// Opcciones para capas WCS
    		if ((lyr instanceof FLyrWCS)) {
    			propsMenuItem = new JMenuItem(PluginServices.getText(this, "propiedades_raster"));
    			getMenu().add( propsMenuItem );
    			propsMenuItem.setFont(FPopupMenu.theFont);
    			getMenu().setEnabled(true);
    			//getMenu().addSeparator();
    	        //Cambio color
    			propsMenuItem.addActionListener(this);   			
     		}
		}
	}
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.gui.toc.TocMenuEntry#execute(com.iver.cit.gvsig.gui.toc.ITocItem)
	 */
	public void actionPerformed(ActionEvent e) {
       	lyr = getNodeLayer();
        
		if(lyr instanceof FLyrWCS && ((FLyrWCS) lyr).getPxRaster()!=null){
			RasterFilterStackManager 	stackManager = null;
			
			FLyrWCS layer = (FLyrWCS) lyr;

			stackManager = new RasterFilterStackManager(layer.getFilterStack());
			
			propsDialog = new WCSRasterPropsDialog(layer, stackManager.getTransparencyRGB());
			
			int alpha = layer.getPxRaster().getAlpha();
			
			FilterRasterDialogPanel filterPanel = ((FilterRasterDialogPanel)propsDialog.getContentPane());
			
			bandSetup = (BandSetupPanel)filterPanel.getPanelByClassName("BandSetupPanel");
			GeoRasterFile[] files = layer.getPxRaster().getFiles();
			bandSetup.addFiles(files);
			for(int i=0; i< files.length;i++)
				propsDialog.addNumBands(files[i].getBandCount());
				
			PxRaster px = layer.getPxRaster();
			int posR = 0, posG = 0, posB = 0;
			
			for(int i=0;i<px.getPosFile(GeoRasterFile.RED_BAND);i++)
				posR += files[i].getBandCount();
			posR += px.getBand(GeoRasterFile.RED_BAND);
			
			for(int i=0;i<px.getPosFile(GeoRasterFile.GREEN_BAND);i++)
				posG += files[i].getBandCount();
			posG += px.getBand(GeoRasterFile.GREEN_BAND);
			
			for(int i=0;i<px.getPosFile(GeoRasterFile.BLUE_BAND);i++)
				posB += files[i].getBandCount();
			posB += px.getBand(GeoRasterFile.BLUE_BAND);
			
					
			bandSetup.assignBand(posR, GeoRasterFile.RED_BAND);
			bandSetup.assignBand(posG, GeoRasterFile.GREEN_BAND);
			bandSetup.assignBand(posB, GeoRasterFile.BLUE_BAND);	
									
			//Asignacin del alpha actual de la imagen al dialogo
			RasterTransparencyPanel rasterTrans = (RasterTransparencyPanel)filterPanel.getPanelByClassName("RasterTransparencyPanel");
			rasterTrans.setOpacity(alpha);
				
			//Asignacin del porcentaje de recorte actual de la imagen al dialogo
			EnhancedPanel pEnhan =  (EnhancedPanel)filterPanel.getPanelByClassName("EnhancedPanel");
						
			propsDialog.setRasterFilterStackManager(stackManager);
			int tail = 0;
			if(	pEnhan != null &&
				stackManager.getStackStats() != null){
				tail = (int)(stackManager.getStackStats().tailPercent*200);
				pEnhan.setPercentTail(tail);
			}
			/*if(stackManager.isActive(stackManager.getTypeFilter("enhanced")))
				pEnhan.setSelectedFilter(1);
			else
				pEnhan.setSelectedFilter(0);*/
							
			propsDialog.readStat();
			
			PluginServices.getMDIManager().addView(propsDialog);       	
		}
	}
}