package com.iver.cit.gvsig.fmap.drivers.geotools;

import java.io.IOException;
import java.util.Date;
import java.util.NoSuchElementException;
import java.util.Vector;

import org.geotools.data.FeatureReader;
import org.geotools.feature.Feature;
import org.geotools.feature.IllegalAttributeException;

import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueFactory;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.core.DefaultFeature;
import com.iver.cit.gvsig.fmap.core.IFeature;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.core.ShapeFactory;
import com.iver.cit.gvsig.fmap.core.v02.FConverter;
import com.iver.cit.gvsig.fmap.drivers.IFeatureIterator;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: WFSFeaturesIterator.java,v 1.3 2006/05/23 13:21:12 jorpiell Exp $
 * $Log: WFSFeaturesIterator.java,v $
 * Revision 1.3  2006/05/23 13:21:12  jorpiell
 * Modificado el datasource para capturar las excepciones de parseo
 *
 * Revision 1.2  2006/05/22 10:10:07  jorpiell
 * El iteartor crea un vector y lo inicializa en el primer recorrido que se hace sobre l. A partir de ah se devuelve un elemento del vector en los posteriores recorridos
 *
 * Revision 1.1  2006/05/19 12:47:12  jorpiell
 * Creado el iteardor de Features
 *
 *
 */
/**
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class WFSFeaturesIterator implements IFeatureIterator{
	private FeatureReader featureReader = null;
	private Vector featureVector = new Vector();
	private boolean finished = false;
	private int position = -1;
	private int length = -2;
	
	public WFSFeaturesIterator(FeatureReader featureReader){
		this.featureReader = featureReader;
	}
	
	/**
	 * Gets the next feature
	 * @return
	 * @throws DriverException
	 */
	public IFeature next() throws DriverException {
		if (!finished){
			Feature feature = null;
			try {
				feature = featureReader.next();
			} catch (NoSuchElementException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			} catch (IllegalAttributeException e) {
				e.printStackTrace();
			}
			Object[] attr = null;
			attr = feature.getAttributes(attr);
			Value[] values = getValues(attr);
			
			try{
				IGeometry g = ShapeFactory.createGeometry(FConverter.jts_to_java2d(feature.getDefaultGeometry()));
				IFeature outPutFeature = new DefaultFeature(g,values,feature.getID());
				featureVector.add(outPutFeature);
				return outPutFeature;
			}catch (NullPointerException e){
				//The geometry can't be created
			}
		}else{
			return (DefaultFeature)featureVector.get(position);			
		}
		return null;
	}
	
	/**
	 * Returns if there is a new feature to retrieve
	 * @return
	 */
	public boolean hasNext(){
		if (length == -2){
			try {
				if (featureReader.hasNext()){
					position++;
					return true;
				}else{
					featureReader.close();
					finished = true;
					length = featureVector.size() - 1;
					return false;
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		}else{
			if (position != length){
				position++;
				return true;
			}else{
				return false;
			}
		}
		return false;
	}
	
	/**
	 * Gets the attributes
	 * @param attr
	 * @return
	 */
	private Value[] getValues(Object[] attr) {
		Value[] values = new Value[attr.length];
		for (int i=1;i<attr.length;i++){
			if (attr[i]!=null){
				if (attr[i] instanceof Double){
					values[i]=ValueFactory.createValue(((Double)attr[i]).doubleValue());
				}else if (attr[i] instanceof String){
					values[i]=ValueFactory.createValue(String.valueOf(attr[i]));
				}else if (attr[i] instanceof Long){
					values[i]=ValueFactory.createValue(((Long)attr[i]).longValue());
				}else if (attr[i] instanceof Integer){
					values[i]=ValueFactory.createValue(((Integer)attr[i]).intValue());
				}else if (attr[i] instanceof Float){
					values[i]=ValueFactory.createValue(((Float)attr[i]).floatValue());
				}else if (attr[i] instanceof Short){
					values[i]=ValueFactory.createValue(((Short)attr[i]).shortValue());
				}else if (attr[i] instanceof Boolean){
					values[i]=ValueFactory.createValue(((Boolean)attr[i]).booleanValue());
				}else if (attr[i] instanceof Date){
					values[i]=ValueFactory.createValue(((Date)attr[i]));
				}					
			}else{
				values[i]=ValueFactory.createValue("");
			}
		}
		return values;
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.IFeatureIterator#closeIterator()
	 */
	public void closeIterator() throws DriverException {
		// TODO Auto-generated method stub		
	}
	
	/**
	 * This method must be executed before to start an iteration
	 * process
	 */
	public void startIteration(){
		position = -1;		
	}
}
