package com.iver.cit.gvsig.fmap.drivers.geotools;

import java.util.Vector;

import org.geotools.filter.CompareFilter;
import org.geotools.filter.FilterFactory;
import org.gvsig.remoteClient.wfs.WFSAttribute;
import org.gvsig.remoteClient.wfs.WFSStatus;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.layers.WFSLayerNode;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: WFSFilter.java,v 1.3 2006/05/25 15:32:01 jorpiell Exp $
 * $Log: WFSFilter.java,v $
 * Revision 1.3  2006/05/25 15:32:01  jorpiell
 * Se ha aadido la funcionalidad para eliminar el namespace de los tipos de atributos
 *
 * Revision 1.2  2006/05/25 10:32:21  jorpiell
 * Se ha renombrado la clase WFSFields por WFSAttributes porque era algo confusa
 *
 * Revision 1.1  2006/05/23 08:07:00  jorpiell
 * Aadido un mtodo para detectar el tipo de geometra
 *
 *
 */
/**
 * This class creates a "filter encoding" filter to be used
 * by the WFS protocol
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class WFSFilter {
	
	public static void createAttributeFilter(WFSStatus status){
//		FilterFactory filterFactory = FilterFactory.createFilterFactory();
//		CompareFilter filter = null;
//		try {
//		    filter = filterFactory.createCompareFilter(CompareFilter.COMPARE_GREATER_THAN);
//		    FeatureType featureType = features.getFeatures().getSchema();
//		    filter.addLeftValue(filterFactory.createAttributeExpression(featureType,"POP_2000"));
//		    filter.addRightValue(filterFactory.createLiteralExpression(100000));
//		} catch (IllegalFilterException e) {
//		    e.printStackTrace();
	}
	
	/**
	 * It return true is the attributeType is a geometry
	 * @param attributeType
	 * Type to compare
	 * @return
	 */
	public static boolean isGeometry(String attributeType){
		if (attributeType == null){
			return false;
		}
		
		if (attributeType.compareTo("gml:GeometryPropertyType") == 0){
			return true;
		}
		if (attributeType.compareTo("gml:MultiLineStringPropertyType") == 0){
			return true;
		}
		if (attributeType.compareTo("gml:PointPropertyType") == 0){
			return true;
		}
		if (attributeType.compareTo("gml:MultiPointPropertyType") == 0){
			return true;
		}
		if (attributeType.compareTo("gml:MultiPolygonPropertyType") == 0){
			return true;
		}
		return false;
	}
	
	/**
	 * Return the layer geometry
	 * @param layer
	 * Layer with a set of attributes
	 * @return
	 */
	public static String getGeometry(WFSLayerNode layer){
		 Vector attributes = layer.getFields();
		
         for (int i=0 ; i<attributes.size() ; i++){
         	WFSAttribute attribute = (WFSAttribute)attributes.get(i);
         	if (WFSFilter.isGeometry(attribute.getType())){
         		return attribute.getType().split(":")[1];
         	}
         }
         return "";
	}
	
	/**
	 * It returns the attribute type. It has to remove the
	 * namespace to can use the translation mechanism 
	 * translations
	 * @param attribute
	 * Attribute to translate
	 */
	public static String getAttributeType(WFSAttribute attribute){
		if (attribute.getType() == null){
			return "";
		}
		if (isGeometry(attribute.getType())){
			return attribute.getType().split(":")[1];
		}
		
		String sAttribute = "";
		if (attribute.getType().split(":").length > 1){
			sAttribute = attribute.getType().split(":")[1];
		}else{
			sAttribute = attribute.getType();
		}		
		return sAttribute;
	}
	
	
}
