package com.iver.cit.gvsig.fmap.drivers.geotools;

import java.awt.geom.Rectangle2D;
import java.io.IOException;
import java.net.ConnectException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;

import org.geotools.data.DefaultQuery;
import org.geotools.data.FeatureReader;
import org.geotools.data.Query;
import org.geotools.data.Transaction;
import org.geotools.data.wfs.WFSDataStoreFactory;
import org.geotools.feature.AttributeType;
import org.gvsig.remoteClient.RemoteClient;
import org.gvsig.remoteClient.wfs.WFSStatus;
import org.gvsig.remoteClient.wms.ICancellable;
import org.xml.sax.SAXException;

import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.core.IFeature;


/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: WFSGeotoolsDriver.java,v 1.6 2006/05/25 10:32:21 jorpiell Exp $
 * $Log: WFSGeotoolsDriver.java,v $
 * Revision 1.6  2006/05/25 10:32:21  jorpiell
 * Se ha renombrado la clase WFSFields por WFSAttributes porque era algo confusa
 *
 * Revision 1.5  2006/05/24 07:21:38  jorpiell
 * Si no ha dado tiempo a inicializar el resultset se debe devolver nulo
 *
 * Revision 1.4  2006/05/23 08:07:37  jorpiell
 * Cambio probocado por el renombramiento de la clase DataStore por WFSDataStore
 *
 * Revision 1.3  2006/05/22 10:32:01  jorpiell
 * El connect debe de tener en cuanta los valores del buffer y el timeout
 *
 * Revision 1.2  2006/05/22 10:10:46  jorpiell
 * El driver recorre las features la primera vez e inicializa el vector
 *
 * Revision 1.1  2006/05/19 12:48:21  jorpiell
 * Driver que implementa la accesibilidad a geotools
 *
 *
 */
/**
 * This class implements the WFS getFetaure method using the
 * Geotools library
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class WFSGeotoolsDriver extends RemoteClient{
	private WFSDataStore dataStore = null;
	private ArrayList features = new ArrayList();
	private Rectangle2D fullExtent = null;
	private Hashtable hashRelate = new Hashtable();
	private WFSStatus currentStatus = null;
	private int numReg = -1;
	private int numField = -1;
	private AttributeType[] attributes;
	private WFSFeaturesIterator iterator = null;
	
	public WFSGeotoolsDriver(String host,WFSStatus status) throws ConnectException{
		if (status != null){
			this.currentStatus = status;
		}
		this.setHost(host);
		if (!connect(null)){
			throw new ConnectException();	
		}		
	}
	
	/**
	 * The GetFeature operation allows retrieval of features from a 
	 * web feature service. A GetFeature request is processed by
	 * a WFS and when the value of the outputFormat attribute is 
	 * set to text/gml a GML instance document, containing the 
	 * result set, is returned to the client.
	 * @param status
	 * WFS client status. Contains all the information to create
	 * the query
	 * @return Object
	 * GML File
	 */
	public Object getFeature(WFSStatus status){
		try { 
			DefaultQuery query = new DefaultQuery(currentStatus.getFeatureName());
			query.setPropertyNames(status.getFields());
			FeatureReader featureReader;
			featureReader = dataStore.getFeatureReader(query,Transaction.AUTO_COMMIT);
			if (numField == -1) {
				numField = featureReader.getFeatureType().getAttributeCount();
				attributes = featureReader.getFeatureType().getAttributeTypes();
			}
			iterator = new WFSFeaturesIterator(featureReader);
			initialize();
			return iterator;
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();			
		} 
		return null;
	}	
	
	/**
	 * Sets the retrieved features
	 * @param transformation
	 * @throws DriverException 
	 * @throws DriverException 
	 */
	private void initialize() {
		int index = 0;
		while (iterator.hasNext()){
			IFeature feature;
			try {
				feature = iterator.next();
				try{
					features.add(feature);
					Rectangle2D rect = feature.getGeometry().getBounds2D();
					
					if (fullExtent == null) {
						fullExtent = new Rectangle2D.Double(rect.getMinX(),
								rect.getMinY(), rect.getWidth(), rect.getHeight());
					} else {
						fullExtent.add(new Rectangle2D.Double(rect.getMinX(),
								rect.getMinY(), rect.getWidth(), rect.getHeight()));
					}
					
					hashRelate.put(feature.getID(), new Integer(index));
				}catch(NullPointerException e){
					//
				}
			} catch (DriverException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}			
			index++;
		}
		numReg = index;
	}

	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.remoteClient.RemoteClient#connect()
	 */
	public boolean connect(ICancellable cancel) {
		URL url;
		try {
			url = new URL(getCapabilitiesRequest());
			dataStore = new WFSDataStore(url, 
					new Boolean(true), 
					currentStatus.getUserName(), 
					currentStatus.getPassword(), 
					currentStatus.getTimeout(), 
					currentStatus.getBuffer());
			return true;
		} catch (MalformedURLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
		return false;         
	}

	/**
	 * @return the host name
	 */
    public String getCapabilitiesRequest() {        
    	StringBuffer req = new StringBuffer();
    	req.append(super.getHost()).append("?").append("REQUEST=GetCapabilities&SERVICE=WFS&");
    	req.append("&EXCEPTIONS=XML");
    	return req.toString();        
    } 

	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.remoteClient.RemoteClient#close()
	 */
	public void close() {
		// TODO Auto-generated method stub
		
	}

	/**
	 * @return Returns the features.
	 */
	public ArrayList getFeatures() {
		return features;
	}

	/**
	 * @return Returns the hashRelate.
	 */
	public Hashtable getHashRelate() {
		return hashRelate;
	}

	/**
	 * @return Returns the fullExtent.
	 */
	public Rectangle2D getFullExtent() {
		return fullExtent;
	}

	/**
	 * @return Returns the numReg.
	 */
	public int getNumReg() {
		return numReg;
	}

	/**
	 * @return Returns the attributes.
	 */
	public AttributeType[] getAttributes() {
		return attributes;
	}

	/**
	 * @return Returns the numField.
	 */
	public int getNumField() {
		return numField;
	}

	/**
	 * @return Returns the iterator.
	 */
	public WFSFeaturesIterator getIterator() {
			if (iterator == null){
				return null;
			}
			iterator.startIteration();
			return iterator;		
	}
}
