package com.iver.cit.gvsig.fmap.drivers.wfs;

import java.awt.geom.Rectangle2D;
import java.io.IOException;
import java.net.ConnectException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;

import org.geotools.feature.AttributeType;
import org.gvsig.remoteClient.wfs.WFSClient;
import org.gvsig.remoteClient.wfs.WFSFeature;
import org.gvsig.remoteClient.wfs.WFSStatus;
import org.gvsig.remoteClient.wfs.WFSProtocolHandler.ServiceInformation;

import com.hardcode.gdbms.engine.data.DataSourceFactory;
import com.hardcode.gdbms.engine.data.driver.ObjectDriver;
import com.hardcode.gdbms.engine.data.edition.DataWare;
import com.hardcode.gdbms.engine.values.Value;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.core.FShape;
import com.iver.cit.gvsig.fmap.core.IFeature;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.drivers.DriverAttributes;
import com.iver.cit.gvsig.fmap.drivers.IFeatureIterator;
import com.iver.cit.gvsig.fmap.drivers.UnsupportedVersionException;
import com.iver.cit.gvsig.fmap.drivers.WFSDriver;
import com.iver.cit.gvsig.fmap.drivers.WFSException;
import com.iver.cit.gvsig.fmap.drivers.geotools.WFSFeaturesIterator;
import com.iver.cit.gvsig.fmap.drivers.geotools.WFSGeotoolsDriver;
import com.iver.cit.gvsig.fmap.layers.WFSLayerNode;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: FMapWFSDriver.java,v 1.7 2006/05/25 10:32:11 jorpiell Exp $
 * $Log: FMapWFSDriver.java,v $
 * Revision 1.7  2006/05/25 10:32:11  jorpiell
 * Se ha renombrado la clase WFSFields por WFSAttributes porque era algo confusa
 *
 * Revision 1.6  2006/05/23 13:21:28  jorpiell
 * Se tiene en cuanta el online resource
 *
 * Revision 1.5  2006/05/22 10:31:35  jorpiell
 * Cambio producido al cambiar una interfaz del geotools driver
 *
 * Revision 1.4  2006/05/22 10:11:10  jorpiell
 * Eliminadas algunas lineas innecesarias
 *
 * Revision 1.3  2006/05/19 12:53:54  jorpiell
 * Se le ha aadido un driver de geottols, por lo que tiene dos drivers. Para hacer el getFeatureInfo y el capablities utilizar elremoteServices y para hacer el getFeature usar el de geotools.
 *
 * Revision 1.2  2006/04/20 16:38:24  jorpiell
 * Ahora mismo ya se puede hacer un getCapabilities y un getDescribeType de la capa seleccionada para ver los atributos a dibujar. Queda implementar el panel de opciones y hacer el getFeature().
 *
 * Revision 1.1  2006/04/19 12:50:16  jorpiell
 * Primer commit de la aplicacin. Se puede hacer un getCapabilities y ver el mensaje de vienvenida del servidor
 *
 *
 */
/**
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class FMapWFSDriver implements WFSDriver,ObjectDriver{
	private WFSClient remoteServicesClient;
	private WFSGeotoolsDriver geotoolsClient;
    private WFSLayerNode[] featuresList;
    private Hashtable hashFeatures;  
        
   private int idFID_FieldName;
	private int numField = -1;
	//private AttributeType[] attributes;
    private String strOriginalSRID;

	public FMapWFSDriver() {
		super();
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.WFSDriver#getCapabilities(java.net.URL)
	 */
	public void getCapabilities(URL server) 
		throws WFSException {
			try {
				getClient(server).connect(null);
			} catch (Exception e) {			
				throw new WFSException(e);
			}		
	}	
	
	 /*
     *  (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.drivers.WFSDriver#describeFeatureType(java.lang.String)
     */
	public void describeFeatureType(String featureType) throws WFSException {
		remoteServicesClient.describeFeatureType(new WFSStatus(featureType));				
	}
	
	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.WFSDriver#getFeature(org.gvsig.remoteClient.wfs.WFSStatus)
	 */
	public void getFeature(WFSStatus wfsStatus) throws WFSException {
		//Using remoteServices
		//return remoteServicesClient.getFeature(wfsStatus);
		//Using geotools
		if (geotoolsClient == null){
			try {
				geotoolsClient = new WFSGeotoolsDriver(remoteServicesClient.getHost(),
						wfsStatus);
			} catch (ConnectException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				throw new WFSException();
			}
		}
		geotoolsClient.getFeature(wfsStatus);
	}	
	
	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialDriver#getShapeType()
	 */
	public int getShapeType() {
		return FShape.MULTI;
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialDriver#getShapeCount()
	 */
	public int getShapeCount() throws IOException {
		return getNumReg();
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialDriver#getDriverAttributes()
	 */
	public DriverAttributes getDriverAttributes() {
		// TODO Auto-generated method stub
		return null;
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialDriver#getFullExtent()
	 */
	public Rectangle2D getFullExtent() throws IOException {
		return geotoolsClient.getFullExtent();
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialDriver#getShape(int)
	 */
	public IGeometry getShape(int index) throws IOException {
		return ((IFeature) getFeatures().get(index)).getGeometry();
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialDriver#reLoad()
	 */
	public void reLoad() throws IOException {
		geotoolsClient.getFeature(null);
	}

	/*
	 *  (non-Javadoc)
	 * @see com.hardcode.driverManager.Driver#getName()
	 */
	public String getName() {
		return "WFS Driver";
	}
    
    
    
	/**
	 * Devuelve WFSClient a partir de su URL.
	 *
	 * @param url URL.
	 *
	 * @return WMSClient.
	 * @throws IOException 
	 * @throws ConnectException 
	 *
	 * @throws UnsupportedVersionException
	 * @throws IOException
	 */
	public WFSClient getClient(URL url) throws ConnectException, IOException {
		if (remoteServicesClient == null) {
			remoteServicesClient = new WFSClient(url.toString());
		}
		return remoteServicesClient;
	}
	
	/**
     * Creates a new instance of a WFSClient.
     * @param 
     * host
	 * @throws IOException 
	 * @throws ConnectException 
     */
    public void createClient(URL host) throws ConnectException, IOException {
        remoteServicesClient = new WFSClient(host.toString());
    }
    
    /**
	 * Establishes the connection to the WfS server. Connecting to a WFS is
	 * an abstraction.<br>
	 * <p>
	 * Actually, it sends a GetCapabilities request to read the necessary 
	 * data for building further DescribeFeatureType requests.
	 * </p>
	 * @throws IOException, DriverException.
	 */
	public boolean connect() throws IOException, DriverException {
		return remoteServicesClient.connect(null);
	}
	
	/**
	 * Returns an array of WFSLayerNode's with the descriptors of 
	 * all features (retrieved using the getCapabilities operation)
	 * @return WFSLayerNode[]
	 */
	public WFSLayerNode[] getLayerList(){
		if (hashFeatures == null) {
			hashFeatures = new Hashtable();
			Hashtable wfsFeatures  = remoteServicesClient.getFeatures();
				
			Iterator keys = wfsFeatures.keySet().iterator();
			featuresList = new WFSLayerNode[wfsFeatures.size()];
			
			for (int i=0 ; i<wfsFeatures.size() ; i++){
				WFSLayerNode lyr = new WFSLayerNode();
				WFSFeature feature = (WFSFeature)wfsFeatures.get(keys.next());
				
				lyr.setName(feature.getName());
				lyr.setTitle(feature.getTitle());
				lyr.setAbstract(feature.getAbstract());
				lyr.setFields(feature.getFields());
				
				featuresList[i] = lyr;
				hashFeatures.put(lyr.getName(), lyr);
			}			
		}		
		return featuresList;
	}
	
	/**
	 * Returns all the feature information retrieved using a 
	 * describeFeatureTypeOpearion
	 * @param layerName
	 * Feature name
	 * @return
	 */
	public WFSLayerNode getLayerInfo(String layerName){
		WFSLayerNode lyr = (WFSLayerNode)hashFeatures.get(layerName);
		if (lyr.getFields().size() == 0){
			try {
				describeFeatureType(layerName);
				WFSFeature feature = (WFSFeature) remoteServicesClient.getFeatures().get(layerName);
				lyr.setFields(feature.getFields());
			} catch (WFSException e) {
				// The feature doesn't has fields
				e.printStackTrace();
			}
		}
		return lyr;
	}
	
	
	/**
     * @return The title of the service offered by the WMS server.
     */
    public String getServiceTitle() {
        return remoteServicesClient.getServiceInformation().title;
    }
    
    /**
     * @return The abstract of the service offered by the WMS server.
     */
    public String getServiceAbstract() {
        return remoteServicesClient.getServiceInformation().abstr;
    }
    
    /**
     * Returns a Hash table containing the values for each online resource.
     * Using as key a String with name of the WMS request and the value returned
     * by the hash is another string containing the corresponding Url  
     * @return HashTable
     */
    public Hashtable getOnlineResources() {
    	Hashtable onlineResources = new Hashtable();
    	ServiceInformation si = remoteServicesClient.getServiceInformation();
    	Iterator it = si.operations.keySet().iterator();
    	while (it.hasNext()) {
    		String key = (String) it.next();
    		String val = (String) si.operations.get(key);
    		if (val==null && (si.online_resource!=null || si.online_resource!= ""))
    			val = si.online_resource;
    		if (val!=null) {
    			onlineResources.put(key, val);
    		}
    	}
    	return onlineResources;
    }
    
    public String getOnlineResource(){
    	ServiceInformation si = remoteServicesClient.getServiceInformation();
    	return si.getOnline_resource();
    }
    
    /**
     * 
     * @return the host
     */
    public String getHost(){
        return remoteServicesClient.getHost();
    }   
    
    /**
     * @return the version of this client.
     */
    public String getVersion() {
        return remoteServicesClient.getVersion();
    }

	/**
	 * @return Returns the features.
	 */
	public ArrayList getFeatures() {
		return geotoolsClient.getFeatures();
	}

	/**
	 * @return Returns the hashRelate.
	 */
	public Hashtable getHashRelate() {
		return geotoolsClient.getHashRelate();
	}
	
    
 	/**
	 * @return Returns the numReg.
	 */
	public int getNumReg() {
		return geotoolsClient.getNumReg();
	}
	
 	/**
	 * @return Returns the numFiled.
	 */
	public int getNumField() {
		return geotoolsClient.getNumField();
	}
	
	/**
	 * @return Returns the attributes.
	 */
	public AttributeType[] getAttributes() {
		return geotoolsClient.getAttributes();
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.WFSDriver#close()
	 */
	public void close() {
		// TODO Auto-generated method stub
		
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.WFSDriver#open()
	 */	
	public void open() throws DriverException {
		// TODO Auto-generated method stub
		
	}

	/*
	 *  (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.driver.ObjectDriver#getPrimaryKeys()
	 */
	public int[] getPrimaryKeys() throws com.hardcode.gdbms.engine.data.driver.DriverException {
		throw new UnsupportedOperationException();
	}

	/*
	 *  (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.driver.ObjectDriver#write(com.hardcode.gdbms.engine.data.edition.DataWare)
	 */
	public void write(DataWare dataWare) throws com.hardcode.gdbms.engine.data.driver.DriverException {
		// TODO Auto-generated method stub
		
	}
	
	/*
	 *  (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.driver.GDBMSDriver#setDataSourceFactory(com.hardcode.gdbms.engine.data.DataSourceFactory)
	 */
	public void setDataSourceFactory(DataSourceFactory dsf) {
		// TODO Auto-generated method stub
		
	}

	/*
	 *  (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.driver.ReadAccess#getFieldValue(long, int)
	 */
	public Value getFieldValue(long rowIndex, int fieldId) throws com.hardcode.gdbms.engine.data.driver.DriverException {
		return ((IFeature) getFeatures().get((int) rowIndex)).getAttribute(fieldId);
	}

	/*
	 *  (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.driver.ReadAccess#getFieldCount()
	 */
	public int getFieldCount() throws com.hardcode.gdbms.engine.data.driver.DriverException {
		return getNumField();
	}

	/*
	 *  (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.driver.ReadAccess#getFieldName(int)
	 */
	public String getFieldName(int fieldId) throws com.hardcode.gdbms.engine.data.driver.DriverException {
		return getAttributes()[fieldId].getName();
	}

	/*
	 *  (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.driver.ReadAccess#getRowCount()
	 */
	public long getRowCount() throws com.hardcode.gdbms.engine.data.driver.DriverException {
		return getNumReg();
	}

	/*
	 *  (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.driver.ReadAccess#getFieldType(int)
	 */
	public int getFieldType(int i) throws com.hardcode.gdbms.engine.data.driver.DriverException {
		getAttributes()[i].getType();

		return FIELD_TYPE_STRING; 
	}

	/*
	 *  (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.driver.ReadAccess#getFieldWidth(int)
	 */
	public int getFieldWidth(int i) throws com.hardcode.gdbms.engine.data.driver.DriverException {
		// TODO Auto-generated method stub
		return 0;
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.WFSDriver#getFeatureIterator(java.awt.geom.Rectangle2D, java.lang.String)
	 */
	public IFeatureIterator getFeatureIterator(Rectangle2D r, String strEPSG) throws DriverException, IOException {
		return geotoolsClient.getIterator();
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.WFSDriver#getRowIndexByFID(com.iver.cit.gvsig.fmap.core.IFeature)
	 */
	public int getRowIndexByFID(IFeature FID) {
		int resul;
		Integer rowIndex = (Integer) getHashRelate().get(FID.getID());
		resul = rowIndex.intValue();
		return resul;
	}
	
}
