package com.iver.cit.gvsig.fmap.layers;

import java.awt.geom.Rectangle2D;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

import org.gvsig.remoteClient.wfs.WFSAttribute;
import org.gvsig.remoteClient.wfs.WFSStatus;

import com.hardcode.gdbms.engine.instruction.FieldNotFoundException;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.drivers.VectorialDriver;
import com.iver.cit.gvsig.fmap.drivers.WFSException;
import com.iver.cit.gvsig.fmap.drivers.wfs.FMapWFSDriver;
import com.iver.cit.gvsig.fmap.operations.strategies.Strategy;
import com.iver.cit.gvsig.fmap.operations.strategies.WFSStrategy;
import com.iver.cit.gvsig.fmap.rendering.LegendFactory;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: FLyrWFS.java,v 1.3 2006/05/25 10:31:55 jorpiell Exp $
 * $Log: FLyrWFS.java,v $
 * Revision 1.3  2006/05/25 10:31:55  jorpiell
 * Se ha renombrado la clase WFSFields por WFSAttributes porque era algo confusa
 *
 * Revision 1.2  2006/05/23 13:21:59  jorpiell
 * Si hay algn problema en la carga se muestra un mensaje de error
 *
 * Revision 1.1  2006/05/19 12:54:11  jorpiell
 * Creada la capa WFS
 *
 *
 */
/**
 *  FMap's WFS Layer class.
 *  
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class FLyrWFS extends FLyrVect{
	private URL host = null;
	private String onlineResource = null;
	private String name = null;
	private String layerName = null;
    private String userName = null;
    private String password = null;
    private WFSAttribute[] fields = null;
    private int numfeatures = 100;
    private int timeout = 10000;
    private FMapWFSDriver wfsDriver = null;
    private VectorialAdapter vectorialAdapter = null;
    private WFSStrategy wfsStrategy = null;

    
    /**
     * Constructor
     */
    public FLyrWFS(){
    	super();   	
    }    
    
    /**
     * Loads the features from the server
     */
    public void load() throws DriverIOException{
    	WFSAdapter adapter = new WFSAdapter();
    	try {
    		wfsDriver.getFeature(getWFSStatus());
    		
    		adapter.setDriver((VectorialDriver) wfsDriver);
    		setSource(adapter);
    		
    		
    		setLegend(LegendFactory.createSingleSymbolLegend(
    				getShapeType()));
    	} catch (WFSException e) {
    		// TODO Auto-generated catch block
    		e.printStackTrace();
    		throw new DriverIOException(e.getMessage());
    	}catch (FieldNotFoundException e) {
    		e.printStackTrace();
    		throw new DriverIOException(e.getMessage());
    	} catch (DriverException e) {
    		e.printStackTrace();
    		throw new DriverIOException(e.getMessage());
    	}
    }
    
    /**
     * Gets the WFS Status
     * @return
     */
    private WFSStatus getWFSStatus(){
    	WFSStatus status = new WFSStatus(getLayerName());
    	status.setUserName(getUserName());
    	status.setPassword(getPassword());
    	status.setBuffer(getNumfeatures());
    	status.setTimeout(getTimeout());
    	status.setFields(getFieldNames());
    	status.setOnlineResource(getOnlineResource());
	   	return status;
    }
 
    /**
     * Adds the vectorial adapter to the layer
     * @param vectorialAdapter
     */
    public void setSource(VectorialAdapter vectorialAdapter) {
    	this.vectorialAdapter = vectorialAdapter;
        super.setSource(vectorialAdapter);
    }
    
	/**
	 * @return Returns the layerName.
	 */
	public String getLayerName() {
		return layerName;
	}
	/**
	 * @param layerName The layerName to set.
	 */
	public void setLayerName(String layerName) {
		this.layerName = layerName;
	}
	/**
	 * @return Returns the numfeatures.
	 */
	public int getNumfeatures() {
		return numfeatures;
	}
	/**
	 * @param numfeatures The numfeatures to set.
	 */
	public void setNumfeatures(int numfeatures) {
		this.numfeatures = numfeatures;
	}
	
	/**
	 * @return Returns the pwd.
	 */
	public String getPassword() {
		return password;
	}
	/**
	 * @param pwd The pwd to set.
	 */
	public void setPassword(String password) {
		this.password = password;
	}
	/**
	 * @return Returns the timeout.
	 */
	public int getTimeout() {
		return timeout;
	}
	/**
	 * @param timeout The timeout to set.
	 */
	public void setTimeout(int timeout) {
		this.timeout = timeout;
	}
	/**
	 * @return Returns the user.
	 */
	public String getUserName() {
		return userName;
	}
	/**
	 * @param user The user to set.
	 */
	public void setUserName(String userName) {
		this.userName = userName;
	}
	/**
	 * @return Returns the fields.
	 */
	public WFSAttribute[] getFields() {
		return fields;
	}
	
	/**
	 * Return the attributes name
	 * @return
	 */
	public String[] getFieldNames(){
		String[] attributes = new String[fields.length];
		for (int i=0 ; i<fields.length ; i++){
			attributes[i] = fields[i].getName();
		}
		return attributes;
	}
	
	/**
	 * @param fields The fields to set.
	 */
	public void setFields(WFSAttribute[] fields) {
		this.fields = fields;
	}
	/**
	 * @return Returns the name.
	 */
	public String getName() {
		return name;
	}
	/**
	 * @param name The name to set.
	 */
	public void setName(String name) {
		this.name = name;
	}
	/**
	 * @return Returns the host.
	 */
	public URL getHost() {
		return host;
	}
	/**
	 * @param host The host to set.
	 */
	public void setHost(URL host) {
		this.host = host;		
	}

	/**
	 * @return Returns the wfs.
	 */
	public FMapWFSDriver getWfsDriver() {
		return wfsDriver;
	}

	/**
	 * @param wfs The wfs to set.
	 */
	public void setWfsDriver(FMapWFSDriver wfs) {
		this.wfsDriver = wfs;
	}
	
	  
    /**
     * @return wfsStrategy
     */
    public Strategy getStrategy() {
        if (wfsStrategy == null){
        	wfsStrategy = new WFSStrategy(this);
        }
    	return wfsStrategy;
    }    
    
    /*
     *  (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.layers.FLayer#getFullExtent()
     */
    public Rectangle2D getFullExtent() throws DriverException {
    	try {
			return wfsDriver.getFullExtent();
		} catch (IOException e) {
			throw new DriverException(e.toString());
		}
	}

	/**
	 * @return Returns the onlineResource.
	 */
	public String getOnlineResource() {
		return onlineResource;
	}

	/**
	 * @param onlineResource The onlineResource to set.
	 */
	public void setOnlineResource(String onlineResource) {
		this.onlineResource = onlineResource;
	}
}
