/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.operations.strategies;

import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.IOException;

import org.cresques.cts.ICoordTrans;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.core.IFeature;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.core.v02.FSymbol;
import com.iver.cit.gvsig.fmap.drivers.IFeatureIterator;
import com.iver.cit.gvsig.fmap.drivers.WFSDriver;
import com.iver.cit.gvsig.fmap.layers.FBitSet;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrVect;
import com.iver.cit.gvsig.fmap.layers.VectorialAdapter;
import com.iver.cit.gvsig.fmap.layers.layerOperations.ClassifiableVectorial;
import com.iver.cit.gvsig.fmap.layers.layerOperations.Selectable;
import com.iver.cit.gvsig.fmap.layers.layerOperations.SingleLayer;
import com.iver.cit.gvsig.fmap.rendering.VectorialLegend;
import com.iver.utiles.swing.threads.Cancellable;
/**
 * Estrategia para WFS.
 *
 * @author Vicente Caballero Navarro
 */
public class WFSStrategy extends DefaultStrategy {
	
	public WFSStrategy(FLayer capa) {
		super(capa);
	}
	
	
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.operations.strategies.Strategy#draw(java.awt.image.BufferedImage, java.awt.Graphics2D, com.iver.cit.gvsig.fmap.ViewPort, com.iver.cit.gvsig.fmap.operations.Cancellable)
	 */
	public void draw(BufferedImage image, Graphics2D g, ViewPort viewPort, Cancellable cancel) throws DriverException {
//		// Nos aprovechamos del SQL para lanzar la consulta
//		// teniendo en cuenta el boundingbox que toca.
//		com.iver.cit.gvsig.gui.View activeView = 
//			(com.iver.cit.gvsig.gui.View) PluginServices.getMDIManager().getActiveView();
		
		FLayer capa = getCapa();
		ICoordTrans ct = getCapa().getCoordTrans();
		VectorialAdapter wfsAdapter =  (VectorialAdapter) ((SingleLayer) capa).getSource();
		if (wfsAdapter==null){
			capa.setActive(false);
			capa.setVisible(false);
			throw new DriverException("Error en la conexin");
		}
		WFSDriver wfsDriver = (WFSDriver) wfsAdapter.getDriver();
		
		
		wfsDriver.open();
		
		FLyrVect flryVect = (FLyrVect)getCapa();
		Selectable selectable = flryVect.getRecordset();
		FBitSet bitSet = selectable.getSelection();
			
		String strEPSG = viewPort.getProjection().getAbrev().substring(5);
		IFeatureIterator geomIt=null;
		try{
			geomIt = wfsDriver.getFeatureIterator(viewPort.getAdjustedExtent(), strEPSG);
		}catch (IOException e) {
			throw new DriverException(e);
		}
		
		if (geomIt == null){
			return;
		}
		
		VectorialLegend vectorialLegend = (VectorialLegend) ((ClassifiableVectorial) capa).getLegend();
		int rowIndex;
		FSymbol symbol;
		while (geomIt.hasNext()){
			if (cancel.isCanceled()){
				wfsDriver.close();
				break;
			}
			IFeature feat = geomIt.next();
			IGeometry geom = feat.getGeometry();
			if (ct != null) {
				geom.reProject(ct);
			}
			rowIndex = wfsDriver.getRowIndexByFID(feat);
			symbol = vectorialLegend.getSymbolByFeature(feat);                
			if (symbol == null){
				continue;
			}
			if (bitSet.get(rowIndex)) {
				symbol = FSymbol.getSymbolForSelection(symbol);
			}
			geom.drawInts(g,viewPort, symbol);
		}			
	}
	
	
}
