package com.iver.cit.gvsig.gui.wizards;

import java.awt.Component;
import java.awt.geom.Rectangle2D;
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;

import com.iver.andami.PluginServices;
import com.iver.andami.persistence.serverData.ServerDataPersistence;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrWFS;
import com.iver.cit.gvsig.gui.WizardPanel;
import com.iver.cit.gvsig.gui.panels.WFSParamsPanel;
import com.iver.utiles.NotExistInXMLEntity;
import com.iver.utiles.XMLEntity;
import com.iver.utiles.swing.jcomboServer.JComboServer;
import com.iver.utiles.swing.jcomboServer.ServerData;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: WFSWizard.java,v 1.8 2006/05/25 16:22:59 jorpiell Exp $
 * $Log: WFSWizard.java,v $
 * Revision 1.8  2006/05/25 16:22:59  jorpiell
 * Se limpia el panel cada vez que se conecta con un servidor distinto
 *
 * Revision 1.7  2006/05/25 16:01:51  jorpiell
 * Se ha aadido la funcionalidad para eliminar el namespace de los tipos de atributos
 *
 * Revision 1.6  2006/05/25 10:29:07  jorpiell
 * Aadido el checkbox para que se  haga uso de la cache
 *
 * Revision 1.5  2006/05/24 12:08:53  jorpiell
 * Cambiado el nombre de la pestaa a WFS
 *
 * Revision 1.4  2006/05/23 13:21:59  jorpiell
 * Si hay algn problema en la carga se muestra un mensaje de error
 *
 * Revision 1.3  2006/05/23 08:09:53  jorpiell
 * Se ha cambiado la forma en la que se leian los valores seleccionados en los paneles y se ha cambiado el comportamiento de los botones
 *
 * Revision 1.2  2006/05/19 12:58:03  jorpiell
 * Modificados algunos paneles
 *
 * Revision 1.1  2006/04/19 12:50:16  jorpiell
 * Primer commit de la aplicacin. Se puede hacer un getCapabilities y ver el mensaje de vienvenida del servidor
 *
 *
 */
/**
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class WFSWizard extends WizardPanel {
	private int page = 0;
	private boolean connected = false;
	private JComboServer cmbHost = null;
	private javax.swing.JButton btnConnect = null;
	private javax.swing.JPanel jPanel = null;
	private javax.swing.JLabel jLabel1 = null;
	private javax.swing.JLabel lblTitle = null;
	private javax.swing.JScrollPane jScrollPane = null;
	private javax.swing.JTextArea txtAbstract = null;
	private javax.swing.JPanel panelPage1 = null;
	private javax.swing.JButton btnSiguiente = null;
	private javax.swing.JButton btnAnterior = null;
	private javax.swing.JPanel jPanel1 = null;
	private WizardListenerSupport listenerSupport = new WizardListenerSupport();
	private WFSParamsPanel wfsParamsPanel = null;
	private WFSWizardData dataSource = null;
	private JLabel lblServerType = null;
	private JLabel lblServerTypeValue = null;
	private JCheckBox chkCaching = null;
	
	/**
	 * This is the default constructor
	 */
	public WFSWizard() {
		super();
		initialize();
	}

	/**
	 * This method initializes this
	 */
	private void initialize() {
		setTabName("WFS");
		lblServerType = new JLabel();
		lblServerType.setBounds(20, 444, 100, 20);
		lblServerType.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
		lblServerType.setHorizontalTextPosition(javax.swing.SwingConstants.RIGHT);
		lblServerType.setFont(new java.awt.Font("Arial", java.awt.Font.PLAIN, 11));
		lblServerType.setText(PluginServices.getText(this, "server_type")+":");
		lblServerTypeValue = new JLabel();
		lblServerTypeValue.setBounds(128, 444, 148, 20);
		lblServerTypeValue.setFont(new java.awt.Font("Arial", java.awt.Font.BOLD, 11));
		lblServerTypeValue.setText("-");		
		this.setLayout(null);
		this.setPreferredSize(new java.awt.Dimension(750, 320));
		this.setVisible(true);
		this.setSize(510, 468);
		this.add(getPanelPage1(), null);
		this.add(getPanelPage2(), null);
		this.add(getBtnAnterior(), null);
		this.add(getBtnSiguiente(), null);
		this.add(lblServerType, null);
		this.add(lblServerTypeValue, null);
		activarVisualizarBotones();
	}

	/**
	 * Adds the host addres to a persistent data storage.
	 * 
	 * con addHost guardamos la informacin del combo de servidores en un
	 * almacn persistente
	 * 
	 * @param host
	 */
	private void addHost(String host) {
		host = host.trim();
		ServerDataPersistence persistence = new ServerDataPersistence(this,ServerData.SERVER_TYPE_WMS);
		persistence.addServerData(new ServerData(host, ServerData.SERVER_TYPE_WMS));
	}

	/**
	 * Fills up the initial WCSWizard controls.
	 * 
	 * Rellena los primeros controles del WCS Wizard
	 * 
	 * jaume
	 */
	private void rellenarControles() {
		try {
			String host = cmbHost.getModel().getSelectedItem().toString();
					
			dataSource.setHost(new URL(host));
			addHost(host);
			getLblTitle().setText(dataSource.getTitle());
			getTxtAbstract().setText(dataSource.getAbstract());
			lblServerTypeValue.setText(dataSource.getServerType());
			wfsParamsPanel.setWizardData(dataSource);
			
			connected = true;
			activarVisualizarBotones();
		} catch (Exception e) {
			listenerSupport.callError(e);
			e.printStackTrace();
		}
	}

	/**
	 * Enables or disables the Next and Previous buttons according with the current
	 * page and the enabled pages.
	 */
	private void activarVisualizarBotones() {
		if (page == 0) {
			getBtnAnterior().setEnabled(false);
			getBtnSiguiente().setVisible(true);
			if (connected) {
				getBtnSiguiente().setEnabled(true);
			} else {
				getBtnSiguiente().setEnabled(false);
			}
		} else if (page < wfsParamsPanel.getNumTabs()) {
			getBtnSiguiente().setEnabled(true);
			getBtnSiguiente().setVisible(true);
			getBtnAnterior().setEnabled(true);
			getBtnAnterior().setVisible(true);
			if (wfsParamsPanel.nextPageEnabled()==-1){
			    getBtnSiguiente().setEnabled(false);
			}
			listenerSupport.callStateChanged(wfsParamsPanel.isCorretlyConfigured());
		} else if (page == wfsParamsPanel.getNumTabs()) {
			getBtnSiguiente().setVisible(false);
			getBtnSiguiente().setEnabled(false);
			listenerSupport.callStateChanged(wfsParamsPanel 
					.isCorretlyConfigured());
		}

	}
	


	/**
	 * This method initializes txtHost from the persistent data storage
	 * 
	 * @return javax.swing.JTextField
	 */
	private javax.swing.JComboBox getTxtHost() {
		if (cmbHost == null) {
			cmbHost = new JComboServer();
			cmbHost.setModel(new DefaultComboBoxModel());
			cmbHost.setPreferredSize(new java.awt.Dimension(350, 20));
			cmbHost.setBounds(11, 26, 454, 20);
			cmbHost.setEditable(true);	

			ServerDataPersistence persistence = new ServerDataPersistence(this,ServerData.SERVER_TYPE_WFS);
			ServerData[] servers = persistence.getArrayOfServerData();

			if (servers.length == 0){
				PluginServices ps = PluginServices.getPluginServices(this);
				XMLEntity xml = ps.getPersistentXML();
				try {
					String[] oldServers = xml.getStringArrayProperty("wfs-servers");
					servers = new ServerData[oldServers.length];
					for (int i=0; i<oldServers.length; i++) {
						servers[i] = new ServerData(oldServers[i],ServerData.SERVER_TYPE_WMS);
					}
					xml.remove("wfs-servers");
					ps.setPersistentXML(xml);
				} catch (NotExistInXMLEntity e) {
					// Old servers doesn't exist
					servers = new ServerData[1];
					servers[0] = new ServerData("http://www2.dmsolutions.ca/cgi-bin/mswfs_gmap",ServerData.SERVER_TYPE_WFS);
				}

				for (int i=0 ; i<servers.length ; i++){
					persistence.addServerData(servers[i]);			       
				}
				persistence.setPersistent();		
			}

			cmbHost.setServerList(servers);
		}
		return cmbHost;
	}

	
	/**
	 * This method initializes btnDetalles
	 * 
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnConnect() {
		if (btnConnect == null) {
			btnConnect = new javax.swing.JButton();
			btnConnect.setPreferredSize(new java.awt.Dimension(100, 20));
			btnConnect.setBounds(366, 50, 100, 20);
			btnConnect.setText(PluginServices.getText(this, "connect"));
			btnConnect.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					rellenarControles();
				}
			});
		}

		return btnConnect;
	}

	/**
	 * This method initializes jPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private javax.swing.JPanel getJPanel() {
		if (jPanel == null) {
			jPanel = new javax.swing.JPanel();
			jPanel.setLayout(null);
			jPanel.setPreferredSize(new java.awt.Dimension(470, 130));
			jPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(null,
					PluginServices.getText(this, "descripcion"),
					javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
					javax.swing.border.TitledBorder.DEFAULT_POSITION, null,
					null));
			jPanel.setBounds(2, 96, 477, 324);
			jPanel.add(getJLabel1(), null);
			jPanel.add(getLblTitle(), null);
			jPanel.add(getJScrollPane(), null);
		}

		return jPanel;
	}

	/**
	 * This method initializes jLabel1
	 * 
	 * @return javax.swing.JLabel
	 */
	private javax.swing.JLabel getJLabel1() {
		if (jLabel1 == null) {
			jLabel1 = new javax.swing.JLabel();
			jLabel1.setText(PluginServices.getText(this, "nombre") + ":");
			jLabel1.setBounds(37, 26, 40, 15);
		}

		return jLabel1;
	}

	/**
	 * This method initializes lblTitle
	 * 
	 * @return javax.swing.JLabel
	 */
	private javax.swing.JLabel getLblTitle() {
		if (lblTitle == null) {
			lblTitle = new javax.swing.JLabel();
			lblTitle.setText("-");
			lblTitle.setPreferredSize(new java.awt.Dimension(350, 16));
			lblTitle.setBounds(82, 26, 350, 16);
		}

		return lblTitle;
	}

	/**
	 * This method initializes jScrollPane
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private javax.swing.JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new javax.swing.JScrollPane();
			jScrollPane.setViewportView(getTxtAbstract());
			jScrollPane.setPreferredSize(new java.awt.Dimension(450, 60));
			jScrollPane.setBounds(10, 47, 450, 267);
		}

		return jScrollPane;
	}

	/**
	 * This method initializes txtAbstract
	 * 
	 * @return javax.swing.JTextArea
	 */
	private javax.swing.JTextArea getTxtAbstract() {
		if (txtAbstract == null) {
			txtAbstract = new javax.swing.JTextArea();
			txtAbstract.setWrapStyleWord(true);
			txtAbstract.setColumns(30);
			txtAbstract.setLineWrap(true);
		}

		return txtAbstract;
	}

	/**
	 * This method initializes panelPage1
	 * 
	 * @return javax.swing.JPanel
	 */
	private javax.swing.JPanel getPanelPage1() {
		if (panelPage1 == null) {
			panelPage1 = new javax.swing.JPanel();
			panelPage1.setLayout(null);
			panelPage1.setPreferredSize(new java.awt.Dimension(480, 220));
			panelPage1.setVisible(true);
			panelPage1.setBounds(15, 5, 480, 427);
			panelPage1.add(getJPanel1(), null);
			panelPage1.add(getJPanel(), null);
		}

		return panelPage1;
	}

	/**
	 * This method initializes panelPage2
	 * 
	 * @return javax.swing.JPanel
	 */
	private javax.swing.JPanel getPanelPage2() {
		if (wfsParamsPanel == null) {
			wfsParamsPanel = new WFSParamsPanel();
			wfsParamsPanel.setListenerSupport(this.listenerSupport);
			wfsParamsPanel.setBounds(15, 5, 480, 428);
			wfsParamsPanel.getJTabbedPane().addMouseListener(
					new java.awt.event.MouseAdapter() {
						public void mouseClicked(java.awt.event.MouseEvent e) {
							page = wfsParamsPanel.currentPage() + 1;
							activarVisualizarBotones();
						}
					});
		}

		return wfsParamsPanel;
	}



	/**
	 * This method initializes btnSiguiente
	 * 
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnSiguiente() {
		if (btnSiguiente == null) {
			btnSiguiente = new javax.swing.JButton();
			btnSiguiente.setPreferredSize(new java.awt.Dimension(100, 30));
			btnSiguiente.setText(PluginServices.getText(this, "siguiente"));
			btnSiguiente.setBounds(395, 444, 100, 20);
			btnSiguiente.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					if (page > 0) {
						// si page es mayor que cero, vamos enfocando las
						// sucesivas SOLAPAS del wfsParamsPanel
						wfsParamsPanel.avanzaTab();
						page = wfsParamsPanel.getIndiceSolapaActual();
					}
					page++;

					getPanelPage1().setVisible(false);
					getPanelPage2().setVisible(true);
					activarVisualizarBotones();
				}
			});
		}
		return btnSiguiente;
	}

	/**
	 * This method initializes btnAnterior
	 * 
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnAnterior() {
		if (btnAnterior == null) {
			btnAnterior = new javax.swing.JButton();
			btnAnterior.setText(PluginServices.getText(this, "anterior"));
			btnAnterior.setPreferredSize(new java.awt.Dimension(100, 30));
			btnAnterior.setBounds(292, 444, 100, 20);
			btnAnterior.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					page --;
					if (page > 0) {
						// Tenemos que retroceder en el wfsParamsPanel
						wfsParamsPanel.retrocedeTab();
						page = wfsParamsPanel.getIndiceSolapaActual() + 1;
						activarVisualizarBotones();
					} else if (page==0){
						activarVisualizarBotones();
						page = 1;
						wfsParamsPanel.cleanupWizard();
						getLblTitle().setText("-");
						getTxtAbstract().setText("");
						wfsParamsPanel.retrocedeTab();
						getPanelPage1().setVisible(true);
						getPanelPage2().setVisible(false);
						getBtnSiguiente().setEnabled(false);
						connected = false; 
					}
					listenerSupport.callStateChanged(wfsParamsPanel.isCorretlyConfigured());
				}
			});
		}
		return btnAnterior;
	}

	/**
	 * This method initializes jPanel1
	 * 
	 * @return javax.swing.JPanel
	 */
	private javax.swing.JPanel getJPanel1() {
		if (jPanel1 == null) {
			jPanel1 = new javax.swing.JPanel();
			jPanel1.setLayout(null);
			jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder(
					null, PluginServices.getText(this, "server"),
					javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
					javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
			jPanel1.setBounds(2, 5, 477, 85);
			jPanel1.add(getTxtHost(), null);
			jPanel1.add(getBtnConnect(), null);
			jPanel1.add(getChkCaching(), null);
		}

		return jPanel1;
	}

	/**
	 * Adds the gvSIG's wizard listener 
	 * 
	 * @param listener
	 */
	public void addWizardListener(WizardListener listener) { 
		listenerSupport.addWizardListener(listener);
	}

	/**
	 * Removes the gvSIG's wizard listener
	 * 
	 * @param listener
	 */
	public void removeWizardListener(WizardListener listener) {
		listenerSupport.removeWizardListener(listener);
	}

	/**
	 * Returns the Wizard's data source
	 * 
	 * @return
	 */
	public WFSWizardData getDataSource() {
		return dataSource;
	}

	/**
	 * sets the wizard's data source
	 * 
	 * @param source
	 */
	public void setDataSource(WFSWizardData source) {
		dataSource = source;
	}

	/**
	 * Returns the rectangle that contains all the others
	 * 
	 * Obtiene el rectngulo que contiene a todos los otros
	 * 
	 * @param rects[]  rectangulos
	 * 
	 * @return Rectangle2D
	 */
	public static Rectangle2D getRectangle(Rectangle2D[] rects) {
		Rectangle2D ret = rects[0];

		for (int i = 1; i < rects.length; i++) {
			ret.add(rects[i]);
		}

		return ret;
	}

	/**
	 * Returns the host typed in the host text field or null if nothing is typed
	 * or it is not a valid URL.
	 * 
	 * Devuelve el host que est escrito en el cuadro de texto del host o null
	 * en caso de que no haya nada escrito o est mal escrito
	 * 
	 * @return URL del host
	 */
	public URL getHost() {
		try {
			return new URL(cmbHost.getModel().getSelectedItem().toString());
		} catch (MalformedURLException e) {
			return null;
		}
	}

 
	public void initWizard() {
		setDataSource(new WFSWizardData());
	}

	public void execute() {
	}
	
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.gui.WizardPanel#getLayer()
	 */
	public FLayer getLayer() {
		FLyrWFS layer = (FLyrWFS)wfsParamsPanel.getLayer();
		layer.setHost(this.getHost());
		layer.setWfsDriver(dataSource.getDriver());
		layer.setOnlineResource(dataSource.getOnlineResource());
		try {
			layer.load();
		} catch (DriverIOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
				 	PluginServices.getText(this,"cantLoad"));
			return null;
		}
		
		return layer;
	}
	
	/**
	 * This method initializes chkCaching	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */    
	private JCheckBox getChkCaching() {
		if (chkCaching == null) {
			chkCaching = new JCheckBox();
			chkCaching.setBounds(7, 51, 349, 20);
			chkCaching.setText(PluginServices.getText(this, "refresh_capabilities"));
			chkCaching.setToolTipText(PluginServices.getText(this, "refresh_capabilities_tooltip"));
			//chkCaching.setSelected(refreshing);
			chkCaching.addItemListener(new java.awt.event.ItemListener() { 
				public void itemStateChanged(java.awt.event.ItemEvent e) {
					//refreshing = chkCaching.isSelected();
				}
			});
			chkCaching.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					//fPrefs.putBoolean("refresh_capabilities", chkCaching.isSelected());
				}
			});

		}
		return chkCaching;
	}



}
