/*
 * Creado el 7-marzo-2005
 */
package com.iver.cit.gvsig.gui.dialogs;

import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.util.ArrayList;
import java.util.Vector;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.filechooser.FileFilter;

import org.cresques.cts.IProjection;
import org.cresques.io.GeoRasterFile;
import org.cresques.io.raster.RasterFilterStackManager;
import org.cresques.px.Extent;
import org.cresques.px.PxRaster;
import org.cresques.ui.raster.BandSetupPanel;
import org.cresques.ui.raster.EnhancedPanel;
import org.cresques.ui.raster.FilterRasterDialogPanel;
import org.cresques.ui.raster.RasterTransparencyPanel;

import com.hardcode.driverManager.Driver;
import com.hardcode.driverManager.DriverLoadException;
import com.iver.andami.PluginServices;
import com.iver.andami.messages.NotificationManager;
import com.iver.andami.ui.mdiManager.View;
import com.iver.andami.ui.mdiManager.ViewInfo;
import com.iver.cit.gvsig.fmap.drivers.RasterDriver;
import com.iver.cit.gvsig.fmap.layers.FLyrWMS;
import com.iver.cit.gvsig.fmap.layers.LayerFactory;
import com.iver.cit.gvsig.fmap.layers.StatusLayerRaster;

/**
 * <P>
 * Dialog for the properties of a WMS layer. It manages the avants and aplies
 * filters to the raster through the filter stack manager according to the user's
 * selection. This dialog contains some panels.
 * </P>
 * <UL>
 * <LI>Propierties</LI>
 * <LI>Band selection</LI>
 * <LI>Transparency</LI>
 * <LI>Enhancement</LI>
 * </UL>
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class WMSRasterPropsDialog extends FilterRasterDialogPanel implements View {
	
	private FilterRasterDialogPanel 				contentPane = null;  	
	private JPanel									propPanel = null;
	private IProjection 							currentProjection = null;
	private FLyrWMS		 							fLayer = null;
	private static final int						nprops = 11;
	private Object[][]								props = null;
	private RasterFilterStackManager 				stackManager = null;
	private Status									status = null;
	private StatusLayerRaster						rasterStatus = null;
	private JFileChooser							fileChooser = null;
	private String									lastPath = new String("./");
	private	PxRaster								px = null;
	private GeoRasterFile							grf = null;
	
	/**
	 * Class that holds the dialog state and restores the initial state if it
	 * is cancelled.
	 * @author Nacho Brodin <brodin_ign@gva.es>
	 */
	class Status{
		public String					inicAlpha;
		public int 						bandR;
		public int 						bandG;
		public int 						bandB;
		private ArrayList				filters = new ArrayList();
		
		public Status(String alpha, int bandR, int bandG, int bandB){
			this.inicAlpha = alpha;
			this.bandR = bandR;
			this.bandG = bandG;
			this.bandB = bandB;
			filters = stackManager.getStringsFromStack();
		}
				
		/**
		 * Restaura el Estado salvado 
		 * @param status Estado
		 */
		public void restoreStatus(WMSRasterPropsDialog props){
			//Devolvemos la pila de filtros al estado inicial
			/*if(stackManager != null)
				stackManager.deleteTempFilters();*/
			
			//Devolvemos el alpha al estado inicial
			int opac = Integer.parseInt(status.inicAlpha);
			opac = (int)((opac*255)/100);
			fLayer.setTransparency(255-opac);
			
			if (fLayer != null) {
				fLayer.setBandR(bandR);
				fLayer.setBandG(bandG);
				fLayer.setBandB(bandB);
			}
			
			//Restauramos los filtros
			if(filters!=null)
				stackManager.createStackFromStrings(filters);
			
			fLayer.getFMap().invalidate();
			
		}
	}
	
	public class DriverFileFilter extends FileFilter{
		
		private Driver driver;
		
		public DriverFileFilter(String driverName) throws DriverLoadException{
			driver = LayerFactory.getDM().getDriver(driverName);
		}

		/**
		 * @see javax.swing.filechooser.FileFilter#accept(java.io.File)
		 */
		public boolean accept(File f) {
			if (f.isDirectory()) return true;
			if (driver instanceof RasterDriver){
				return ((RasterDriver) driver).fileAccepted(f);
			}else{
				throw new RuntimeException("Tipo no reconocido");
			}
		}

		/**
		 * @see javax.swing.filechooser.FileFilter#getDescription()
		 */
		public String getDescription() {
			return ((Driver) driver).getName();
		}
	}
	
	/**
	 * Dialog window constructor.
	 * 
	 * @param app
	 */
	public WMSRasterPropsDialog(FLyrWMS layer, ArrayList ranges){
		super();
		fLayer = layer;
		this.px = layer.getPxRaster();
		this.grf = layer.getGeoRasterFile();
		if(fLayer.getStatus()==null){
			rasterStatus = new StatusLayerRaster();
			fLayer.setStatus(rasterStatus);
		}else
			rasterStatus = (StatusLayerRaster)fLayer.getStatus();
		init();
		this.setRanges(ranges);
		FilterRasterDialogPanel fr = ((FilterRasterDialogPanel)this.getContentPane());
		BandSetupPanel bandSetup = (BandSetupPanel)fr.getPanelByClassName("BandSetupPanel");
		bandSetup.getFileList().getJButtonAdd().setEnabled(false);
		bandSetup.getFileList().getJButtonRemove().setEnabled(false);
		this.setTranslation();
		
	}
	
	/**
	 * Asigna los textos a los paneles
	 */
	private void setTranslation(){
		FilterRasterDialogPanel fr = ((FilterRasterDialogPanel)this.getContentPane());
		BandSetupPanel bandSetup = (BandSetupPanel)fr.getPanelByClassName("BandSetupPanel");
		bandSetup.getFileList().getJButtonAdd().setText(PluginServices.getText(this,"add"));
		bandSetup.getFileList().getJButtonRemove().setText(PluginServices.getText(this,"remove"));
		bandSetup.getFileList().lbandasVisibles.setText(PluginServices.getText(this,"bands"));
		
		RasterTransparencyPanel tpan = (RasterTransparencyPanel)fr.getPanelByClassName("RasterTransparencyPanel");
				
		tpan.getTransparencyCheck().setText(PluginServices.getText(this,"transparencia"));
		tpan.getOpacityCheck().setText(PluginServices.getText(this,"opacidad"));
			
		/*EnhancedPanel ep =  (EnhancedPanel)fr.getPanelByClassName("EnhancedPanel");
		ep.lLineal.setText(PluginServices.getText(this,"lineal_directo"));
		ep.lQueue.setText(PluginServices.getText(this,"recorte_colas"));
		ep.lWithoutEnhanced.setText(PluginServices.getText(this,"sin_realce"));
		ep.lCut.setText(PluginServices.getText(this,"recorte"));
		ep.lRemove.setText(PluginServices.getText(this,"eliminar_extremos"));*/
		
		//Recorremos los Tab y traducimos el nombre
		for(int i=0;i<this.getTab().getTabCount();i++){
			if(this.getTab().getTitleAt(i).equals("Info"))
				this.getTab().setTitleAt(i,PluginServices.getText(this,"info"));
			if(this.getTab().getTitleAt(i).equals("Transparencia"))
				this.getTab().setTitleAt(i,PluginServices.getText(this,"transparencia"));
			if(this.getTab().getTitleAt(i).equals("Bands"))
				this.getTab().setTitleAt(i,PluginServices.getText(this,"bands"));
			if(this.getTab().getTitleAt(i).equals("Realce"))
				this.getTab().setTitleAt(i,PluginServices.getText(this,"realce"));
		}
		
		this.getAcceptButton().setText(PluginServices.getText(this,"ok"));
		this.getApplyButton().setText(PluginServices.getText(this,"apply"));
		this.getCancelButton().setText(PluginServices.getText(this,"cancel"));
	}
	
	/**
	 * Assigns a FLayerRaster
	 * @param layer	capa a asignar
	 */
	public void setFLyrWMS(FLyrWMS layer){
		fLayer = layer;
	}
	
	/**
	 * Dialog window constructor.
	 */
	public WMSRasterPropsDialog() {
		init();
	}
			
	/**
	 * Loads the info panel data.
	 */
	private void loadInfoData(){
		Rectangle2D r = fLayer.getFullExtent();
		if(fLayer!=null){
			props = new Object[nprops][2];
			props[0][0] = new String(PluginServices.getText(this,"Fichero"));
			props[0][1] = "http://"+fLayer.host.getHost()+fLayer.host.getFile();
			props[1][0] = new String(PluginServices.getText(this,"num_bandas"));
			props[1][1] = new String(String.valueOf( grf.getBandCount()));
			props[2][0] = new String(PluginServices.getText(this,"ancho_alto"));
			props[2][1] = (int)(fLayer.getWidth())+" X "+(int)(fLayer.getHeight());
			props[3][0] = new String(PluginServices.getText(this,"format"));
			props[3][1] = fLayer.m_Format;
			props[4][0] = new String(PluginServices.getText(this,"tipo_dato"));
			String type = null;
			switch(grf.getDataType()){
				case 0: type = new String("BYTE");break;
				case 1: type = new String("USHORT");break;
				case 2: type = new String("SHORT");break;
				case 3: type = new String("INT");break;
				case 4: type = new String("FLOAT");break;
				case 5: type = new String("DOUBLE");break;
				default: type = new String("UNDEFINED");break;
			}
		    props[4][1] = type;
			props[5][0] = new String(PluginServices.getText(this,"coor_geograficas"));
			props[6][0] = new String(PluginServices.getText(this,"xmin"));
			props[6][1] = String.valueOf(fLayer.getMinX());
			props[7][0] = new String(PluginServices.getText(this,"ymin"));
			props[7][1] = String.valueOf(fLayer.getMinY());
			props[8][0] = new String(PluginServices.getText(this,"xmax"));
			props[8][1] = String.valueOf(fLayer.getMaxX());
			props[9][0] = new String(PluginServices.getText(this,"ymax"));
			props[9][1] = String.valueOf(fLayer.getMaxY());
			
		}else{
			props = new Object[1][2];
			props[0][0] = new String("No props");
			props[0][1] = new String("-");
		}
		
	}
	
	/**
	 * Aade bandas al contador de bandas del FilterRasterDialogPanel
	 * @param numBands Nmero de bandas a aadir
	 */
	public void addNumBands(int numBands){
		nbands += numBands;
		FilterRasterDialogPanel fr = ((FilterRasterDialogPanel)this.getContentPane());
		RasterTransparencyPanel rasterTrans = (RasterTransparencyPanel)fr.getPanelByClassName("RasterTransparencyPanel");
		if(rasterTrans != null && rasterTrans.getPTranspByPixel().isControlEnabled())
			rasterTrans.setActiveTransparencyControl(true); 
	}
	
	/**
	 * Inits the jDialog	
	 */    
	public void init() {
		
		//this.setLayout(new FlowLayout());
			
		setName("filterRaster");
			   		
   		this.loadInfoData();
		super.init(props);
		this.setTabVisible("Pansharpening", false);
		
		//this.add(getContentPane());
	
		this.setSize(486, 338);
		
		//contentPane.getAcceptButton().setEnabled(false);
		this.getAcceptButton().addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				acceptButtonActionPerformed(evt);
				closeJDialog();
			}
		});
		this.getCancelButton().addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				cancelButtonActionPerformed(evt);
				closeJDialog();
			}
		});
		this.getApplyButton().addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				acceptButtonActionPerformed(evt);
			}
		});
		BandSetupPanel bandSetup = (BandSetupPanel)((FilterRasterDialogPanel)this.getContentPane()).getPanelByClassName("BandSetupPanel");
		bandSetup.getFileList().getJButtonAdd().addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt){
				addFileBand(evt);
				
			}
		});
		bandSetup.getFileList().getJButtonRemove().addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt){
				delFileBand(evt);
			}
		});

	
	}
	
	/**
	 * Saves the initial state to restore it if cancel.
	 */
	public void readStat(){
		FilterRasterDialogPanel fr = ((FilterRasterDialogPanel)this.getContentPane());
		RasterTransparencyPanel rasterTrans = (RasterTransparencyPanel)fr.getPanelByClassName("RasterTransparencyPanel");
		status = new Status(rasterTrans.getOpacityText().getText(),
							getAssignedBand(GeoRasterFile.RED_BAND),
							getAssignedBand(GeoRasterFile.GREEN_BAND),
							getAssignedBand(GeoRasterFile.BLUE_BAND)
							);				
	}
	
	/**
	 * This method initializes jContentPane		
	 */    
	public Container getContentPane() {
		return this;
	}
	
	/**
	 * Sets a projection.
	 * 
	 * @param prj
	 */
	public void setProjection(IProjection prj) {
		this.currentProjection = prj;
	}
	
	
	
	public void closeJDialog() {
		PluginServices.getMDIManager().closeView(WMSRasterPropsDialog.this);
	}
	
	/**
	 * Sets the RasterFilterStackManager
	 * @param stackManager
	 */
	public void setRasterFilterStackManager(RasterFilterStackManager stackManager){
		this.stackManager = stackManager;
		stackManager.resetTempFilters();
	}
	
	/**
	 * 
	 * @param flag
	 * @return
	 */
	public int getAssignedBand(int flag) {
		BandSetupPanel bandSetup = (BandSetupPanel)((FilterRasterDialogPanel)this.getContentPane()).getPanelByClassName("BandSetupPanel");
		return bandSetup.getAssignedBand(flag);
	}
	
	/**
	 * Pressing OK with the bands panel selected makes its values to be
	 * processed.
	 * 
	 * @return true if the bands panel was selected and the action is processed, false it
	 * was not selected.
	 */
	public boolean processBandPanel(){
		fLayer.setBandR(getAssignedBand(GeoRasterFile.RED_BAND));
		fLayer.setBandG(getAssignedBand(GeoRasterFile.GREEN_BAND));
		fLayer.setBandB(getAssignedBand(GeoRasterFile.BLUE_BAND));
		rasterStatus.bandR = getAssignedBand(GeoRasterFile.RED_BAND);
		rasterStatus.bandG = getAssignedBand(GeoRasterFile.GREEN_BAND);
		rasterStatus.bandB = getAssignedBand(GeoRasterFile.BLUE_BAND);
			
		//Comprobamos si hay alguna banda que no est asignada y aplicamos el filtro
		StringBuffer sb = new StringBuffer();
		if(getAssignedBand(GeoRasterFile.RED_BAND) == -1)
			sb.append("R");
		if(getAssignedBand(GeoRasterFile.GREEN_BAND) == -1)
			sb.append("G");
		if(getAssignedBand(GeoRasterFile.BLUE_BAND) == -1)
			sb.append("B");
			
		if(!sb.toString().equals(""))
			stackManager.addRemoveBands(sb.toString());
		else
			stackManager.removeFilter(stackManager.getTypeFilter("removebands"));
			
		fLayer.getFMap().invalidate();
		return true;
	}
	
	/**
	 * Pressing OK with the transparency panel selected makes its values to be processed.
	 * @return true if the transparency panel was selected and the action is processed, false it
	 * was not selected.
	 */
	public boolean processTransparencyPanel(){
		//OPACIDAD
		RasterTransparencyPanel tpan = (RasterTransparencyPanel)((FilterRasterDialogPanel)this.getContentPane()).getPanelByClassName("RasterTransparencyPanel");
		String sOpac = tpan.getOpacityText().getText();
		if(!sOpac.equals("") && tpan.getOpacityCheck().isSelected()){
			int opac = Integer.parseInt(sOpac);
			opac = (int)((opac*255)/100);
			px.setTransparency(true);
			//px.setTransparency(255-opac);
			fLayer.setTransparency(255-opac);
			rasterStatus.transparency = 255-opac;
		}else{
			px.setTransparency(false);
			fLayer.setTransparency(0);
			rasterStatus.transparency = 0;
		}
				
		//TRANSPARENCIA
		if(	tpan.getTransparencyCheck().isSelected()){
			
			stackManager.addTransparencyFilter(		tpan.getPTranspByPixel().getEntries(),
													0x10,	//Transparencia
													0xff,	//Color Transparencia R
													0xff,	//Color Transparencia G
													0xff);	//Color Transparencia B
													
		}else
			px.filterStack.removeFilter(stackManager.getTypeFilter("transparency"));
			
		fLayer.getFMap().invalidate();
		return true;	
	}
	
	/**
	 * Pressing OK with the enhancement panel selected makes its values to be processed.
	 * @return true if the enhancement panel was selected and the action is processed, false it
	 * was not selected.
	 */
	public boolean processEnhancedPanel(){
		//Filtro lineal seleccionado
		EnhancedPanel ep = (EnhancedPanel)((FilterRasterDialogPanel)this.getContentPane()).getPanelByClassName("EnhancedPanel");
		if(	ep.getLinealDirectoRadioButton().isSelected()){
			if(	ep.getRemoveCheck().isSelected() &&
				!ep.getTailCheck().isSelected())
				stackManager.addEnhancedFilter(true);
			else
				stackManager.addEnhancedFilter(false);
							
			//Recorte de colas seleccionado
			if(ep.getTailCheck().isSelected()){
				stackManager.removeFilter(stackManager.getTypeFilter("computeminmax"));
				double recorte = Double.parseDouble(ep.getTailText().getText())/100;
				if(ep.getRemoveCheck().isSelected())
					stackManager.addTailFilter( recorte, 0D, true);
				else
					stackManager.addTailFilter( recorte, 0D, false);								
			}else{
				stackManager.removeFilter(stackManager.getTypeFilter("tail"));
				stackManager.addComputeMinMaxFilter();
			}	
		}
			
		//Sin filtro lineal seleccionado
		if(ep.getSinRealceRadioButton().isSelected()){
			stackManager.removeFilter(stackManager.getTypeFilter("computeminmax"));
			stackManager.removeFilter(stackManager.getTypeFilter("tail"));
			stackManager.removeFilter(stackManager.getTypeFilter("enhanced"));
		}
		fLayer.getFMap().invalidate();
			
		return true;
	}
	
	/**
	 * Manages the action perfomed when Apply/OK is pressed or Apply at the raster
	 * properties control.
	 * @param e
	 */
	private void acceptButtonActionPerformed(ActionEvent e) {
		this.processBandPanel();
		this.processTransparencyPanel();	
		//this.processEnhancedPanel();	
	}
	
	/**
	 * Adds a band to the raster.
	 * @param e
	 */
	private void addFileBand(ActionEvent e){
		String[] driverNames = null;
		String rasterDriver = null;
				
		//Creacin del dialogo para seleccin de ficheros
		
		fileChooser = new JFileChooser(lastPath);
		fileChooser.setMultiSelectionEnabled(true);
		fileChooser.setAcceptAllFileFilterUsed(false);
        try {
			driverNames = LayerFactory.getDM().getDriverNames();
			FileFilter defaultFileFilter = null, auxF;
			for (int i = 0; i < driverNames.length; i++) {
				
				if (driverNames[i].endsWith("gvSIG Image Driver")){
					rasterDriver = driverNames[i];
				    auxF = new DriverFileFilter(driverNames[i]);
					fileChooser.addChoosableFileFilter(auxF);
					defaultFileFilter = auxF;
				}
			}
		} catch (DriverLoadException e1) {
			NotificationManager.addError("No se pudo acceder a los drivers", e1);
		}
		int result = fileChooser.showOpenDialog(WMSRasterPropsDialog.this);
		
		if(result == JFileChooser.APPROVE_OPTION){
			File[] files = fileChooser.getSelectedFiles();
		 	FileFilter filter = fileChooser.getFileFilter();
		 	BandSetupPanel bandSetup = (BandSetupPanel)((FilterRasterDialogPanel)this.getContentPane()).getPanelByClassName("BandSetupPanel");
		 	lastPath = files[0].getPath();
		 	
		 	//Lo aadimos a la capa si no esta
		 	
		 	Vector v = new Vector();
            for(int i=0;i<files.length;i++){
            	boolean exist = false;
            	for(int j=0;j<px.getFiles().length;j++){
            		if(px.getFiles()[j].getName().endsWith(files[i].getName()))
            			exist = true;
            	}
            	if(!exist){
            		try{
            			Rectangle2D extentOrigin = fLayer.getFullExtent();
            			
            			Extent extentNewFile = GeoRasterFile.openFile(fLayer.getProjection(), files[i].getAbsolutePath()).getExtent();
            			
						//Comprobamos que el extent y tamao del fichero aadido sea igual al 
						//fichero original. Si no es as no abrimos la capa y mostramos un aviso
						
            			double widthNewFile = (extentNewFile.getMax().getX()-extentNewFile.getMin().getX());
            			double heightNewFile = (extentNewFile.getMax().getY()-extentNewFile.getMin().getY());
            				                                                                                                
            			if( (widthNewFile-extentOrigin.getWidth()) > 1.0 ||
            				(widthNewFile-extentOrigin.getWidth()) < -1.0 ||
            				(heightNewFile-extentOrigin.getHeight()) > 1.0 ||
            				(heightNewFile-extentOrigin.getHeight()) < -1.0){       
            				JOptionPane.showMessageDialog(	null,
            												PluginServices.getText(this, "extents_no_coincidentes"), 
															"",
															JOptionPane.ERROR_MESSAGE);
            				return;
            			}
            				                                
            			if(	(extentNewFile.getMax().getX()-extentNewFile.getMin().getX())!=extentOrigin.getWidth() ||
								(extentNewFile.getMax().getY()-extentNewFile.getMin().getY())!=extentOrigin.getHeight()	){
            				JOptionPane.showMessageDialog(null, 
									PluginServices.getText(this, "extents_no_coincidentes"), "", JOptionPane.ERROR_MESSAGE);
							return;
            			}
														
            		}catch(Exception exc){
            			exc.printStackTrace();
            		}
            		
            		//Lo aadimos a la capa
            		px.addFile(files[i].getAbsolutePath());

            	}else{
            		JOptionPane.showMessageDialog(null, 
							PluginServices.getText(this, "fichero_existe")+" "+files[i].getAbsolutePath(), "", JOptionPane.ERROR_MESSAGE);
            	}
            }
		 	            
            //Aadimos los georasterfile a la tabla del Panel
            
            v = new Vector();
            for(int i=0;i<px.getFiles().length;i++){
            	boolean exist = false;
            	for(int j=0;j<bandSetup.getNBands();j++){
            		if(px.getFiles()[i].getName().endsWith(bandSetup.getBandName(j)))
            			exist = true;
            	}
            	if(!exist)
            		v.add(px.getFiles()[i]);
            }
            
            GeoRasterFile[] grf = new GeoRasterFile[v.size()];
            for(int i=0;i<grf.length;i++){
            	grf[i] = (GeoRasterFile)v.get(i);
            }
            bandSetup.addFiles(grf);
		}
	}
	
	/**
	 * Deletes a band from the raster. If there is only a file or no band is
	 * selected then it does nothing.
	 * 
	 * @param e
	 */
	private void delFileBand(ActionEvent e){
		BandSetupPanel bandSetup = (BandSetupPanel)((FilterRasterDialogPanel)this.getContentPane()).getPanelByClassName("BandSetupPanel");
			
		if(	bandSetup.getFileList().getJList().getSelectedValue()!=null &&
			bandSetup.getFileList().getNFiles() > 1){
			String pathName = bandSetup.getFileList().getJList().getSelectedValue().toString();
			px.delFile(pathName);
			String file = pathName.substring(pathName.lastIndexOf("/")+1);
			file = file.substring(file.lastIndexOf("\\")+1);
			bandSetup.removeFile(file);
			
		}		
	}
	
	/**
	 * The cancel button restores the previous state to the loading of this dialog
	 * @param e	Evento
	 */
	private void cancelButtonActionPerformed(ActionEvent e) {
		this.status.restoreStatus(this);
		fLayer.getFMap().invalidate();
	}
	
	/**
	 * @see com.iver.mdiApp.ui.MDIManager.View#getViewInfo()
	 */
	public ViewInfo getViewInfo() {
		ViewInfo m_viewinfo=new ViewInfo(ViewInfo.MODALDIALOG);
    		m_viewinfo.setTitle(PluginServices.getText(this, "propiedades_raster"));
		return m_viewinfo;
	}
	
	
}  
