/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/
package com.iver.cit.gvsig.gui.toc;

import java.awt.event.ActionEvent;
import java.util.ArrayList;

import javax.swing.JMenuItem;

import org.cresques.io.GeoRasterFile;
import org.cresques.io.raster.RasterFilterStackManager;
import org.cresques.px.PxRaster;
import org.cresques.ui.raster.BandSetupPanel;
import org.cresques.ui.raster.EnhancedPanel;
import org.cresques.ui.raster.FilterRasterDialogPanel;
import org.cresques.ui.raster.RasterTransparencyPanel;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrWMS;
import com.iver.cit.gvsig.gui.dialogs.WMSRasterPropsDialog;

/**
 * 
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class WMSRasterPropsTocMenuEntry  extends TocMenuEntry {
	private JMenuItem propsMenuItem;
	FLayer lyr = null;
	private WMSRasterPropsDialog	propsDialog = null;
	private BandSetupPanel 			bandSetup = null;
	
	public void initialize(FPopupMenu m) {
		super.initialize(m);
		
		if (isTocItemBranch()) {
			lyr = getNodeLayer();
    		// Opcciones para capas WCS
    		if ((lyr instanceof FLyrWMS)) {
    			propsMenuItem = new JMenuItem(PluginServices.getText(this, "propiedades_raster"));
    			getMenu().add( propsMenuItem );
    			propsMenuItem.setFont(FPopupMenu.theFont);
    			getMenu().setEnabled(true);
    			//getMenu().addSeparator();
    	        //Cambio color
    			propsMenuItem.addActionListener(this);   			
     		}
		}
	}
	
	/**
	 * Selecciona las bandas activas en el panel de bandas a partir
	 * del los filtros contenidos en el FilterStack 
	 * @param stackManager RasterFilterStackManager
	 * @param posR	posicin de la banda R en el selector
	 * @param posG  posicin de la banda G en el selector
	 * @param posB  posicin de la banda B en el selector
	 */
	private void bandSelector(RasterFilterStackManager stackManager, int posR, int posG, int posB){
		ArrayList stackList = stackManager.getStringsFromStack();
		String hideBands = null;
		for(int i=0;i<stackList.size();i++){
			if(((String)stackList.get(i)).startsWith("filter.removebands.bands"))
				hideBands = stackManager.getValue((String)stackList.get(i));
		}
		
		//Seleccin de 1, 2 o 3 bandas para visualizar
		if(hideBands != null){
			int pos = 2;
			if(hideBands.length() == 1)
				pos = 1;
			else if(hideBands.length() == 2)
				pos = 0;
			bandSetup.getFileList().getJComboBox().setSelectedIndex(pos);
			
			//Reseteamos los controles de la tabla
			
			for(int i=0;i<bandSetup.getRGBTable().getModel().getRowCount();i++)
				for(int j=0;j<3;j++)
					bandSetup.getRGBTable().getModel().setValueAt(new Boolean(false), i, j);
			
			if(hideBands.equals("GB") || hideBands.equals("G") || hideBands.equals("B") )
				bandSetup.getRGBTable().getModel().setValueAt(new Boolean(true), posR, 0);
			
			if(hideBands.equals("RB") || hideBands.equals("R") || hideBands.equals("B") )
				bandSetup.getRGBTable().getModel().setValueAt(new Boolean(true), posG, 1);
			
			if(hideBands.equals("RG") || hideBands.equals("R") || hideBands.equals("G") )
				bandSetup.getRGBTable().getModel().setValueAt(new Boolean(true), posB, 2);
		}
	}
	
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.gui.toc.TocMenuEntry#execute(com.iver.cit.gvsig.gui.toc.ITocItem)
	 */
	public void actionPerformed(ActionEvent e) {
       	lyr = getNodeLayer();
        
		if(lyr instanceof FLyrWMS && ((FLyrWMS)lyr).getPxRaster()!=null){
			RasterFilterStackManager 	stackManager = null;
			
			FLyrWMS layer = (FLyrWMS)lyr;

			stackManager = new RasterFilterStackManager(layer.getFilterStack());
			
			propsDialog = new WMSRasterPropsDialog(layer, stackManager.getTransparencyRGB());
		       			
			int alpha = layer.getPxRaster().getAlpha();
			
			bandSetup = (BandSetupPanel)((FilterRasterDialogPanel)propsDialog.getContentPane()).getPanelByClassName("BandSetupPanel");
			GeoRasterFile[] files = layer.getPxRaster().getFiles();
			bandSetup.addFiles(files);
			for(int i=0; i< files.length;i++)
				propsDialog.addNumBands(files[i].getBandCount());
				
			PxRaster px = layer.getPxRaster();
			int posR = 0, posG = 0, posB = 0;
			
			for(int i=0;i<px.getPosFile(GeoRasterFile.RED_BAND);i++)
				posR += files[i].getBandCount();
			posR += px.getBand(GeoRasterFile.RED_BAND);
			
			for(int i=0;i<px.getPosFile(GeoRasterFile.GREEN_BAND);i++)
				posG += files[i].getBandCount();
			posG += px.getBand(GeoRasterFile.GREEN_BAND);
			
			for(int i=0;i<px.getPosFile(GeoRasterFile.BLUE_BAND);i++)
				posB += files[i].getBandCount();
			posB += px.getBand(GeoRasterFile.BLUE_BAND);
			
					
			bandSetup.assignBand(posR, GeoRasterFile.RED_BAND);
			bandSetup.assignBand(posG, GeoRasterFile.GREEN_BAND);
			bandSetup.assignBand(posB, GeoRasterFile.BLUE_BAND);	
									
			//Asignacin del alpha actual de la imagen al dialogo
			RasterTransparencyPanel rasterTrans = (RasterTransparencyPanel)((FilterRasterDialogPanel)propsDialog.getContentPane()).getPanelByClassName("RasterTransparencyPanel");
			rasterTrans.setOpacity(alpha);
				
			//Asignacin del porcentaje de recorte actual de la imagen al dialogo
			EnhancedPanel pEnhan =  (EnhancedPanel)((FilterRasterDialogPanel)propsDialog.getContentPane()).getPanelByClassName("EnhancedPanel");
			
			propsDialog.setRasterFilterStackManager(stackManager);
			int tail = 0;
			if(	pEnhan != null &&
				stackManager.getStackStats() != null){
				tail = (int)(stackManager.getStackStats().tailPercent*200);
				pEnhan.setPercentTail(tail);
			}
			/*if(stackManager.isActive(stackManager.getTypeFilter("enhanced")))
				pEnhan.setSelectedFilter(1);
			else
				pEnhan.setSelectedFilter(0);*/
			
			bandSelector(stackManager, posR, posG, posB);
			
			propsDialog.readStat();
			
			PluginServices.getMDIManager().addView(propsDialog);       	
		}
	}
}