/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.core.mdiManager;

import com.iver.andami.plugins.PluginClassLoader;
import com.iver.andami.ui.mdiFrame.MainFrame;
import com.iver.andami.ui.mdiFrame.NoSuchMenuException;
import com.iver.andami.ui.mdiManager.SingletonView;
import com.iver.andami.ui.mdiManager.View;
import com.iver.andami.ui.mdiManager.ViewInfo;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import java.util.Enumeration;
import java.util.Hashtable;


/**
 *
 */
public class ViewInfoSupport {
	private static int serialId = 0;
	private FrameViewSupport fvs;

	/** Correspondencias entre las vistas y su informacion */
	private Hashtable viewInfo = new Hashtable();
	private Hashtable infoView = new Hashtable();
	private ViewPropertyChangeListener viewInfoListener = new ViewPropertyChangeListener();
	private SingletonViewSupport svs;
	private MainFrame mdiFrame;

	/**
	 * Creates a new ViewInfoSupport object.
	 *
	 * @param frame DOCUMENT ME!
	 * @param fvs DOCUMENT ME!
	 * @param svs
	 */
	public ViewInfoSupport(MainFrame frame, FrameViewSupport fvs,
		SingletonViewSupport svs) {
		this.fvs = fvs;
		this.svs = svs;
		this.mdiFrame = frame;
	}

	/**
	 * Devuelve la vista cuyo identificador es el parametro
	 *
	 * @param id Identificador de la vista que se quiere obtener
	 *
	 * @return La vista o null si no hay ninguna vista con ese identificador
	 */
	public View getViewById(int id) {
		Enumeration en = infoView.keys();

		while (en.hasMoreElements()) {
			ViewInfo vi = (ViewInfo) en.nextElement();

			if (vi.getId() == id) {
				return (View) infoView.get(vi);
			}
		}

		return null;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param v DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public synchronized ViewInfo getViewInfo(View v) {
		ViewInfo vi = (ViewInfo) viewInfo.get(v);

		if (vi == null) {
			vi = v.getViewInfo();

			//Para el ttulo
			if (vi.getHeight() != -1) {
				vi.setHeight(vi.getHeight() + 40);
			}

			vi.addPropertyChangeListener(viewInfoListener);
			viewInfo.put(v, vi);
			infoView.put(vi, v);
			vi.setId(serialId++);
		}

		return vi;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param p DOCUMENT ME!
	 */
	public void deleteViewInfo(View p) {
		ViewInfo vi = (ViewInfo) viewInfo.remove(p);
		infoView.remove(vi);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @author $author$
	 * @version $Revision: 1.8 $
	 */
	public class ViewPropertyChangeListener implements PropertyChangeListener {
		/**
		 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
		 */
		public void propertyChange(PropertyChangeEvent evt) {
			ViewInfo v = (ViewInfo) evt.getSource();
			View view = (View) infoView.get(v);

			if (view instanceof SingletonView) {
				SingletonView sv = (SingletonView) view;

				if (evt.getPropertyName().equals("x")) {
					svs.setX(sv, ((Integer) evt.getNewValue()).intValue());
				} else if (evt.getPropertyName().equals("y")) {
					svs.setY(sv, ((Integer) evt.getNewValue()).intValue());
				} else if (evt.getPropertyName().equals("height")) {
					svs.setHeight(sv, ((Integer) evt.getNewValue()).intValue());
				} else if (evt.getPropertyName().equals("width")) {
					svs.setWidth(sv, ((Integer) evt.getNewValue()).intValue());
				} else if (evt.getPropertyName().equals("title")) {
					svs.setTitle(sv, (String) evt.getNewValue());

					try {
						mdiFrame.changeMenuName(new String[] {
								"ventana", (String) evt.getOldValue()
							}, (String) evt.getNewValue(),
							(PluginClassLoader) getClass().getClassLoader());
					} catch (NoSuchMenuException e) {
						/*
						 * No se hace nada porque puede modificarse el ttulo de
						 * una ventana antes de ser aadida a Andami
						 */
					}
				}
			} else {
				if (evt.getPropertyName().equals("x")) {
					fvs.setX(view, ((Integer) evt.getNewValue()).intValue());
				} else if (evt.getPropertyName().equals("y")) {
					fvs.setY(view, ((Integer) evt.getNewValue()).intValue());
				} else if (evt.getPropertyName().equals("height")) {
					fvs.setHeight(view, ((Integer) evt.getNewValue()).intValue());
				} else if (evt.getPropertyName().equals("width")) {
					fvs.setWidth(view, ((Integer) evt.getNewValue()).intValue());
				} else if (evt.getPropertyName().equals("title")) {
					fvs.setTitle(view, (String) evt.getNewValue());
					try{
						mdiFrame.changeMenuName(new String[] {
								"ventana", (String) evt.getOldValue()
							}, (String) evt.getNewValue(),
							(PluginClassLoader) getClass().getClassLoader());
					} catch (NoSuchMenuException e) {
						/*
						 * No se hace nada porque puede modificarse el ttulo de
						 * una ventana antes de ser aadida a Andami
						 */
					}
				}
			}
		}
	}
}
