/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.io.raster;

import java.util.ArrayList;

import org.cresques.io.GeoRasterFile;


/**
 * Interfaz que deben implementar los Gestores de pila de filtros
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */
public interface IStackManager {
    /**
     * Convierte cada filtro o elemento de un filtro de la pila en un String de la forma
     * elemento=valor
     * @return ArrayList donde cada elemento es una cadena
     */
    public ArrayList getStringsFromStack();
    
    /**
     * Convierte cada filtro o elemento de un filtro de la pila en un String de la forma
     * elemento=valor. Esta versin la implementan las extensiones que registren filtros
     * de forma externa para que pueda ser ejecutado por el RasterFilterStackManager.
     * @return ArrayList donde cada elemento es una cadena
     */
    public ArrayList getStringsFromStack(ArrayList filterList, RasterFilter rf);

    /**
     * Crea una pila de filtros a partir de un Array de Strings. Cada elemento del array debe
     * tener la forma elemento=valor.
     * @param filters
     */
    public void createStackFromStrings(ArrayList f, Integer pos);
    
    /**
     * Crea una pila de filtros a partir de un Array de Strings. Cada elemento del array debe
     * tener la forma elemento=valor. Esta versin la implementan las extensiones que registren filtros
     * de forma externa para que pueda ser ejecutado por el RasterFilterStackManager.
     * @param fil Cadena que representa el identificador del elemento 
     * @param filters	Lista de filtros
     * @param grfList	Lista de geoRasterFile
     * @param filteri Nmero de filtro de la lista analizado
     * @return filteri modificado
     */    
    public int createStackFromStrings(ArrayList filters, String fil, GeoRasterFile[] grfList, int filteri);
}

