/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.io.raster;


/**
 * Realzado Lineal (Amplitude Rescaling) para imagenes de 16 bits.
 * @author Luis W. Sevilla (sevilla_lui@gva.es)
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class LinearEnhancementShortFilter extends LinearEnhancementFilter {
    /**
     * Constructor
     */
    public LinearEnhancementShortFilter() {
        super();
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#pre()
     */
    public void pre() {
        //Obtencin de parmetros
        this.raster = (RasterBuf) params.get("raster");
        this.stats = (RasterStats) params.get("stats");
        this.removeExtrema = ((Boolean) params.get("remove")).booleanValue();
        this.filename = (String) params.get("filename");
        height = raster.getHeight();
        width = raster.getWidth();
        super.pre();
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#process(int, int)
     */
    public void process(int x, int y) {
        raster.getElemInt(x, y, px);

        int bandsCnt = 3; // Esto hay que sacarlo de fuera, lgicamente.

        for (int i = 0; i < bandsCnt; i++) {
            if (px[i] > maxBandValue[i]) {
                px[i] = maxBandValue[i];
            } else if (px[i] < minBandValue[i]) {
                px[i] = minBandValue[i];
            }

            px[i] = (((int) ((((double) px[i]) * scale[i]) + offset[i])) &
                    0xff);
        }

        raster.setElemInt(x, y, px);
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getResult(java.lang.String)
     */
    public Object getResult(String name) {
        if (name.equals("raster")) {
            return (Object) this.raster;
        } else {
            return null;
        }
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getInRasterDataType()
     */
    public int getInRasterDataType() {
        return RasterBuf.TYPE_SHORT;
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getOutRasterDataType()
     */
    public int getOutRasterDataType() {
        return RasterBuf.TYPE_SHORT;
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#post()
     */
    public void post() {
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.RasterFilter#processLine(int)
     */
    public void processLine(int y) {
    }
}
