/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.io.raster;

import java.awt.Point;


/**
 * Rectangulo de pixeles. parecido a java.awt.image.Raster
 * @author Luis W. Sevilla (sevilla_lui@gva.es)
 */
public class RasterBuf implements IRaster {
    private byte[][][] byteBuf;
    private short[][][] shortBuf;
    private int[][][] intBuf;
    private int width;
    private int height;
    private int bandNr;
    private int dataType;

    /**
     * Constructor
     * @param dataType Tipo de dato
     * @param width Ancho
     * @param height Alto
     * @param bandNr Banda
     * @param orig
     */
    public RasterBuf(int dataType, int width, int height, int bandNr, Point orig) {
        this.dataType = dataType;
        this.width = width;
        this.height = height;
        this.bandNr = bandNr;

        if (dataType == TYPE_BYTE) {
            byteBuf = new byte[height][width][bandNr];
        } else if ((dataType == TYPE_SHORT) | (dataType == TYPE_USHORT)) {
            shortBuf = new short[height][width][bandNr];
        } else if (dataType == TYPE_INT) {
            intBuf = new int[height][width][bandNr];
        }
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRaster#getWidth()
     */
    public int getWidth() {
        return width;
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRaster#getHeight()
     */
    public int getHeight() {
        return height;
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRaster#getBandNr()
     */
    public int getBandNr() {
        return bandNr;
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRaster#getDataType()
     */
    public int getDataType() {
        return dataType;
    }

    /**
     * Obtiene el tipo de dato
     * @return Tipo de dato
     */
    public int getDataSize() {
        if (dataType == TYPE_BYTE) {
            return 1;
        } else if ((dataType == TYPE_SHORT) | (dataType == TYPE_USHORT)) {
            return 2;
        } else if (dataType == TYPE_INT) {
            return 4;
        }

        return 0;
    }

    /**
     * Obtiene el tamao del buffer
     * @return tamao del buffer
     */
    public int sizeof() {
        return getDataSize() * width * height * bandNr;
    }

    public byte[][] getLineByte(int y) {
        return byteBuf[y];
    }

    public short[][] getLineShort(int y) {
        return shortBuf[y];
    }

    public int[][] getLineInt(int y) {
        return intBuf[y];
    }

    public void getElemShort(int x, int y, int[] px) {
        for (int i = 0; i < bandNr; i++)
            px[i] = shortBuf[y][x][i];
    }

    public void getElemInt(int x, int y, int[] px) {
        for (int i = 0; i < bandNr; i++)
            px[i] = intBuf[y][x][i];
    }

    public void setElemShort(int x, int y, int[] px) {
        for (int i = 0; i < bandNr; i++)
            shortBuf[y][x][i] = (short) px[i];
    }

    public void setElemInt(int x, int y, int[] px) {
        for (int i = 0; i < bandNr; i++)
            intBuf[y][x][i] = px[i];
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRaster#getElemInt(int, int, int)
     */
    public int getElemInt(int x, int y, int band) {
        return intBuf[y][x][band];
    }

    /**
     * Convierte un tipo de dato a cadena
     * @param type Tipo de dato
     * @return cadena  que representa el tipo de dato
     */
    public static String typesToString(int type) {
        switch (type) {
        case RasterBuf.TYPE_IMAGE:
            return new String("Image");

        case RasterBuf.TYPE_BYTE:
            return new String("Byte");

        case RasterBuf.TYPE_DOUBLE:
            return new String("Double");

        case RasterBuf.TYPE_FLOAT:
            return new String("Float");

        case RasterBuf.TYPE_INT:
        case RasterBuf.TYPE_USHORT:
        case RasterBuf.TYPE_SHORT:
            return new String("Short");
        }

        return null;
    }
}
