/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.io.raster;


/**
 * Proceso del filtro de recorte de colas aplicado a imgenes 16 bits
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */
public class RemoveBandsShortFilter extends RemoveBandsFilter {
    public RemoveBandsShortFilter() {
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#pre()
     */
    public void pre() {
        //Obtencin de parmetros
        this.raster = (RasterBuf) params.get("raster");
        height = raster.getHeight();
        width = raster.getWidth();
        super.pre();
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#process(int, int)
     */
    public void process(int x, int y) {
        raster.getElemInt(x, y, px);
        px[0] = px[0] & m1;
        px[1] = px[1] & m2;
        px[2] = px[2] & m3;
        raster.setElemInt(x, y, px);
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getInRasterDataType()
     */
    public int getInRasterDataType() {
        return RasterBuf.TYPE_SHORT;
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getOutRasterDataType()
     */
    public int getOutRasterDataType() {
        return RasterBuf.TYPE_SHORT;
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.RasterFilter#processLine(int)
     */
    public void processLine(int y) {
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getResult(java.lang.String)
     */
    public Object getResult(String name) {
        if (name.equals("raster")) {
            return (Object) this.image;
        } else {
            return null;
        }
    }

    public void post() {
        super.post();
    }
}
