/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.io.raster;

import java.io.IOException;
import java.util.ArrayList;


/**
 * Clase base para los filtros de transparencia en sus diferentes tipos
 * de datos.
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */
public abstract class TransparencyFilter extends RasterFilter {
    //Parmetros del filtro
	/**
	 * Lista de TransparencyRange
	 */
	protected ArrayList	rangesList = null;
	
    /**
     * Valor de transparencia
     */
    public int alpha = 0x10;

    /**
     * Color en la banda del rojo para la transparencia
     */
    public int transparencyColorRed = 0xff;

    /**
     * Color en la banda del verde para la transparencia
     */
    public int transparencyColorGreen = 0xff;

    /**
     * Color en la banda del azul para la transparencia
     */
    public int transparencyColorBlue = 0xff;

    /**
     * Constructor
     *
     */
    public TransparencyFilter() {
        super();
    }
    
    /**
     * Obtiene parmetros del filtro desde la tabla Hash
     */
    public void pre() {
        //Obtencin de parmetros comunes a todos
    	rangesList = (ArrayList)params.get("rangesList");
    	/*for(int i=0;i<rangesList.size();i++)
    		((TransparencyRange)rangesList.get(i)).show();*/
        alpha = ((Integer) params.get("alpha")).intValue();
        transparencyColorRed = ((Integer) params.get("transparencyRed")).intValue();
        transparencyColorGreen = ((Integer) params.get("transparencyGreen")).intValue();
        transparencyColorBlue = ((Integer) params.get("transparencyBlue")).intValue();
    }
}
