/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.io.raster;


/**
 * Filtro de transparencia aplicada a un rango de valores de un pixel sobre un objeto image
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */
public class TransparencyShortFilter extends TransparencyFilter {
    /**
     * Constructor
     *
     */
    public TransparencyShortFilter() {
        super();
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#pre()
     */
    public void pre() {
        //Obtencin de parmetros
        this.raster = (RasterBuf) params.get("raster");
        height = raster.getHeight();
        width = raster.getWidth();
        super.pre();
    }

    /**
     * Asigna al pixel pasado como parmetro como transparente con 
     * el color de transparencia que est seleccionado
     * @param px Pixel a asignarle transparencia
     */
    private void setPixelTransparent(int[] px){
		px[3] = this.alpha;
        px[0] = this.transparencyColorRed;
        px[1] = this.transparencyColorGreen;
        px[2] = this.transparencyColorBlue;	
    }
    
    /**     
     * @param range rango
     * @param banda banda
     * @param px    pixel
     */
    private void processRange(int[] px) {
    	for (int i = 0; i < rangesList.size(); i++) {
    		TransparencyRange tr = ((TransparencyRange)rangesList.get(i));

    		if(tr.isAnd()){
    			if(tr.getRed() != null)
    				if (px[0] < tr.getRed()[0] || px[0] > tr.getRed()[1])
    					continue;
    			if(tr.getGreen() != null)
    				if (px[1] < tr.getGreen()[0] || px[1] > tr.getGreen()[1])
    					continue;
    			if(tr.getBlue() != null)
    				if (px[2] < tr.getBlue()[0] || px[2] > tr.getBlue()[1])
    					continue;
    			if(tr.getRed() != null || tr.getGreen() != null || tr.getBlue() != null)
    				setPixelTransparent(px);
    			
    		}else{
    			if(tr.getRed() != null){
    				if (px[0] >= tr.getRed()[0] && px[0] <= tr.getRed()[1]){
    					setPixelTransparent(px);
    					continue;
    				}
    			}
    			if(tr.getGreen() != null){
    				if (px[1] >= tr.getGreen()[0] && px[1] <= tr.getGreen()[1]){
    					setPixelTransparent(px);
    					continue;
    				}
    			}
    			if(tr.getBlue() != null){
    				if (px[2] >= tr.getBlue()[0] && px[2] <= tr.getBlue()[1]){
    					setPixelTransparent(px);
    					continue;
    				}
    			}
    		}
    	}
    	
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#process(int, int)
     */
    public void process(int x, int y) {
        raster.getElemInt(x, y, px);
        processRange(px);
        raster.setElemInt(x, y, px);
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getInRasterDataType()
     */
    public int getInRasterDataType() {
        return RasterBuf.TYPE_SHORT;
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getOutRasterDataType()
     */
    public int getOutRasterDataType() {
        return RasterBuf.TYPE_SHORT;
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getResult(java.lang.String)
     */
    public Object getResult(String name) {
        if (name.equals("raster")) {
            return (Object) this.raster;
        } else {
            return null;
        }
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.RasterFilter#processLine(int)
     */
    public void processLine(int y) {
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#post()
     */
    public void post() {
    }
}
