/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.ui;

import java.awt.GridBagConstraints;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;


/**
 * Panel del que heredaran los paneles que contengan tabs
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class DefaultTabPanel extends DefaultDialogPanel implements ComponentListener, ChangeListener{
    final private static long serialVersionUID = -3370601314380922368L;
    protected JTabbedPane tabbedPane = null;

    /**
     * Contructor
     */
    public DefaultTabPanel() {
        super();
        this.setBounds(0, 0, 355, 230);
        this.getContentPanel();
        this.getTabPane();
        GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
        gridBagConstraints6.gridy = 0;
        gridBagConstraints6.gridx = 0;
        contentPane.add(tabbedPane, gridBagConstraints6);
        tabbedPane.addChangeListener(this);
    }


	/**
     * Obtiene el Componente JTabbedPane
     * @return Componente JTabbedPane
     */
    private JTabbedPane getTabPane() {
        if (tabbedPane == null) {
            tabbedPane = new JTabbedPane();
            tabbedPane.setLocation(0,0);
        }

        return tabbedPane;
    }

    /**
     * Aade un tab al JTabbedPane
     * @param title        Ttulo del tab
     * @param panel        Panel del nuevo tab
     */
    public void addTab(String title, JPanel panel) {
        tabbedPane.add(title, panel);
    }

    /**
     * Obtiene el JTabbedPane
     * @return JTabbedPane
     */
    public JTabbedPane getTab() {
        return tabbedPane;
    }

	public void componentResized(ComponentEvent e) {
		if(e.getSource() ==  this){
			super.componentResized(e);
			//int difWidth = this.getSize().width - cWidth;
			//int difHeight = this.getSize().height - cHeight;
			this.tabbedPane.setSize(this.tabbedPane.getSize().width + difWidth, this.tabbedPane.getSize().height + difHeight);
			this.setLocation(0,0);
			
		}
		
	}

	public void stateChanged(ChangeEvent ev) {
		//this.tabbedPane.setSize(this.tabbedPane.getSize().width + difWidth, this.tabbedPane.getSize().height + difHeight);
		//System.out.println("X: " + this.tabbedPane.getSize().width + " Y: " + this.tabbedPane.getSize().height);
		
		if (ev.getSource().getClass().toString().endsWith("JTabbedPane")) {
			if (((JTabbedPane)ev.getSource()).getSelectedIndex() == 1){
				super.resizeWindow();
				this.tabbedPane.setSize(this.tabbedPane.getSize().width + difWidth, this.tabbedPane.getSize().height + difHeight);
				this.setLocation(0,0);
			}
		}
	}
}  //  @jve:decl-index=0:visual-constraint="13,18"
