/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.ui.raster;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.TreeMap;

import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.cresques.io.GeoRasterFile;
import org.cresques.io.raster.IRasterOperationsRegistrablePanels;
import org.cresques.io.raster.TransparencyRange;
import org.cresques.ui.DefaultTabPanel;


/**
 * Panel que contiene todos los paneles de los tabs del cuadro de dialogo
 * de bandas y transparencias
 *
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class FilterRasterDialogPanel extends DefaultTabPanel
    implements ActionListener, ChangeListener, MouseListener, FocusListener,
               KeyListener {
    final private static long serialVersionUID = -3370601314380922368L;

    /**
     * Tamao en X del panel interior.El tab de dentro es proporcional
     */
    protected int sizePanelX = 460;
    
    /**
     * Tamao en Y del panel interior. El tab de dentro es proporcional
     */
    protected int sizePanelY = 275;
    
    /**
     * Propiedades guardadas en una matriz Nx2 con la forma propiedad/valor.
     */
    public Object[][] props = null;
    
    /**
     * Nmero de bandas.
     */
    protected int nbands = 0;
    
    /**
     * Lista de georrasterfiles correspondiente a los ficheros de bandas
     */
    protected GeoRasterFile[] grf = null;
    private TreeMap shadePanel = new TreeMap();
    
    //Paneles que se registran dentro de esta misma clase
    private RasterTransparencyPanel pTrans = null;
    private BandSetupPanel pSetup = null;
    private EnhancedPanel pEnhan = null;
    private InfoPanel pInfo = null;
    
	    
    /**
     * Constructor. Inicializa los paneles y propiedades
     * @param props        Propiedades
     */
    public FilterRasterDialogPanel(Object[][] props) {
        super();
        init(props);
    }

    /**
     * Constructor
     */
    public FilterRasterDialogPanel() {
        super();
    }

    /**
     * Inicializa las propiedades
     * @param props        Propiedades. Guardadas en una matriz Nx2 con la forma proiedad/valor
     */
    public void init(Object[][] props) {
        contentPane.setPreferredSize(new java.awt.Dimension(sizePanelX + this.difWidth,
                                                            sizePanelY + this.difHeight));
        this.tabbedPane.setPreferredSize(new java.awt.Dimension(sizePanelX + this.difWidth,
                                                                sizePanelY + difHeight - 5));
        this.props = props;
        initPanels();
        
    }

    /**
     * This method initializes this
     *
     * @return void
     */
    protected void initPanels() {
        this.setBounds(0, 0, 355, 230);
        pTrans = new RasterTransparencyPanel(this);
        pSetup = new BandSetupPanel();
        pEnhan = new EnhancedPanel(this);
        pInfo = new InfoPanel(this);
        
        this.addTab(pInfo.getName(), pInfo);
        this.addTab(pSetup.getName(), pSetup);
        this.addTab(pTrans.getName(), pTrans);
        //this.addTab(pEnhan.getName(), pEnhan);
   
        pTrans.getOpacityCheck().addActionListener(this);
        pTrans.getTransparencyCheck().addActionListener(this);
        pTrans.getOpacitySlider().addChangeListener(this);
        pTrans.getOpacityText().addActionListener(this);
        pTrans.getOpacityText().addFocusListener(this);
        pTrans.addFocusListener(this);
        this.getTab().addFocusListener(this);

        pEnhan.getTailCheck().addActionListener(this);
        pEnhan.getTailSlider().addChangeListener(this);
        pEnhan.getSinRealceRadioButton().addMouseListener(this);
        pEnhan.getLinealDirectoRadioButton().addMouseListener(this);
        pEnhan.getTailText().addActionListener(this);
        pEnhan.getTailText().addFocusListener(this);

        //Ocultamos el botn de aplicar
        //this.getApplyButton().setVisible(false);
    }

    /**
     * Asigna la visibilidad de un tab a verdadero o falso. La
     * seleccin del tab se hace por el identificador.
     * @param tab	Identificador del tab. Variable nom del mismo
     * @param active	True o false para visible o invisible.
     */
    public void setTabVisible(String tab, boolean active){
    	if(active == false){
	    	for(int i=0; i<this.getTab().getTabCount(); i++){
	    		if(tab.equals( ((JPanel)this.getTab().getComponentAt(i)).getName() )){
	    			shadePanel.put(tab, this.getTab().getComponentAt(i));
	    			this.getTab().removeTabAt(i);
	    		}
	    	}
    	}else
    		this.addTab(tab, ((JPanel)shadePanel.get(tab)) );
    }
    
    /**
     * Selecciona el panel indicado por index
     * @param index        panel seleccionado
     */
    public void setSelectedIndex(int index) {
        tabbedPane.setSelectedIndex(index);
    }

    /**
     * Obtiene un panel a travs del nombre de su clase
     * @param name Nombre de la clase del panel
     */
    public JPanel getPanelByClassName(String name){
    	for(int i=0;i<super.getTab().getTabCount() ;i++){
    		JPanel p = (JPanel)super.getTab().getComponentAt(i);
    		if(p.getClass().getName().endsWith(name))
    			return p;
    	}
    	return null;
    }
    
    /**
     * Eventos sobre TextField y CheckBox. Controla eventos de checkbox de opacidad, transparencia,
     * recorte de colas y los textfield de opacidad, valores de transparencia por banda y
     * porcentaje de recorte.
     */
    public void actionPerformed(ActionEvent e) {
        //Evento sobre el checkbox de opacidad
        if (e.getSource().equals(pTrans.getOpacityCheck())) {
            //Check de opacidad activado -> Activar controles de opacidad
            if (pTrans.getOpacityCheck().isSelected()) {
                pTrans.getOpacitySlider().setEnabled(true);
                pTrans.getOpacityText().setEnabled(true);
            } else {
                pTrans.getOpacitySlider().setEnabled(false);
                pTrans.getOpacityText().setEnabled(false);
            }
        }

        //Evento sobre el checkbox de transparencia
        if (e.getSource().equals(pTrans.getTransparencyCheck())) {
            //Check de opacidad activado -> Activar controles de opacidad
            if (pTrans.getTransparencyCheck().isSelected()) {
            	pTrans.getPTranspByPixel().setControlEnabled(true);
            	pTrans.getPTranspByPixel().getPRGBInput().setActiveBands(nbands);
            }else
            	pTrans.getPTranspByPixel().setControlEnabled(false);
        }

        //Evento sobre el textfield de opacidad
        if (e.getSource().equals(pTrans.getOpacityText())) {
            checkOpacityText();
        }

        //Evento sobre el checkbox de Recorte de colas
        if (e.getSource().equals(pEnhan.getTailCheck())) {
            if (pEnhan.getTailCheck().isSelected()) {
                pEnhan.setActiveTailControl(true);
            } else {
                pEnhan.setActiveTailControl(false);
            }
        }

        //Evento sobre el textfield de porcentaje de recorte
        if (e.getSource().equals(pEnhan.getTailText())) {
            checkTailText();
        }
    } 

    /**
     * Asigna valores para los rangos.
     * @param ranges
     */
    public void setRanges(ArrayList ranges) {      
      if(ranges != null){    	  
    	  TransparencyByPixelPanel panel = (TransparencyByPixelPanel)pTrans.getPTranspByPixel();
    	  pTrans.getTransparencyCheck().setSelected(true);
    	  panel.setControlEnabled(true);
    	  panel.setEntries(ranges);
	      for(int i=0;i<ranges.size();i++)
	    	  panel.getListModel().addElement( ((TransparencyRange)ranges.get(i)).getStrEntry() );
      }
    }

    /**
     * Aade las bandas de los georrasterfile a los paneles 
     * que lo necesitan
     * @param files
     */
    public void addFiles(GeoRasterFile[] files){
    	for(int i=0;i<this.getTab().getTabCount();i++){
    		if(this.getTab().getComponentAt(i) instanceof IRasterOperationsRegistrablePanels)
    		 	((IRasterOperationsRegistrablePanels)getTab().getComponentAt(i)).addFiles(files);
    	}
    		
        if(pSetup != null)
        	pSetup.addFiles(files);
    }
    
    /**
     * Elimina las bandas de los georrasterfile a los paneles
     * @param file Banda a eliminar
     */
    public void removeFile(String file) {
    	for(int i=0;i<this.getTab().getTabCount();i++){
    		if(this.getTab().getComponentAt(i) instanceof IRasterOperationsRegistrablePanels);
    		 	((IRasterOperationsRegistrablePanels)getTab().getComponentAt(i)).removeFile(file);
    	}
        if(pSetup != null)
        	pSetup.removeFile(file);
    }
    
    /**
     * Controla que si el formato introducido en el textfield
     * de opacidad es numerico se actualiza la posicin del
     * slider.
     */
    private void checkOpacityText() {
        String op = pTrans.getOpacityText().getText();
        int value = 0;

        try {
            if (!op.equals("")) {
                value = Integer.parseInt(op);
            }

            pTrans.getOpacitySlider().setValue(value);
        } catch (NumberFormatException exc) {
            System.err.println("Formato no numerico");
            pTrans.getOpacityText().setText("100");
            pTrans.getOpacitySlider().setValue(100);
        }
    }

    /**
     * Controla que si el formato introducido en el textfield
     * de Recorte de colas es numerico se actualiza la posicin del
     * slider
     */
    private void checkTailText() {
        String op = pEnhan.getTailText().getText();
        double value = 0;

        try {
            if (!op.equals("")) {
                value = (Double.parseDouble(op) * 100) / 50;
            }

            pEnhan.getTailSlider().setValue((int) Math.round(value));
            pEnhan.getTailText().setText(op);
        } catch (NumberFormatException exc) {
            System.err.println("Formato no numerico");
            pEnhan.getTailText().setText("0.0");
            pEnhan.getTailSlider().setValue(0);
        }
    }

    /*
     *
     */
    public void stateChanged(ChangeEvent e) {
    	super.stateChanged(e);
        //Ponemos el valor del texto de la opacidad de pendiendo de la posicin del Slider
        if (e.getSource().equals(pTrans.getOpacitySlider())) {
            pTrans.getOpacityText().setText(
            		String.valueOf(pTrans.getOpacitySlider().getValue()));
        }

        if (e.getSource().equals(pEnhan.getTailSlider())) {
            pEnhan.getTailText().setText(
            		String.valueOf((pEnhan.getTailSlider().getValue() >> 1)) + ".0");
        }
        
    }

    public void focusGained(FocusEvent e) {
        //pTrans.updateTextBox();
    }

    public void focusLost(FocusEvent e) {
        checkOpacityText();
        checkTailText();
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseClicked(MouseEvent e) {
    }

    public void mousePressed(MouseEvent e) {
        //Evento sobre los radio button de Realce 
        if (e.getSource().equals(pEnhan.getSinRealceRadioButton())) {
            if (pEnhan.getSinRealceRadioButton().isEnabled()) {
                pEnhan.getTailCheck().setEnabled(false);
                pEnhan.setActiveTailControl(false);
                pEnhan.getSinRealceRadioButton().setSelected(true);
                pEnhan.setActiveRemoveCheck(false);
            }
        }

        if (e.getSource().equals(pEnhan.getLinealDirectoRadioButton())) {
            pEnhan.getTailCheck().setEnabled(true);
            pEnhan.setActiveRemoveCheck(true);

            if (pEnhan.getTailCheck().isSelected()) {
                pEnhan.setActiveTailControl(true);
            }
        }
    }

    public void keyTyped(KeyEvent e) {
    }

    public void keyPressed(KeyEvent e) {
    }

    public void keyReleased(KeyEvent e) {
    }

}
