/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.ui.raster;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;

import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import org.cresques.ui.raster.FilterRasterDialogPanel;

/**
 * 
 * @author Miguel Angel Querol Carratal <querol_mig@gva.es>
 * Panel de informacion de la imagen rster de la que se muestran las propiedades
 *
 */
public class InfoPanel extends JPanel implements ComponentListener{

	private final String bgColor0 = "\"#FEEDD6\""; // light salmon
    private final String bgColor1 = "\"#EAEAEA\""; // light grey
    private final String bgColor2 = "\"#F2FEFF\""; // light blue
    private final String bgColor3 = "\"#FBFFE1\""; // light yellow
	private JScrollPane jScrollPane = null;
	private JEditorPane jEditorPane = null;
    
	/**
	 * Nombre del Panel
	 */
	private String nom = "Info";
	
	/**
	 * Dialogo padre que contiene a este
	 */
	public FilterRasterDialogPanel parent = null;
	
	/**
	 * Matriz de tamao Nx2 que contiene a las propiedades en forma propiedad/valor
	 */
	public Object[][] props = null;
	
	/**
	 * Cabecera de las columnas del dialogo
	 */
	public Object[] columnNames = { "Propiedad", "Valor" };
	/**
	 * This is the default constructor
	 */
	public InfoPanel(FilterRasterDialogPanel parent) {
		super();
		this.parent = parent;
		this.props = parent.props;
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	public void initialize() {
		GridBagConstraints gridBagConstraints = new GridBagConstraints();
		gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 0;
		gridBagConstraints.weightx = 1.0;
		gridBagConstraints.weighty = 1.0;
		gridBagConstraints.insets = new java.awt.Insets(5,8,5,8);
		this.setLayout(new GridBagLayout());
		this.setSize(445, 239);
		this.setPreferredSize(new java.awt.Dimension(445,239));
		this.add(getJScrollPane(), gridBagConstraints);
		this.addComponentListener(this);
		refresh(props, columnNames);
		
	}

	/**
	 * This method initializes jScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setPreferredSize(new java.awt.Dimension(440,260));
			jScrollPane.setViewportView(getJEditorPane());
			

		}
		return jScrollPane;
	}

	/**
	 * This method initializes jEditorPane	
	 * 	
	 * @return javax.swing.JEditorPane	
	 */
	private JEditorPane getJEditorPane() {
		if (jEditorPane == null) {
			jEditorPane = new JEditorPane();
			jEditorPane.setEditable(false);
			jEditorPane.setSize(new java.awt.Dimension(430,220));
			
		}
		return jEditorPane;
	}
	
	public void resetTable(){
		this.jEditorPane = null;
	}
	
	/**
	 * Obtiene el nombre del panel.
	 */
	public String getName(){
    	return this.nom;
    }
	
	/**
	 *Dibuja las tablas HTML con la informacin general de la imaagen raster 
	 * @param props
	 * @param columnNames
	 */
	
	public void refresh(Object props[][], Object columnNames[]){
		String propiedades="";
		String html;
		String color = "\"#FEEDD6\"";
		 
		
		String cabecera = 
			"  <tr valign=\"top\">" +
            "     <td width=\"140\" height=\"18\" bgcolor="+bgColor3+"align=\"center\"><font face=\"Arial\" size=\"3\" align=\"right\"><b>"+columnNames[0]+"</b></font></td>" +
            "     <td width=\"1586\" height=\"18\" bgcolor="+bgColor3+"align=\"left\"><font face=\"Arial\" size=\"3\"><b>"+columnNames[1]+"</b></font></td>" +
            "  </tr>";
		
		
		for(int i = 0; i<props.length; i++){
			
			if ((i%2 == 0) || i == 0) color = "\"#FEEDD6\"";
			else color = "\"#F2FEFF\"";
			
			if(props[i][1] == null) props[i][1] = "";
			
			if(props[i][0] != null){
				String propiedad = 
					"  <tr valign=\"top\">" +
					"     <td width=\"140\" height=\"18\" bgcolor="+bgColor1+"align=\"right\"><font face=\"Arial\" size=\"3\" align=\"right\">"+props[i][0]+"</font></td>" +
					"     <td width=\"1586\" height=\"18\" bgcolor="+color+"align=\"left\"><font face=\"Arial\" size=\"3\">"+props[i][1]+"</font></td>" +
					"  </tr>";
				propiedades = propiedades + propiedad; 
			}
		}
		
			
		html = "<html>"+cabecera + propiedades+"</html>";
		
		this.getJEditorPane().setContentType("text/html");
		this.getJEditorPane().setText(html);
		
	}

	public void componentHidden(ComponentEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void componentMoved(ComponentEvent e) {
		// TODO Auto-generated method stub
		
	}
	/**
	 * Cambia el tamao del panel cuando se redimensiona su contenedor
	 */
	public void componentResized(ComponentEvent e) {
		if(e.getSource() == this){
			this.getJScrollPane().setSize(this.getJScrollPane().getWidth() + 1, this.getJScrollPane().getHeight() + 1);
			this.getJEditorPane().setSize(this.getJEditorPane().getWidth() + 1, this.getJEditorPane().getHeight() + 1);
			//System.out.println("-->"+jScrollPane.getVerticalScrollBar().getValue());
			jScrollPane.getVerticalScrollBar().setValue(0);
		}
		
	}

	public void componentShown(ComponentEvent e) {
		// TODO Auto-generated method stub
		
	}
	
}


