package org.cresques.ui.raster;

import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.io.IOException;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * Panel con los 3 campos para introducir listas de valores RGB. Este
 * gestiona los eventos de FocusListener para validar que los valores 
 * que se introducen son correctos. Si el TextField pierde el foco y
 * tiene un valor incorrecto resetea el contenido.
 * 
 * Los mtodos publicos getRangeXXX devuelven el intervalo de valores
 * en forma de array multidimensional de 2xN elementos. Cada pareja de 
 * valores es un rango de color en ese valor de pixel. Un rango de valores
 * podria ser por ejemplo {{1,3},{15,30},{200, 255}}
 *  
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */
public class RGBInputPanel extends JPanel implements FocusListener{

	private JPanel 			pRed = null;
	private JPanel			pGreen = null;
	private JPanel 			pBlue = null;
	private JLabel			lRed = null;
	private JLabel 			lGreen = null;
	private JLabel 			lBlue = null;
	private JTextFieldExt	tRed = null;
	private JTextFieldExt	tGreen = null;
	private JTextFieldExt	tBlue = null;
	private int[] 			rangeRed = null;
    private int[] 			rangeGreen = null;
    private int[] 			rangeBlue = null;
    private boolean			validValue = false;

    /**
     * Componente JTextField que se pone con el color de fondo cuando
     * est desactivado y blanco cuando est activo.
     * @author Nacho Brodin (brodin_ign@gva.es)
     */
    public class JTextFieldExt extends JTextField {
        final private static long serialVersionUID = -3370601314380922368L;
        private Color color = null;

        public JTextFieldExt(Color color) {
            this.color = color;
        }

        public JTextFieldExt() {
            super();
        }

        public void setEnabled(boolean enabled) {
            super.setEnabled(enabled);

            if (!enabled) {
                if (color == null) {
                    this.setBackground(new Color(210, 210, 210));
                } else {
                    this.setBackground(color);
                }
            } else {
                this.setBackground(new Color(255, 255, 255));
            }
        }
    }
    
	/**
	 * This is the default constructor
	 */
	public RGBInputPanel() {
		super();
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
		gridBagConstraints2.gridx = 0;
		gridBagConstraints2.gridy = 2;
		GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
		gridBagConstraints1.gridx = 0;
		gridBagConstraints1.gridy = 1;
		GridBagConstraints gridBagConstraints = new GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 0;
		this.setLayout(new GridBagLayout());
		this.setSize(77, 60);
		this.setPreferredSize(new java.awt.Dimension(77,60));
		this.add(getPRed(), gridBagConstraints);
		this.add(getPGreen(), gridBagConstraints1);
		this.add(getPBlue(), gridBagConstraints2);
		this.getTBlue().addFocusListener(this);
		this.getTGreen().addFocusListener(this);
		this.getTRed().addFocusListener(this);
	}

	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPRed() {
		if (pRed == null) {
			FlowLayout flowLayout = new FlowLayout();
			flowLayout.setHgap(0);
			flowLayout.setVgap(0);
			lRed = new JLabel();
			lRed.setText("R :");
			pRed = new JPanel();
			pRed.setLayout(flowLayout);
			pRed.setPreferredSize(new java.awt.Dimension(76,20));
			pRed.add(lRed, null);
			pRed.add(getTRed(), null);
		}
		return pRed;
	}

	/**
	 * This method initializes jPanel1	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPGreen() {
		if (pGreen == null) {
			FlowLayout flowLayout1 = new FlowLayout();
			flowLayout1.setHgap(0);
			flowLayout1.setVgap(0);
			lGreen = new JLabel();
			lGreen.setText("G :");
			pGreen = new JPanel();
			pGreen.setLayout(flowLayout1);
			pGreen.setPreferredSize(new java.awt.Dimension(77,20));
			pGreen.add(lGreen, null);
			pGreen.add(getTGreen(), null);
		}
		return pGreen;
	}

	/**
	 * This method initializes jPanel2	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPBlue() {
		if (pBlue == null) {
			FlowLayout flowLayout2 = new FlowLayout();
			flowLayout2.setHgap(0);
			flowLayout2.setVgap(0);
			lBlue = new JLabel();
			lBlue.setText("B :");
			pBlue = new JPanel();
			pBlue.setLayout(flowLayout2);
			pBlue.setPreferredSize(new java.awt.Dimension(76,20));
			pBlue.add(lBlue, null);
			pBlue.add(getTBlue(), null);
		}
		return pBlue;
	}

	/**
	 * This method initializes jTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	public JTextField getTRed() {
		if (tRed == null) {
			tRed = new JTextFieldExt();
			tRed.setPreferredSize(new java.awt.Dimension(60,19));
		}
		return tRed;
	}

	/**
	 * This method initializes jTextField1	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	public JTextField getTGreen() {
		if (tGreen == null) {
			tGreen = new JTextFieldExt();
			tGreen.setPreferredSize(new java.awt.Dimension(60,19));
		}
		return tGreen;
	}

	/**
	 * This method initializes jTextField2	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	public JTextField getTBlue() {
		if (tBlue == null) {
			tBlue = new JTextFieldExt();
			tBlue.setPreferredSize(new java.awt.Dimension(60,19));
		}
		return tBlue;
	}

	public void focusGained(FocusEvent e) {
		// TODO Auto-generated method stub
		
	}

	/**
     * Obtiene el rango de valores a partir de la cadena de texto introducida por el usuario. 
     * Sirve para parsear el contenido de los campos de texto que contienen listas
     * de valores para RGB.
     * @param values
     */
    public static int[] stringToInterval(String values) throws IOException {
        int[] rangeTransparency = new int[2];

        boolean dots = false;
        for (int i = 0; i < values.length(); i++){
        	char c = values.charAt(i);
        	if(c != ':' && (c < 48 || c > 57))
        		throw new IOException("Caracteres incorrectos en la cadena.");
        	if(c == ':' && dots)
        		throw new IOException("Se repite el caracter :.");
        	if(c == ':' && !dots)
        		dots = true;
        	if(c == ':' && (i == 0 || i == (values.length() - 1)))
        		throw new IOException("No puede comenzar ni acabar por :.");        	
        }
        
        if(dots){
	        String[] interv = values.split(":");
	        int val1 = Integer.parseInt(interv[0]);
	        int val2 = Integer.parseInt(interv[1]);
	        if(val1 > 255 || val2 > 255)
	        	throw new IOException("Valor de pixel erroneo.");
	        else{
	        	if(val1 < val2){
	        		rangeTransparency[0] = val1;
	        		rangeTransparency[1] = val2;
	        	}else{
	        		rangeTransparency[0] = val2;
	        		rangeTransparency[1] = val1;
	        	}
	        	return rangeTransparency;
	        }
        }else{
        	rangeTransparency[0] = rangeTransparency[1] = Integer.parseInt(values);
        	if(rangeTransparency[0] > 255)
        		throw new IOException("Valor de pixel erroneo.");
        	return rangeTransparency;
        }

    }
    
	public void focusLost(FocusEvent e) {
		validValue = true;
		rangeRed = null;
		rangeGreen = null;
		rangeBlue = null;
		if (!getTRed().getText().equals("")) {
            try {
                rangeRed = RGBInputPanel.stringToInterval(getTRed().getText());
            } catch (IOException exc) {
            	exc.printStackTrace();
                getTRed().setText("");
                validValue = false;
            }
        }

        if (!getTGreen().getText().equals("")) {
            try {
                rangeGreen = RGBInputPanel.stringToInterval(getTGreen().getText());
            } catch (IOException exc) {
            	exc.printStackTrace();
                getTGreen().setText("");
                validValue = false;
            }
        }

        if (!getTBlue().getText().equals("")) {
            try {
                rangeBlue = RGBInputPanel.stringToInterval(getTBlue().getText());
            } catch (IOException exc) {
            	exc.printStackTrace();
                getTBlue().setText("");
                validValue = false;
            }
        }
	}

	/**
	 * Obtiene la lista de valores del campo Blue.
	 * @return array multidimensional con los intervalos
	 */
	public int[] getRangeBlue() {
		return rangeBlue;
	}

	/**
	 * Obtiene la lista de valores del campo Green.
	 * @return array multidimensional con los intervalos
	 */
	public int[] getRangeGreen() {
		return rangeGreen;
	}

	/**
	 * Obtiene la lista de valores del campo Red.
	 * @return array multidimensional con los intervalos
	 */
	public int[] getRangeRed() {
		return rangeRed;
	}

	/**
	 * Obtiene true si los valores de RGB que han introducido son 
	 * validos y false si no lo son.
	 * @return
	 */
	public boolean isValidValue() {
		return validValue;
	}

	/**
	 * Activa o desactiva el control
	 * @param enable True activa el control y false lo desactiva
	 */
	public void setControlEnabled(boolean enabled){
		getTRed().setEnabled(enabled);
		getTGreen().setEnabled(enabled);
		getTBlue().setEnabled(enabled);
	}

	/**
	 * Asigna el nmero de bandas activas 
	 * @param n Nmero de bandas
	 */
	public void setActiveBands(int n){
		this.getTRed().setEnabled(true);
		switch(n){
		case 1:	this.getTGreen().setEnabled(false);
				this.getTBlue().setEnabled(false);
				break;
		case 2:	this.getTGreen().setEnabled(true);
				this.getTBlue().setEnabled(false);
				break;
		case 3:	this.getTGreen().setEnabled(true);
				this.getTBlue().setEnabled(true);
				break;
		}
	}
}
