/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.ui.raster;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;

import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;


/**
 * Dialogo para asignar la transparencia por pixel y global al raster.
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class RasterTransparencyPanel extends JPanel implements ComponentListener{
    final private static long serialVersionUID = -3370601314380922368L;

    /**
     * Nombre del panel
     */
    private String nom = "Transparencia";
    FilterRasterDialogPanel parent = null;

    /**
     * Nmero de bandas del raster
     */
    public int nBands = 3;
    private JPanel jPanel = null;
    private JPanel jPanel1 = null;
    private JCheckBox cbTransparencia = null;

    /**
     * Variable pblica para la traduccin
     */
    public JPanel pGeneralTrans = null;

    private JCheckBox cbOpacidad = null;
    private JSlider slOpacidad = null;
    private JTextField tOpacidad = null;

	private TransparencyByPixelPanel pTranspByPixel = null;

	private JPanel pTransparencyByPixel = null;
		
    /**
     * Constructor. Asigna la variable panel que contiene este
     * @param parent FilterRasterDialogPanel
     */
    public RasterTransparencyPanel(FilterRasterDialogPanel parent) {
        this.parent = parent;
        initialize();
    }

    /**
     * This method initializes this
     *
     * @return void
     */
    private void initialize() {
        this.setLayout(new BorderLayout());
        this.setBounds(0, 0, 445, 239);
        this.setPreferredSize(new java.awt.Dimension(445, 239));
        this.add(getJPanel(), java.awt.BorderLayout.CENTER);
        initControls();
        this.addComponentListener(this);
    }

    /**
     * Obtiene el nombre del panel
     * @return Cadena con el nombre del panel
     */
    public String getName(){
    	return this.nom;
    }
    
    /**
     * Asigna el nmero de bandas de la imagen
     * @param nBands
     */
    public void setBands(int nBands) {
        this.nBands = nBands;
    }

    /**
     * Inicializa controles a sus valores por defecto
     */
    public void initControls() {
        this.getOpacityText().setText("100");
        this.setActiveOpacityControl(false);
        this.setActiveTransparencyControl(false);
    }

    /**
     * This method initializes jPanel
     *
     * @return javax.swing.JPanel
     */
    private JPanel getJPanel() {
        if (jPanel == null) {
            GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
            gridBagConstraints1.gridx = 0;
            gridBagConstraints1.gridy = 0;
            GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
            jPanel = new JPanel();
            jPanel.setLayout(new GridBagLayout());
            gridBagConstraints7.gridwidth = 1;
            gridBagConstraints7.gridy = 2;
            gridBagConstraints7.gridx = 0;
            gridBagConstraints7.insets = new java.awt.Insets(5,0,0,0);
            jPanel.setPreferredSize(new java.awt.Dimension(333, 113));
            jPanel.add(getJPanel4(), gridBagConstraints7);
            jPanel.add(getPTransparencyByPixel(), gridBagConstraints1);
        }

        return jPanel;
    }

    /**
     * This method initializes jPanel1
     *
     * @return javax.swing.JPanel
     */
    private JPanel getJPanel1() {
        if (jPanel1 == null) {
            jPanel1 = new JPanel();
            jPanel1.setLayout(new BorderLayout());
            jPanel1.setPreferredSize(new java.awt.Dimension(415,25));
            jPanel1.setBorder(javax.swing.BorderFactory.createEmptyBorder(0,0,0,0));
            jPanel1.add(getTransparencyCheck(), java.awt.BorderLayout.NORTH);
        }

        return jPanel1;
    }

    /**
     * This method initializes jCheckBox
     *
     * @return javax.swing.JCheckBox
     */
    public JCheckBox getTransparencyCheck() {
        if (cbTransparencia == null) {
            cbTransparencia = new JCheckBox();
            cbTransparencia.setText("Transparencia por pixel");
        }

        return cbTransparencia;
    }

    /**
     * This method initializes jPanel4
     *
     * @return javax.swing.JPanel
     */
    private JPanel getJPanel4() {
        if (pGeneralTrans == null) {
        	pGeneralTrans = new JPanel();
        	pGeneralTrans.setBorder(javax.swing.BorderFactory.createLineBorder(java.awt.Color.gray, 1));
        	pGeneralTrans.setPreferredSize(new java.awt.Dimension(415, 35));
        	pGeneralTrans.add(getOpacityCheck(), null);
        	pGeneralTrans.add(getOpacitySlider(), null);
        	pGeneralTrans.add(getOpacityText(), null);
        }

        return pGeneralTrans;
    }

    /**
     * This method initializes jCheckBox1
     *
     * @return javax.swing.JCheckBox
     */
    public JCheckBox getOpacityCheck() {
        if (cbOpacidad == null) {
            cbOpacidad = new JCheckBox();
            cbOpacidad.setText("Opacidad");
        }

        return cbOpacidad;
    }

    /**
     * This method initializes jSlider
     *
     * @return javax.swing.JSlider
     */
    public JSlider getOpacitySlider() {
        if (slOpacidad == null) {
            slOpacidad = new JSlider();
            slOpacidad.setPreferredSize(new java.awt.Dimension(280, 16));
        }

        return slOpacidad;
    }

    /**
     * This method initializes jTextField3
     *
     * @return javax.swing.JTextField
     */
    public JTextField getOpacityText() {
        if (tOpacidad == null) {
            tOpacidad = new JTextField();
            tOpacidad.setPreferredSize(new java.awt.Dimension(30, 19));
        }

        return tOpacidad;
    }

    /**
     * Activa/Desactiva los controles de opacidad
     * @param active
     */
    public void setActiveOpacityControl(boolean active) {
        this.getOpacityCheck().setSelected(active);
        this.getOpacitySlider().setEnabled(active);
        this.getOpacityText().setEnabled(active);
    }

    /**
     * Activa/Desactiva los controles de transparencia
     * @param active
     */
    public void setActiveTransparencyControl(boolean active) {
        this.getTransparencyCheck().setSelected(active);
        RGBInputPanel rgbPanel = this.getPTranspByPixel().getPRGBInput();
        rgbPanel.getTRed().setEnabled(active);
        
        if (parent.nbands == 2) {
        	rgbPanel.getTGreen().setEnabled(active);
        	rgbPanel.getTBlue().setEnabled(false);
        }

        if (parent.nbands == 3) {
        	rgbPanel.getTGreen().setEnabled(active);
        	rgbPanel.getTBlue().setEnabled(active);
        }
    }

    /**
     * Asigna el valor de opacidad a los controles de la ventana
     * para que cuando esta se abra tenga los valores seleccionados en la
     * imagen.
     * @param alpha
     */
    public void setOpacity(int alpha) {
        int opacityPercent = (int) ((alpha * 100) / 255);
        this.getOpacityText().setText(String.valueOf(opacityPercent));
        this.getOpacitySlider().setValue(opacityPercent);
        this.setActiveOpacityControl(true);

        if (opacityPercent == 100) {
            this.setActiveOpacityControl(false);
        }
    }

	/**
	 * This method initializes jPanel2	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	public TransparencyByPixelPanel getPTranspByPixel() {
		if (pTranspByPixel == null) {
			pTranspByPixel = new TransparencyByPixelPanel();
			pTranspByPixel.setPreferredSize(new java.awt.Dimension(415,128));
			pTranspByPixel.setBorder(javax.swing.BorderFactory.createEmptyBorder(0,0,0,0));
			pTranspByPixel.setControlEnabled(false);
		}
		return pTranspByPixel;
	}

	/**
	 * This method initializes jPanel2	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPTransparencyByPixel() {
		if (pTransparencyByPixel == null) {
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.gridx = 0;
			gridBagConstraints.gridy = 1;
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.anchor = GridBagConstraints.WEST;
			gridBagConstraints2.insets = new Insets(0, 0, 5, 0);
			gridBagConstraints2.gridheight = 1;
			gridBagConstraints2.gridwidth = 0;
			gridBagConstraints2.gridx = -1;
			gridBagConstraints2.gridy = -1;
			gridBagConstraints2.ipady = 0;
			gridBagConstraints2.fill = GridBagConstraints.NONE;
			pTransparencyByPixel = new JPanel();
			pTransparencyByPixel.setLayout(new GridBagLayout());
			pTransparencyByPixel.setBorder(javax.swing.BorderFactory.createLineBorder(java.awt.Color.gray,1));
			pTransparencyByPixel.add(getJPanel1(), gridBagConstraints2);
			pTransparencyByPixel.add(getPTranspByPixel(), gridBagConstraints);
		}
		return pTransparencyByPixel;
	}

	public void componentHidden(ComponentEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void componentMoved(ComponentEvent e) {
		// TODO Auto-generated method stub
		
	}
	/**
	 * Redimensiona el panel cuando se redimensiona el contenedor de ste
	 */
	public void componentResized(ComponentEvent e) {
		if (e.getSource() == this){
			int nWidth = this.getSize().width;
			int nHeight = this.getSize().height;
			int difWidth = nWidth - 445;
			int difHeight = nHeight - 239;
			this.pTransparencyByPixel.setSize(this.pTransparencyByPixel.getSize().width + difWidth, this.pTransparencyByPixel.getSize().height + difHeight);
			this.pTransparencyByPixel.setLocation(14, 19);
			this.pGeneralTrans.setSize(this.pGeneralTrans.getSize().width + difWidth, this.pGeneralTrans.getSize().height);
			this.pGeneralTrans.setLocation(14, this.pGeneralTrans.getLocation().y + difHeight/2);
			this.slOpacidad.setSize(this.slOpacidad.getSize().width + difWidth, this.slOpacidad.getSize().height);
			this.tOpacidad.setLocation(this.tOpacidad.getLocation().x + difWidth, this.tOpacidad.getLocation().y);
			this.pTranspByPixel.setLocation(this.pTranspByPixel.getLocation().x + difWidth/2, this.pTranspByPixel.getLocation().y + difHeight/2);
						
		}
		
	}

	public void componentShown(ComponentEvent e) {
		// TODO Auto-generated method stub
		
	}
	
	
} //  @jve:decl-index=0:visual-constraint="36,15"
