/*
 * Created on 26-oct-2005
 *
 * gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 * 
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 * 
 *    or
 * 
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 * 
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.drivers;

import java.awt.geom.Rectangle2D;
import java.sql.Connection;
import java.sql.Types;

import com.hardcode.gdbms.engine.instruction.Field;

public class DBLayerDefinition extends LayerDefinition {
	private Connection conn;
    private String catalogName;
    private String tableName;
    // private String[] fieldNames; // GeometryField not included
    private String fieldID; // Estos 2 campos en PostGIS los sabremos 
    private String fieldGeometry; // (gid y the_geom), pero por si acaso se cambian
    private String whereClause;
    private Rectangle2D workingArea;
    private String SRID_EPSG;
    private String classToInstantiate;
    
    // private int idFieldGeometry = -1;
    private int idFieldID = -1;
	private int dimension = 2;
    
    public String getCatalogName() {
        return catalogName;
    }
    public void setCatalogName(String catalogName) {
        this.catalogName = catalogName;
    }
    public String getFieldGeometry() {
        return fieldGeometry;
    }
    public void setFieldGeometry(String fieldGeometry) {
        this.fieldGeometry = fieldGeometry;
    }
    public String getFieldID() {
        return fieldID;
    }
    public void setFieldID(String fieldID) {
        this.fieldID = fieldID;
    }
    /**
     * @return GeometryField is not included in this list
     */
    public String[] getFieldNames() {
    	String[] fieldNames = new String[fieldsDesc.length];
    	for (int i=0; i< fieldsDesc.length; i++)
    	{
    		fieldNames[i] = fieldsDesc[i].getFieldName();
    	}
        return fieldNames;
    }
    /**
     * Geometry field not included in this list
     * @param fieldNames
     */
    public void setFieldNames(String[] fieldNames) {
        idFieldID = -1;
        // idFieldGeometry = -1;
        if (fieldsDesc == null)
        	fieldsDesc = new FieldDescription[fieldNames.length];
        if (fieldsDesc.length != fieldNames.length)
        	fieldsDesc = new FieldDescription[fieldNames.length];
        for (int i=0; i < fieldNames.length; i++)
        {
        	fieldsDesc[i] = new FieldDescription();
        	fieldsDesc[i].setFieldName(fieldNames[i]);
        }
    }
    public String getTableName() {
        return tableName;
    }
    public void setTableName(String tableName) {
        this.tableName = tableName;
    }
    /**
     * The returned string must contain "WHERE " clause
     * @return
     */
    public String getWhereClause() {
        return whereClause;
    }
    public void setWhereClause(String whereClause) {
        this.whereClause = whereClause;
    }
    public Rectangle2D getWorkingArea() {
        return workingArea;
    }
    public void setWorkingArea(Rectangle2D workingArea) {
        this.workingArea = workingArea;
    }
    
    /**
     * @param fieldName
     * @return index of field (0 based). -1 if not found
     */
    public int getIdField(String fieldName)
    {
        int result = -1;
        for (int i=0; i < fieldsDesc.length; i++)
        {
            if (fieldName.equalsIgnoreCase(fieldsDesc[i].getFieldName()))
            {
                result = i;
                break;
            }
        }
        return result;
    }
    
    /**
     * 0 based index of the ID field
     * @return Returns the idFieldID.
     */
    public int getIdFieldID() {
        if (idFieldID == -1)
            idFieldID = getIdField(fieldID);        
        return idFieldID;
    }
    /**
     * @return Returns the sRID_EPSG.
     */
    public String getSRID_EPSG() {
        return SRID_EPSG;
    }
    /**
     * @param srid_epsg The sRID_EPSG to set.
     */
    public void setSRID_EPSG(String srid_epsg) {
        SRID_EPSG = srid_epsg;
    }
    /**
     * @return Returns the classToInstantiate.
     */
    public String getClassToInstantiate() {
        return classToInstantiate;
    }
    /**
     * @param classToInstantiate The classToInstantiate to set.
     */
    public void setClassToInstantiate(String classToInstantiate) {
        this.classToInstantiate = classToInstantiate;
    }
    
    public int getFieldIdByName(String name)
    {
        int resul = -1;
        for (int i=0; i<fieldsDesc.length; i++)
        {
            if (fieldsDesc[i].getFieldName().equals(name))
            {
                resul = i;
                break;
            }
        }
        return resul;
    }
	public Connection getConnection() {
		return conn;
	}
	
	public void setConnection(Connection connection) {
		conn = connection;
	}
	public int getDimension() {
		return dimension;
	}

	public void setDimension(int dim) {
		dimension = dim;
	}
	
}
