/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.drivers.shp;

import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.channels.FileChannel;
import java.util.Properties;
import java.util.Random;

import org.apache.log4j.Logger;

import com.hardcode.gdbms.driver.DriverUtilities;
import com.iver.cit.gvsig.fmap.core.FShape;
import com.iver.cit.gvsig.fmap.core.GeneralPathX;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.core.ShapeFactory;
import com.iver.cit.gvsig.fmap.drivers.BoundedShapes;
import com.iver.cit.gvsig.fmap.drivers.DriverAttributes;
import com.iver.cit.gvsig.fmap.drivers.ExternalData;
import com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver;
import com.iver.cit.gvsig.fmap.edition.EditionException;
import com.iver.cit.gvsig.fmap.edition.IRowEdited;
import com.iver.cit.gvsig.fmap.edition.ISpatialWriter;
import com.iver.cit.gvsig.fmap.edition.writers.shp.ShpWriter;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.utiles.bigfile.BigByteBuffer2;


/**
 * Driver del formato SHP. Usa ByteBuffer2, que no consume
 * memoria (bueno, 8KBytes... !) y no le importa que le pidan
 * entidades desordenadas del fichero, lo cual es indispensable
 * para trabajar con ndices espaciales.
 * Adems, el tiempo de espera para abrir un fichero y empezar a pintar es
 * prcticamente nulo, tanto desde disco duro como por red.
 * La nica pega es que en ficheros grandes puede ser unos
 * milisegundos un poco ms lento que el otro, pero muy poco.
 *
 * @author Francisco Jos Pearrubia
 */
public class IndexedShpDriver implements VectorialFileDriver, BoundedShapes,
	ExternalData, ISpatialWriter {
	private static Logger logger = Logger.getLogger(IndexedShpDriver.class.getName());
	private static String tempDirectoryPath = System.getProperty("java.io.tmpdir");
	private File fileShp;
	private File fTemp;
	private BigByteBuffer2 bb;
	private FileChannel channel;
	private FileInputStream fin;
	private int type;
	// private long[] m_posShapes;
	private int numReg;
	private Rectangle2D extent;

    // private File shxFile;
    private BigByteBuffer2 bbShx;
    private FileChannel channelShx;
    private FileInputStream finShx;

    private ShpWriter shpWriter = new ShpWriter();


	/**
	 * Cierra el fichero.
	 *
	 * @throws IOException
	 *
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#close()
	 */
	public void close() throws IOException {
		IOException ret = null;

		try {
			channel.close();
            channelShx.close();
		} catch (IOException e) {
			ret = e;
		} finally {
			try {
				fin.close();
			} catch (IOException e1) {
				ret = e1;
			}
		}

		if (ret != null) {
			throw ret;
		}
		else // Si todo ha ido bien, preparamos para liberar memoria.
        {
		    bb = null;
            bbShx = null;
        }
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#open(java.io.File)
	 */
	public void open(File f) throws IOException {
		fileShp = f;

		fin = new FileInputStream(f);

		// Open the file and then get a channel from the stream
		channel = fin.getChannel();

		// long size = channel.size();

		// Get the file's size and then map it into memory
		// bb = channel.map(FileChannel.MapMode.READ_ONLY, 0, size);
        bb = new BigByteBuffer2(channel, FileChannel.MapMode.READ_ONLY);

        finShx = new FileInputStream(getShxFile(f));

        // Open the file and then get a channel from the stream
        channelShx = finShx.getChannel();

        long sizeShx = channelShx.size();

        // Get the file's size and then map it into memory
        // bb = channel.map(FileChannel.MapMode.READ_ONLY, 0, size);
        // bbShx = channelShx.map(FileChannel.MapMode.READ_ONLY, 0, sizeShx);
        bbShx = new BigByteBuffer2(channelShx, FileChannel.MapMode.READ_ONLY);
        bbShx.order(ByteOrder.BIG_ENDIAN);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param index DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 *
	 * @throws IOException
	 *
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#getShape(int)
	 */

	/* public FShape getShapeByID(int ID) {
	   Point2D.Double p = new Point2D.Double();
	   Point2D.Double pAnt = null;
	   int numParts;
	   int numPoints;
	   int i;
	   int j;
	   int numReg;
	   int numeroPuntos;
	   int hasta;
	   int desde;
	   Rectangle2D.Double BoundingBox = new Rectangle2D.Double();

	           SHPShape shapeShp=null;
	   FShape resulShape = null;
	   try {
	       bb.position(m_posShapes[ID]);
	       bb.order(ByteOrder.LITTLE_ENDIAN);
	       int tipoShape = bb.getInt();
	       m_shapeType = tipoShape;
	       // retrieve that shape.
	       // tempRecord.setShape(readShape(tempShapeType, tempContentLength, in));
	       if (tipoShape == FConstant.SHAPE_TYPE_POINT) {
	           p = readPoint(bb);
	           resulShape = new FShape(new FPoint(p.getX(),p.getY()),FConstant.SHAPE_TYPE_POINT);
	           //Comprobacin punto.
	           //System.err.println("p.x = "+p.x);
	           //System.err.println("p.y = "+p.y);

	       } else if ((tipoShape == FConstant.SHAPE_TYPE_POLYLINE) ||
	               (tipoShape == FConstant.SHAPE_TYPE_POLYGON)) {
	           // BoundingBox
	           BoundingBox = readRectangle(bb);
	           numParts = bb.getInt();
	           numPoints = bb.getInt();
	           // part indexes.
	           // Geometry geom = GeometryFactory.toGeometryArray();
	           GeneralPathX elShape = new GeneralPathX(GeneralPathX.WIND_EVEN_ODD,
	                   numPoints);
	           int[] tempParts = new int[numParts];
	           for (i = 0; i < numParts; i++) {
	               tempParts[i] = bb.getInt();
	           }
	           j = 0;
	           ///Line2D.Double line2D;
	           FPoint[] points=new FPoint[numPoints];
	           for (i = 0; i < numPoints; i++) {
	                   p=readPoint(bb);
	               points[i] = new FPoint(p.x,p.y);
	               // System.out.println("x= " + p.x + " y=" + p.y);
	               // System.out.println("x= " + (float) p.x + " y=" + (float) p.y);
	               if (i == tempParts[j]) {
	                   elShape.moveTo(p.x, p.y);
	                   if (j < (numParts - 1)) {
	                       j++;
	                   }
	               } else {
	                   elShape.lineTo(p.x, p.y);
	               }
	           }
	           //FGeometry pol=new FPolyLine(points,tempParts,BoundingBox);

	           resulShape = new FShape(tipoShape,elShape);
	       } else if (tipoShape == FConstant.SHAPE_TYPE_MULTIPOINT) {
	           // BoundingBox
	           BoundingBox = readRectangle(bb);
	           numPoints = bb.getInt();
	           FPoint[] tempPoints = new FPoint[numPoints];
	           for (i = 0; i < numPoints; i++) {
	                   Point2D p2=readPoint(bb);
	               tempPoints[i] = new FPoint(p2.getX(),p2.getY(),0);
	           }
	           FMultiPoint multipoint = new FMultiPoint(tempPoints,BoundingBox);
	           resulShape = new FShape(multipoint,tipoShape);
	       } else if (tipoShape == FConstant.SHAPE_TYPE_POINTZ) {
	           FPoint p3d = new FPoint();
	           p3d.read(bb);
	           resulShape = new FShape(p3d,tipoShape);
	       } else if ((tipoShape == FConstant.SHAPE_TYPE_POLYLINEZ) ||
	               (tipoShape == FConstant.SHAPE_TYPE_POLYGONZ)) {
	           // BoundingBox
	           BoundingBox = readRectangle(bb);
	           numParts = bb.getInt();
	           numPoints = bb.getInt();
	           // part indexes.
	           // Geometry geom = GeometryFactory.toGeometryArray();

	           GeneralPathX elShape = new GeneralPathX(GeneralPathX.WIND_EVEN_ODD,
	                   numPoints);
	           int[] tempParts = new int[numParts];
	           for (i = 0; i < numParts; i++) {
	               tempParts[i] = bb.getInt();
	           }
	           j = 0;
	          //Line2D.Double line2D;
	           FPoint[] points=new FPoint[numPoints];
	           for (i = 0; i < numPoints; i++) {
	               p = readPoint(bb);
	               points[i]=new FPoint(p.x,p.y);

	               if (i == tempParts[j]) {
	                   elShape.moveTo(p.x, p.y);
	                   if (j < (numParts - 1)) {
	                       j++;
	                   }
	               } else {
	                   elShape.lineTo(p.x, p.y);
	               }

	           }

	           double[] boxZ = new double[2];
	           boxZ[0] = bb.getDouble();
	           boxZ[1] = bb.getDouble();
	           double[] pZ = new double[numPoints];
	           for (i = 0; i < numPoints; i++) {
	               pZ[i] = bb.getDouble();
	           }
	           //FGeometry pol=new FPolyLine(points,tempParts,BoundingBox);
	           resulShape = new FShape(tipoShape, elShape, pZ);
	       } else if (tipoShape == FConstant.SHAPE_TYPE_MULTIPOINTZ) {
	           // BoundingBox
	           BoundingBox = readRectangle(bb);
	           numPoints = bb.getInt();
	           FPoint[] tempPoints3D = new FPoint[numPoints];
	           for (i = 0; i < numPoints; i++) {
	               tempPoints3D[i] = new FPoint();
	               tempPoints3D[i].read(bb);
	           }
	           FMultiPoint multipoint3D = new FMultiPoint(tempPoints3D,BoundingBox);
	           resulShape = new FShape(multipoint3D,tipoShape);
	       }
	   } catch (Exception e) {
	       System.err.println("Fallo en getShapeByID. ID=" + ID +
	           " m_posShapes[ID]=" + m_posShapes[ID]);
	       System.err.println("getShapeByID: " + e.getMessage());
	       e.printStackTrace();
	   }
	   return resulShape;
	   }
	 */

	/**
	 * Devuelve la geometria a partir de un ndice.
	 *
	 * @param index DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 *
	 * @throws IOException DOCUMENT ME!
	 */
	public synchronized IGeometry getShape(int index) throws IOException {
		Point2D.Double p = new Point2D.Double();
		int numParts;
		int numPoints;
		int i;
		int j;
		/* int numReg;
		int numeroPuntos;
		int hasta;
		int desde; */
		int shapeType;

		//Rectangle2D.Double BoundingBox;
        // if (m_posShapes[index] == 0)

		// bb.position(m_posShapes[index]);
        bb.position(getPositionForRecord(index));
		bb.order(ByteOrder.LITTLE_ENDIAN);

		///bb.position(bb.position()+4);
		shapeType = bb.getInt();
		//el shape tal con tema tal y nmro tal es null
		if (shapeType==SHP.NULL){
			logger.info("El shape ="+index+ " de la capa ="+this.toString()+" es null");
			return null;
		}

		// retrieve that shape.
		// tempRecord.setShape(readShape(tempShapeType, tempContentLength, in));
		switch (type) {
			case (SHP.POINT2D):
				p = readPoint(bb);

				return ShapeFactory.createPoint2D(p.getX(), p.getY());

			case (SHP.POLYLINE2D):

				//BoundingBox = readRectangle(bb);
				//bb.getDouble();
				//bb.getDouble();
				//bb.getDouble();
				//bb.getDouble();
				bb.position(bb.position() + 32);
				numParts = bb.getInt();
				numPoints = bb.getInt();

				// part indexes.
				// Geometry geom = GeometryFactory.toGeometryArray();
				GeneralPathX elShape = new GeneralPathX(GeneralPathX.WIND_EVEN_ODD,
						numPoints);

				int[] tempParts = new int[numParts];

				for (i = 0; i < numParts; i++) {
					tempParts[i] = bb.getInt();
				}

				j = 0;

				for (i = 0; i < numPoints; i++) {
					p = readPoint(bb);

					if (i == tempParts[j]) {
						elShape.moveTo(p.x, p.y);

						if (j < (numParts - 1)) {
							j++;
						}
					} else {
						elShape.lineTo(p.x, p.y);
					}
				}

				return ShapeFactory.createPolyline2D(elShape);

			case (SHP.POLYGON2D):

				//	            	BoundingBox = readRectangle(bb);
				bb.getDouble();
				bb.getDouble();
				bb.getDouble();
				bb.getDouble();

				numParts = bb.getInt();

				numPoints = bb.getInt();

				// part indexes.
				// Geometry geom = GeometryFactory.toGeometryArray();
				elShape = new GeneralPathX(GeneralPathX.WIND_EVEN_ODD, numPoints);

				tempParts = new int[numParts];

				for (i = 0; i < numParts; i++) {
					tempParts[i] = bb.getInt();
				}

				j = 0;

				for (i = 0; i < numPoints; i++) {
					p = readPoint(bb);

					if (i == tempParts[j]) {
						elShape.moveTo(p.x, p.y);

						if (j < (numParts - 1)) {
							j++;
						}
					} else {
						elShape.lineTo(p.x, p.y);
					}
				}

				return ShapeFactory.createPolygon2D(elShape);

			case (SHP.POINT3D):

				double x = bb.getDouble();
				double y = bb.getDouble();
				double z = bb.getDouble();

				return ShapeFactory.createPoint3D(x, y, z);

			case (SHP.POLYLINE3D):
				bb.position(bb.position() + 32);
				numParts = bb.getInt();
				numPoints = bb.getInt();
				elShape = new GeneralPathX(GeneralPathX.WIND_EVEN_ODD, numPoints);
				tempParts = new int[numParts];

				for (i = 0; i < numParts; i++) {
					tempParts[i] = bb.getInt();
				}

				j = 0;

				for (i = 0; i < numPoints; i++) {
					p = readPoint(bb);

					if (i == tempParts[j]) {
						elShape.moveTo(p.x, p.y);

						if (j < (numParts - 1)) {
							j++;
						}
					} else {
						elShape.lineTo(p.x, p.y);
					}
				}

				double[] boxZ = new double[2];
				boxZ[0] = bb.getDouble();
				boxZ[1] = bb.getDouble();

				double[] pZ = new double[numPoints];

				for (i = 0; i < numPoints; i++) {
					pZ[i] = bb.getDouble();
				}

				return ShapeFactory.createPolyline3D(elShape, pZ);
			case (SHP.POLYGON3D):
			bb.position(bb.position() + 32);
			numParts = bb.getInt();
			numPoints = bb.getInt();
			elShape = new GeneralPathX(GeneralPathX.WIND_EVEN_ODD, numPoints);
			tempParts = new int[numParts];

			for (i = 0; i < numParts; i++) {
				tempParts[i] = bb.getInt();
			}

			j = 0;

			for (i = 0; i < numPoints; i++) {
				p = readPoint(bb);

				if (i == tempParts[j]) {
					elShape.moveTo(p.x, p.y);

					if (j < (numParts - 1)) {
						j++;
					}
				} else {
					elShape.lineTo(p.x, p.y);
				}
			}

			double[] boxpoZ = new double[2];
			boxpoZ[0] = bb.getDouble();
			boxpoZ[1] = bb.getDouble();

			double[] poZ = new double[numPoints];

			for (i = 0; i < numPoints; i++) {
				poZ[i] = bb.getDouble();
			}

			return ShapeFactory.createPolygon3D(elShape, poZ);

			case (SHP.MULTIPOINT2D):
				bb.position(bb.position() + 32);
				numPoints = bb.getInt();

				double[] tempX = new double[numPoints];
				double[] tempY = new double[numPoints];

				for (i = 0; i < numPoints; i++) {
					tempX[i] = bb.getDouble();
					tempY[i] = bb.getDouble();
				}

				return ShapeFactory.createMultipoint2D(tempX, tempY);

			case (SHP.MULTIPOINT3D):
				bb.position(bb.position() + 32);
				numPoints = bb.getInt();

				double[] temX = new double[numPoints];
				double[] temY = new double[numPoints];
				double[] temZ = new double[numPoints];

				for (i = 0; i < numPoints; i++) {
					temX[i] = bb.getDouble();
					temY[i] = bb.getDouble();
					//temZ[i] = bb.getDouble();
				}

				for (i = 0; i < numPoints; i++) {
					temZ[i] = bb.getDouble();
				}
				return ShapeFactory.createMultipoint3D(temX, temY, temZ);
		}

		return null;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#getShapeCount()
	 */
	public int getShapeCount() {
		return numReg;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialDriver#getShapeType()
	 */
	public int getShapeType() {
		int auxType = 0;

		switch (type) {
			case (SHP.POINT2D):
			case (SHP.POINT3D):
				auxType = auxType | FShape.POINT;

				break;

			case (SHP.POLYLINE2D):
			case (SHP.POLYLINE3D):
				auxType = auxType | FShape.LINE;

				break;

			case (SHP.POLYGON2D):
			case (SHP.POLYGON3D):
				auxType = auxType | FShape.POLYGON;

				break;
			case (SHP.MULTIPOINT2D):
			case (SHP.MULTIPOINT3D):
				auxType = auxType | FShape.MULTIPOINT;

				break;
		}

		return auxType;
	}


	/**
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#initialize()
	 */
	public void initialize() throws IOException {
		// create a new header.
		ShapeFileHeader2 myHeader = new ShapeFileHeader2();

		bb.position(0);

		// read the header
		myHeader.readHeader(bb);

		extent = new Rectangle2D.Double(myHeader.myXmin, myHeader.myYmin,
				myHeader.myXmax - myHeader.myXmin,
				myHeader.myYmax - myHeader.myYmin);

		type = myHeader.myShapeType;

		double x = myHeader.myXmin;
		double y = myHeader.myYmin;
		double w = myHeader.myXmax - myHeader.myXmin;
		double h = myHeader.myYmax - myHeader.myYmin;

		if (w == 0) {
			x -= 0.1;
			w = 0.2;
		}

		if (h == 0) {
			y -= 0.1;
			h = 0.2;
		}

		// String strFichDbf = m_Path.toLowerCase().replaceAll("\\.shp", ".dbf");
		String strFichDbf = fileShp.getAbsolutePath().replaceAll("\\.shp", ".dbf");
		strFichDbf = strFichDbf.replaceAll("\\.SHP", ".DBF");

		DbaseFileNIO m_FichDbf = new DbaseFileNIO();

		m_FichDbf.open(new File(strFichDbf));
		numReg = m_FichDbf.getRecordCount();

		// shpWriter.initialize(file);
		// m_posShapes = new long[numReg];

		// read the records.
		/* int tempCurrentLength = myHeader.getHeaderLength();
		int numReg = 0;

		long pos1;

		while (tempCurrentLength < myHeader.myFileLength) {
			// read the record header
			// ShapeFileRecord tempRecord = new ShapeFileRecord();
			// Bytes 0 to 4 represent the record number in the file, these may be out of order.
			bb.order(ByteOrder.BIG_ENDIAN);

			// tempRecord.setIndex(in.readInt());
			bb.getInt();

			// read the content length of this record in 16 bit words, excluding the index.
			// in.setLittleEndianMode(false);
			int tempContentLength = bb.getInt();

			pos1 = bb.position();

			m_posShapes[numReg] = bb.position();

			// Posicionamos
			bb.position((pos1 + (2 * tempContentLength)));
			numReg = numReg + 1;

			// update the current length the 4 is for the index, and content length.
			tempCurrentLength = tempCurrentLength + 4 + tempContentLength;
		} */
	}

	/**
	 * Reads the Point from the shape file.
	 *
	 * @param in ByteBuffer.
	 *
	 * @return Point2D.
	 */
	private synchronized Point2D.Double readPoint(BigByteBuffer2 in) {
		// create a new point
		Point2D.Double tempPoint = new Point2D.Double();

		// bytes 1 to 4 are the type and have already been read.
		// bytes 4 to 12 are the X coordinate
		in.order(ByteOrder.LITTLE_ENDIAN);
		tempPoint.setLocation(in.getDouble(), in.getDouble());

		return tempPoint;
	}

	/**
	 * Lee un rectngulo del fichero.
	 *
	 * @param in ByteBuffer.
	 *
	 * @return Rectngulo.
	 *
	 * @throws IOException
	 */
	private synchronized Rectangle2D.Double readRectangle(BigByteBuffer2 in)
		throws IOException {
		Rectangle2D.Double tempRect = new Rectangle2D.Double();
		in.order(ByteOrder.LITTLE_ENDIAN);
		tempRect.x = in.getDouble();
		tempRect.y = in.getDouble();

		tempRect.width = in.getDouble() - tempRect.x;

		if (tempRect.width == 0) {
			tempRect.width = 0.2;
			tempRect.x -= 0.1;
		}

		tempRect.height = in.getDouble() - tempRect.y;

		if (tempRect.height == 0) {
			tempRect.height = 0.2;
			tempRect.y -= 0.1;
		}

		return tempRect;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#getFullExtent()
	 */
	public Rectangle2D getFullExtent() throws IOException {
		return extent;
	}

	/**
	 * Obtiene el extent del shape a partir de un ndice.
	 *
	 * @param index ndice.
	 *
	 * @return Rectngulo.
	 *
	 * @throws IOException
	 *
	 * @see com.iver.cit.gvsig.fmap.drivers.BoundedShapes#getShapeBounds()
	 */
	public synchronized Rectangle2D getShapeBounds(int index) throws IOException {
		Point2D p = new Point2D.Double();
		Rectangle2D BoundingBox = new Rectangle2D.Double();
		bb.position(getPositionForRecord(index));
		bb.order(ByteOrder.LITTLE_ENDIAN);

		int tipoShape = bb.getInt();

		if (tipoShape != SHP.NULL) {
			type = tipoShape;
		}

		// retrieve that shape.
		// tempRecord.setShape(readShape(tempShapeType, tempContentLength, in));
		switch (tipoShape) {
			case (SHP.POINT2D):
			case (SHP.POINT3D):
				p = readPoint(bb);
				BoundingBox = new Rectangle2D.Double(p.getX() - 0.1,
						p.getY() - 0.1, 0.2, 0.2);

				break;

			case (SHP.POLYLINE2D):
			case (SHP.POLYGON2D):
			case (SHP.MULTIPOINT2D):
			case (SHP.POLYLINE3D):
			case (SHP.POLYGON3D):
			case (SHP.MULTIPOINT3D):

				// BoundingBox
				BoundingBox = readRectangle(bb);

				break;
		}

		return BoundingBox;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#accept(java.io.File)
	 */
	public boolean accept(File f) {
		return (f.getName().toUpperCase().endsWith("SHP"));
	}

	/**
	 * @see com.hardcode.driverManager.Driver#getType()
	 */
	public String getName() {
		// Para que se use este driver en lugar del viejo DemoShpDriver.

		// return "gvSIG indexed shp driver";
		return "gvSIG shp driver";
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#getDataDriverName()
	 */
	public String getDataDriverName() {
		return "gdbms dbf driver";
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#getDataFile(java.io.File)
	 */
	public File getDataFile(File f) {
		String str = f.getAbsolutePath();

		return new File(str.substring(0, str.length() - 3) + "dbf");
	}

    public File getShxFile(File f) {
        String str = f.getAbsolutePath();

        return new File(str.substring(0, str.length() - 3) + "shx");
    }


	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.BoundedShapes#getShapeType(int)
	 */
	public int getShapeType(int index) {
		// Por ahora todos los fichero .shp contienen
		// entidades del mismo tipo. Si trabajamos con
		// alguno mixto, tendremos que cambiar esta funcin.
		return getShapeType();
	}

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.drivers.VectorialDriver#getDriverAttributes()
     */
    public DriverAttributes getDriverAttributes() {
        return null;
    }

    private synchronized long getPositionForRecord(int numRec)
    {
        // shx file has a 100 bytes header, then, records
        // 8 bytes length, one for each entity.
        // first 4 bytes are the offset
        // next 4 bytes are length

        int posIndex = 100 + (numRec * 8);
        // bbShx.position(posIndex);
        long pos = 8 + 2*bbShx.getInt(posIndex);

        return pos;
    }

	public File getFile() {
		return fileShp;
	}

	public void reLoad() throws IOException {
		initialize();
	}

	public boolean canWriteGeometry(int gvSIGgeometryType) {
		return shpWriter.canWriteGeometry(gvSIGgeometryType);
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.edition.IWriter#preProcess()
	 */
	public void preProcess() throws EditionException {
		// Necesitamos crear un fichero temporal porque a la vez que esic
		// La cabecera ya existe, as que no la sobreescribimos.
		// shpWriter.setWriteHeaders(false);
		shpWriter.preProcess();

	}

	public void process(IRowEdited row) throws EditionException {
		shpWriter.process(row);

	}

	public void postProcess() throws EditionException {
		shpWriter.postProcess();

		try {

			// close();

			// Shp
			FileChannel fcinShp = new FileInputStream(fTemp).getChannel();
			FileChannel fcoutShp = new FileOutputStream(fileShp).getChannel();
			DriverUtilities.copy(fcinShp, fcoutShp);

			// Shx
			File shxFile = getShxFile(fTemp);
			FileChannel fcinShx = new FileInputStream(shxFile).getChannel();
			FileChannel fcoutShx = new FileOutputStream(getShxFile(fileShp)).getChannel();
			DriverUtilities.copy(fcinShx, fcoutShx);

			// Dbf
			File dbfFile = getDataFile(fTemp);
			FileChannel fcinDbf = new FileInputStream(dbfFile).getChannel();
			FileChannel fcoutDbf = new FileOutputStream(getDataFile(fileShp)).getChannel();
			DriverUtilities.copy(fcinDbf, fcoutDbf);

			// Borramos los temporales
			fTemp.delete();
			shxFile.delete();
			dbfFile.delete();
			reLoad();



		} catch (FileNotFoundException e) {
			throw new EditionException(e);
		} catch (IOException e) {
			throw new EditionException(e);
		}


	}

	public String getCapability(String capability) {
		return shpWriter.getCapability(capability);
	}

	public void setCapabilities(Properties capabilities) {
		shpWriter.setCapabilities(capabilities);

	}

	public boolean canWriteAttribute(int sqlType) {
		return shpWriter.canWriteAttribute(sqlType);
	}

	public ShpWriter getShpWriter() {
		return shpWriter;
	}

	public void setShpWriter(ShpWriter shpWriter) {
		this.shpWriter = shpWriter;
	}

	public void initialize(FLayer layer) throws EditionException {
		int aux = (int)(Math.random() * 1000);
		fTemp = new File(tempDirectoryPath + "/tmpShp" + aux + ".shp");
		shpWriter.setFile(fTemp);
		shpWriter.initialize(layer);

	}

	public void setFlatness(double flatness) {
		shpWriter.setFlatness(flatness);
	}

}
