/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.drivers.shp.write;

import com.iver.cit.gvsig.fmap.core.IGeometry;

import java.nio.ByteBuffer;
import java.nio.MappedByteBuffer;


/**
 * Interfaz de todos los tipos de formato shape.
 *
 * @author Vicente Caballero Navarro
 */
public interface SHPShape {
	/**
	 * Devuelve el tipo de shape de que se trata.
	 *
	 * @return Tipo de shape.
	 */
	public int getShapeType();

	/**
	 * Lee del buffer el shape y crea una nueva geometra.
	 *
	 * @param buffer Buffer de donde se lee.
	 * @param type Tipo de shape en concreto.
	 *
	 * @return Nueva geometra.
	 */
	public IGeometry read(MappedByteBuffer buffer, int type);

	/**
	 * Escribe en el buffer la geometra que se pasa como parmetro.
	 *
	 * @param buffer Buffer donde escribir.
	 * @param geometry Geometra a escribir.
	 */
	public void write(ByteBuffer buffer, IGeometry geometry);

	/**
	 * Devuelve el tamao de la geometra.
	 *
	 * @param fgeometry Geometra a medir.
	 *
	 * @return Tamao de la geometra.
	 */
	public int getLength(IGeometry fgeometry);

	/**
	 * Obtiene los puntos y partes del GeneralPathXIterator del shape.
	 *
	 * @param iter
	 */
	public void obtainsPoints(IGeometry g);

	public void setFlatness(double flatness);
}
