package com.iver.cit.gvsig.fmap.edition;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;

import com.hardcode.driverManager.DriverLoadException;
import com.hardcode.gdbms.engine.data.DataSourceFactory;
import com.hardcode.gdbms.engine.data.NoSuchTableException;
import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.data.driver.ObjectDriver;
import com.hardcode.gdbms.engine.data.edition.DataWare;
import com.hardcode.gdbms.engine.values.Value;
import com.iver.cit.gvsig.fmap.MapControl.CancelDraw;
import com.iver.cit.gvsig.fmap.core.DefaultRow;
import com.iver.cit.gvsig.fmap.core.IRow;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.edition.commands.AddRowCommand;
import com.iver.cit.gvsig.fmap.edition.commands.Command;
import com.iver.cit.gvsig.fmap.edition.commands.CommandCollection;
import com.iver.cit.gvsig.fmap.edition.commands.CommandRecord;
import com.iver.cit.gvsig.fmap.edition.commands.MemoryCommandRecord;
import com.iver.cit.gvsig.fmap.edition.commands.ModifyRowCommand;
import com.iver.cit.gvsig.fmap.edition.commands.RemoveRowCommand;
import com.iver.cit.gvsig.fmap.layers.FBitSet;
import com.iver.cit.gvsig.fmap.layers.LayerFactory;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;
import com.iver.cit.gvsig.fmap.operations.Cancel;

/**
 * DOCUMENT ME!
 *
 * @author Vicente Caballero Navarro
 */
public class EditableAdapter implements IEditableSource {
	protected boolean isEditing = false;

	private SelectableDataSource ds = null;

	protected FBitSet delRows = new FBitSet();

	private CommandRecord cr;

	/**
	 * Flag que indica que hay que tomar las siguientes operaciones como una
	 * operacin atmica
	 */
	private boolean complex = false;

	private CommandCollection commands = null;

	/*
	 * Establece una relacin entre los ndices de las geometras en el
	 * EditableFeatureSource y los ndices en el fichero de expansin FJP:
	 * CAMBIO: NECESITAMOS TRABAJAR CON FEATURE Y FEATUREITERATOR PARA IR
	 * PREPARANDO EL CAMINO, GUARDAMOS EL FEATUREID (STRING) COMO CLAVE, Y COMO
	 * VALOR, EL INDICE DENTRO DEL FICHERO DE EXPANSION (Integer). Lo de que
	 * FeatureId sea un String es por compatibilidad con OGC. Segn OGC, una
	 * Feature tiene que tener un Id string En el caso de los randomaccess,
	 * sern el id de registro En los casos de base de datos espaciales, supongo
	 * que siempre ser numrico tambin, pero lo tendremos que convertir a
	 * string. Lo que est claro es que NO se puede confiar nunca en que sea
	 * algo correlativo (1, 2, 3, 4, 5, ... => FALSO!!)
	 */
	protected HashMap relations = new HashMap();

	/*
	 * Fichero en el que se guardan las nuevas geometras, producto de adiciones
	 * o de modificaciones
	 */
	protected ExpansionFile expansionFile;

	protected int numAdd = 0;

	private ObjectDriver editingDriver = new myObjectDriver();

	private SelectableDataSource ods;

	private ArrayList editionListeners = new ArrayList();

	/**
	 * Crea un nuevo EditableAdapter.
	 */
	public EditableAdapter() {
		expansionFile = new MemoryExpansionFile();
		cr = new MemoryCommandRecord();
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param ds
	 *            DOCUMENT ME!
	 */
	public void setOriginalDataSource(SelectableDataSource ds) {
		this.ods = ds;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @throws EditionException
	 *             DOCUMENT ME!
	 */
	public void startEdition(int sourceType) throws EditionException {
		isEditing = true;
		fireStartEditionEvent(sourceType);
	}


	/**
	 * Se ejecuta preProcess() del IWriter, luego se itera por los registros
	 * borrados por si el IWriter los quiere borrar (solo ser necesario cuando
	 * escribimos sobre la misma tabla) y luego se itera por los nuevos
	 * registros llamando a process con el registro correcto. (Aadidos,
	 * modificados). Para finalizar, se ejecuta PostProcess
	 *
	 * @param writer
	 *            IWriter que recibir las llamadas.
	 *
	 * @throws EditionException
	 *             DOCUMENT ME!
	 */
	public void stopEdition(IWriter writer, int sourceType) throws EditionException {

		writer.preProcess();

		try {

			// Procesamos primero los borrados.
			// Cuando se genere un tema nuevo, no se les debe hacer caso
			// a estos registros

			for (int i = delRows.nextSetBit(0); i >= 0; i = delRows
					.nextSetBit(i + 1)) {
				int calculatedIndex = i;
				Integer integer = new Integer(calculatedIndex);
				// Si no est en el fichero de expansin, es de los originales
				// y hay que borrarlo
				DefaultRowEdited edRow = null;
				if (!relations.containsKey(integer)) {
					edRow = new DefaultRowEdited(new DefaultRow(ods
							.getRow(calculatedIndex)),
							DefaultRowEdited.STATUS_DELETED, calculatedIndex);
					writer.process(edRow);
				} else {
					int num = ((Integer) relations.get(integer)).intValue();

					// return expansionFile.getRow(num);
					IRowEdited rowFromExpansion = expansionFile.getRow(num);
					// Habra que hacer aqu setID(index + "")?
					edRow = new DefaultRowEdited(rowFromExpansion
							.getLinkedRow().cloneRow(), DefaultRowEdited.STATUS_DELETED
							, calculatedIndex);
					writer.process(edRow);
				}

			}

			int rowCount = getRowCount();
			for (int i = 0; i < rowCount; i++) {
				IRowEdited rowEdited = getRow(i);

				if (rowEdited != null) {
					writer.process(rowEdited);
				}
			}
		} catch (DriverIOException e) {
			e.printStackTrace();
			throw new EditionException(e);
		} catch (IOException e) {
			e.printStackTrace();
			throw new EditionException(e);
		} catch (DriverException e) {
			e.printStackTrace();
			throw new EditionException(e);
		}

		writer.postProcess();
		isEditing = false;
		fireStopEditionEvent(sourceType);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @throws IOException
	 *             DOCUMENT ME!
	 */
	public void cancelEdition(int sourceType) throws IOException {
		isEditing = false;
		expansionFile.close();
		fireCancelEditionEvent(sourceType);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.iver.cit.gvsig.fmap.edition.IEditableSource#getRow(int)
	 */
	public IRowEdited getRow(int index) throws DriverIOException, IOException {
		int calculatedIndex = getCalculatedIndex(index);
		Integer integer = new Integer(calculatedIndex);
		DefaultRowEdited edRow = null;
		// Si no est en el fichero de expansin
		if (!relations.containsKey(integer)) {
			try {
				/*
				 * edRow = new DefaultRowEdited(new
				 * DefaultRow(ods.getRow(calculatedIndex), "" + index),
				 * DefaultRowEdited.STATUS_ORIGINAL, index);
				 */
				edRow = new DefaultRowEdited(new DefaultRow(ods
						.getRow(calculatedIndex)),
						DefaultRowEdited.STATUS_ORIGINAL, index);
			} catch (DriverException e) {
				throw new DriverIOException(e);
			}

			return edRow;
		} else {
			int num = ((Integer) relations.get(integer)).intValue();

			// return expansionFile.getRow(num);
			IRowEdited rowFromExpansion = expansionFile.getRow(num);
			// Habra que hacer aqu setID(index + "")?
			edRow = new DefaultRowEdited(rowFromExpansion.getLinkedRow()
					.cloneRow(), rowFromExpansion.getStatus(), index);
			return edRow;
		}
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 *
	 * @throws DriverIOException
	 *             DOCUMENT ME!
	 * @throws IOException
	 *             DOCUMENT ME!
	 */
	public int getRowCount() throws DriverIOException, IOException {
		try {
			return (int) (ods.getRowCount() + numAdd) - delRows.cardinality();// -
																				// expansionFile.getInvalidRows().cardinality();
		} catch (DriverException e) {
			throw new DriverIOException(e);
		}

	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.edition.IEditableSource#addRow(com.iver.cit.gvsig.fmap.core.IRow, java.lang.String)
	 */
	public int addRow(IRow row, String descrip, int sourceType) throws DriverIOException,
			IOException {

		int calculatedIndex = doAddRow(row, sourceType);
		Command command = new AddRowCommand(this, row, calculatedIndex, sourceType);
		command.setDescription(descrip);
		if (complex) {
			commands.add(command);
		} else {
			cr.pushCommand(command);
		}

		return calculatedIndex;
	}


	/**
	 * DOCUMENT ME!
	 *
	 * @throws DriverIOException
	 *             DOCUMENT ME!
	 * @throws IOException
	 *             DOCUMENT ME!
	 */
	public void undo() throws DriverIOException, IOException {
		// seleccion.clear();
		if (moreUndoCommands()) {
			cr.undoCommand();
		}
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @throws DriverIOException
	 *             DOCUMENT ME!
	 * @throws IOException
	 *             DOCUMENT ME!
	 */
	public void redo() throws DriverIOException, IOException {
		// seleccion.clear();
		if (moreRedoCommands()) {
			cr.redoCommand();
		}
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public boolean moreUndoCommands() {
		return cr.moreUndoCommands();
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public boolean moreRedoCommands() {
		return cr.moreRedoCommands();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.iver.cit.gvsig.fmap.edition.IEditableSource#removeRow(int)
	 */
	public void removeRow(int index, String descrip, int sourceType) throws IOException,
			DriverIOException {

		int calculatedIndex = getCalculatedIndex(index);
		Command command = new RemoveRowCommand(this, calculatedIndex, sourceType);
		command.setDescription(descrip);
		if (complex) {
			commands.add(command);
		} else {
			cr.pushCommand(command);
		}
		doRemoveRow(calculatedIndex, sourceType);

	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.iver.cit.gvsig.fmap.edition.IEditableSource#modifyRow(int,
	 *      com.iver.cit.gvsig.fmap.core.IRow)
	 */
	public int modifyRow(int index, IRow row, String descrip, int sourceType)
			throws IOException, DriverIOException {

		int calculatedIndex = getCalculatedIndex(index);
		int pos = doModifyRow(calculatedIndex, row, sourceType);
		Command command = new ModifyRowCommand(this, calculatedIndex, pos, row,sourceType);
		command.setDescription(descrip);
		if (complex) {
			commands.add(command);
		} else {
			cr.pushCommand(command);
		}

		return pos;
	}

	/**
	 * DOCUMENT ME!
	 */
	public void compact() {
		expansionFile.compact(relations);
	}

	/**
	 * DOCUMENT ME!
	 */
	public void startComplexRow() {
		complex = true;
		commands = new CommandCollection();
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @throws IOException
	 *             DOCUMENT ME!
	 * @throws DriverIOException
	 *             DOCUMENT ME!
	 */
	public void endComplexRow() throws IOException, DriverIOException {
		cr.pushCommand(commands);
		complex = false;
	}

	/**
	 * Actualiza en el mapa de ndices, la posicin en la que estaba la
	 * geometra antes de ser modificada. Se marca como vlida, en caso de que
	 * fuera una modificacin de una geometra que estuviese en el fichero de
	 * expansin antes de ser modificada y se pone el puntero de escritura del
	 * expansion file a justo despues de la penultima geometra
	 *
	 * @param geometryIndex
	 *            ndice de la geometra que se quiere deshacer su modificacin
	 * @param previousExpansionFileIndex
	 *            ndice que tena antes la geometra en el expansionFile. Si
	 *            vale -1 quiere decir que es una modificacin de una geometra
	 *            original y por tanto no hay que actualizar el mapa de indices
	 *            sino eliminar su entrada.
	 *
	 * @throws IOException
	 * @throws DriverIOException
	 */
	public void undoModifyRow(int geometryIndex, int previousExpansionFileIndex, int sourceType)
			throws IOException, DriverIOException {

		if (previousExpansionFileIndex == -1) {
			DefaultRowEdited edRow=null;
			try {
				edRow = new DefaultRowEdited(new DefaultRow(ods
						.getRow(geometryIndex)),
						DefaultRowEdited.STATUS_ORIGINAL,geometryIndex);
			} catch (DriverException e) {
				e.printStackTrace();
			}
			boolean cancel = fireBeforeModifyRow(edRow,geometryIndex,sourceType);
			if (cancel)
				return;
			// Se elimina de las relaciones y del fichero de expansin
			relations.remove(new Integer(geometryIndex));
			expansionFile.deleteLastRow();
		} else {
			boolean cancel = fireBeforeModifyRow(expansionFile.getRow(previousExpansionFileIndex),geometryIndex,sourceType);
			if (cancel)
				return;
			// Se actualiza la relacin de ndices
			relations.put(new Integer(geometryIndex), new Integer(
					previousExpansionFileIndex));
		}
		fireAfterModifyRow(geometryIndex,sourceType);
	}

	/**
	 * Elimina una geometria. Si es una geometra original de la capa en edicin
	 * se marca como eliminada (haya sido modificada o no). Si es una geometra
	 * aadida posteriormente se invalida en el fichero de expansin, para que
	 * una futura compactacin termine con ella.
	 *
	 * @param index
	 *            ndice de la geometra.
	 *
	 * @throws DriverIOException
	 * @throws IOException
	 */
	public IRow doRemoveRow(int index, int sourceType) throws DriverIOException, IOException {
		boolean cancel = fireBeforeRemoveRow(index, sourceType);
		if (cancel)
			return null;
		// Llega un calculatedIndex
		delRows.set(index, true);
		System.err.println("Elimina una Row en la posicin: " + index);
		// TODO: Con tablas no es necesario devolver la anterior feature. Por
		// ahora.
		fireAfterRemoveRow(index, sourceType);
		return null;
	}

	/**
	 * Si se intenta modificar una geometra original de la capa en edicin se
	 * aade al fichero de expansin y se registra la posicin en la que se
	 * aadi. Si se intenta modificar una geometria que se encuentra en el
	 * fichero de expansin (por ser nueva o original pero modificada) se invoca
	 * el mtodo modifyGeometry y se actualiza el ndice de la geometria en el
	 * fichero.
	 *
	 * @param index
	 *            DOCUMENT ME!
	 * @param feat
	 *            DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 *
	 * @throws IOException
	 * @throws DriverIOException
	 */
	public int doModifyRow(int index, IRow feat, int sourceType) throws IOException,
			DriverIOException {
		boolean cancel = fireBeforeModifyRow(feat,index, sourceType);
		if (cancel)
			return -1;

		int pos = -1;
		Integer integer = new Integer(index);
		System.err.println("Modifica una Row en la posicin: " + index);
		// Si la geometra no ha sido modificada
		if (!relations.containsKey(integer)) {
			int expansionIndex = expansionFile.addRow(feat, IRowEdited.STATUS_MODIFIED);
			relations.put(integer, new Integer(expansionIndex));
		} else {
			// Obtenemos el ndice en el fichero de expansin
			int num = ((Integer) relations.get(integer)).intValue();
			pos = num;

			/*
			 * Se modifica la geometra y nos guardamos el ndice dentro del
			 * fichero de expansin en el que se encuentra la geometra
			 * modificada
			 */
			num = expansionFile.modifyRow(num, feat);

			/*
			 * Actualiza la relacin del ndice de la geometra al ndice en el
			 * fichero de expansin.
			 */
			relations.put(integer, new Integer(num));
		}
		fireAfterModifyRow(index, sourceType);
		return pos;
	}

	/**
	 * Aade una geometria al fichero de expansin y guarda la correspondencia
	 * en la tabla relations.
	 *
	 * @param feat
	 *            geometra a guardar.
	 *
	 * @return calculatedIndex
	 *
	 * @throws DriverIOException
	 * @throws IOException
	 */
	public int doAddRow(IRow feat,int sourceType) throws DriverIOException, IOException {
		boolean cancel = fireBeforeRowAdded(sourceType);
		if (cancel)
			return -1;
		// Aade la geometra
		// int virtualIndex = 0;
		int calculatedIndex = -1;

		try {
			calculatedIndex = (int) ods.getRowCount() + numAdd;
			// int externalIndex = getRowCount();
			// calculatedIndex = getCalculatedIndex(externalIndex);
		} catch (DriverException e) {
			throw new DriverIOException(e);
		}

		int pos = expansionFile.addRow(feat, IRowEdited.STATUS_ADDED);
		relations.put(new Integer(calculatedIndex), new Integer(pos));
		numAdd++;
		System.err.println("Aade una Row en la posicin: " + calculatedIndex);
		fireAfterRowAdded(calculatedIndex, sourceType);
		return calculatedIndex;
	}

	/**
	 * Se desmarca como invalidada en el fichero de expansion o como eliminada
	 * en el fichero original
	 *
	 * @param index
	 *            DOCUMENT ME!
	 *
	 * @throws IOException
	 * @throws DriverIOException
	 */
	public void undoRemoveRow(int index, int sourceType) throws IOException, DriverIOException {
		boolean cancel = fireBeforeRowAdded(sourceType);
		if (cancel)
			return;
		delRows.set(index, false);
		fireAfterRowAdded(index, sourceType);
	}

	/**
	 * Se elimina del final del fichero de expansin poniendo el puntero de
	 * escritura apuntando al final de la penltima geometra. Deber quitar la
	 * relacin del mapa de relaciones
	 *
	 * @param index
	 *            ndice de la geometra que se aadi
	 *
	 * @throws DriverIOException
	 * @throws IOException
	 */
	public void undoAddRow(int calculatedIndex,int sourceType) throws DriverIOException,
			IOException {
		boolean cancel = fireBeforeRemoveRow(calculatedIndex,sourceType);
		if (cancel)
			return;
		expansionFile.deleteLastRow();
		relations.remove(new Integer(calculatedIndex));
		numAdd--;
		fireAfterRemoveRow(calculatedIndex,sourceType);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.iver.cit.gvsig.fmap.layers.VectorialAdapter#getRecordset()
	 */
	public SelectableDataSource getRecordset() throws DriverLoadException {
		if (isEditing) {
			if (ds == null) {
				String name = LayerFactory.getDataSourceFactory()
						.addDataSource((ObjectDriver) editingDriver);

				try {
					ds = new SelectableDataSource(LayerFactory
							.getDataSourceFactory().createRandomDataSource(
									name, DataSourceFactory.AUTOMATIC_OPENING));
					ds.setSelectionSupport(ods.getSelectionSupport());

				} catch (NoSuchTableException e) {
					throw new RuntimeException(e);
				} catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
					throw new RuntimeException(e);
				}
			}

			return ds;
		} else {
			return ods;
		}
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param sds
	 *            DOCUMENT ME!
	 */
	public void setRecordSet(SelectableDataSource sds) {
		this.ods = sds;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return
	 */
	public FBitSet getSelection() {
		/*
		 * try { return getRecordset().getSelection(); } catch
		 * (DriverLoadException e) { // TODO Auto-generated catch block
		 * e.printStackTrace(); } return null;
		 */
		return getRecordset().getSelection();
	}

	public void setSelection(FBitSet selection) {
		/*
		 * try { getRecordset().setSelection(selection); } catch
		 * (DriverLoadException e) { // TODO Auto-generated catch block
		 * e.printStackTrace(); }
		 */
		getRecordset().setSelection(selection);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public boolean isEditing() {
		return isEditing;
	}

	protected int getInversedIndex(long rowIndex) {
		int intervalNotDeleted = 0;
		int antDeleted = -1;
		int idPedido = (int) rowIndex;
		int numNotDeleted = 0;
		int numBorradosAnt = 0;

		for (int i = delRows.nextSetBit(0); i >= 0; i = delRows
				.nextSetBit(i + 1)) {
			intervalNotDeleted = i - antDeleted - 1;
			numNotDeleted += intervalNotDeleted;
			if (i > idPedido) {
				numNotDeleted = numNotDeleted + (i - idPedido);
				break;
			}
			numBorradosAnt++;
			antDeleted = i;
		}
		numNotDeleted = idPedido - numBorradosAnt;
		// System.out.println("Piden Viejo : "+ rowIndex + " y devuelvo como
		// nuevo " + (numNotDeleted));
		return numNotDeleted;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param rowIndex
	 *            DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public int getCalculatedIndex(long rowIndex) {
		int numNotDeleted = 0;
		int intervalNotDeleted = 0;
		int antDeleted = -1;
		int calculatedIndex;
		int idPedido = (int) rowIndex;
		int numBorradosAnt = 0;

		for (int i = delRows.nextSetBit(0); i >= 0; i = delRows
				.nextSetBit(i + 1)) {
			intervalNotDeleted = i - antDeleted - 1;
			numNotDeleted += intervalNotDeleted;
			if (numNotDeleted > idPedido) {
				break;
			}
			numBorradosAnt++;
			antDeleted = i;
		}
		calculatedIndex = numBorradosAnt + idPedido;
		// System.out.println("Piden Registro : "+ rowIndex + " y devuelvo el "
		// + (calculatedIndex));
		return calculatedIndex;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @author Vicente Caballero Navarro
	 */
	private class myObjectDriver implements ObjectDriver {
		/*
		 * (non-Javadoc)
		 *
		 * @see com.hardcode.gdbms.engine.data.driver.ObjectDriver#getPrimaryKeys()
		 */
		public int[] getPrimaryKeys() throws DriverException {
			return ods.getPrimaryKeys();
			// int[] pk=new int[1];
			/*
			 * for (int i=0;i<getRowCount();i++){ pk[i]=i; }
			 */
			// pk[0]=1;
			// return pk;
		}

		/*
		 * (non-Javadoc)
		 *
		 * @see com.hardcode.gdbms.engine.data.driver.ObjectDriver#write(com.hardcode.gdbms.engine.data.edition.DataWare)
		 */
		public void write(DataWare dataWare) throws DriverException {
			DataWare dataWareOrig = ods
					.getDataWare(DataSourceFactory.DATA_WARE_DIRECT_MODE);
			dataWareOrig.commitTrans();
		}

		/*
		 * (non-Javadoc)
		 *
		 * @see com.hardcode.gdbms.engine.data.driver.GDBMSDriver#setDataSourceFactory(com.hardcode.gdbms.engine.data.DataSourceFactory)
		 */
		public void setDataSourceFactory(DataSourceFactory dsf) {
			ods.setDataSourceFactory(dsf);
		}

		/*
		 * (non-Javadoc)
		 *
		 * @see com.hardcode.driverManager.Driver#getName()
		 */
		public String getName() {
			return ods.getName();
		}

		/*
		 * (non-Javadoc)
		 *
		 * @see com.hardcode.gdbms.engine.data.driver.ReadAccess#getFieldValue(long,
		 *      int)
		 */
		public Value getFieldValue(long rowIndex, int fieldId)
				throws DriverException {
			// Si no est en el fichero de expansin
			Integer integer = new Integer(getCalculatedIndex(rowIndex));

			try {
				if (!relations.containsKey(integer)) {
					return ods.getFieldValue(rowIndex, fieldId);
				} else {
					int num = ((Integer) relations.get(integer)).intValue();
					DefaultRowEdited feat = (DefaultRowEdited) expansionFile
							.getRow(num);

					if (feat == null) {
						return null;
					}

					return feat.getAttribute(fieldId);
				}
			} catch (DriverException e) {
				e.printStackTrace();
				throw new DriverException(e);
			} catch (IOException e) {
				e.printStackTrace();
				throw new DriverException(e);
			}

			/**
			 * try { if (!relations.containsKey(integer)) { // Si ha sido
			 * eliminada if (delRows.get(integer.intValue())) { return null; }
			 * else { return ods.getFieldValue(rowIndex, fieldId); }} else { int
			 * num = ((Integer) relations.get(integer)).intValue();
			 * DefaultRowEdited feat = (DefaultRowEdited)
			 * expansionFile.getRow(num); if (feat==null)return null; return
			 * feat.getAttribute(fieldId); }} catch (DriverException e) {
			 * e.printStackTrace(); throw new DriverException(e); } catch
			 * (IOException e) { e.printStackTrace(); throw new
			 * DriverException(e); }
			 */
		}

		/*
		 * (non-Javadoc)
		 *
		 * @see com.hardcode.gdbms.engine.data.driver.ReadAccess#getFieldCount()
		 */
		public int getFieldCount() throws DriverException {
			// TODO Por ahora, no dejamos que se aadan campos
			return ods.getFieldCount();
		}

		/*
		 * (non-Javadoc)
		 *
		 * @see com.hardcode.gdbms.engine.data.driver.ReadAccess#getFieldName(int)
		 */
		public String getFieldName(int fieldId) throws DriverException {
			return ods.getFieldName(fieldId);
		}

		/*
		 * (non-Javadoc)
		 *
		 * @see com.hardcode.gdbms.engine.data.driver.ReadAccess#getRowCount()
		 */
		public long getRowCount() {
			try {
				return (int) (ods.getRowCount() + numAdd)
						- delRows.cardinality();// -
												// expansionFile.getInvalidRows().cardinality();
			} catch (DriverException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			return 0;
		}

		/*
		 * (non-Javadoc)
		 *
		 * @see com.hardcode.gdbms.engine.data.driver.ReadAccess#getFieldType(int)
		 */
		public int getFieldType(int i) throws DriverException {
			return ods.getFieldType(i);
		}

		public int getFieldWidth(int i) throws DriverException {
			return ods.getFieldWidth(i);
		}
	}

	public CommandRecord getCommandRecord() {
		return cr;
	}

	protected void fireAfterRemoveRow(int index,int sourceType) {
		AfterRowEditEvent event = new AfterRowEditEvent(this, index, EditionEvent.CHANGE_TYPE_DELETE, sourceType);
		for (int i=0; i < editionListeners.size(); i++)
		{
			IEditionListener listener = (IEditionListener) editionListeners.get(i);
			listener.afterRowEditEvent(event);
		}

	}

	protected boolean fireBeforeRemoveRow(int index,int sourceType) {
		Cancel cancel = new Cancel();
		BeforeRowEditEvent event = new BeforeRowEditEvent(this, index, EditionEvent.CHANGE_TYPE_DELETE, cancel, sourceType);
		for (int i=0; i < editionListeners.size(); i++)
		{
			IEditionListener listener = (IEditionListener) editionListeners.get(i);
			listener.beforeRowEditEvent(null,event);
			if (cancel.isCanceled())
				return true;
		}
		return false;
	}

	protected void fireAfterRowAdded(int calculatedIndex,int sourceType) {
		AfterRowEditEvent event = new AfterRowEditEvent(this, calculatedIndex, EditionEvent.CHANGE_TYPE_ADD, sourceType);
		for (int i=0; i < editionListeners.size(); i++)
		{
			IEditionListener listener = (IEditionListener) editionListeners.get(i);
			listener.afterRowEditEvent(event);
		}
	}

	protected boolean fireBeforeRowAdded(int sourceType) throws DriverIOException, IOException {
		Cancel cancel = new Cancel();
		BeforeRowEditEvent event = new BeforeRowEditEvent(this, getRowCount(), EditionEvent.CHANGE_TYPE_ADD, cancel, sourceType);
		for (int i=0; i < editionListeners.size(); i++)
		{
			IEditionListener listener = (IEditionListener) editionListeners.get(i);
			listener.beforeRowEditEvent(null,event);
			if (cancel.isCanceled())
				return true;
		}
		return false;
	}

	protected boolean fireBeforeModifyRow(IRow feat,int index, int sourceType) {
		Cancel cancel = new Cancel();
		BeforeRowEditEvent event = new BeforeRowEditEvent(this, index, EditionEvent.CHANGE_TYPE_MODIFY, cancel,sourceType);
		for (int i=0; i < editionListeners.size(); i++)
		{
			IEditionListener listener = (IEditionListener) editionListeners.get(i);
			listener.beforeRowEditEvent(feat, event);
			if (cancel.isCanceled())
				return true;
		}
		return false;
	}

	protected void fireAfterModifyRow(int index,int sourceType) {
		AfterRowEditEvent event = new AfterRowEditEvent(this, index, EditionEvent.CHANGE_TYPE_MODIFY,sourceType);
		for (int i=0; i < editionListeners.size(); i++)
		{
			IEditionListener listener = (IEditionListener) editionListeners.get(i);
			listener.afterRowEditEvent(event);
		}


	}

	protected void fireStartEditionEvent(int sourceType) {
		EditionEvent ev = new EditionEvent(this, EditionEvent.START_EDITION, sourceType);
		for (int i=0; i < editionListeners.size(); i++)
		{
			IEditionListener listener = (IEditionListener) editionListeners.get(i);
			listener.processEvent(ev);
		}

	}

	protected void fireStopEditionEvent(int sourceType) {
		EditionEvent ev = new EditionEvent(this, EditionEvent.STOP_EDITION, sourceType);
		for (int i=0; i < editionListeners.size(); i++)
		{
			IEditionListener listener = (IEditionListener) editionListeners.get(i);
			listener.processEvent(ev);
		}

	}

	protected void fireCancelEditionEvent(int sourceType) {
		EditionEvent ev = new EditionEvent(this, EditionEvent.CANCEL_EDITION, sourceType);
		for (int i=0; i < editionListeners.size(); i++)
		{
			IEditionListener listener = (IEditionListener) editionListeners.get(i);
			listener.processEvent(ev);
		}

	}


	public void addEditionListener(IEditionListener listener){
		if (!editionListeners.contains(listener))
			editionListeners.add(listener);
	}

	public void removeEditionListener(IEditionListener listener)
	{
		editionListeners.remove(listener);
	}

}
