package com.iver.cit.gvsig.fmap.edition.writers.dbf;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.channels.FileChannel;
import java.nio.channels.WritableByteChannel;
import java.sql.Types;

import com.hardcode.driverManager.DriverLoadException;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.core.IRow;
import com.iver.cit.gvsig.fmap.drivers.shp.DbaseFileHeaderNIO;
import com.iver.cit.gvsig.fmap.drivers.shp.DbaseFileWriterNIO;
import com.iver.cit.gvsig.fmap.edition.EditionException;
import com.iver.cit.gvsig.fmap.edition.IEditableSource;
import com.iver.cit.gvsig.fmap.edition.IRowEdited;
import com.iver.cit.gvsig.fmap.edition.IWriter;
import com.iver.cit.gvsig.fmap.edition.writers.AbstractWriter;
import com.iver.cit.gvsig.fmap.layers.FBitSet;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;

public class DbfWriter extends AbstractWriter {
	private String dbfPath;

	private DbaseFileWriterNIO dbfWrite;

	private DbaseFileHeaderNIO myHeader;

	private int numRows;
	private Object[] record;

	// private FLyrVect lyrVect;
	private FBitSet selection = null;

	private void setFile(File f)
	{
		String strFichDbf = f.getAbsolutePath().replaceAll("\\.shp", ".dbf");
		dbfPath = strFichDbf.replaceAll("\\.SHP", ".DBF");
	}

	private WritableByteChannel getWriteChannel(String path)
				throws IOException
	{
		WritableByteChannel channel;

		File f = new File(path);

		if (!f.exists()) {
			System.out.println("Creando fichero " + f.getAbsolutePath());

			if (!f.createNewFile()) {
				System.err.print("Error al crear el fichero " +
					f.getAbsolutePath());
				throw new IOException("Cannot create file " + f);
			}
		}

		RandomAccessFile raf = new RandomAccessFile(f, "rw");
		channel = raf.getChannel();

		return channel;
	}



	/*public DbfWriter(File shpFile, IEditableSource ies) throws IOException, DriverException, DriverLoadException
	{
		SelectableDataSource sds = ies.getRecordset();
		myHeader = DbaseFileHeaderNIO.createDbaseHeader(sds);

		initialize(shpFile);
	}
*/

	public void initialize(File shpFile, IEditableSource ies)throws IOException, DriverException, DriverLoadException{
		SelectableDataSource sds = ies.getRecordset();
		myHeader = DbaseFileHeaderNIO.createDbaseHeader(sds);

		initialize(shpFile);
	}
	private void initialize(File shpFile) throws IOException //, FLyrVect lyrVect, FBitSet selection) throws IOException, DriverException
	{
		setFile(shpFile);
	}

	public void preProcess() throws EditionException {
		// Por ahora solo escribimos los primeros bytes
		// de las cabeceras. Luego en el postprocess los escribiremos
		// correctamente, con el fullExtent y el numero de
		// registros que tocan.
		if (selection == null)
		{

			try {
				myHeader.setNumRecords(0);
				dbfWrite = new DbaseFileWriterNIO(myHeader,
						(FileChannel) getWriteChannel(dbfPath));
				record = new Object[myHeader.getNumFields()];
				numRows = 0;

			} catch (IOException e) {
				e.printStackTrace();
				throw new EditionException(e);
			}
		}


	}

	public void process(IRowEdited row) throws EditionException {
		IRow rowEdit = (IRow) row.getLinkedRow();

		try {
			for (int i=0; i < record.length; i++)
				record[i] = rowEdit.getAttribute(i);
			dbfWrite.write(record);
			numRows++;

		} catch (IOException e) {
			e.printStackTrace();
			throw new EditionException(e);
		}

	}

	public void postProcess() throws EditionException {
		try {
			myHeader.setNumRecords(numRows);
			dbfWrite = new DbaseFileWriterNIO(myHeader,
					(FileChannel) getWriteChannel(dbfPath));
		} catch (IOException e) {
			e.printStackTrace();
			throw new EditionException(e);
		}


	}

	public String getName() {
		return "DBF Writer";
	}

	public boolean canWriteAttribute(int sqlType) {
		switch (sqlType)
		{
		case Types.DOUBLE:
		case Types.FLOAT:
		case Types.INTEGER:
		case Types.BIGINT:
			return true;
		case Types.DATE:
			return true;
		case Types.BIT:
		case Types.BOOLEAN:
			return true;
		case Types.VARCHAR:
		case Types.CHAR:
		case Types.LONGVARCHAR:
			return true; // TODO: Revisar esto, porque no creo que admita campos muy grandes

		}

		return false;
	}

	public void setFlatness(double flatness) {
		// TODO Auto-generated method stub

	}

}
