package com.hardcode.gdbms.driver.hsqldb;

import com.hardcode.gdbms.engine.data.DataSourceFactory;
import com.hardcode.gdbms.engine.data.db.JDBCSupport;
import com.hardcode.gdbms.engine.data.driver.AlphanumericDBDriver;
import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueWriter;

import java.sql.Connection;
import java.sql.Date;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Time;
import java.sql.Timestamp;

import java.text.DateFormat;
import java.text.SimpleDateFormat;

import java.util.HashMap;


/**
 * DOCUMENT ME!
 *
 * @author Fernando Gonzlez Corts
 */
public class HSQLDBDriver implements AlphanumericDBDriver {
    private static Exception driverException;
    private static DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
    private static DateFormat timeFormat = new SimpleDateFormat("HH:mm:ss");
    private static DateFormat timestampFormat = new SimpleDateFormat(
            "yyyy-MM-dd HH:mm:ss.SSSSSSSSS");
  
    static {
        try {
            Class.forName("org.hsqldb.jdbcDriver").newInstance();
        } catch (Exception ex) {
            driverException = ex;
        }
    }

    private ValueWriter vWriter = ValueWriter.internalValueWriter;
    protected JDBCSupport jdbcSupport;
    private Connection con = null;
    
    /**
     * @see com.hardcode.gdbms.engine.data.driver.DBDriver#getConnection(java.lang.String,
     *      int, java.lang.String, java.lang.String, java.lang.String)
     */
    public Connection getConnection(String host, int port, String dbName,
        String user, String password) throws SQLException {
        if (driverException != null) {
            throw new RuntimeException(driverException);
        }

        String connectionString = "jdbc:hsqldb:file:" + dbName;
        if (con == null){
        	con = DriverManager.getConnection(connectionString, user, password);
        }
        return con;
    }

    /**
     * @see com.hardcode.gdbms.engine.data.driver.DBDriver#open(java.sql.Connection,
     *      java.lang.String, com.hardcode.gdbms.engine.data.HasProperties)
     */
    public void open(Connection con, String sql) throws SQLException {
        jdbcSupport = JDBCSupport.newJDBCSupport(con, sql);
    }

    /**
     * @see com.hardcode.gdbms.engine.data.driver.DBDriver#execute(java.sql.Connection,
     *      java.lang.String, com.hardcode.gdbms.engine.data.HasProperties)
     */
    public void execute(Connection con, String sql) throws SQLException {
        JDBCSupport.execute(con, sql);
    }

    /**
     * @see com.hardcode.gdbms.engine.data.driver.DBDriver#close()
     */
    public void close() throws SQLException {
        jdbcSupport.close();
    }

    /**
     * @see com.hardcode.gdbms.engine.data.driver.ReadAccess#getFieldValue(long,
     *      int)
     */
    public Value getFieldValue(long rowIndex, int fieldId)
        throws DriverException {
        return jdbcSupport.getFieldValue(rowIndex, fieldId);
    }

    /**
     * @see com.hardcode.gdbms.engine.data.driver.ReadAccess#getFieldCount()
     */
    public int getFieldCount() throws DriverException {
        return jdbcSupport.getFieldCount();
    }

    /**
     * @see com.hardcode.gdbms.engine.data.driver.ReadAccess#getFieldName(int)
     */
    public String getFieldName(int fieldId) throws DriverException {
        return jdbcSupport.getFieldName(fieldId);
    }

    /**
     * @see com.hardcode.gdbms.engine.data.driver.ReadAccess#getRowCount()
     */
    public long getRowCount() throws DriverException {
        return jdbcSupport.getRowCount();
    }

    /**
     * @see com.hardcode.gdbms.engine.data.driver.ReadAccess#getFieldType(int)
     */
    public int getFieldType(int i) throws DriverException {
        return jdbcSupport.getFieldType(i);
    }

    /**
     * @see com.hardcode.driverManager.Driver#getName()
     */
    public String getName() {
        return "GDBMS HSQLDB driver";
    }

    /**
     * @see com.hardcode.gdbms.engine.data.driver.DriverCommons#getDriverProperties()
     */
    public HashMap getDriverProperties() {
        return null;
    }

    /**
     * @see com.hardcode.gdbms.engine.data.driver.DriverCommons#setDataSourceFactory(com.hardcode.gdbms.engine.data.DataSourceFactory)
     */
    public void setDataSourceFactory(DataSourceFactory dsf) {
    }

    /**
     * @see com.hardcode.gdbms.engine.data.driver.DBDriver#getInternalTableName(java.lang.String)
     */
    public String getInternalTableName(String tableName) {
        return tableName.toUpperCase();
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getNullStatementString() {
        return vWriter.getNullStatementString();
    }

    /**
     * DOCUMENT ME!
     *
     * @param b DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getStatementString(boolean b) {
        return vWriter.getStatementString(b);
    }

    /**
     * DOCUMENT ME!
     *
     * @param binary DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getStatementString(byte[] binary) {
        return vWriter.getStatementString(binary);
    }

    /**
     * DOCUMENT ME!
     *
     * @param d DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getStatementString(Date d) {
        return vWriter.getStatementString(d);
    }

    /**
     * DOCUMENT ME!
     *
     * @param d DOCUMENT ME!
     * @param sqlType DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getStatementString(double d, int sqlType) {
        return vWriter.getStatementString(d, sqlType);
    }

    /**
     * DOCUMENT ME!
     *
     * @param i DOCUMENT ME!
     * @param sqlType DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getStatementString(int i, int sqlType) {
        return vWriter.getStatementString(i, sqlType);
    }

    /**
     * DOCUMENT ME!
     *
     * @param i DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getStatementString(long i) {
        return vWriter.getStatementString(i);
    }

    /**
     * DOCUMENT ME!
     *
     * @param str DOCUMENT ME!
     * @param sqlType DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getStatementString(String str, int sqlType) {
        return vWriter.getStatementString(str, sqlType);
    }

    /**
     * DOCUMENT ME!
     *
     * @param t DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getStatementString(Time t) {
        return vWriter.getStatementString(t);
    }

    /**
     * DOCUMENT ME!
     *
     * @param ts DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getStatementString(Timestamp ts) {
        return vWriter.getStatementString(ts);
    }

	public int getFieldWidth(int i) throws DriverException {
		return jdbcSupport.getFieldWidth(i);
	}
}
