package com.hardcode.gdbms.driver.postgresql;

import com.hardcode.gdbms.engine.data.DataSourceFactory;
import com.hardcode.gdbms.engine.data.db.JDBCSupport;
import com.hardcode.gdbms.engine.data.driver.AlphanumericDBDriver;
import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.values.Value;

import java.sql.Connection;
import java.sql.Date;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Time;
import java.sql.Timestamp;

import java.util.HashMap;


/**
 *
 */
public class PostgreSQLDriver implements AlphanumericDBDriver {
    private static Exception driverException;

    static {
        try {
            Class.forName("org.postgresql.Driver").newInstance();
        } catch (Exception ex) {
            driverException = ex;
        }
    }

    protected JDBCSupport jdbcSupport;

    /**
     * DOCUMENT ME!
     *
     * @param host DOCUMENT ME!
     * @param port DOCUMENT ME!
     * @param dbName DOCUMENT ME!
     * @param user DOCUMENT ME!
     * @param password DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     *
     * @throws SQLException
     * @throws RuntimeException DOCUMENT ME!
     *
     * @see com.hardcode.gdbms.engine.data.driver.DBDriver#connect(java.lang.String)
     */
    public Connection getConnection(String host, int port, String dbName,
        String user, String password) throws SQLException {
        if (driverException != null) {
            throw new RuntimeException(driverException);
        }

        String connectionString = "jdbc:postgresql://" + host;

        if (port != -1) {
            connectionString += (":" + port);
        }

        connectionString += ("/" + dbName);

        if (user != null) {
            connectionString += ("?user=" + user + "&password=" + password);
        }

        return DriverManager.getConnection(connectionString);
    }

    /**
     * @see com.hardcode.driverManager.Driver#getName()
     */
    public String getName() {
        return "postgresql";
    }

    /**
     * @see com.hardcode.gdbms.engine.data.driver.DBDriver#executeSQL(java.sql.Connection,
     *      java.lang.String)
     */
    public void open(Connection con, String sql) throws SQLException {
        jdbcSupport = JDBCSupport.newJDBCSupport(con, sql);
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     *
     * @throws DriverException DOCUMENT ME!
     */
    public int getFieldCount() throws DriverException {
        return jdbcSupport.getFieldCount();
    }

    /**
     * DOCUMENT ME!
     *
     * @param fieldId DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     *
     * @throws DriverException DOCUMENT ME!
     */
    public String getFieldName(int fieldId) throws DriverException {
        return jdbcSupport.getFieldName(fieldId);
    }

    /**
     * DOCUMENT ME!
     *
     * @param i DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     *
     * @throws DriverException DOCUMENT ME!
     */
    public int getFieldType(int i) throws DriverException {
        return jdbcSupport.getFieldType(i);
    }

    /**
     * DOCUMENT ME!
     *
     * @param rowIndex DOCUMENT ME!
     * @param fieldId DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     *
     * @throws DriverException DOCUMENT ME!
     */
    public Value getFieldValue(long rowIndex, int fieldId)
        throws DriverException {
        return jdbcSupport.getFieldValue(rowIndex, fieldId);
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     *
     * @throws DriverException DOCUMENT ME!
     */
    public long getRowCount() throws DriverException {
        return jdbcSupport.getRowCount();
    }

    /**
     * @see com.hardcode.gdbms.engine.data.driver.DBDriver#close()
     */
    public void close() throws SQLException {
        jdbcSupport.close();
    }

    /**
     * @see com.hardcode.gdbms.engine.data.driver.DriverCommons#getDriverProperties()
     */
    public HashMap getDriverProperties() {
        return null;
    }

    /**
     * @see com.hardcode.gdbms.engine.data.driver.DriverCommons#setDataSourceFactory(com.hardcode.gdbms.engine.data.DataSourceFactory)
     */
    public void setDataSourceFactory(DataSourceFactory dsf) {
    }

    /**
     * @see com.hardcode.gdbms.engine.data.driver.DBDriver#execute(java.sql.Connection,
     *      java.lang.String, com.hardcode.gdbms.engine.data.HasProperties)
     */
    public void execute(Connection con, String sql) throws SQLException {
        JDBCSupport.execute(con, sql);
    }

    /**
     * @see com.hardcode.gdbms.engine.data.driver.DBDriver#getInternalTableName(java.lang.String)
     */
    public String getInternalTableName(String tableName) {
        return tableName;
    }

    /**
     * @see com.hardcode.gdbms.engine.data.driver.DBDriver#getStatementString(long)
     */
    public String getStatementString(long i) {
        // TODO Auto-generated method stub
        return null;
    }

    /**
     * @see com.hardcode.gdbms.engine.data.driver.DBDriver#getStatementString(int,
     *      int)
     */
    public String getStatementString(int i, int sqlType) {
        // TODO Auto-generated method stub
        return null;
    }

    /**
     * @see com.hardcode.gdbms.engine.data.driver.DBDriver#getStatementString(double,
     *      int)
     */
    public String getStatementString(double d, int sqlType) {
        // TODO Auto-generated method stub
        return null;
    }

    /**
     * @see com.hardcode.gdbms.engine.data.driver.DBDriver#getStatementString(java.lang.String,
     *      int)
     */
    public String getStatementString(String str, int sqlType) {
        // TODO Auto-generated method stub
        return null;
    }

    /**
     * @see com.hardcode.gdbms.engine.data.driver.DBDriver#getStatementString(java.sql.Date)
     */
    public String getStatementString(Date d) {
        // TODO Auto-generated method stub
        return null;
    }

    /**
     * @see com.hardcode.gdbms.engine.data.driver.DBDriver#getStatementString(java.sql.Time)
     */
    public String getStatementString(Time t) {
        // TODO Auto-generated method stub
        return null;
    }

    /**
     * @see com.hardcode.gdbms.engine.data.driver.DBDriver#getStatementString(java.sql.Timestamp)
     */
    public String getStatementString(Timestamp ts) {
        // TODO Auto-generated method stub
        return null;
    }

    /**
     * @see com.hardcode.gdbms.engine.data.driver.DBDriver#getStatementString(byte[])
     */
    public String getStatementString(byte[] binary) {
        // TODO Auto-generated method stub
        return null;
    }

    /**
     * @see com.hardcode.gdbms.engine.data.driver.DBDriver#getStatementString(boolean)
     */
    public String getStatementString(boolean b) {
        // TODO Auto-generated method stub
        return null;
    }

    /**
     * @see com.hardcode.gdbms.engine.values.ValueWriter#getNullStatementString()
     */
    public String getNullStatementString() {
        return null;
    }
	public int getFieldWidth(int i) throws DriverException {
		return jdbcSupport.getFieldWidth(i);
	}
}
