package org.gvsig.remoteClient.wfs;

import java.io.File;
import java.io.IOException;
import java.net.ConnectException;
import java.util.Hashtable;
import java.util.Vector;

import org.gvsig.remoteClient.RemoteClient;
import org.gvsig.remoteClient.wms.ICancellable;
import org.gvsig.remoteClient.wms.WMSProtocolHandler;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: WFSClient.java,v 1.7 2006/05/25 10:28:25 jorpiell Exp $
 * $Log: WFSClient.java,v $
 * Revision 1.7  2006/05/25 10:28:25  jorpiell
 * Se ha aadido un atributo al mtodo connect
 *
 * Revision 1.6  2006/05/23 13:23:22  jorpiell
 * Se ha cambiado el final del bucle de parseado y se tiene en cuenta el online resource
 *
 * Revision 1.4  2006/04/20 16:39:16  jorpiell
 * Aadida la operacion de describeFeatureType y el parser correspondiente.
 *
 * Revision 1.3  2006/04/19 12:51:35  jorpiell
 * Aadidas algunas de las clases del servicio WFS
 *
 *
 */
/**
 * Represents the class the with the necessary logic to connect to a 
 * OGCWFS and interpretate the data 
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class WFSClient extends RemoteClient{
	private WFSProtocolHandler handler;
		
	/**
	 * Constructor.
	 * the parameter host, indicates the WFS host to connect.
	 * @throws ConnectException,IOException 
	 *
	 */
	public WFSClient(String host) throws ConnectException,IOException {
		setHost(host);
		
		try {        	
			handler = WFSProtocolHandlerFactory.negotiate(host);
			handler.setHost(host);        
		} catch(ConnectException conE) {
			conE.printStackTrace();
			throw conE; 
		} catch(IOException ioE) {
			ioE.printStackTrace();
			throw ioE; 
		} catch(Exception e) {
			e.printStackTrace();       	
		} 
	}
	
	/**
	 * Every OGC Web Service (OWS), including a Web Feature Service,
	 * must have the ability to describe its capabilities by returning
	 * service metadata in response to a GetCapabilities request.
	 * @param status
	 * WFS client status. Contains all the information to create
	 * the query
	 */
	public void getCapabilities(WFSStatus status) {
		handler.getCapabilities(status);		
	}
	
	/**
	 * The function of the DescribeFeatureType operation is to 
	 * generate a schema description of feature types serviced 
	 * by a WFS implementation. The schema descriptions define 
	 * how a WFS implementation expects feature instances to 
	 * be encoded on input (via Insert and Update requests) 
	 * and how feature instances will be generated on output 
	 * (in response to GetFeature and GetGmlObject requests). 
	 * @param status
	 * WFS client status. Contains all the information to create
	 * the query
	 */
	public void describeFeatureType(WFSStatus status) {
		handler.describeFeatureType(status);		
	}
	
	/**
	 * The GetFeature operation allows retrieval of features from a 
	 * web feature service. A GetFeature request is processed by
	 * a WFS and when the value of the outputFormat attribute is 
	 * set to text/gml a GML instance document, containing the 
	 * result set, is returned to the client.
	 * @param status
	 * WFS client status. Contains all the information to create
	 * the query
	 * @return File
	 * GML File
	 */
	public File getFeature(WFSStatus status){
		return handler.getFeature(status);
	}
	
	/**
	 * <p>Checks the connection to de remote WFS and requests its 
	 * capabilities.</p>
	 * 
	 */
	public boolean connect(ICancellable cancel) 
	{
		try {
			if (handler == null) {
				if (getHost().trim().length() > 0) {                   
					handler = WFSProtocolHandlerFactory.negotiate(getHost());
					handler.setHost(getHost());
				} else {
					// must to specify host first!!!!
					return false;
				}
			}
			getCapabilities(null);			
			
			return true;
			
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.remoteClient.RemoteClient#close()
	 */
	public void close() {
		// TODO Auto-generated method stub
		
	}
	
	 public String getVersion()
	 {
		 return handler.getVersion();
	 }	 
	 
	 /**
     * Gets the Service information included in the Capabilities
     */
    
    public WFSProtocolHandler.ServiceInformation getServiceInformation()
    {
        return handler.getServiceInformation();  
    }
    
    /**
     * Returns the features list
     * @return
     */
    public Hashtable getFeatures()
    {
        return handler.getFeatures();  
    }	
	
}
