package org.gvsig.remoteClient.wfs.wfs_1_0_0;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Vector;

import org.gvsig.remoteClient.utils.CapabilitiesTags;
import org.gvsig.remoteClient.wfs.WFSSchemaParser;
import org.gvsig.remoteClient.wfs.WFSAttribute;
import org.gvsig.remoteClient.wfs.WFSProtocolHandler;
import org.gvsig.remoteClient.wms.wms_1_1_0.WMSLayer1_1_0;
import org.kxml2.io.KXmlParser;
import org.xmlpull.v1.XmlPullParserException;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: WFSProtocolHandler1_0_0.java,v 1.5 2006/05/25 10:20:57 jorpiell Exp $
 * $Log: WFSProtocolHandler1_0_0.java,v $
 * Revision 1.5  2006/05/25 10:20:57  jorpiell
 * Se ha cambiado el nombre de la clase WFSField por la clase WFSAttribute, porque resultaba confuso
 *
 * Revision 1.4  2006/05/23 13:23:22  jorpiell
 * Se ha cambiado el final del bucle de parseado y se tiene en cuenta el online resource
 *
 * Revision 1.2  2006/04/20 16:39:16  jorpiell
 * Aadida la operacion de describeFeatureType y el parser correspondiente.
 *
 * Revision 1.1  2006/04/19 12:51:35  jorpiell
 * Aadidas algunas de las clases del servicio WFS
 *
 *
 */
/**
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class WFSProtocolHandler1_0_0 extends WFSProtocolHandler{
	public WFSProtocolHandler1_0_0(){
		this.version = "1.0.0";
		this.name = "WFS1.0.0";		
	}	
	
	public boolean parseCapabilities(File f) {
		FileReader reader = null;       
		try
		{
			reader = new FileReader(f);
			BufferedReader br = new BufferedReader(reader);
			char[] buffer = new char[100];
			br.read(buffer);
			StringBuffer st = new StringBuffer(new String(buffer));
			String searchText = "encoding=\"";
			int index = st.indexOf(searchText);
			if (index>-1) {
				st.delete(0, index+searchText.length());
				encoding = st.substring(0, st.indexOf("\""));
			}
		} catch (FileNotFoundException ex) {
			ex.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		int tag;
		KXmlParser parser = null;
		parser = new KXmlParser();
		try
		{
			parser.setInput(new FileInputStream(f), encoding);        
			parser.nextTag();
			
			if ( parser.getEventType() != KXmlParser.END_DOCUMENT ) 
			{     
				parser.require(KXmlParser.START_TAG, null, CapabilitiesTags.WFS_CAPABILITIES_ROOT1_0_0);             
				tag = parser.nextTag();
				while(tag != KXmlParser.END_DOCUMENT)
				{
					switch(tag)
					{
					case KXmlParser.START_TAG:
						if (parser.getName().compareTo(CapabilitiesTags.SERVICE)==0)
						{
							parseServiceTag(parser);
						} 
						else if (parser.getName().compareTo(CapabilitiesTags.CAPABILITY)==0)
						{
							parseCapabilityTag(parser);
						} 
						else if (parser.getName().compareTo(CapabilitiesTags.WFS_FEATURETYPELIST)==0)
						{
							parseFeatureTypeListTag(parser);
						} 
						break;
					case KXmlParser.END_TAG:                            
						break;
					case KXmlParser.TEXT:
						if (parser.getName()!=null)
							System.out.println("[TEXT]["+parser.getText().trim()+"]");                         
						break;
					}
					tag = parser.next();
				}
				parser.require(KXmlParser.END_DOCUMENT, null, null);                
			}
		}
		catch(XmlPullParserException parser_ex){
			System.out.println(parser_ex.getMessage());
			parser_ex.printStackTrace();
			return false;
		}
		catch (IOException ioe) {           
			ioe.printStackTrace();
			return false;
		}
		return true;
	}
	
	/**
	 * <p>Parses the Service Information </p>
	 */    
	private void parseServiceTag(KXmlParser parser) throws IOException, XmlPullParserException 
	{
		int currentTag;
		boolean end = false;
		
		parser.require(KXmlParser.START_TAG, null, CapabilitiesTags.SERVICE);
		currentTag = parser.next();
		
		while (!end) 
		{
			switch(currentTag)
			{
			case KXmlParser.START_TAG:
				if (parser.getName().compareToIgnoreCase(CapabilitiesTags.NAME)==0)
				{
					serviceInfo.name = parser.nextText(); 
				}   
				else if (parser.getName().compareTo(CapabilitiesTags.WFS_TITLE)==0)
				{
					serviceInfo.title = parser.nextText(); 
				}
				else if (parser.getName().compareTo(CapabilitiesTags.WFS_ABSTRACT)==0)
				{
					serviceInfo.abstr = parser.nextText(); 
				} 
				else if (parser.getName().compareTo(CapabilitiesTags.ONLINERESOURCE)==0)
				{
			       	serviceInfo.online_resource = parser.nextText();
			      
				}	                       
				break;
			case KXmlParser.END_TAG:
				if (parser.getName().compareTo(CapabilitiesTags.SERVICE) == 0)
					end = true;
				break;
			case KXmlParser.TEXT:                   
				break;
			}
			if (!end){
				currentTag = parser.next();
			}
		}     
	}
	
	private void parseCapabilityTag(KXmlParser parser) throws IOException, XmlPullParserException
	{   
		int currentTag;
		boolean end = false;
		
		parser.require(KXmlParser.START_TAG, null, CapabilitiesTags.CAPABILITY);
		currentTag = parser.next();
		
		while (!end) 
		{
			switch(currentTag)
			{
			case KXmlParser.START_TAG:
				if (parser.getName().compareTo(CapabilitiesTags.REQUEST)==0)
				{
					parseRequestTag(parser); 
				}   
				else if (parser.getName().compareTo(CapabilitiesTags.EXCEPTION)==0)
				{
					//TODO Parse exception tags...
					parser.skipSubTree();
				}
				else if ((parser.getName().compareTo(CapabilitiesTags.VENDORSPECIFICCAPABILITIES)==0) ||
						(parser.getName().compareTo(CapabilitiesTags.USERDEFINEDSYMBOLIZATION )==0))
				{
					parser.skipSubTree();
				}                   
				break;
			case KXmlParser.END_TAG:
				if (parser.getName().compareTo(CapabilitiesTags.CAPABILITY) == 0)
					end = true;
				break;
			case KXmlParser.TEXT:                   
				break;
			}
			if (!end){
				currentTag = parser.next();
			}
		}
	}
	
	/**
	 * <p>Parses the Request tag </p>
	 */ 
	private void parseRequestTag(KXmlParser parser) throws IOException, XmlPullParserException
	{   
		int currentTag;
		boolean end = false;
		
		parser.require(KXmlParser.START_TAG, null, CapabilitiesTags.REQUEST);
		currentTag = parser.next();
		
		while (!end) 
		{
			switch(currentTag)
			{
			case KXmlParser.START_TAG:
				if (parser.getName().compareTo(CapabilitiesTags.GETCAPABILITIES)==0)
				{
					currentTag = parser.nextTag();
					parser.require(KXmlParser.START_TAG, null, CapabilitiesTags.DCPTYPE);
					{			
						currentTag = parser.nextTag();
						if(parser.getName().compareTo(CapabilitiesTags.HTTP)==0)
						{
							currentTag = parser.nextTag();
							if(parser.getName().compareTo(CapabilitiesTags.GET)==0)
							{
								currentTag = parser.nextTag();
								if (parser.getName().compareTo(CapabilitiesTags.ONLINERESOURCE)==0)
								{
									String value = new String();
									value = parser.getAttributeValue("", CapabilitiesTags.XLINK_HREF);
									if (value != null){
										serviceInfo.operations.put(value, null);
									}
								}
							}
						}
					}	
					
				}   
				else if (parser.getName().compareTo(CapabilitiesTags.DESCRIBECOVERAGE)==0)
				{
					currentTag = parser.nextTag();
					parser.require(KXmlParser.START_TAG, null, CapabilitiesTags.DCPTYPE);
					{			
						currentTag = parser.nextTag();
						if(parser.getName().compareTo(CapabilitiesTags.HTTP)==0)
						{
							currentTag = parser.nextTag();
							if(parser.getName().compareTo(CapabilitiesTags.GET)==0)
							{
								currentTag = parser.nextTag();
								if (parser.getName().compareTo(CapabilitiesTags.ONLINERESOURCE)==0)
								{
									String value = new String();
									value = parser.getAttributeValue("", CapabilitiesTags.XLINK_HREF);
									if (value != null){
										serviceInfo.operations.put(value, null);
									}
								}
							}
						}
					}	                                                                        
				}
				else if (parser.getName().compareTo(CapabilitiesTags.GETCOVERAGE)==0)
				{
					currentTag = parser.nextTag();
					parser.require(KXmlParser.START_TAG, null, CapabilitiesTags.DCPTYPE);
					{			
						currentTag = parser.nextTag();
						if(parser.getName().compareTo(CapabilitiesTags.HTTP)==0)
						{
							currentTag = parser.nextTag();
							if(parser.getName().compareTo(CapabilitiesTags.GET)==0)
							{
								currentTag = parser.nextTag();
								if (parser.getName().compareTo(CapabilitiesTags.ONLINERESOURCE)==0)
								{
									String value = new String();
									value = parser.getAttributeValue("", CapabilitiesTags.XLINK_HREF);
									if (value != null){
										serviceInfo.operations.put(value, null);
									}
								}
							}
						}
					}	
				}               
				break;
			case KXmlParser.END_TAG:
				if (parser.getName().compareTo(CapabilitiesTags.REQUEST) == 0)
					end = true;
				break;
			case KXmlParser.TEXT:                   
				break;
			}
			if (!end){
				currentTag = parser.next();
			}
		}
		// TODO: does not get such a tag when arrives here!!!!!!
		//parser.require(KXmlParser.END_TAG, null, CapabilitiesTags.REQUEST);       
	}
	
	
	private void parseFeatureTypeListTag(KXmlParser parser) throws IOException, XmlPullParserException
	{
		int currentTag;
		boolean end = false;
		
		parser.require(KXmlParser.START_TAG, null, CapabilitiesTags.WFS_FEATURETYPELIST);
		currentTag = parser.next();
		
		while (!end) 
		{
			switch(currentTag)
			{
			case KXmlParser.START_TAG:
				if (parser.getName().compareToIgnoreCase(CapabilitiesTags.WFS_FEATURETYPE)==0)
				{
					WFSFeature1_0_0 feature = new WFSFeature1_0_0();
					feature.parse(parser);	
					features.put(feature.getName(),feature);
				} 				         
				break;
			case KXmlParser.END_TAG:
				if (parser.getName().compareTo(CapabilitiesTags.WFS_FEATURETYPELIST) == 0)
					end = true;
				break;
			case KXmlParser.TEXT:                   
				break;
			}
			if (!end){
				currentTag = parser.next();
			}
		}     
	}	
	
	public boolean parseDescribeFeatureType(File f) {
		FileReader reader = null;       
		try
		{
			reader = new FileReader(f);
			BufferedReader br = new BufferedReader(reader);
			char[] buffer = new char[100];
			br.read(buffer);
			StringBuffer st = new StringBuffer(new String(buffer));
			String searchText = "encoding=\"";
			int index = st.indexOf(searchText);
			if (index>-1) {
				st.delete(0, index+searchText.length());
				encoding = st.substring(0, st.indexOf("\""));
			}
		} catch (FileNotFoundException ex) {
			ex.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		int tag;
		WFSSchemaParser parser = null;
		parser = new WFSSchemaParser();
		try
		{
			parser.setInput(new FileInputStream(f), encoding);        
			parser.nextTag();
			
			if ( parser.getEventType() != KXmlParser.END_DOCUMENT ) 
			{     
				parser.setSchemaFromMainTag(parser.getName());
				     
				parser.require(KXmlParser.START_TAG, null, CapabilitiesTags.WFS_SCHEMAROOT); 
				tag = parser.nextTag();
				while(tag != KXmlParser.END_DOCUMENT)
				{
					switch(tag)
					{
					case KXmlParser.START_TAG:
						if (parser.getName().compareTo(CapabilitiesTags.COMPLEXTYPE)==0)
						{							
							parseComplexType(parser);
						} 						
						break;
					case KXmlParser.END_TAG:                            
						break;
					case KXmlParser.TEXT:
						if (parser.getName()!=null)
							System.out.println("[TEXT]["+parser.getText().trim()+"]");                         
						break;
					}
					tag = parser.next();
				}
				parser.require(KXmlParser.END_DOCUMENT, null, null);                
			}
		}
		catch(XmlPullParserException parser_ex){
			System.out.println(parser_ex.getMessage());
			parser_ex.printStackTrace();
			return false;
		}
		catch (IOException ioe) {           
			ioe.printStackTrace();
			return false;
		}
		return true;
	}
	
	private void parseComplexType(WFSSchemaParser parser) throws IOException, XmlPullParserException
	{   
		int currentTag;
		boolean end = false;
		
		parser.require(KXmlParser.START_TAG, null, CapabilitiesTags.COMPLEXTYPE);
		currentTag = parser.next();
		
		while (!end) 
		{
			switch(currentTag)
			{
			case KXmlParser.START_TAG:
				if (parser.getName().compareTo(CapabilitiesTags.COMPLEXCONTENT)==0)
				{
					parseComplexContent(parser); 
				}   
				break;
			case KXmlParser.END_TAG:
				if (parser.getName().compareTo(CapabilitiesTags.COMPLEXTYPE) == 0)
					end = true;
				break;
			case KXmlParser.TEXT:                   
				break;
			}
			if (!end){
				currentTag = parser.next();
			}	
		}
	}
	
	private void parseComplexContent(WFSSchemaParser parser) throws IOException, XmlPullParserException
	{   
		int currentTag;
		boolean end = false;
		
		parser.require(KXmlParser.START_TAG, null, CapabilitiesTags.COMPLEXCONTENT);
		currentTag = parser.next();
		
		while (!end) 
		{
			switch(currentTag)
			{
			case KXmlParser.START_TAG:
				if (parser.getName().compareTo(CapabilitiesTags.EXTENSION )==0)
				{
					parseExtension(parser); 
				}   
				break;
			case KXmlParser.END_TAG:
				if (parser.getName().compareTo(CapabilitiesTags.COMPLEXCONTENT) == 0)
					end = true;
				break;
			case KXmlParser.TEXT:                   
				break;
			}
			if (!end){
				currentTag = parser.next();
			}	
		}
	}
	
	private void parseExtension(WFSSchemaParser parser) throws IOException, XmlPullParserException
	{   
		int currentTag;
		boolean end = false;
		
		parser.require(KXmlParser.START_TAG, null, CapabilitiesTags.EXTENSION);
		currentTag = parser.next();
		
		while (!end) 
		{
			switch(currentTag)
			{
			case KXmlParser.START_TAG:
				if (parser.getName().compareTo(CapabilitiesTags.SEQUENCE)==0)
				{
					parseSequence(parser); 
				}   
				break;
			case KXmlParser.END_TAG:
				if (parser.getName().compareTo(CapabilitiesTags.EXTENSION) == 0)
					end = true;
				break;
			case KXmlParser.TEXT:                   
				break;
			}
			if (!end){
				currentTag = parser.next();
			}			
		}
	}
	
	private void parseSequence(WFSSchemaParser parser) throws IOException, XmlPullParserException
	{   
		int currentTag;
		boolean end = false;
		
		parser.require(KXmlParser.START_TAG, null, CapabilitiesTags.SEQUENCE);
		currentTag = parser.next();
		
		Hashtable htFields = new Hashtable();
		
		while (!end) 
		{
			switch(currentTag)
			{
			case KXmlParser.START_TAG:
				if (parser.getName().compareTo(CapabilitiesTags.ELEMENT)==0)
				{
					WFSAttribute field = new WFSAttribute();
					field.parse(parser);
					if (field.getName() != null){
						htFields.put(field.getName(),field);
					}
				}   
				break;
			case KXmlParser.END_TAG:
				if (parser.getName().compareTo(CapabilitiesTags.SEQUENCE) == 0)
					end = true;
				break;
			case KXmlParser.TEXT:                   
				break;
			}
			if (!end){
				currentTag = parser.next();
			}
		}
		this.setFields(htFields);
		
	}

	public File parseGetFeature(File file) {
		FileReader reader = null;       
		try
		{
			reader = new FileReader(file);
			BufferedReader br = new BufferedReader(reader);
			char[] buffer = new char[100];
			br.read(buffer);
			StringBuffer st = new StringBuffer(new String(buffer));
			String searchText = "encoding=\"";
			int index = st.indexOf(searchText);
			if (index>-1) {
				st.delete(0, index+searchText.length());
				encoding = st.substring(0, st.indexOf("\""));
			}
		} catch (FileNotFoundException ex) {
			ex.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		int tag;
		WFSSchemaParser parser = null;
		parser = new WFSSchemaParser();
		try
		{
			parser.setInput(new FileInputStream(file), encoding);        
			parser.nextTag();
			
			if ( parser.getEventType() != KXmlParser.END_DOCUMENT ) 
			{     
				parser.setSchemaFromMainTag(parser.getName());
				     
				parser.require(KXmlParser.START_TAG, null, CapabilitiesTags.WFS_SCHEMAROOT); 
				tag = parser.nextTag();
				while(tag != KXmlParser.END_DOCUMENT)
				{
					switch(tag)
					{
					case KXmlParser.START_TAG:
						if (parser.getName().compareTo(CapabilitiesTags.COMPLEXTYPE)==0)
						{							
							parseComplexType(parser);
						} 						
						break;
					case KXmlParser.END_TAG:                            
						break;
					case KXmlParser.TEXT:
						if (parser.getName()!=null)
							System.out.println("[TEXT]["+parser.getText().trim()+"]");                         
						break;
					}
					tag = parser.next();
				}
				parser.require(KXmlParser.END_DOCUMENT, null, null);                
			}
		}
		catch(XmlPullParserException parser_ex){
			System.out.println(parser_ex.getMessage());
			parser_ex.printStackTrace();
			return null;
		}
		catch (IOException ioe) {           
			ioe.printStackTrace();
			return null;
		}
		return null;
	}
	
}
